/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  
 *  Modifications :
 *  2006 Creation P.Delrieu
 * 
 */
package org.openmobileis.module.core;

import java.util.Enumeration;
import java.util.Hashtable;

import org.openmobileis.common.util.OpenMISSerializable;
import org.openmobileis.common.util.collection.Array;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: define a module managed by the ProfilManager. A module is a set of
 * files and classes synchronized as a whole. 
 * A module is made of rubric. A rubric is a subset of a module.
 * Properties can be attached to a modules.
 * To define module data use the profile xml file.
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class ProfileModule  implements OpenMISSerializable {
  static final long serialVersionUID = 5521257935120563452L;
	
	private String name;
	private String version;
    private String terminalType;
	private Hashtable properties;
    private String serverjar;
    private String terminaljar;
	private Array rubrics;
    private Array dependModuleList;
	
	public ProfileModule() {
		this.properties = new Hashtable();
		this.rubrics = new Array();
    dependModuleList = new Array();
	}
  
  public void addDependProfilModule(DependProfileModule moduleName) {
    dependModuleList.add(moduleName);
  }
  
  public Array getDependModuleList()  {
    return dependModuleList;
  }
  
	public void setName(String name) {
		this.name = name;
	}
	
	public String getName() {
		return this.name;
	}
	
	public void setVersion(String version) {
		this.version = version;
	}
	
	public String getVersion() {
		return this.version;
	}
	
	public void addProperty(String name, String value) {
		properties.put(name, value);
	}
	
	public String getProperty(String name) {
		return (String)this.properties.get(name);
	}
	
	public String[] getAllPropertiesNames() {
		Enumeration enumkeys = this.properties.keys();
		Array keys = new Array();
		while (enumkeys.hasMoreElements()) {
			keys.add((String)enumkeys.nextElement());
		}
		String[] result = new String[keys.size()];
		keys.toArray(result);
		return result;
	}
	
	public void addModuleRubric(ProfileModuleRubric rubric) {
		this.rubrics.add(rubric);
	}
	
	public Array getModuleRubrics() {
		return this.rubrics;
	}

  public String getServerjar() {
    return serverjar;
  }

  public void setServerjar(String serverjar) {
    this.serverjar = serverjar;
  }

  public String getTerminaljar() {
    return terminaljar;
  }

  public void setTerminaljar(String terminaljar) {
    this.terminaljar = terminaljar;
  }

  public String getTerminalType() {
    return terminalType;
  }

  public void setTerminalType(String terminalType) {
    this.terminalType = terminalType;
  }

}
