/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  
 *  Modifications :
 *  2006 Creation P.Delrieu
 * 
 */
package org.openmobileis.module.core;

import java.util.Enumeration;
import java.util.Hashtable;

import org.openmobileis.common.util.OpenMISSerializable;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.module.profiles.terminal.ProfileRubricSynchroListener;
import org.openmobileis.modules.profiles.terminal.RubricLoader;

/**
 * Title: OpenMobileIS project source <BR>
 * Description: define a rubric. A rubric is a part of a module. A rubric define
 * services to be loaded and SynchroListener to register. Rubric properties is
 * defined in the profil file. Use the RubricLoader class to define rubric
 * service and listener.
 * 
 * @author Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class ProfileModuleRubric  implements OpenMISSerializable {
  static final long serialVersionUID = 5521257935120563452L;

  private String name;

  private Hashtable properties;

  private String loaderClass;

  private Array synchroListenerList;

  private String[] dependsList;

  public ProfileModuleRubric() {
    this.properties = new Hashtable();
    synchroListenerList = new Array();
    dependsList = new String[0];
  }

  public void setName(String name) {
    this.name = name;
  }

  public String getName() {
    return this.name;
  }

  public void addSynchroListener(ProfileRubricSynchroListener synchrolistener) {
    synchroListenerList.add(synchrolistener);
  }

  public Array getSynchroListenerList() {
    return synchroListenerList;
  }

  public void addProperty(String name, String value) {
    properties.put(name, value);
  }

  public String getProperty(String name) {
    return (String) this.properties.get(name);
  }

  public String[] getAllPropertiesNames() {
    Enumeration enumkeys = this.properties.keys();
    Array keys = new Array();
    while (enumkeys.hasMoreElements()) {
      keys.add((String) enumkeys.nextElement());
    }
    String[] result = new String[keys.size()];
    keys.toArray(result);
    return result;
  }

  public String getLoaderClass() {
    return loaderClass;
  }

  public void setLoaderClass(String loader) {
    this.loaderClass = loader;
  }

  public String[] getDependsList() {
    return dependsList;
  }

  public void setDependsList(String[] dependsList) {
    this.dependsList = dependsList;
  }

}
