/*
 *        OpenMobileIS - a free Java Framework for mobile applications
 *
 *   Copyright (C) 2002  Philippe Delrieu.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Philippe Delrieu kept the rigth to distribute all code Copyrighted by philippe Delrieu
 *  under other licence term even commercial one.
 *  
 *  Modifications :
 *  2002 Creation P.Delrieu
 * 
 */

package org.openmobileis.module.profiles.server;

import org.openmobileis.common.user.User;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.user.profile.Profile;
import org.openmobileis.common.user.profile.ProfileDataManager;
import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.context.*;
import org.openmobileis.module.profiles.terminal.OpenMSPProfileSyncListener;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener;
import org.openmobileis.synchro.security.auth.Credential;



/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Default implentation of the ProfilManagerService.
 * Use to manage profil synchronisation.

 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class ProfilManagerSyncTarget extends DefaultOpenMSPSynchroTargetListener {

  public ProfilManagerSyncTarget() {
  }

  public String getTargetName() {
    return OpenMSPProfileSyncListener.profilSynchroTargetName;
   }

  protected ContainerMessage processMapCommand(Credential cred,ContainerMessage mapContainer)   throws OpenMSPException {
    throw new OpenMSPException("MAP command not implemented.");
  }
  
  protected void notifySyncAction(long sessionId,Credential cred, Command syncCommand)  throws OpenMSPException {
    //do nothing. Sync action is managed in the GET command.
  }

  protected  void processResultCommand(Credential cred,Result resultCommande)  throws OpenMSPException  {
    throw new OpenMSPException("RESULT command not implemented.");
  }

  protected void processStatusCommand(Credential cred,Status statusCommande)  throws OpenMSPException   {
    throw new OpenMSPException("STATUS not implemented.");
  }

  protected Status processAddCommand(Credential cred, ContainerMessage addContainer) throws OpenMSPException    {
    throw new OpenMSPException("ADD command not implemented.");
  }

  protected Status processDeleteCommand(Credential cred, ContainerMessage deleteContainer) throws OpenMSPException  {
    throw new OpenMSPException("DELETE command not implemented.");
  }
 
  protected Status processReplaceCommand(Credential cred, ContainerMessage replaceContainer) throws OpenMSPException    {
    throw new OpenMSPException("REPLACE command not implemented.");
  }

  protected void beginProcessOpenMSpCommand(Credential cred)  throws OpenMSPException   {
  }

  protected ContainerMessage[] endProcessOpenMSpCommand()  throws OpenMSPException  {
    //do nothing.
    return null;
  }

  protected ContainerMessage[] processGetCommand(Credential cred, ContainerMessage getContainer)  throws OpenMSPException   {
    try {
      Element getCommand = getContainer.getElement(); //getCommand is a RequestCommand
   //   long sessionID = getCommand.getSourceSessionID();
      Result resultCommand = new Result(getCommand.getCmdId(), this.getTargetName(), OpenMSPProfileSyncListener.profilSynchroSourceName);
   //   resultCommand.setMetaInformation(Long.toString(newSessionId));
      ContainerMessage resultContainer = new ContainerMessage(resultCommand);
      SessionContext context = SessionContextManager.getManager().getSessionContext();
      User user = UserManager.getManager().getUser(context.getUserGroup(), context.getUserId());
      Profile profil = ProfileDataManager.getManager().getProfil(context.getUserGroup(), user.getProfil());
      String encodedProfil =  new String(GeneralCoder.encodeBase64(profil.toXML().getBytes()));
      DataItem newItem = new DataItem(Element.ITEM, "xmlprofil", encodedProfil, null, null);
      resultContainer.add(newItem);
      return new ContainerMessage[]{resultContainer};
    } catch (Throwable ex)  {
      throw new OpenMSPException(ex);
    }

  }

}