/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  
 *  Modifications :
 *  2006 Creation P.Delrieu
 * 
 */

package org.openmobileis.module.profiles.terminal;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import org.openmobileis.common.user.profile.Profile;
import org.openmobileis.common.user.profile.impl.TerminalXmlFileProfilDataFactory;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.xmlpull.v1.XmlPullParserException;

/**
 * Title: OpenMobileIS project source <BR>
 * Description: Manage profil on the terminal side. Profile are store in a XML file. The filename and path is defined by the PropertiesManager property
 * org.openmobileis.profil.file By default (ie the property is not defined), the file is stored in the directory
 * "user.dir"/WEB-INF/conf/properties/profilData.xml.
 * 
 * @author Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class TerminalProfileManager {

  private static TerminalProfileManager manager;

  private String profilFile;

  private Profile profil = null;

  private TerminalProfileManager() {
    profilFile = PropertiesManager.getManager().getProperty("org.openmobileis.profil.file");
    if (profilFile == null) {
      profilFile = System.getProperty("user.dir") + File.separator + "WEB-INF" + File.separator + "conf" + File.separator + "properties" + File.separator
          + "profildata.xml";
    }
  }

  public static TerminalProfileManager getManager() {
    if (manager == null) {
      synchronized (org.openmobileis.modules.profiles.old.ProfileManager.class) {
        if (manager == null) {
          manager = new TerminalProfileManager();
          try {
            manager.parseFile();
          } catch (Throwable ex) {
            LogManager.traceError(0, ex);
          }
        }
      }
    }
    return manager;
  }

  public void saveProfilFile(String fileData) throws ServiceException {
    try {
      FileWriter out = new FileWriter(this.profilFile);
      try {
        out.write(fileData.toCharArray());
        out.flush();
      } finally {
        out.close();
      }
      this.parseFile();
    } catch (Throwable ex) {
      throw new ServiceException(ex);
    }
  }

  public Profile getProfil() {
    return profil;
  }

  private void parseFile() throws IOException, XmlPullParserException {
    try {
      FileReader reader = new FileReader(this.profilFile);
      try {
        TerminalXmlFileProfilDataFactory parser = new TerminalXmlFileProfilDataFactory(reader);
        this.profil = parser.getParsedProfil();
      } finally {
        reader.close();
      }
    } catch (FileNotFoundException ex) {
      LogManager.traceWarning(0, "Warning Profil file not found.");
    }

  }
}
