/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.module.server;

import java.io.File;
import java.util.HashMap;

import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.openmsp.server.synctarget.SynchroTargerManager;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class ModuleServerManager {
  private static ModuleServerManager manager;
  /**
   * 
   */
  private ModuleServerManager() {
    this.initManager();
  }
  
  public static ModuleServerManager getManager()    {
    if (manager == null)    {
      synchronized(ModuleServerManager.class)   {
        if (manager == null)    {
          manager = new ModuleServerManager();
        }
      }
    }
    return manager;
  }
  
  private void initManager() {
    String repositoryPath = PropertiesManager.getManager().getProperty("org.openmobileis.bundle.module.repositorypath");
    if (repositoryPath == null) {
      LogManager.traceWarning(0, "org.openmobileis.bundle.module.repositorypath not set. Use user.dir/repository");
      repositoryPath = System.getProperty("user.dir")+"/repository";
    }
    File repFile = new File(repositoryPath);
    if (!repFile.exists())  {
      LogManager.traceError(0, "Error, repository path :"+repositoryPath+" does not exist. Stop loading module");
      return;
    }
    if (!repFile.isDirectory())  {
      LogManager.traceError(0, "Error, repository path :"+repositoryPath+" is not a directory. Stop loading module");
      return;
    }
    
    HashMap jarMap = new HashMap(20);
    Array xmlList = new Array(20);
    File[] fileList = repFile.listFiles();
    //TODO to do add subdirectory reading and loading
    for (int i=0; i<fileList.length; i++)   {
      if (fileList[i].getName().endsWith(".jar"))   {
        jarMap.put(fileList[i].getName(), fileList);
      } else if(fileList[i].getName().endsWith(".xml"))   {
        xmlList.add(fileList[i]);
      }
    }
    
    //load xml files
    for (int i=0; i<xmlList.size(); i++)    {
        File toload = (File)xmlList.get(i);
        try   {
        ModuleJarSynchroTarget listener = new ModuleJarSynchroTarget(toload);
        //TODO add support for multiple terminal
        SynchroTargerManager.getManager().addSynchroTargetListener(listener, listener.getVersion(), null);
      } catch(Throwable ex) {
        LogManager.traceError(0, "An exception occurs during the loading of module xml file :"+toload.getName());
        LogManager.traceError(0, ex);
      }
    }
  }

}
