/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.module.terminal;

import java.io.File;

import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.module.core.ProfileModule;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.journal.JournalManager;
import org.openmobileis.synchro.journal.SimpleLogRenderer;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.client.DefaultOpenMSPSyncListener;
import org.openmobileis.synchro.openmsp.client.core.NumSyncManagerDB;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Item;
import org.openmobileis.synchro.openmsp.protocol.Message;
import org.openmobileis.synchro.openmsp.protocol.RequestCommand;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class ModuleJarSyncListener extends DefaultOpenMSPSyncListener {
  private boolean syncGoOn = false;
  protected String jarPath;
  protected String synchroPath;
  protected String moduleName;
  protected ProfileModule module;
  protected String xmlPath;

  /**
   * 
   */
  public ModuleJarSyncListener() {
    super();
    //register the log renderer.
    JournalManager.getManager().registerJournalLogRenderer(new SimpleLogRenderer(this.getSyncName(), this.getSyncName()+" application"));
    jarPath = PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.jar");
    if (jarPath == null)    {
      LogManager.traceWarning(0, "Property org.openmobileis.module.repository.jar not set. Module synchro can't be done.");
    } else  {
      File file = new File(jarPath);
      if (!file.exists()) file.mkdirs();
    }
    
    xmlPath = PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.xml");
    if (xmlPath == null)    {
      LogManager.traceWarning(0, "Property org.openmobileis.module.repository.xml not set. Module synchro can't be done.");
    } else  {
      File file = new File(xmlPath);
      if (!file.exists()) file.mkdirs();
    }
 }
  public void setModule(ProfileModule module)   {
    this.moduleName = module.getName();
    this.module = module;
 }
  
  private String getModuleXmlFileName()   {
   StringBuffer modrep = new StringBuffer(this.xmlPath);
   modrep.append('/');
   modrep.append(this.moduleName);
   modrep.append(".xml");
   return modrep.toString();
  }

  public void startSync(Credential cred, SynchroDescriptor synchrodescriptor) throws OpenMSPException {
    super.startSync(cred, synchrodescriptor);
    this.syncGoOn = true;
    synchroPath = synchrodescriptor.getSynchroConduit().getIntallPath();
 }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#sendData(org.openmobileis.synchro.openmsp.protocol.Message)
   */
  public void sendData(Message message) throws OpenMSPException {
    if (!this.isSyncOK()) {
      return; // return if an error occurs during synchro.
    }
    //send jar last modification date in long.
    if (!syncGoOn) { // synchronize data only for the first call.
    return;
    }
    
    //get modified files
    long ns = NumSyncManagerDB.getManager().getSyncNumberForService(this.getSyncName());

    try {
      Command getCommand = new Command(Element.GET, this.getSyncName(), this.getSyncName());
      getCommand.setSourceSessionID(ns);
      ContainerMessage getContainer = new ContainerMessage(getCommand);
      Item newItem = null;
      newItem = new DataItem(
                  Element.ITEM
                  , "Jar Server file name"
                  , ""
                  , null
                  , null
              );
      getContainer.add(newItem);
      message.add(getContainer);
      syncGoOn = false;
   } catch (Throwable ex)  {
      this.setSynchroStatus(Status.STATUS_FAILED);
      LogManager.trace(new SynchroException("DefaultDbSyncListener "+this.getSyncName()+" Send data exception ", ex));
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#getSyncName()
   */
  public String getSyncName() {
    return this.moduleName;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveMapCommand(org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  public void receiveMapCommand(ContainerMessage mapContainer) throws OpenMSPException {

  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveResultCommand(org.openmobileis.synchro.openmsp.protocol.ContainerMessage, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  public void receiveResultCommand(ContainerMessage resultContainer, ContainerMessage initialCommand) throws OpenMSPException {
    Result resultCommand = (Result)resultContainer.getElement(); //result
    Array copiedFiles = new Array();
    try {
      long newSessionID = Long.parseLong(resultCommand.getMetaInformation());
      while (resultContainer.hasMoreMessage())  {
        DataItem item = (DataItem) resultContainer.nextMessage().getElement();
        String meta = item.getMetaInformation();
        if (meta.equals("jarfile")) {
          this.updateJarFile(item.getData(), copiedFiles); 
        }
        if (meta.equals("xmlfile")) {
          this.updateXmlFile(item.getData()); 
          copiedFiles.add(item.getData());
        }
      }
      
      if (this.isSyncOK())  {
      	ModuleManager.getManager().notifyModuleUpdateAfterSynchro(this.module);
        NumSyncManagerDB.getManager().saveSyncNumberForService(newSessionID, this.getSyncName());
        //notify module update 
        
      }
      
    } catch (Throwable ex)  {
    	//remove all copied jar file to avoid to remove old one. 
    	int size = copiedFiles.size();
    	for (int i=0; i<size; i++){
    		String filename = (String)copiedFiles.get(i);
    		File file = new File(filename);
    		file.delete();
    	}
      throw new OpenMSPException(ex);
    }
  }

  private void updateJarFile (String filename, Array copiedFiles)  {
     String inPath = this.synchroPath+File.separator+"filedir"+File.separator+filename;
     String outPath = this.jarPath+File.separator+filename;
     FileUtilities.moveFile(inPath, outPath);
     copiedFiles.add(outPath);
  }

  private void updateXmlFile (String filename)  {
     String inPath = this.synchroPath+File.separator+"filedir"+File.separator+filename;
     String outPath = this.getModuleXmlFileName();
     FileUtilities.moveFile(inPath, outPath);
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveStatusCommand(org.openmobileis.synchro.openmsp.protocol.Status, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  public void receiveStatusCommand(Status statusCommande, ContainerMessage initialCommand) throws OpenMSPException {
//    int cmlRef = statusCommande.getCmdRef();
    int status = statusCommande.getStatus();
    if (status != Status.STATUS_OK)  {
      this.notifySynchroFailure();
    }   
  }

}
