/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  Creation P.Delrieu
 * 
 */
package org.openmobileis.module.terminal;

import java.io.File;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.db.FODBCollection;
import org.openmobileis.module.core.ProfileModule;
import org.openmobileis.module.core.ProfileModuleRubric;
import org.openmobileis.modules.profiles.terminal.RubricLoader;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.client.EmptyOpenMSPSyncListener;
import org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;
import org.openmobileis.synchro.openmsp.client.core.NumSyncManagerDB;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * @author phil
 *
 */
public final class StaticModuleManagerListener extends EmptyOpenMSPSyncListener implements ModuleManagerListener {
	private static Array collectionToremoveList;
	private boolean restartAppli = false;

	/**
	 * 
	 */
	public StaticModuleManagerListener() {
		collectionToremoveList = new Array();
	}
	
  public void initManager() throws ServiceException	{
  	
  	//copy all synchronized jar to the lib dir.
  	File copyDir = new File(this.getJarInstallPath());
  	File[] jarList = copyDir.listFiles();
  	String baseDestDir = System.getProperty("user.dir")+"/WEB-INF/lib/";
  	for (int i=0; i<jarList.length; i++)	{
  		File temp1 = jarList[i].getAbsoluteFile();
  		String temp2 = baseDestDir+jarList[i].getName();
  		jarList[i].getAbsoluteFile().renameTo(new File(baseDestDir+jarList[i].getName()));
  	}
		OpenMSPSynchroManager.getManager().addListener(this, null);
  }

	/**
	* Return the name used to identiy the of the listener in the synchro processus.
	* This name correspond to the source in the OpenMS protocol.
	*/
	 public String getSyncName()	{
		 return "FWK StaticModuleManagerListener";
	 }

	/* (non-Javadoc)
	 * @see org.openmobileis.module.terminal.ModuleManagerListener#getJarInstallPath()
	 */
	public String getJarInstallPath() {
		return System.getProperty("user.dir")+"/WEB-INF/module/jar";
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.module.terminal.ModuleManagerListener#getXMLInstallPath()
	 */
	public String getXMLInstallPath() {
		return System.getProperty("user.dir")+"/WEB-INF/module/xml";
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.module.terminal.ModuleManagerListener#notifyModuleLoading(org.openmobileis.module.core.ProfileModule)
	 */
	public void notifyModuleLoading(ProfileModule module) throws ServiceException {
		Array rubList = module.getModuleRubrics();
		for (int i=0; i<rubList.size(); i++)	{
			ProfileModuleRubric rubric = (ProfileModuleRubric) rubList.get(i);
			String loaderClassName = rubric.getLoaderClass();
			if (loaderClassName != null)	{
				try	{
					RubricLoader loader = (RubricLoader) Class.forName(loaderClassName).newInstance();
					ServiceManager.getManager().loadRubricLoader(loader);
				} catch (Throwable ex)	{
					LogManager.traceError(0, "StaticModuleManagerListener notifyModuleLoading : Exception during module "+module.getName()+" loading :"+ex.getMessage());
					throw new ServiceException(ex);
				}
			}
		}
		

	}

	/* (non-Javadoc)
	 * @see org.openmobileis.module.terminal.ModuleManagerListener#notifyModuleLoading(org.openmobileis.module.core.ProfileModule)
	 */
	public void notifyModuleUpdate(ProfileModule module) throws ServiceException {
		try	{
			if (FastObjectDBManager.isFodbStarted())	{
				Array rubrics = module.getModuleRubrics(); //all database collection is a rubric in the profil.
				FODBCollection[] collist = FastObjectDBManager.getCurrentFODB().getDatabaseCollectionArray();
				int rubsize = rubrics.size();
				for (int i=0; i<rubsize; i++)	{
					ProfileModuleRubric rubric = (ProfileModuleRubric)rubrics.get(i);
					for (int j=0; j<collist.length; j++)	{
						if (rubric.getName().equals(collist[j].getName()))	{
							//validate collection synchro OK.
							OpenMSPSyncListener listener = OpenMSPSynchroManager.getManager().getListenerByName(collist[j].getName());
							if (listener != null){ //synchronized collection
								if (!listener.isSyncOK())	{							
									throw new ServiceException(module.getName()+" collection "+collist[j].getName()+" synchro error. Module can't be updated. Remove synchro module files");
								}
								// add collecion to the remove list.
								collectionToremoveList.add(collist[j].getName());
								//force complete synchro for the listener.
				        NumSyncManagerDB.getManager().saveSyncNumberForService(0, collist[j].getName());
							}
						}
					}
				}
			}
		} catch (Exception ex)	{
			throw new ServiceException(ex);
		}
		this.restartAppli = true;
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#startSync(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.client.SynchroDescriptor)
	 */
	public void startSync(Credential cred, SynchroDescriptor synchrodescriptor) throws OpenMSPException {
		this.restartAppli = false;
		collectionToremoveList.clear();
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#endSync()
	 */
	public void endSync() throws OpenMSPException {
		if (this.restartAppli)	{
			Thread thread = new Thread(new Runnable()	{
				public void run() {
					try	{
						Thread.currentThread().sleep(10000); //way 10 second the synchronisation finised.
					} catch (Throwable ex)	{
						LogManager.traceError(0, ex);
					}
					System.exit(0);
				};
			});
			thread.start();
//			remove fodb collection before stop.
//			restart appli. show return page and after stop
//			ajout du message l'application va tre redmarrer dans quelques instant veuillez patienter.
//			Ajouter au redmarrage l'ajout du message, pour terminer la mise  jour vous devez raliser une nouvelle synchronisation.
		}
	}

}
