package org.openmobileis.oscar.bundlemanager;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.file.ZipUtilities;
import org.openmobileis.modules.profiles.old.ProfileManager;
import org.openmobileis.modules.profiles.old.ProfileModule;
import org.openmobileis.oscar.OscarBundleUtils;
import org.osgi.framework.BundleContext;

public class OscarBundleManager {
	
	private static BundleContext context = null;
	private static OscarBundleManager manager = null;
	private static OscarBundleUtils utils;
	
	private String packagesPath = null;
	
	private OscarBundleManager() {
		utils = new OscarBundleUtils(context, System.getProperty("user.dir")+File.separator+"WEB-INF"+File.separator+"bundles"+File.separator);
		this.packagesPath = System.getProperty("user.dir")+File.separator+"synchro"+File.separator+"packages"+File.separator;
	}
	
	public static OscarBundleManager getManager() {
		if (manager == null) {
	        synchronized(OscarBundleManager.class) {
	            if (manager == null)  {
	              manager = new OscarBundleManager();
	            }
	        }

		}
		return manager;
	}
	
	public static void setContext(BundleContext context) {
		OscarBundleManager.context = context;
		if (utils != null) {
			utils.setContext(context);
		} else {
			OscarBundleManager.getManager();
		}
	}
	
	private void checkUtils() throws ServiceException {
		if (utils == null) {
			throw new ServiceException("OscarBundleManager: utils no loaded");
		}
		
	}
	
	public void startBundles() throws ServiceException {
		ServiceException firstException = null;
		Array modules = ProfileManager.getManager().getProfilModules();
		if (modules != null) {
			for (int i = 0; i < modules.size(); i++) {
				ProfileModule module = (ProfileModule)modules.get(i);
				try {
					System.out.println("********* DEBUG "+module.getName());
					if (isLoadablePackage(module.getName())) {
						this.loadPackage(module.getName());
					}
					this.install(module.getName());
					this.start(module.getName());
				} catch (ServiceException ex) {
					System.out.println("Unable to start bundle : "+module.getName());
					if (firstException == null) {
						firstException = ex;
					}
				}
			}
		}
		
		if (firstException != null) {
			throw firstException;
		}
	}
	
	public void install(String name) throws ServiceException {
		this.checkUtils();
		utils.installWithFileName(name);
	}
	
	public void update(String name) throws ServiceException {
		this.checkUtils();
		utils.updateBundleWithFileName(name);
	}
	
	public void uninstall(String name) throws ServiceException {
		this.checkUtils();
		utils.uninstallBundleWithFileName(name);
	}
	
	public void start(String name) throws ServiceException {
		this.checkUtils();
		utils.startBundleWithFileName(name);
	}
	
	public void stop(String name) throws ServiceException {
		this.checkUtils();
		utils.stopBundleWithFileName(name);
	}
	
	public void restart(String name) throws ServiceException {
		this.checkUtils();
		utils.restartBundleWithFileName(name);
	}
	
	public void refresh(String name) throws ServiceException {
		this.checkUtils();
		utils.refreshBundleWithFileName(name);
	}
	
	public void shutdown() throws ServiceException {
		this.checkUtils();
		utils.shutdown();
	}
	
	public void loadPackage(String packageName) throws ServiceException {
		File packageFile = new File(this.packagesPath+packageName+".pkg");
		if (packageFile.length() > Integer.MAX_VALUE) {
			// File is too large
			throw new ServiceException("File is too long : "+packageFile.length()+" bytes !!!");
		}
		try {
			FileInputStream instr = new FileInputStream(packageFile);
			BufferedInputStream bufstr = new BufferedInputStream(instr);
			ZipUtilities.unZipData(bufstr, System.getProperty("user.dir")+File.separator);
			packageFile.delete();
		} catch (IOException ex) {
			throw new ServiceException(ex);
		}
	}
	
	public boolean isLoadablePackage(String packageName) {
		File packageFile = new File(this.packagesPath+packageName+".pkg");
		if (packageFile.exists() && packageFile.canRead() && packageFile.isFile()) {
			return true;
		}
		return false;
	}
	
	public boolean isInstalledBundle(String name) {
		return false;
	}
	
}
