/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.common.context;

import java.util.*;

/**
 * Application Context are use to share application specific data between application objects.
 * Context is not persistent.
 * The application context can be access using the ApplicationContextManager class.
 * @author	Philippe Delrieu
 * @author Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0
 */
public class ApplicationContext {

	private Hashtable contextMap = null;

	private String[] applicationExecArguments;
  private Plateform plateform;

	public ApplicationContext() {
		contextMap = new Hashtable(10);
    plateform = new DefaultPlateform();
}

	/** 
	 * add the object with specified name to the context
	 *
	 * @param objectName : object name
	 * @param object : object added to the context.
	 */
	public void addObject(String objectName, Object object) {
		if ((objectName != null) && (object != null)) {
			contextMap.put(objectName, object);
		}
	}

	/** 
	 * remove the specified object name from the context
	 *
	 * @param objectName : object name to remove
	 */
	public void removeObject(String objectName) {
		if (objectName != null) {
			contextMap.remove(objectName);
		}
	}

	/** 
	 * return the application context with specified name
	 *
	 * @param objectName : object name to return
	 * @return return the object with specified object name or null if not found.
	 */
	public Object getObject(String objectName) {
		if (objectName != null) {
			return contextMap.get(objectName);
		}
		return null;
	}

	/** 
	 * return the application arguments if initialized
	 *
	 * @return return the application parameters if initialized
	 */
	public String[] getApplicationExecParameters() {
		return applicationExecArguments;
	}

	/** 
	 * set the application arguments
	 */
	public void setApplicationExecArguments(String[] args) {
		applicationExecArguments = args;
	}

  /**
   * return the plateform description of the hardware where the application is executed.
   * By default the DefaultPlateform instance is return.
   * @return
   */
  public Plateform getPlateform() {
    return plateform;
  }


  /**
   * set the plateform description of the hardware where the application is executed.
   * If not set, by default the DefaultPlateform instance is set.
   * @return
   */
 public void setPlateform(Plateform plateform) {
    this.plateform = plateform;
  }
 
 /**
  * return objects'keys in the context Map
  * @return
  */
 public Set getObjectNames()  {
	return contextMap.keySet();
  }

}