/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */


package org.openmobileis.common.context;

import org.openmobileis.common.util.collection.Array;

/**
 * Hold the application context. Singleton classe that provide access to the application Context.
 * Application Context are use to save application specific data. These data are share among all application classes
 * Data are not persistent.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 * @see ApplicationContext
 */
public class ApplicationContextManager {
	private static ApplicationContextManager manager = null;

	private ApplicationContext applicationContext;

	private Array managerList;

	public ApplicationContextManager() {
		managerList = new Array(10);
		applicationContext = new ApplicationContext();
 	}

	public final static ApplicationContextManager getManager() {
		if (manager == null) {
			synchronized (org.openmobileis.common.context.ApplicationContextManager.class) {
				if (manager == null) {
					manager = new ApplicationContextManager();
				}
			}

		}
		return manager;
	}

	/** 
	 * facility use to reference application singleton manager to avoid to have them garbage collected
	 *
	 * @param objectName : object a manager
	 */
	public void addManager(Object manager) {
		managerList.add(manager);
	}

	public void setApplicationContext(ApplicationContext context) {
		applicationContext = context;
	}

	public ApplicationContext getApplicationContext() {
		return applicationContext;
	}
}