/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.common.context;

import java.util.Enumeration;
import java.util.Properties;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class DefaultPlateform implements Plateform {
  public String os;
  public String osversion;
  public String jvm;
  public String jvmspec;
  /**
   * 
   */
  public DefaultPlateform() {
    super();
    //manage OS
    Properties props = System.getProperties();
    String osname = props.getProperty("os.name");
    if (osname.indexOf("Linux") != -1)  {
      this.os = Plateform.LINUXOS;
    } else  if (osname.indexOf("Windows CE")!=-1) {
 //   } else  if (osname.indexOf("PPC")!=-1) {
      this.os = Plateform.POCKETPCOS;
    } else  if (osname.indexOf("Windows")!=-1) {
      this.os = Plateform.WINOS;
    } else  {
      this.os = Plateform.UNKNOWNOS;
    }
    
    //os version
    this.osversion = props.getProperty("os.version");
    
    //jvm vendor
    this.jvm = props.getProperty("java.vendor");
    
    //jvm spec
    this.jvmspec = props.getProperty("java.specification.version");
    
    //trace info for test
 //   this.main(null);
    
    
  }

  /* (non-Javadoc)
   * @see org.openmobileis.common.context.Plateform#getOS()
   */
  public String getOS() {
     return this.os;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.common.context.Plateform#getOSVersion()
   */
  public String getOSVersion() {
    return this.osversion;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.common.context.Plateform#getJVM()
   */
  public String getJVMVendor() {
    return this.jvm;
  }
  
  /**
   * Return the JVM Specification version.
   * Default implementation return the JVM property java.specification.version
   * @return
   */
  public String getJVMSpecificationVersion()  {
    return this.jvmspec;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.common.context.Plateform#getOpenMobileISMajorVersion()
   */
  public String getOpenMobileISMajorVersion() {
    return "1";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.common.context.Plateform#getOpenMobileISMinorVersion()
   */
  public String getOpenMobileISMinorVersion() {
   return "R2";
  }
  
  public static void main(String[] args) {
    Properties props = System.getProperties();
    Enumeration enume = props.keys();
    while (enume.hasMoreElements()) {
      String key = (String) enume.nextElement();
      System.out.println("Property :"+key+" value:"+props.getProperty(key));
    }
  }

}
