/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.common.context;

import java.util.HashMap;
import java.util.Iterator;


public class SessionContextManager {


  private static SessionContextManager manager;
  private HashMap sessionListbyId;
  private HashMap sessionListbythread;
  private static long oldID = 0;
  
  
  public SessionContextManager() {
    super();
    sessionListbyId = new HashMap(10);
    sessionListbythread = new HashMap(10);
    
  }
  
  public final static void registerManager(SessionContextManager newManager) {
    manager = newManager;
  }
	
  public final static SessionContextManager getManager()  {
    if (manager == null)  {  
      manager = new SessionContextManager();
	}
	  return manager;
  }

  /**
   * return the session context with the specified id. return null if not found
   */
  public synchronized SessionContext getSessionContext(String id) {
    if (id != null) {
      SessionContext session = (SessionContext) sessionListbyId.get(id);
      if (session == null)  {
        session = this.createSessionContext(id);
      } else {
        session.SetIsNew(false);
      }
      return session;
    }
    return null;
  }

  /**
   * return the session context associted with the currentThread.
   * Null if not session is associated with the current thread.
   * The method joinSessionContext must have been call before.
   */
  public synchronized SessionContext getSessionContext() {
    SessionContext session = (SessionContext) sessionListbythread.get(Thread.currentThread());
    if (session != null)  {
      session.SetIsNew(false);
    } else  {
      String id= "OPENMISCONTEXT"+Long.toString(getNewID());
      session = this.createSessionContext(id);
      sessionListbythread.put(Thread.currentThread(), session);
    }
    return session;
  }
  
  /**
   * create a new session context with the specified Id.
   */
  public synchronized SessionContext createSessionContext(String id) {
    SessionContext session = null;
    if ((id != null) && (!sessionListbyId.containsKey(id))) {
      session = this.realCreateSessionContext(id);
      session.SetIsNew(true);
      sessionListbyId.put (id, session);
    } else  if (id != null) {
      session = this.getSessionContext(id);
    }
    return session;
  }
  
  protected SessionContext realCreateSessionContext(String id) {
    return new SessionContext(id);
  }
  
  
  /**
   * Join the current thread to the SessionContext with the specified id. If no SessionContext has the id
   * create it. A thread can only join one SessionContext.
   */
  public synchronized void joinSessionContext(String id) {
    SessionContext session = (SessionContext) sessionListbyId.get(id);
    if (session == null)  {
      session = createSessionContext(id);
    }
    sessionListbythread.put(Thread.currentThread(), session);
  }
  
  
  public synchronized void invalidateSession(String id)  {
    SessionContext session = (SessionContext) sessionListbyId.get(id);
    if (session != null)  {
      if (sessionListbythread.containsValue(session))  {
        Iterator iter = sessionListbythread.keySet().iterator();
        while (iter.hasNext())  {
          Object obj = iter.next();
          SessionContext se = (SessionContext) sessionListbythread.get(obj);
          if (se.getId().equals(id))  {
            sessionListbythread.remove(obj);
          }
        }
      }
      sessionListbyId.remove(id);
    }
  }
  
  
  /**
   * After a thread has use a session context. Before ending it must leave the session context.
   */
  public synchronized void leaveSessionContext() {
    sessionListbythread.remove(Thread.currentThread());
  }

	
  static synchronized long getNewID()	{
    long id = System.currentTimeMillis() - (943293708615L);  // to have int conversion positive
    while (id < oldID)	{
      id ++;
    }
    oldID = ++id;
    	return oldID;
  }

}
