/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.common.user.profil;

import org.openmobileis.common.util.OpenMISSerializable;

/**
 * Struct definition : ProfilProperty
 * 
*/
public final class ProfileProperty implements OpenMISSerializable  {
  static final long serialVersionUID = 5521257935120563452L;
    /**
     * Struct member key
     */
    public String key;

    /**
     * Struct member value
     */
    public String value;
    
    /**
     * name of the profil.
     */
    public String moduleName;

    /**
     * Default constructor
     */
    public ProfileProperty()   { }

    /**
     * Constructor with fields initialization
     * @param key key struct member
     * @param value value struct member
     */
    public ProfileProperty(String key, String value)    {
        this.key = key;
        this.value = value;
    }
    
    public boolean equals(Object obj) {
      if (obj instanceof ProfileProperty) {
        ProfileProperty nobj = (ProfileProperty) obj;
        if (nobj.key.equals(this.key) && nobj.value.equals(this.value))    {
          //validate modulename
          if (nobj.moduleName != null && this.moduleName == null) {
            return false;
          } else  if (nobj.moduleName == null && this.moduleName != null)  {
            return false;
          } else if (nobj.moduleName != null && this.moduleName != null) {
            if (!nobj.moduleName.equals(this.moduleName))  {
              return false;
            }
          }
          return true;
        }
      }
      return false;
    }
    
    public int hashCode() {
      StringBuffer buff = new StringBuffer(this.key);
      buff.append("/");
      buff.append(this.value);
      if (this.moduleName != null)  {
        buff.append(":");
        buff.append(this.moduleName);
      }
      return buff.hashCode();
    }

}
