/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.common.user.profile;

import org.openmobileis.common.user.profile.ProfileProperty;
import org.openmobileis.common.user.profile.ProfileRubric;
import org.openmobileis.common.util.OpenMISSerializable;
import org.openmobileis.common.util.collection.Array;

/**
 * Struct definition : Profil
 * 
*/
public final class Profile implements OpenMISSerializable {
  static final long serialVersionUID = 5521257935120563452L;
    /**
     * Struct member userID
     */
    public String userID;

    /**
     * Struct member profilProperties
     */
    public ProfileProperty[] profilProperties;

    /**
     * Struct member profilRubric
     */
    public ProfileRubric[] profilRubric;
    
    /**
     * name of the profil.
     */
   public String profilName;

    /**
     * Default constructor
     */
    public Profile()  { }

    /**
     * Constructor with fields initialization
     * @param userID userID struct member
     * @param profilProperties profilProperties struct member
     * @param profilRubric profilRubric struct member
     * @param completeSynchro completeSynchro struct member
     */
    public Profile(String userID, ProfileProperty[] profilProperties, ProfileRubric[] profilRubric)  {
        this.userID = userID;
        this.profilProperties = profilProperties;
        this.profilRubric = profilRubric;
    }
    
    public String toXML()   {
      StringBuffer buff = new StringBuffer();
      buff.append("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n\t");
      buff.append("<PROFILLIST>");
      buff.append("<PROFILNAME name=\""+this.profilName+"\">");
      for (int i=0; i<profilRubric.length; i++) {
        buff.append("<ENTRY moduleName=\""+profilRubric[i].moduleName+"\" rubricName=\""+profilRubric[i].rubricName+"\"");
        if (profilRubric[i].description != null)    {
          buff.append(" description=\""+profilRubric[i].description+"\"");
        }
        if (profilRubric[i].version != null)    {
          buff.append(" version=\""+profilRubric[i].version+"\"");
        }
        buff.append(" />");
      }
      for (int i=0; i<profilProperties.length; i++) {
        buff.append("<PROPERTY key=\""+profilProperties[i].key+"\" value=\""+profilProperties[i].value+"\" ");
        if (profilProperties[i].moduleName!= null)    {
          buff.append(" module=\""+profilProperties[i].moduleName+"\"");
        }
        buff.append(" />");
      }
      buff.append("</PROFILNAME>");
      buff.append("</PROFILLIST>");
      return buff.toString();
    }
    
    public boolean equals(Object obj) {
      if (obj instanceof Profile) {
        Profile nobj = (Profile) obj;
        if (nobj.profilName.equals(this.profilName))    {
           //validate profil rubric
          Array thisrubric = new Array(this.profilRubric);
          for (int i=0; i<nobj.profilRubric.length; i++)    {
            if (!thisrubric.contains(nobj.profilRubric[i])) {
              return false;
            }
          }
          Array nobjrubric = new Array(nobj.profilRubric);
          for (int i=0; i<this.profilRubric.length; i++)    {
            if (!nobjrubric.contains(this.profilRubric[i])) {
              return false;
            }
          }
          //validate profil properties
          Array thisprop = new Array(this.profilProperties);
          for (int i=0; i<nobj.profilProperties.length; i++)    {
            if (!thisprop.contains(nobj.profilProperties[i])) {
              return false;
            }
          }
          Array nobjprop= new Array(nobj.profilProperties);
          for (int i=0; i<this.profilProperties.length; i++)    {
            if (!nobjprop.contains(this.profilProperties[i])) {
              return false;
            }
          }
          
          return true;
        }
      }
      return false;
    }
    
    public int hashCode() {
      return profilName.hashCode();
    }

}
