/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.common.user.profile;

import java.util.Hashtable;

import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.user.profile.Profile;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Manage profil data access. Hide real implementation.

 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class ProfileDataManager {
  private static ProfileDataManager manager;
  private Hashtable factoryList;
  
  private ProfileDataManager() {
    factoryList = new Hashtable();
  }

  public static ProfileDataManager getManager() {
    if (manager == null)  {
      synchronized (UserManager.class) {
        if (manager == null)  {
          manager = new ProfileDataManager();
          ApplicationContextManager.getManager().addManager(manager);
        }
      }
    }
    return manager;
  }
  
  public void registerProfilDataFactoryForGroup(String group, ProfileDataFactory factory)	{
    factoryList.put(group, factory);
  }

  /**
   * return the idl profil for specified profil name.
   * null if not found
   */
  public Profile getProfil(String group, String profilName) throws ProfileNotFoundException {
    ProfileDataFactory factory = (ProfileDataFactory) factoryList.get(group);
    if (factory != null)  {
      return factory.getProfil(profilName);
    } else  {
      throw new ProfileNotFoundException("No ProfilDataFactory registered for group :"+group);
    }
  }

}