/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.common.user.profile.impl;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

import java.util.*;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.openmobileis.common.user.profile.Profile;
import org.openmobileis.common.user.profile.ProfileDataFactory;
import org.openmobileis.common.user.profile.ProfileProperty;
import org.openmobileis.common.user.profile.ProfileRubric;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.log.*;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.common.util.exception.DefaultException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Manage profil data in an xml file.

 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class XmlFileProfilDataFactory implements ProfileDataFactory {
  private HashMap profilList;

  class ProfilListParser  extends DefaultHandler  {
    private String currentProfilName;
    private HashMap rubricList;
    private ArrayList propertyList;

    ProfilListParser()  {
      rubricList = new HashMap(10);
      propertyList = new ArrayList(10);
    }

    /** Start document. */
    public void startDocument() {
    } // startDocument()

    /** Start element. */
    public void startElement(java.lang.String uri,
                         java.lang.String qname,
                         java.lang.String name,
                         Attributes attrs)
                  throws SAXException {
      if (name.equals("PROFILNAME")) {
        rubricList.clear();
        propertyList.clear();
        currentProfilName = attrs.getValue("name");
        if (currentProfilName == null)  {
          throw new SAXException("TAG PROFILNAME with no attribut name");
        }
      } else if (name.equals("ENTRY"))  {
        String module = attrs.getValue("moduleName");
        if (module == null)  {
          module = attrs.getValue("channelName"); //put here for back compatibility with version before 1.0R5
         if (module == null) throw new SAXException("TAG ENTRY with no attribut moduleName");
        }
        String rubric = attrs.getValue("rubricName");
        if (rubric == null)  {
          throw new SAXException("TAG ENTRY with no attribut rubricName");
        }
        ProfileRubric rub = new ProfileRubric(module, rubric, null, new ProfileProperty[0]);
        String description = attrs.getValue("description");
        if (description != null)  {
          rub.description = description;
        } else  {
          rub.description = "";
        }
        rubricList.put(module+"/"+rubric, rub);
     } else if (name.equals("PROPERTY")) {
        String key = attrs.getValue("key");
        if (key == null)  {
          throw new SAXException("TAG PROPERTY with no attribut key for property");
        }
        String value = attrs.getValue("value");
        if (value == null)  {
          throw new SAXException("TAG PROPERTY with no attribut value for property");
        }
        ProfileProperty prop = new ProfileProperty(key, value);
        String moduleName = attrs.getValue("module");
        if (moduleName != null)  {
          prop.moduleName = moduleName;
        }
        propertyList.add(prop);
      }
    } // startElement(String,AttributeList)

    public void endElement (String uri, String qname, String localName) throws SAXException {
      if (localName.equals("PROFILNAME")) {
        ProfileRubric[] rubs = new ProfileRubric[rubricList.size()];
        rubricList.values().toArray(rubs);
        ProfileProperty[] props = new ProfileProperty[propertyList.size()];
        propertyList.toArray(props);
        Profile newidlprofil = new Profile("0", props, rubs);
        profilList.put(currentProfilName, newidlprofil);
      }
    }


    /** Characters. */
    public void characters(char ch[], int start, int length) {
    }
  }

  public XmlFileProfilDataFactory() {
    profilList = new HashMap(10);
    try {
      String file =  PropertiesManager.getManager().getProperty("org.openmobileis.common.user.profil.xmlprofilfile.path");
      if (file == null) {
        LogManager.traceError(LogServices.DEFAULTSERVICE, "No profil list XML file defined. Profil management is not initialized correctly");
        return;
      }
      file = FileUtilities.convertFileNameToSystem(file);
      ProfilListParser handler = new ProfilListParser();
      SAXParserFactory factory = SAXParserFactory.newInstance();
      // Parse the input 
      SAXParser saxParser = factory.newSAXParser();
      saxParser.parse(new InputSource(new java.io.FileReader(file)), handler);
    } catch (Exception e) {
      LogManager.trace(new DefaultException ("Problem occurs during loading profil list file", e));
    }
  }

  /**
   * return the profil for specified profil name.
   * null if not found
   */
  public Profile getProfil(String profilName)  {
    return (Profile) profilList.get(profilName);
  }
}