/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.common.util;

import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Generate an unique id for the PDA based  on the current time.
 * ID is not unique between several PDA. To have an unique id between PDA, use the method
 * getNewStringID that add the userid to the generated long id.
 * userid is get from SessionContext. 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class UniqueIdGenerator {
  
  private static UniqueIdGenerator manager;
  private static long oldID = 0;

  /** Creates a new instance of UniqueIdGenerator */
  public UniqueIdGenerator() {
  }
  public static UniqueIdGenerator getManager () {
    if (manager == null) {
      synchronized (UniqueIdGenerator.class) {
        if (manager == null)  {
          manager = new UniqueIdGenerator();
        }
      }
    }
    return manager;
  }
  
  public synchronized long getNewID()  {
    long id = System.currentTimeMillis() - (943293708615L);  // to have int conversion positive
    while (id < oldID)  {
      id ++;
    }
    oldID = ++id;
    return oldID;
  }

  
  public String getNewStringID() {
    String longid = Long.toString(this.getNewID());
    SessionContext context = SessionContextManager.getManager().getSessionContext();
    if (context != null)  {
      String userid = context.getUserId();
      if (userid != null) {
        longid = userid+longid;
      }
    }   
    return longid;
  }

}
