/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.common.util.codec;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Base64, QuotedPrintable, rfc2047 codec. Use HTTPClient lib.
 * Use to hide the general coding library use.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
import java.io.*;
import java.io.CharArrayWriter;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.net.QuotedPrintableCodec;

// import HTTPClient.Codecs;
// import HTTPClient.NVPair;

public class GeneralCoder {

  public GeneralCoder() {
  }

  public static byte[] decodeBase64(byte[] inByte) throws IOException{
                  // decode data
  	return Base64.decodeBase64(inByte);
  }

  public static byte[] encodeBase64(byte[] inByte) throws IOException {
  	return Base64.encodeBase64(inByte);
  }

  /**
   * return null if an error occurs.
   */
  public static String decodeQuotedPrintable(String data) {
    try {
      return new String(QuotedPrintableCodec.decodeQuotedPrintable(data.getBytes()));
    } catch (Exception ex)  {
      return null;
    }
  }
  
  public static String encodeQuotedPrintable(String data) {
  	try {
  		return new String(QuotedPrintableCodec.encodeQuotedPrintable(null,data.getBytes()));
  	} catch (Exception ex) {
  		return null;
  	}
  }
  
 /* public static NVPair[] mpFormDataDecode(byte[] data, String cont_type, String dir) {
  	try {
  		return Codecs.mpFormDataDecode(data, cont_type, dir);
  	} catch (Exception ex) {
  		return null;
  	}
  } */
  
  /**
   decode string encoded with RFC 2047. Mostly use for Mime message subject. 
  */
  public static String decoderfc2047(String data){
    try {
      char[] encoded = data.toCharArray();
      CharArrayWriter out = new CharArrayWriter();
      int i=0;
      while (i<encoded.length)  {
        if (encoded[i] == '=')  { // get first '=?
          if ((i<(encoded.length-1)) &&(encoded[i+1] == '?')) {
            // begin decode rfc
            i+=2;
            int step = 0;
            StringBuffer tempString = new StringBuffer();
            char encoding = 0;
            while(i<encoded.length) {
              if (encoded[i] == '?')  {              
                // manage current tempString
                switch (step) {
                  case 0 :
                    //does not manage char set (we suppose its ok
                    tempString.setLength(0);
                    break;
                  case 1 :
                    //manage encoding
                    encoding = tempString.charAt(0);
                    tempString.setLength(0);
                    break;
                  case 2 :
                    //decode content
                    if (encoding == 'Q')  {
                      String content = GeneralCoder.decodeQuotedPrintable(tempString.toString());
                      out.write(content.toCharArray());
                    } else if (encoding == 'B') {
                      byte[] content = GeneralCoder.decodeBase64(tempString.toString().getBytes());
                      out.write(new String(content).toCharArray());
                    }
                    tempString.setLength(0);
                    //end of encoded remove final equals and continue string parsing
                    i++;
                    break;
                }
                step ++;
                i++;
                if (step == 3) break;
              } else {
                tempString.append(encoded[i]);
                i++;
              }
            }
            continue;
          }
        }
        out.write(encoded[i]);
        i++;
      }
      out.close();
      return out.toString();    
    }catch (Throwable ex) {
      return data; // if an error occurs return original data
    }
  }
  
  public static void main(String[] args)  {
    String totest[] = {"From: =?ISO-8859-1?Q?Patrik_F=E4ltstr=F6m?= <paf@nada.kth.se>"
      , ""
      , "normal"
      , "avec un ="
      , "avec un equals = non fin"
      , "=?ISO-8859-1?Q?a" //a
      ,"=?ISO-8859-1?Q?a?= b" //a b
      ,"=?ISO-8859-1?Q?a?= =?ISO-8859-1?Q?b?=" //ab
      ,"=?ISO-8859-1?Q?a?=  =?ISO-8859-1?Q?b?=" //ab
      ,"=?ISO-8859-1?Q?a_b?=" //a b
      ,"=?ISO-8859-1?Q?a?= =?ISO-8859-2?Q?_b?=" //a b
      
    };
    try {
      for (int i=0; i<totest.length; i++) {
        String decode = GeneralCoder.decoderfc2047(totest[i]);
        System.out.println(decode);
      }
    } catch (Throwable ex)  {
      ex.printStackTrace();
    }


  }
}