/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.common.util.log;

import java.util.Properties;
import java.io.PrintStream;

/**
 * Default tracer for log management
 * 		   Use to trace to the console
 * @author P.Delrieu
 * @version 1.0
 * @since JDK1.1
 */
public class DefaultLogTracer implements LogTracer {

  protected PrintStream outStream;

  public DefaultLogTracer() {

  }

  public void setLogTracerProperties(Properties props)  {
  }
  
  /**
  * Set the static outpuStream used by trace methods by default
  * outStream == System.out</code>.
  */
  public void setLogStream(java.io.PrintStream newStream) {
    outStream = newStream;
  }

  /**
   * Trace the message to the stream defined showing the specified priority and service.
   */
  public synchronized void trace(int service, int priority, String message)  {
    outStream.print(new java.util.Date().toString()
      +" Service :"+LogServices.toString(service)
      +" Priority:"+LogPriorities.toString(priority)
      +" Message:"+message+"\n");
     outStream.flush();

  }

  public PrintStream getLogPrintStream()  {
    return outStream;
  }

  /**
  * Called by the system.
  */
  public void finalize() {
    try {
      if (outStream != null)  {
        outStream.flush();
        outStream.close();
      }
    } catch (java.lang.Exception exc) {
      System.out.println("exception: " + exc.getMessage());
      exc.printStackTrace();
    }
  }
}