/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */


package org.openmobileis.common.util.log;

import java.util.Properties;
import java.io.*;

/**
 * Implement a LogTracer that Log in a File. The File is define by specifing the property LOGFILE.
 * Implements an archiving system. Define the property LOGDIRARCHIVE to define the directory where log are archived.
 * @author P.Delrieu
 * @version 1.0
 * @since JDK1.1
 */
public class FileLogTracer extends DefaultLogTracer {

  protected FileOutputStream fileStream;
  protected String logFile;

  public FileLogTracer() {
  }
  
  /**
   * Two property are defined :
   *  LOGFILE : file where log are written.
   *  LOGDIRARCHIVE : directory where old log are archived.
   */
  public void setLogTracerProperties(Properties props)  {
    try {
      logFile = props.getProperty("org.openmobileis.common.log.file");
      if (logFile == null)  {
        System.out.println("org.openmobileis.common.log.file property is not defined");
        System.out.println("LOG to console");
        outStream = System.out;
        return;
      }
      File logfile = new File(logFile);
      if (logfile.exists()) {
        String archivedir = props.getProperty("org.openmobileis.common.log.archivefile");
        if (archivedir != null) {
          int index = logFile.lastIndexOf(File.separator);
          String fileName = logFile;
          if (index !=-1) {
            fileName = logFile.substring(index+1);
          }
          java.text.SimpleDateFormat formatter
            = new java.text.SimpleDateFormat ("dd-MM-yy#hh-mm-ss", java.util.Locale.FRANCE);
          String dateString = formatter.format(new java.util.Date());
          String logArchiveFile = archivedir+dateString+"-"+fileName;
          FileInputStream input  = new FileInputStream(logFile);
          FileOutputStream output = new FileOutputStream(logArchiveFile);
          try {
            byte[] buf = new byte[4096];
            int len = 0;
            while ( ( len = input.read( buf ) ) != -1 )
              output.write( buf, 0, len );
            output.flush();
          } finally {
            input.close();
            output.close();
          }
        }
      }
      fileStream = new FileOutputStream(logFile);
      outStream = new PrintStream(fileStream, true);
      System.setOut(outStream);
      System.setErr(outStream);
    } catch (Exception e) {
      System.out.println("Error in FileTraceManager Init" +e);
    }
  }

}