/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.common.util.log;

import java.util.Properties;
import java.io.*;

import org.openmobileis.common.util.PropertiesManager;

/**
 * Title:        OpenMobileIS project source <BR>
 * Implement a LogTracer that Log in a File. The File is define by specifing the property LOGFILE.
 * Implements an archiving system. Define the property LOGDIRARCHIVE to define the directory where log are archived.
 * @author P.Delrieu
 * @version 1.0
 * @since JDK1.1
 */
public class FileOpenCloseLogTracer extends DefaultLogTracer {

  protected FileOutputStream fileStream;
  protected String logFile;

  public FileOpenCloseLogTracer() {
  }
  /**
   * Two property are defined :
   *  LOGFILE : file where log are written.
   *  LOGDIRARCHIVE : directory where old log are archived.
   */

  public void setLogTracerProperties(Properties props)  {
    try {
      logFile = props.getProperty("org.openmobileis.common.log.file");
      if (logFile == null)  {
        System.out.println("LOGFILE property is not defined");
        System.out.println("LOG to console");
        outStream = System.out;
        return;
      }
      PropertiesManager.getManager().addProperty("org.openmobileis.common.log.file", logFile);
      outStream = System.out;
//      System.setOut(outStream);
//      System.setErr(outStream);
    } catch (Exception e) {
      System.out.println("Error in FileTraceManager Init" +e);
    }
  }

	/**
	 * Trace the message to the stream defined showing the specified priority and service.
	 */
	public synchronized void trace(int service, int priority, String message)  {
		try	{
			fileStream = new FileOutputStream(logFile, true);
			try	{
				outStream = new PrintStream(fileStream, true);
				super.trace(service, priority, message);
			} finally	{
				outStream = System.out;
				fileStream.flush();
				fileStream.close();
			}
		} catch (Throwable ex)	{
			ex.printStackTrace();
		}

	}
  
  public File getLogFile()	{
    return  new File(logFile);
  }

}