/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

/**
 * Index descriptor use to index String method or field.
 */
public class FODBStringIndexDescriptor extends FODBIndexDescriptor {
	static final long serialVersionUID = 5521257935120563452L;
	private int keyLen;
	private int incTabSize;
	private boolean caseSensitive = false;
	
	public FODBStringIndexDescriptor(String newName, int newType, String newMemberName, int btreeOrder, int newKeyLen) {
		super(newName, newType, newMemberName, btreeOrder);
		keyLen = newKeyLen;
		incTabSize = 5;
	}
	
	public FODBStringIndexDescriptor(String newName, int newType, String newMemberName, int btreeOrder, int newKeyLen, int btreeMultipleIndexIncrement) {
		super(newName, newType, newMemberName, btreeOrder);
		keyLen = newKeyLen;
		incTabSize = btreeMultipleIndexIncrement;
	}
    
    public FODBStringIndexDescriptor() {
    }
   
    public void writeExternal(ObjectOutput out) throws IOException {
      super.writeExternal(out);  
      out.writeInt(keyLen);
      out.writeInt(incTabSize);
      out.writeBoolean(caseSensitive);
    }
    
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
      super.readExternal(in);  
      keyLen = in.readInt();
      incTabSize = in.readInt();
      caseSensitive = in.readBoolean();
    }
	
	public int getIncTabSize() {
		return incTabSize;
	}
	
	public int getKeyLen() {
		return keyLen;
	}
  /**
   * @return
   */
  public boolean isCaseSensitive() {
    return caseSensitive;
  }

  /**
   * @param b
   */
  public void setCaseSensitive(boolean b) {
    caseSensitive = b;
  }

}
