/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */
package org.openmobileis.database.fastobjectdb.db.crypto;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import Acme.Crypto.BlowfishCipher;
import Acme.Crypto.EncryptedInputStream;
import Acme.Crypto.EncryptedOutputStream;

/**
 * Blowfish cypher use to encrypt a collection object data.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class AcmeBlowfishCypher implements FODBCypher {
  static final long serialVersionUID = 5521257935120563452L;
	private BlowfishCipher cypher;
	/**
	 * 
	 */
	public AcmeBlowfishCypher() {
		super();
		cypher = new BlowfishCipher("abcdefghijklmnopqrstuvwxyz");
	}
	/**
	 * 
	 */
	public AcmeBlowfishCypher(String key) {
		super();
		cypher = new BlowfishCipher(key);
	}

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.db.crypto.FODBCypher#encryptObject(byte[])
   */
  public byte[] encryptObject(byte[] object) throws IOException {
		ByteArrayOutputStream encrypted = new ByteArrayOutputStream();
		EncryptedOutputStream encrout = new EncryptedOutputStream(cypher, encrypted);
		encrout.write(object);
		encrout.flush();
		encrout.close();
/*  	byte[] eigthbyte = new byte[8];
		byte[] cipherdata = new byte[8];
  	int i=0;
  	while (true)	{
  		if (i+8 >= object.length)	{ //does not encrypt remaining bytes.
  			for (int j=i; j<object.length; j++)	{
					encrypted.write((int)object[j]);
  			}
  			break;
  		} else	{
  			System.arraycopy(object, i, eigthbyte, 0, 8);
  			i+=8;
				cypher.encrypt( eigthbyte, cipherdata );
				encrypted.write(cipherdata);
  		}
  	} */
		return encrypted.toByteArray();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.db.crypto.FODBCypher#decryptObject(byte[])
   */
  public  byte[] decryptObject(byte[] object) throws IOException {
		ByteArrayOutputStream ret = new ByteArrayOutputStream();
		ByteArrayInputStream decrypted = new ByteArrayInputStream(object);
		EncryptedInputStream encint = new EncryptedInputStream(cypher, decrypted);
		int read = 0;
		while ((read = encint.read()) != -1)	{
			ret.write(read);
		}
		ret.flush();
		return ret.toByteArray();
  }
  
	public void setKey( byte[] key )	{
		cypher = new BlowfishCipher(key);
	}
	  
	public void setKey(String key )	{
		cypher = new BlowfishCipher(key);
	}
	
	public static void main(String[] args)	{
		//test encryption data
		AcmeBlowfishCypher cypher = new AcmeBlowfishCypher();
		try	{
			byte[] test0 = new byte[0];
			byte[] res = cypher.encryptObject(test0);
			res = cypher.decryptObject(res);
			byte[] test1 = {1, 2, 2, 2, 2, 3};
			res = cypher.encryptObject(test1);
			res = cypher.decryptObject(res);
			byte[] test2 = {1, 2, 2, 2, 2, 2, 2, 3};
			res = cypher.encryptObject(test2);
			res = cypher.decryptObject(res);
			byte[] test3 = {1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3};
			res = cypher.encryptObject(test3);
			res = cypher.decryptObject(res);
			byte[] test4 = {1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3};
			res = cypher.encryptObject(test4);
			res = cypher.decryptObject(res);
			byte[] test5 = {1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3};
			res = cypher.encryptObject(test5);
			res = cypher.decryptObject(res);
			System.out.println("END");
		} catch (Throwable ex)	{
			ex.printStackTrace();
		}
		
		
	}

}
