/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.index;

import java.io.*;

import org.openmobileis.database.fastobjectdb.db.index.node.Node;


/**
 * Base table that store index position in collection data file.
 */

public class FODBIndexTable implements Externalizable {
  static final long serialVersionUID = 5521257935120563452L;

  private static int DEF_TAB_SIZE = 20;
	
	private int incTabSize;
	private int currentSize;
	
	private long[] indexsPos;
	
	public FODBIndexTable() {
		incTabSize = DEF_TAB_SIZE;				// default value;
		constructorHelper();
	}
	
	public FODBIndexTable(int tSize) {
		incTabSize = tSize;
		constructorHelper();
	}
	
	protected void constructorHelper() {
		currentSize = 0;
		indexsPos = new long[incTabSize];
		
		for (int i = 0; i < incTabSize; i++) {
			indexsPos[i] = Node.NO_NODE;
		}
	}
	
	public int size() {
		return currentSize;
	}
	
	public int allocatedSize() {
		return indexsPos.length;
	}
	
	public synchronized boolean add(long pos) {
		int tabSize = indexsPos.length;
		
		if (currentSize < tabSize) {
			indexsPos[currentSize] = pos;
			currentSize++;
			return false;
		}
		else { // currentSize == tabSize
			long[] newIndexsPos = new long[tabSize + incTabSize];
			
			System.arraycopy(indexsPos, 0, newIndexsPos, 0, indexsPos.length);
			
			newIndexsPos[currentSize] = pos;
			
			for (int i = currentSize+1; i<newIndexsPos.length; i++)  {
				newIndexsPos[i] = Node.NO_NODE;
			}
			
			indexsPos = newIndexsPos;
			currentSize++;
			return true;
		}
		
	}
	
	public long getPosByPlace(int place) {
		if (place >= currentSize || place < 0) {
			return Node.NO_NODE;
		}
		return indexsPos[place];
	}
	
	public String toString() {
		String result = "       FODBIndexTable[" + incTabSize + ", " + currentSize + "]\n";
		
		result = result.concat("         [");
		for (int i = 0; i < currentSize; i++) {
			result = result.concat("(" + indexsPos[i] + ")");
		}
		result = result.concat("]\n");
		
		return result;
	}
	
	public void writeExternal(ObjectOutput out) throws IOException {
		out.writeInt(incTabSize);
		out.writeInt(currentSize);
		out.writeInt(indexsPos.length);
		for (int i = 0; i < indexsPos.length; i++) {
			out.writeLong(indexsPos[i]);
		}
	}
	
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
		incTabSize = in.readInt();
		currentSize = in.readInt();
		int realTabSize = in.readInt();
		
		indexsPos = new long[realTabSize];
		
		for (int i = 0; i < realTabSize; i++) {
			indexsPos[i] = in.readLong();
		}
	}

}
		