/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.index;

import java.io.IOException;
import java.lang.reflect.*;
import org.openmobileis.common.util.collection.*;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;
import org.openmobileis.database.fastobjectdb.db.index.node.Node;
import org.openmobileis.database.fastobjectdb.db.index.node.StringNode;
import org.openmobileis.database.fastobjectdb.db.query.soda.SodaStringIndexComparator;
import org.openmobileis.database.fastobjectdb.db.store.*;

/**
 */
public class FODBMultipleStringIndex extends FODBStringIndex implements FODBMultipleIndex {
	
	public FODBMultipleStringIndex(FODBIndexHeader newHeader, FODBCollectionIndexFile cFile, AccessibleObject accObj) throws FODBException {
		super(newHeader, cFile, accObj);
	}
	
	public FODBMultipleStringIndex(FODBStringIndexDescriptor descriptor, FODBCollectionIndexFile cFile, AccessibleObject accObj) throws FODBException {
		super(descriptor, cFile, accObj);
	}
	
	public int getType() {
		return FODBIndex.MULTIPLE;
	}

	/**
	  replace the old ptr with new if present. Add if old not present
	*/
  protected void writeKeyPtr(Node node, int pos, long newptr) throws IOException, ClassNotFoundException {
    long arrayPtr = node.getNodePtrAtPos(pos);
    long[] ptrList = (long[]) colFile.readLongPtr(arrayPtr);
    int size = ptrList.length;

    int i = 0;
    for (i = 0; i < size; i++) {
      if (ptrList[i] == Node.NO_NODE) {
        break;
      }
    }
    if (i == size) { // increment +indexPosTabSize
      long[] newptrlist = new long[size + ((FODBStringIndexDescriptor)header.descriptor).getIncTabSize()];
      System.arraycopy(ptrList, 0, newptrlist, 0, ptrList.length);
      newptrlist[size] = newptr;
      for (i = size + 1; i < newptrlist.length; i++) {
        newptrlist[i] = Node.NO_NODE;
      }
      colFile.deleteLongPtr(arrayPtr);
      long modified = colFile.writeLongPtr(newptrlist);
      node.ptr[pos] = modified;
      colFile.rewriteNode(node, node.filePtr);
      return;
    } else {
      ptrList[i] = newptr;
      colFile.rewriteLongPtr(ptrList, arrayPtr);
      return;
    }
  }
	
	protected long createPtrArray(long ptr) throws IOException, ClassNotFoundException {
	  long[] newptrlist = new long[((FODBStringIndexDescriptor)header.descriptor).getIncTabSize()];
	  newptrlist[0] = ptr;
	  for (int i=1; i<newptrlist.length; i++)  {
			newptrlist[i] = Node.NO_NODE;
	  }
	  return colFile.writeLongPtr(newptrlist);
	}
	
/*	protected void addSearchResult(long pos, LongArray array) throws IOException, ClassNotFoundException  {
		long ptrList[] = this.readPtrAtPos(pos);
		array.add(ptrList);
	} */

	protected void addSearchResult(StringNode node, int pos, LongArray array, SodaStringIndexComparator comparator) throws IOException, ClassNotFoundException {
	  if (comparator.isDistinct())	{
	    long[] ptrList = this.readPtrAtPos(node, pos);
	  	array.add(ptrList[0]);
	  } else	{
	    long[] ptrList = this.readPtrAtPos(node, pos);
	  	array.add(ptrList);
	  }

	}
	
	/*private long[] readPtrAtPos(long pos) throws IOException, ClassNotFoundException {
	  
	  long[] arrayList = (long[])colFile.readLongPtr(pos);
	  // remove null;
	  int i = 0;
	  for (i=0; i<arrayList.length; i++)  {
		if (arrayList[i] == Node.NO_NODE) {
		  break;
		}
	  }
	  if (i==arrayList.length)  {
		return arrayList;
	  }
	  long[] newArrayList = new long[i];
	  System.arraycopy(arrayList, 0, newArrayList, 0, newArrayList.length);
	  return newArrayList;
	} */
	
	private long[] readPtrAtPos(Node pg, int pos) throws IOException, ClassNotFoundException {
	  long arrayPtr = pg.ptr[pos];
	  
	  long[] arrayList = (long[])colFile.readLongPtr(arrayPtr);
	  // remove null;
	  int i = 0;
	  for (i=0; i<arrayList.length; i++)  {
		if (arrayList[i] == Node.NO_NODE) {
		  break;
		}
	  }
	  if (i==arrayList.length)  {
		return arrayList;
	  }
	  long[] newArrayList = new long[i];
	  System.arraycopy(arrayList, 0, newArrayList, 0, newArrayList.length);
	  return newArrayList;
	}
	
	/**
	  remove the specified pointer from the ptr array.
	  return true if array is deleted (empty)
	*/
	protected boolean removeKeyPtr(Node node, int pos, long pointer) throws IOException, ClassNotFoundException {
	  long arrayPtr = node.ptr[pos];
	  long[] ptrList = (long[])colFile.readLongPtr(arrayPtr);

	  // find pointer position
	  int i=0;
	  int size = ptrList.length;
	  for (i=0; i<size; i++)  {
		if (ptrList[i] == pointer)  {
		  ptrList[i] = Node.NO_NODE;
		  break;
		}
	  }
	  if (i == size)  { // Pointer not found
	  	throw new IOException("Pointer not found !!!");
	  }
	  int nbnull = 1;
	  for (int j=i+1; j<size ; j++) {
		if (ptrList[j] == Node.NO_NODE) {
		  nbnull++;
		} else  {
		  ptrList[j-1] = ptrList[j];
		  ptrList[j] = Node.NO_NODE;
		}
	  }
      int inctabsize = ((FODBStringIndexDescriptor)header.descriptor).getIncTabSize();
	  if (nbnull == inctabsize)  {  //decrease
		if (size > inctabsize) {
		  long[] newptrlist = new long[size - inctabsize];
		  System.arraycopy(ptrList, 0, newptrlist, 0, newptrlist.length);
		  colFile.deleteLongPtr(arrayPtr);
		  node.ptr[pos] = colFile.writeLongPtr(newptrlist);
		  // write modified page
		  colFile.rewriteNode(node, node.filePtr);
		  return false;
		} else  {     // is empty
		  colFile.deleteLongPtr(arrayPtr);
		  return true;
		}
	  }
	  // nbnull < header.incTabSize
	  colFile.rewriteLongPtr(ptrList, arrayPtr);
	  return false;
	}
}
