/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.index;

import java.io.IOException;
import java.lang.reflect.AccessibleObject;

import org.openmobileis.common.util.collection.LongArray;
import org.openmobileis.database.fastobjectdb.FODBLongIndexDescriptor;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;
import org.openmobileis.database.fastobjectdb.db.index.node.LongNode;
import org.openmobileis.database.fastobjectdb.db.index.node.Node;
import org.openmobileis.database.fastobjectdb.db.query.soda.BaseConstraint;
import org.openmobileis.database.fastobjectdb.db.query.soda.SodaLongIndexComparator;
import org.openmobileis.database.fastobjectdb.db.store.FODBCollectionIndexFile;

/**
 */
public class FODBUniqueLongIndex extends FODBLongIndex implements FODBUniqueIndex {

	public FODBUniqueLongIndex(FODBIndexHeader newHeader, FODBCollectionIndexFile cFile, AccessibleObject accObj) throws FODBException {
		super(newHeader, cFile, accObj);
	}
	
	public FODBUniqueLongIndex(FODBLongIndexDescriptor descriptor, FODBCollectionIndexFile cFile, AccessibleObject accObj) throws FODBException {
		super(descriptor, cFile, accObj);
	}


	/* (non-Javadoc)
	 * @see org.openmobileis.embedded.fastobjectdb.db.FODBIndex#getType()
	 */
	public int getType() {
		return FODBIndex.UNIQUE;
	}

	protected void writeKeyPtr(Node pg, int pos, long newptr) throws IOException, ClassNotFoundException {
		// TODO : This case shoudn't happen... Think about removing this.
		pg.ptr[pos] = newptr;
		// rewrite modified page
		colFile.rewriteNode(pg, pg.filePtr);
	  	
		return;
	}
	
	protected void addSearchResult(LongNode pg, int pos, LongArray array) throws IOException, ClassNotFoundException {
		array.add(pg.ptr[pos]);
	}
	
	public long getKeyPosition(Object keytosearch) throws FODBException {
		if (keytosearch == null) {
			return Node.NO_NODE;
		}
		if (!(keytosearch instanceof Long))	{
			throw new FODBException("Key type in getKeyPosition for LongUniqueIndex");
		}
		long key = ((Long)keytosearch).longValue();

		try	{
			SodaLongIndexComparator comparator = new SodaLongIndexComparator();
			comparator.setOperand(new Long(key));
			comparator.addComparator(BaseConstraint.EQUALS);
			long preResult[] = this.query(comparator);
	
			if (preResult == null) { // Shouldn't happen !
				throw new FODBException("Internal error !!!");
			}
			if (preResult.length > 1) { // Shouldn't happen in a UniqueIndex
				throw new FODBException("Internal error !!!");
			}
			
			if (preResult.length == 0) {
				return Node.NO_NODE;
			}
			
			// If we got there, then we had ONE result, has expected
			return preResult[0];
		} catch (Throwable ex) {
			throw new FODBException(ex);
		}
	}
	
	protected boolean removeKeyPtr(Node pg, int pos, long pointer) throws IOException, ClassNotFoundException {
		return true;
	}
}
