/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.index;

import java.io.IOException;
import java.lang.reflect.*;
import org.openmobileis.common.util.collection.*;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;
import org.openmobileis.database.fastobjectdb.db.index.node.*;
import org.openmobileis.database.fastobjectdb.db.query.soda.BaseConstraint;
import org.openmobileis.database.fastobjectdb.db.query.soda.SodaStringIndexComparator;
import org.openmobileis.database.fastobjectdb.db.store.*;

/**
 */
public class FODBUniqueStringIndex extends FODBStringIndex implements FODBUniqueIndex {
	
	public FODBUniqueStringIndex(FODBIndexHeader newHeader, FODBCollectionIndexFile cFile, AccessibleObject accObj) throws FODBException	{
		super(newHeader, cFile, accObj);
	}
	
	public FODBUniqueStringIndex(FODBStringIndexDescriptor descriptor, FODBCollectionIndexFile cFile, AccessibleObject accObj) throws FODBException {
		super(descriptor, cFile, accObj);
	}

	public int getType() {
		return FODBIndex.UNIQUE;
	}
	
	protected void writeKeyPtr(Node node, int pos, long newptr) throws IOException, ClassNotFoundException {
  	throw new IOException("rewrite unique String  index Ptr not allowed");
	}
	
	
/*	protected void addSearchResult(long pos, LongArray array) throws IOException, ClassNotFoundException  {
		array.add(pos);
	} */
	
	protected void addSearchResult(StringNode node, int pos, LongArray array, SodaStringIndexComparator comparator) throws IOException, ClassNotFoundException {
		array.add(node.getNodePtrAtPos(pos));
	}
	

	
	public long getKeyPosition(Object keytosearch) throws FODBException {
		if (keytosearch == null) {
			return Node.NO_NODE;
		}
		if (!(keytosearch instanceof String))	{
			throw new FODBException("Key type in getKeyPosition for StringUniqueIndex");
		}
		String key = (String) keytosearch;
		if (key.length() == 0) {
			return Node.NO_NODE;
		}
		
		try	{
			SodaStringIndexComparator comparator = new SodaStringIndexComparator(((FODBStringIndexDescriptor)this.getIndexDescriptor()).isCaseSensitive());
			comparator.setOperand(key);
			comparator.addComparator(BaseConstraint.EQUALS);
			long preResult[] = this.query(comparator);
	
			if (preResult == null) { // Shouldn't happen !
				throw new FODBException("Internal error !!!");
			}
			if (preResult.length > 1) { // Shouldn't happen in a UniqueIndex
				throw new FODBException("Internal error !!!");
			}
			
			if (preResult.length == 0) {
				return Node.NO_NODE;
			}
			
			// If we got there, then we had ONE result, has expected
			return preResult[0];
		} catch (Throwable ex)	{
			throw new FODBException(ex);
		}
	}
	
	protected boolean removeKeyPtr(Node pg, int pos, long pointer) throws IOException, ClassNotFoundException {
		return true;
	}

}
