/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.query.soda;

import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;
import org.openmobileis.database.fastobjectdb.db.index.FODBIndex;
import org.openmobileis.database.fastobjectdb.db.index.FODBIntIndex;
import org.openmobileis.database.fastobjectdb.db.index.FODBLongIndex;
import org.openmobileis.database.fastobjectdb.db.index.FODBStringIndex;

/**
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class FODBSodaIndexPath {
	private FODBIndex index;
	private SodaIndexComparator comparator;
  /**
   * 
   */
  public FODBSodaIndexPath(FODBIndex index) {
    super();
    this.index = index;
    if (index instanceof FODBStringIndex)	{
    	FODBStringIndexDescriptor descr = (FODBStringIndexDescriptor)index.getIndexDescriptor();
    	comparator = new SodaStringIndexComparator(descr.isCaseSensitive());
		} else	if (index instanceof FODBIntIndex)	{
			comparator = new SodaIntIndexComparator();    	
		} else	if (index instanceof FODBLongIndex)	{
			comparator = new SodaLongIndexComparator();    	
    }
  }
	SodaIndexComparator getIndexComparator()	{
		return comparator;
	}
  
	void mergeIndexComporator(SodaIndexComparator index)	{
		comparator.mergeIndexComporator(index);
	}
  
  public String getIndexMember()	{
  	return this.index.getIndexDescriptor().getMemberName();
  }
  
	public boolean setOperand(Object ope)	{
  	if (comparator != null)	{
  		return comparator.setOperand(ope);
  	}
  	return false;
	}
	public long[] execute() throws FODBException {
		return index.query(comparator);
	}
  
	public boolean addComparator(int comp)	{
		if (comparator != null)	{
			return comparator.addComparator(comp);
		}
		return false;
		
	}
  
}
