/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.query.soda;

import org.odbms.ObjectSet;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.database.fastobjectdb.db.FODBCollection;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;

/**
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class FODBSodaObjectSet implements ObjectSet {
  protected Object[] objArray;
  private int currentCursor = 0;

  /**
   * 
   */
  public FODBSodaObjectSet() {
    super();
    this.objArray = new Object[0];
  }

  /**
   * 
   */
  public FODBSodaObjectSet(Object[] array) {
    super();
    this.objArray = array;
  }

  /* (non-Javadoc)
   * @see org.odbms.ObjectSet#hasNext()
   */
  public boolean hasNext() {
    return (currentCursor < objArray.length);
  }

  /* (non-Javadoc)
   * @see org.odbms.ObjectSet#next()
   */
  public Object next() {
    return objArray[currentCursor++];
  }

  /* (non-Javadoc)
   * @see org.odbms.ObjectSet#reset()
   */
  public void reset() {
    currentCursor = 0;
  }

  /* (non-Javadoc)
   * @see org.odbms.ObjectSet#size()
   */
  public int size() {
    return objArray.length;
  }

  protected void addCollectionPtrs(long[] ptrArray, FODBCollection collection) throws FODBException {
    Object[] retObj = new Object[ptrArray.length];
    int size = ptrArray.length;
    for (int i = 0; i < size; i++) {
      retObj[i] = collection.getElementAtPos(ptrArray[i]);
    }
    if (objArray.length > 0) {
      int newSize = objArray.length + ptrArray.length;
      Object[] newArray = new Object[newSize];
      System.arraycopy(objArray, 0, newArray, 0, objArray.length);
      System.arraycopy(retObj, 0, newArray, objArray.length, retObj.length);
      this.objArray = newArray;
    } else {
      this.objArray = retObj;
    }
  }
  
  public Array getArrayFromSet()	{
  	return new Array(objArray);
  }

}
