/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.database.fastobjectdb.db.test;

import java.io.File;
import java.io.IOException;
import java.util.Properties;

import org.odbms.ObjectSet;
import org.odbms.Query;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.user.impl.MonoUserUserManagerFactory;
import org.openmobileis.common.user.profile.ProfileDataManager;
import org.openmobileis.common.user.profile.impl.XmlFileProfilDataFactory;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.exception.BadDataFormatException;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.FODBIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.synchro.client.SynchroFastObjectDBManager;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBSynchroManager;
import org.openmobileis.embedded.webserver.WebServer;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberManager;
import org.openmobileis.synchro.algo.syncnumber.impl.EmptySyncNumberManagerDelegate;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.client.SynchroManager;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;
import org.openmobileis.synchro.openmsp.client.conduit.HttpOpenMSPSynchroConduit;
import org.openmobileis.synchro.openmsp.client.core.NumSyncManagerDB;
import org.openmobileis.synchro.openmsp.server.synctarget.SynchroTargerManager;
import org.openmobileis.synchro.openmsp.server.synctarget.proxy.GenericProxySynchroTarget;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class TestFODBSynchro {

  /**
   * 
   */
  public TestFODBSynchro() {
    super();
  }
  
  private void prepateEnvironement() throws InterruptedException {
    //client environement
   PropertiesManager.getManager().addProperty("org.openmobileis.synchro.client.openmsp.numsyncpropsfile",System.getProperty("user.dir") + "/WEB-INF/conf/numsync.txt");
    //synchro environement is declared by FODBSynchroManager with default parameters.
    
    //server enviromenent;
    PropertiesManager.getManager().addProperty("org.openmobileis.common.user.profil.xmlprofilfile.path",System.getProperty("user.dir") + "/WEB-INF/conf/defaultprofils.xml");
    UserManager.getManager().registerUserManagerFactoryForGroup("default", new MonoUserUserManagerFactory());
    ProfileDataManager.getManager().registerProfilDataFactoryForGroup("default", new XmlFileProfilDataFactory());
    FODBSynchroManager.getManager().registerCollection(new TestFODBSyncTarget());
    SynchroTargerManager.getManager().registerProxyTargetForTerminal(new GenericProxySynchroTarget(), null);
    SyncNumberManager.getManager().registerDelegate(new EmptySyncNumberManagerDelegate());
    //start server
    Runnable startWebServer = new Runnable() {

      public void run() {
        try {

          System.out.println("start WebServer");

          Properties props = new Properties();
          props.put("LOGFILE", System.getProperty("user.dir") + "/WEB-INF/log/server/log");
          LogManager.registerLogManager(props); //log to a file

          WebServer server = null;
          server = new WebServer();

          server.serve();
        } catch (Throwable ex) {
          ex.printStackTrace();
        }
      }

    };
    Thread th = new Thread(startWebServer);
    th.start();
    //do synchro
    //wait 1 second until the webserver is started.
    Thread.currentThread().sleep(1000);

  }
  
  private void initDatabase() throws DatabaseException, ClassNotFoundException, IOException, BadDataFormatException  {
    String tempdir = System.getProperty("user.dir") + File.separator;
    //init db system parameters
    PropertiesManager.getManager().addProperty("fastobjectdb.database.path", tempdir+"database");
    PropertiesManager.getManager().addProperty("fastobjectdb.database.name", "testdbsync");
    SynchroFastObjectDBManager.getCurrentFODB(); //init synchro db to be use as default db
    
    FastObjectDB db = FastObjectDBManager.getCurrentFODB();
    if (!db.isCollectionExist("CLIENT")) {
      db.createCollection("CLIENT", TestData.class);
      FODBStringIndexDescriptor KeyDescriptor = new FODBStringIndexDescriptor("KEY", FODBIndexDescriptor.UNIQUE, "getKey()", 12, 15);
      db.addIndex("CLIENT", KeyDescriptor);
      FODBStringIndexDescriptor debKeyDescriptor = new FODBStringIndexDescriptor("DEBKEY", FODBIndexDescriptor.MULTIPLE, "getDebKey()", 10,
          TestData.NB_BEGIN_KEY_LETTER, 5);
      db.addIndex("CLIENT", debKeyDescriptor);
    }
    if (!db.isCollectionExist("SERVER")) {
      db.createCollection("SERVER", TestData.class);
      FODBStringIndexDescriptor KeyDescriptor = new FODBStringIndexDescriptor("KEY", FODBIndexDescriptor.UNIQUE, "getKey()", 12, 15);
      db.addIndex("SERVER", KeyDescriptor);
      FODBStringIndexDescriptor debKeyDescriptor = new FODBStringIndexDescriptor("DEBKEY", FODBIndexDescriptor.MULTIPLE, "getDebKey()", 10,
          TestData.NB_BEGIN_KEY_LETTER, 5);
      db.addIndex("SERVER", debKeyDescriptor);
    }
   
  }
  
  private void prepareServerDataForSynchro() throws DatabaseException {
    TestData data = this.getServerTestData("FIRSTS");
    if (data == null) { //first synchro create data
     data = new TestData("FIRSTS");
     data.setToModify("0S");
     FastObjectDBManager.getCurrentFODB().add("SERVER", data);
    }
    
  }
  
  private TestData getClientTestData(String key) throws DatabaseException {
    Query q = FastObjectDBManager.getCurrentFODB().query();
    q.constrain("CLIENT");
    Query q2 = q.descend("getKey()");
    q2.constrain(key).equal();
    ObjectSet set = q.execute();
    if (set.hasNext())  {
      return (TestData) set.next();
    }
    return null;
  }
  
  private TestData getServerTestData(String key) throws DatabaseException {
    Query q = FastObjectDBManager.getCurrentFODB().query();
    q.constrain("SERVER");
    Query q2 = q.descend("getKey()");
    q2.constrain(key).equal();
    ObjectSet set = q.execute();
    if (set.hasNext())  {
      return (TestData) set.next();
    }
    return null;
  }
  
  private void doSynchro() throws SynchroException {
    Credential cred = new Credential("test", "test");
    SynchroDescriptor descriptor = new SynchroDescriptor();
    descriptor.addProperty("OpenMSPsynchrotype", "DR");
    String useragent = OpenMSPSynchroManager.getManager().getSynchroUserAgent(descriptor);
    descriptor.setSynchroConduit(new HttpOpenMSPSynchroConduit(System.getProperty("user.dir") + "/synchro",useragent));
    descriptor.setSynchroGroup("default");
    descriptor.setServerURL("http://localhost:9091/services/openmspservice");
    
    SynchroManager.getManager().doSynchro(cred, descriptor);
  }

  /**
   * @param args
   */
  public static void main(String[] args) {
    try  {
      TestFODBSynchro test = new TestFODBSynchro();
      test.prepateEnvironement();
      test.initDatabase();
      test.prepareServerDataForSynchro();
      NumSyncManagerDB.getManager().saveSyncNumberForService(0, "CLIENT"); //set to 0 for first synchro
      test.doSynchro();
      TestData data = test.getClientTestData("FIRSTS");
      if (data != null) LogManager.traceError(0, "First Synchro OK");
      else LogManager.traceError(0, "First Synchro NOK");
      
      // add client size data and modif server side
      data = new TestData("SECONDC");
      data.setToModify("0C");
      TestData datadb = test.getClientTestData("SECONDC");
      if (datadb == null) FastObjectDBManager.getCurrentFODB().add("CLIENT", data);
      test.doSynchro();
      data = test.getServerTestData("SECONDC");
      TestData databis = test.getClientTestData("FIRSTS");
      if ((data != null) && (databis != null) && (databis.getToModify().equals("1S"))) LogManager.traceError(0, "Second Synchro OK");
      else LogManager.traceError(0, "Second Synchro NOK");
 
      // update client size data and delete server side
      data = new TestData("SECONDC");
      data.setToModify("1C");
      FastObjectDBManager.getCurrentFODB().replace("CLIENT", data);
      test.doSynchro();
      data = test.getServerTestData("SECONDC");
      databis = test.getClientTestData("FIRSTS");
      if ((data != null) && (databis == null) && (data.getToModify().equals("1C"))) LogManager.traceError(0, "Second Synchro OK");
      else LogManager.traceError(0, "Second Synchro NOK");

      // delete client and server complete synchro
      FastObjectDBManager.getCurrentFODB().deleteWithId("CLIENT", "SECONDC");
      test.doSynchro();
      data = test.getServerTestData("SECONDC");
      databis = test.getClientTestData("THIRD");
      TestData databis2 = test.getClientTestData("FOURTH");
      if ((data == null) && (databis != null) && (databis2 != null)) LogManager.traceError(0, "Third Synchro OK");
      else LogManager.traceError(0, "Second Synchro NOK");
    } catch (Throwable ex)  {
      LogManager.traceError(0, ex);
    }

  }

}
