/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.db.transaction;

import java.util.HashMap;
import java.util.Iterator;

import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;

/**
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class TransactionManager {
	private HashMap transactionListbythread;
	private HashMap transactionFileListbyCollection;
	private HashMap collectionFileByThread;
  private boolean oneusermode = true;
  private Array threadToStop;
  /**
   * 
   */
  public TransactionManager() {
    super();
		transactionListbythread = new HashMap(10);
		transactionFileListbyCollection = new HashMap(20);
		collectionFileByThread = new HashMap(10);
    threadToStop = new Array();
    String oneuser = PropertiesManager.getManager().getProperty("org.openmobileis.database.fastobjectdb.usermode");
    if ((oneuser !=null) && (oneuser.equals("false"))) {
      oneusermode = false;
    }
  }
  
  private Transaction getTransaction()	{
  	Thread th = Thread.currentThread();
  	Transaction ret = (Transaction)transactionListbythread.get(th);
  	if (ret==null)	{
  		ret = new Transaction(th);
			transactionListbythread.put(th, ret);
  	}
  	return ret;
  }
  
  public synchronized void registerTransactionFile(TransactionFile file, String collectionName)	{
		transactionFileListbyCollection.put(collectionName, file);
  }
  
  public void validateThread(Thread th) throws FODBException {
    if (oneusermode)  {
      synchronized(threadToStop)  {
        if (threadToStop.size()>0)  {
          if (threadToStop.contains(th))  {
            throw new FODBException("Thread stopped by other thread. One User mode.");
          }
        }
      }
    }
  }
  
	public synchronized void enterTransaction(String collectionName)	throws FODBException {
 //   LogManager.traceDebug(0, "TE");
		try	{
	  	Thread th = Thread.currentThread();
	  	Transaction othertr = null;
			synchronized (collectionFileByThread)	{
	  	 othertr = (Transaction)collectionFileByThread.get(collectionName);
			}
	  	if ((othertr != null) && (!othertr.getTransactionThread().equals(th))) { // a new thread arrive
        if (oneusermode)  {
          synchronized(threadToStop)  {
            threadToStop.add(othertr.getTransactionThread());
          }
        }
	  	  synchronized(th)	{
		  	  othertr.pushThread(th);
		  	  th.wait();
	  	  }
	  	}
	//    LogManager.traceDebug(0, "TEOK");
		  TransactionFile file = (TransactionFile)transactionFileListbyCollection.get(collectionName);
			if (file != null)	{
			  Transaction tr = this.getTransaction();
				tr.addTransactionFile(file);
				synchronized (collectionFileByThread)	{
				  collectionFileByThread.put(collectionName, tr); //use to lock transaction until commit
				}
			} else	{
				throw new FODBException("ERROR TransactionManager Unregistered Collection :"+collectionName+" try to enter transaction.");
			}
		} catch (Throwable ex){
			throw new FODBException(ex);
		}
	}
  
  public void begin()	throws FODBException {
 //   LogManager.traceDebug(0, "TB");
		this.getTransaction().begin();
  }
  
  public void commit() throws FODBException	{
  //  LogManager.traceDebug(0, "TC");
	  Transaction tr = this.getTransaction();
		boolean effective = tr.commit();
		if (effective)	{
      this.processEndTransaction(tr);
		}
   }
  
  private void processEndTransaction(Transaction tr) {
    transactionListbythread.remove(Thread.currentThread());
    if (oneusermode)  {
      synchronized(threadToStop)  {
        if (threadToStop.size()>0)  {
          threadToStop.remove(tr.getTransactionThread());
        }
      }
    }
    //remove transaction from collectionFileByThread
    synchronized (collectionFileByThread) {
      Iterator iter = collectionFileByThread.keySet().iterator();
      Array keylist = new Array();
      while (iter.hasNext())  {
        Object key = iter.next();
        Object maptr = collectionFileByThread.get(key);
        if (maptr.equals(tr)) keylist.add(key);
      }
      for (int i=0; i<keylist.size(); i++)  {
        collectionFileByThread.remove(keylist.get(i));
      }
    }
    tr.popallthread();
  }
  
  public void rollback() throws FODBException	{
    //  LogManager.traceDebug(0, "TR");
	  Transaction tr = this.getTransaction();
		boolean effective = tr.rollback();
		if (effective)	{
      this.processEndTransaction(tr);
		}
  }

}
