/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.database.fastobjectdb.synchro.client;

import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.synchro.openmsp.client.db.ObjectDBImportQueryManager;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class FODBImportQueryManager extends ObjectDBImportQueryManager {
  private SynchroFODBReturnListener synchroListener;
  private String collectionName;
  /**
   * 
   */
  public FODBImportQueryManager(String collectionName) {
    super();
    this.collectionName = collectionName;
  }
  
  public void registerSynchroListener(SynchroFODBReturnListener listener) {
    this.synchroListener = listener;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.db.ObjectDBImportQueryManager#addObject(java.lang.String, java.lang.Object)
   */
  public void addObject(String uid, Object object) throws DatabaseException {
    try {
      if (this.synchroListener !=null)  {
        Object newOne = this.synchroListener.notifyBeginDBUpdate(object);
        if (newOne != null) {
          object = newOne;
        }
      }
      SynchroFastObjectDB db = (SynchroFastObjectDB)FastObjectDBManager.getCurrentFODB();
      boolean exist = db.isObjectInCollection(collectionName, object);
      if (exist)  {
        db.replaceFODB(collectionName, object);
      } else  {
        db.addFODB(collectionName, object);
      }
    } catch (Throwable ex)  {
      throw new DatabaseException(ex);
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.db.ObjectDBImportQueryManager#replaceObject(java.lang.String, java.lang.Object)
   */
  public void replaceObject(String uid, Object object) throws DatabaseException {
    this.addObject(uid, object);

  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.db.ObjectDBImportQueryManager#deleteObject(java.lang.String)
   */
  public void deleteObject(String uid) throws DatabaseException {
    try {
      SynchroFastObjectDB db = (SynchroFastObjectDB)FastObjectDBManager.getCurrentFODB();
      db.deleteWithIdFODB(collectionName, uid);
      if (this.synchroListener !=null) this.synchroListener.notifyDBDelete(uid);
    } catch (Throwable ex)  {
      throw new DatabaseException(ex);
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.db.ObjectDBImportQueryManager#beginImport()
   */
  protected void beginImport() throws DatabaseException {

  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.client.db.ObjectDBImportQueryManager#endImport()
   */
  protected void endImport() throws DatabaseException {
    // TODO Auto-generated method stub

  }

  public SynchroFODBReturnListener getSynchroReturnListener() {
    return synchroListener;
  }
}
