/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.database.fastobjectdb.synchro.client;



import org.odbms.ObjectSet;
import org.odbms.Query;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.database.fastobjectdb.FODBCollectionDescriptor;
import org.openmobileis.database.fastobjectdb.FODBIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.db.query.soda.FODBSodaObjectSet;
import org.openmobileis.synchro.openmsp.client.db.ActionDB;
import org.openmobileis.synchro.openmsp.client.db.SyncActionDBQueryManager;


/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Database access for the SyncActionDBManager.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class FODBSyncActionDBQueryManager implements SyncActionDBQueryManager {

  private static int SERVICE_NAME_MAX_LENGTH = 40;
  
  private static int uidkeylength;





  public FODBSyncActionDBQueryManager() {
    uidkeylength = 100;
  }

  public FODBSyncActionDBQueryManager(int uidlength) {
    uidkeylength = uidlength;
  }


 /*

 * check if the table exists. IF it does not exist, create it

 */

  public void initDB() throws DatabaseException {

    try {
			FastObjectDB db = FastObjectDBManager.getCurrentFODB();
			if (!db.isCollectionExist("FWKACTIONDB"))	{
        FODBCollectionDescriptor descriptor = new FODBCollectionDescriptor("FWKACTIONDB", ActionDB.class);
        descriptor.setSynchronize(false);
				db.createCollection(descriptor);
        int keylength = SERVICE_NAME_MAX_LENGTH + uidkeylength;
				FODBStringIndexDescriptor KeyDescriptor = new FODBStringIndexDescriptor("ID", FODBIndexDescriptor.UNIQUE, "getActionID()", 10, keylength);
				db.addIndex("FWKACTIONDB", KeyDescriptor);
				FODBStringIndexDescriptor debKeyDescriptor = new FODBStringIndexDescriptor("SERV", FODBIndexDescriptor.MULTIPLE, "getActionServiceName()", 10, SERVICE_NAME_MAX_LENGTH, 10);
				db.addIndex("FWKACTIONDB", debKeyDescriptor);
			}
    } catch (Throwable ex)  {
      throw new DatabaseException(ex);
    }

  }

  public Array getActionsForService(String serviceName) throws DatabaseException  {
    if ((serviceName== null) || (serviceName.length() > SERVICE_NAME_MAX_LENGTH)) {
      throw new DatabaseException("SyncActionDBQueryManager getActionsForService ServiceName to long or null abort "+serviceName);
    }
		//query SELECT * FROM TESTSEARCH
		Query q = FastObjectDBManager.getCurrentFODB().query();
		q.constrain(ActionDB.class);
		Query subq = q.descend("getActionServiceName()");
		subq.constrain(serviceName).equal();
		FODBSodaObjectSet set = (FODBSodaObjectSet)q.execute();  
		return set.getArrayFromSet();  
  }
  
  public ActionDB getActionDB(String serviceName, String objectUID) throws DatabaseException  {
    if ((serviceName== null) || (serviceName.length() > SERVICE_NAME_MAX_LENGTH)) {
      throw new DatabaseException("SyncActionDBQueryManager getAction ServiceName to long or null abort "+serviceName);
    }
		String actionID = ActionDB.getActionIdFromServiceAndUID(serviceName, objectUID);
		Query q = FastObjectDBManager.getCurrentFODB().query();
		q.constrain(ActionDB.class);
		Query subq = q.descend("getActionID()");
		subq.constrain(actionID).equal();
		ObjectSet set = q.execute();  
		if (set.hasNext())	{
			return (ActionDB) set.next();
		}
		return null;  
  }



  // insert a row

  public void create (ActionDB action) throws DatabaseException  {
    if ((action.getActionServiceName()== null) || (action.getActionServiceName().length() > SERVICE_NAME_MAX_LENGTH)) {
      throw new DatabaseException("SODA SyncActionDBQueryManager create ServiceName to long or null abort "+action.getActionServiceName());
    }
    if ((action.getActionObjectUID()== null) || (action.getActionObjectUID().length() > uidkeylength)) {
      throw new DatabaseException("SODA SyncActionDBQueryManager create Action uid to long or null abort "+action.getActionObjectUID());
    }
      /* resolve action conflic. Rules :
        if new action add remove old action and add new one
        if new action delete and old update remove update add delete
        if new action delete and old add remove add do not add delete
        if new action update and old add keep old add
        if new action update and old update keep old.
        if new action update and old delete keep old.
        for update if oldaction do nothing
       
       if no old action add action.
       */
      
      ActionDB oldAction =  this.getActionDB(action.getActionServiceName(), action.getActionObjectUID());
      if (oldAction != null)  { //resolve conflic
        switch (action.getActionType()) {
          case ActionDB.ADD_ACTION :
            FastObjectDBManager.getCurrentFODB().replace("FWKACTIONDB", action);
            break;
          case ActionDB.DELETE_ACTION :
            if (oldAction.getActionType() == ActionDB.ADD_ACTION) {
							FastObjectDBManager.getCurrentFODB().delete("FWKACTIONDB", oldAction);
            } else  {
							FastObjectDBManager.getCurrentFODB().replace("FWKACTIONDB", action);
            }
            break;
          case ActionDB.UPDATE_ACTION :
          default :
            break;
        }
      } else  {
				FastObjectDBManager.getCurrentFODB().add("FWKACTIONDB", action);
      }
  }
  
  public void deleteAllActionForService(String serviceName)  throws DatabaseException  {
    if ((serviceName== null) || (serviceName.length() > SERVICE_NAME_MAX_LENGTH)) {
      throw new DatabaseException("SODA SyncActionDBQueryManager getActionsForService ServiceName to long or null abort "+serviceName);
    }
    Array array  = this.getActionsForService(serviceName);
    for (int i=0; i<array.size(); i++)	{
    	ActionDB action = (ActionDB) array.get(i);
			FastObjectDBManager.getCurrentFODB().delete("FWKACTIONDB", action);
    }
  }

  // delete a row

  public void delete (String serviceName, String uid) throws DatabaseException  {
		FastObjectDBManager.getCurrentFODB().delete("FWKACTIONDB", new ActionDB(serviceName, uid, 0));
  }

}
