/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.synchro.client;

import java.io.Serializable;

import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.synchro.openmsp.OpenMSPException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public interface SynchroFODBReturnListener {
  
  /**
   * return synchro meta data.
   * Meta data are send inside synchronisation. On the server meta data are transfered to
   * the synchro target with the SynchroTarget method setSendSynchroMetaData(Object metadata)
   * Meta data can be use to transfert a context with the synchronisation.
   * @return
   * @throws OpenMSPException
   */
  public Serializable getSendSynchroMetada() throws OpenMSPException;
	/**
	 * Call before a server object is written in FODB.
	 * Only call during row synchronisation.
	 * If the synchronized object has to be modified before database write, 
	 * modify it and return it. Null otherwise.
	 * @param object
	 * @return null or the modified object to be store in the database.
	 * @throws DatabaseException
	 */
	public Object notifyBeginDBUpdate(Object object) throws OpenMSPException;
	
	/**
	 * Call after a server FODB object with objectId has been deleted in FODB.
	 * Only call during row synchronisation.
	 * @param objectId
	 * @throws DatabaseException
	 */
	public void notifyDBDelete(String objectId) throws OpenMSPException;
	
	/**
	 * Call when synchro begin
	 *
	 */
	public void notifySynchroBegin();
	
	/**
	 * Call when synchro end
	 *
	 */
	public void notifySynchroEnd();
	
	/**
	 * Call when a FODB object has been synchronized from the embedded device 
	 * to the server.
     * The ObjectId of the collection unique index type. ObjectId are evaluated using the 
     * collection unique index. If it's a String index, by default string index are not case sentitive.
     * So with default String index ObjectId will be in upper case.
     * To avoid this behaviour, the collection unique index descriptor must be declared case sensitive.
	 * @param ObjectId
	 */
	public void notifyFODBObjectSynchroDone(String ObjectId) throws OpenMSPException ;


}
