/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.database.fastobjectdb.synchro.client;

import java.util.Enumeration;
import java.util.Hashtable;

import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.db.FODBCollection;
import org.openmobileis.database.fastobjectdb.db.crypto.FODBCypher;
import org.openmobileis.synchro.openmsp.client.db.SyncActionDBManager;


/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * 
 * Factory pattern to manage one FODb inside an application. Use to get the current db factory
 * instance. If the db is not open or created, use the fastobjectdb.database.path fastobjectdb.database.name properties to get db path and name.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class SynchroFastObjectDBManager extends FastObjectDBManager {
	private static Hashtable cypherList;
	
	/**
	 * return the current FODB instance. Open or create it if not done.
	 * 
	 * @return the current database instance.
	 * @throws DatabaseException
	 */
	public static FastObjectDB getCurrentFODB() throws DatabaseException	{
		if (db == null)	{
			String dbPath = PropertiesManager.getManager().getProperty("fastobjectdb.database.path");
			String dbname = PropertiesManager.getManager().getProperty("fastobjectdb.database.name");
			if ((dbPath == null) || (dbname == null))	{
				throw new DatabaseException("FPDB ERROR : FODB path or name property is not set. Can't create db");
			}
			synchronized(SynchroFastObjectDBManager.class)	{
				if (db==null)	{
					try	{
						db = SynchroFastObjectDB.open(dbPath, dbname);
						ApplicationContextManager.getManager().addManager(db);
						if (cypherList != null)	{
						    Enumeration keys = cypherList.keys();
						    while (keys.hasMoreElements())	{
						        String colName = (String) keys.nextElement();
						        FODBCollection col = db.getCollection(colName);
                    if (col == null)  {
                      throw new DatabaseException("FPDB ERROR : FODB collection does not exist for declared collection cypher "+colName);
                    }
						        col.setCollectionCypher((FODBCypher)cypherList.get(colName));
						    }
						    cypherList = null;
						}
						//activate synchro action manager.
						SyncActionDBManager.registerSyncActionDBQueryManager(new FODBSyncActionDBQueryManager());
						SyncActionDBManager.getManager();
					} catch (Throwable ex)	{
						throw new DatabaseException(ex);
					}
				}
			}
		}
		return db;
	}
	
	/**
	 * Use to reload all db file headers.
	 */ 
	public static void flushDB() throws DatabaseException{
	    if (db != null)	{	
	        FODBCollection[] colList = db.getDatabaseCollectionArray();
	        cypherList = new Hashtable();
	        for (int i=0; i<colList.length; i++)	{
	            FODBCypher cypher = colList[i].getCollectionCypher();
	            if (cypher != null)	{
	                cypherList.put(colList[i].getName(), cypher);
	            }
	        }
	    }
	    db=null;
		SynchroFastObjectDBManager.getCurrentFODB();
	}
}
