/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.embedded.webserver;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

public class PathTreeDictionary {
    
    Node root_node;

    PathTreeDictionary() {
      root_node = new Node();
    }

    void put(String path, Object value) {
      StringTokenizer st = new StringTokenizer(path, "\\/");
      Node cur_node = root_node;
      while (st.hasMoreTokens()) {
        String nodename = st.nextToken();
        Node node = (Node) cur_node.get(nodename);
        if (node == null) {
          node = new Node();
          cur_node.put(nodename, node);
        }
        cur_node = node;
      }
      cur_node.object = value;
    }

    /**
     * This function looks up in the directory to find the perfect match
     * and remove matching part from path, so if you need to keep
     * original path, save it somewhere
     */
    Object[] get(String path) {
      Object[] result = new Object[2];
      if (path == null)
        return result;
      char[] ps = path.toCharArray();
      Node cur_node = root_node;
      int p0 = 0, lm = 0; // last match
      result[0] = cur_node.object;
      boolean div_state = true;
      for (int i = 0; i < ps.length; i++) {
        if (ps[i] == '/' || ps[i] == '\\') {
          if (div_state)
            continue;
          Node node = (Node) cur_node.get(new String(ps, p0, i - p0));
          if (node == null) {
            result[1] = new Integer(lm);
            return result;
          }
          if (node.object != null) {
            result[0] = node.object;
            lm = i;
          }
          cur_node = node;
          div_state = true;
        } else {
          if (div_state) {
            p0 = i;
            div_state = false;
          }
        }
      }
      cur_node = (Node) cur_node.get(new String(ps, p0, ps.length - p0));
      if (cur_node != null && cur_node.object != null) {
        result[0] = cur_node.object;
        lm = ps.length;
      }
      result[1] = new Integer(lm);
      return result;
    }

    Enumeration keys() {
      Vector result = new Vector();
      addSiblingNames(root_node, result, "");
      return result.elements();
    }

    void addSiblingNames(Node node, Vector result, String path) {
      Enumeration e = node.keys();
      while (e.hasMoreElements()) {
        String pc = (String) e.nextElement();
        Node childNode = (Node) node.get(pc);
        pc = path + '/' + pc;
        if (childNode.object != null)
          result.addElement(pc);
        addSiblingNames(childNode, result, pc);
      }
    }

    Enumeration elements() {
      Vector result = new Vector();
      addSiblingObjects(root_node, result);
      return result.elements();
    }

    void addSiblingObjects(Node node, Vector result) {
      Enumeration e = node.keys();
      while (e.hasMoreElements()) {
        Node childNode = (Node) node.get(e.nextElement());
        if (childNode.object != null)
          result.addElement(childNode.object);
        addSiblingObjects(childNode, result);
      }
    }

    class Node extends Hashtable {
      static final long serialVersionUID = 5521257935120563452L;
      Object object;
    }

}
