/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.embedded.webserver;

import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.ServletOutputStream;

import org.openmobileis.embedded.webserver.WebServerConnection;;

public class ServeOutputStream extends ServletOutputStream {

    // underneath stream
    private OutputStream out;
    //private BufferedWriter writer; // for top speed
    private WebServerConnection conn;
    private boolean returnedAsStream, returnedAsWriter;

    public ServeOutputStream(OutputStream out, WebServerConnection conn) {

      this.out = out;
      this.conn = conn;
    }

    public void print(String s) throws IOException {
      write(s.getBytes());
    }

    public void write(int b) throws IOException {
      conn.writeHeaders();
      out.write(b);
    }

    public void write(byte[] b) throws IOException {
      write(b, 0, b.length);
    }

    public void write(byte[] b, int off, int len) throws IOException {
      conn.writeHeaders();
      out.write(b, off, len);
    }

    public void flush() throws IOException {
      conn.writeHeaders();
      out.flush();
    }

    public void close() throws IOException {
      conn.writeHeaders();
      out.close();
    }

    boolean isReturnedAsStream() {
      return returnedAsStream;
    }

    void setReturnedAsStream(boolean _set) {
      returnedAsStream = _set;
    }

    boolean isReturnedAsWriter() {
      return returnedAsWriter;
    }

    void setReturnedAsWriter(boolean _set) {
      returnedAsWriter = _set;
    }
    
}
