/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.embedded.webserver;
import java.net.*;

import org.openmobileis.common.util.log.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Runnable that handle connexion process in a thread.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class ServerThread implements Runnable {

  private WebServerConnection connexion;
  private Object waiter;
  private ServerThreadPool threadPool;
  private boolean mustStop = false;

  public ServerThread() {
    waiter = new Object();
    connexion = new WebServerConnection();
    Thread thread = new Thread( this );
    thread.start();
  }

  public void run() {
    while (mustStop == false)  {
      try {
        synchronized (waiter)  {
          try {
            if (threadPool != null) {
              threadPool.disposeObject(this);
            }
            waiter.wait();
          } catch (Exception ex)  {
              LogManager.traceError(0, ex);
          }
        }
        if (mustStop) {
        	break;
        }
        connexion.run();
      } catch (Throwable ex)  {
        LogManager.traceError(LogServices.WEBSERVICE,"ServerThread run connexion error");
        LogManager.traceError(LogServices.WEBSERVICE,ex);
      }
    }
  }

  public void init(WebServer webserver, Socket socket,ServerThreadPool pool)  {
    threadPool = pool;
    connexion.init(webserver, socket);
    synchronized (waiter)  {
      waiter.notify();
    }
  }
  
  public WebServerConnection getConnexion() {
    return connexion;
  }
  
  public void notifyStop() {
	  this.mustStop = true;
	  synchronized (waiter) {
		  waiter.notify();
	  }
  }


} 