/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.data;

import java.io.Serializable;

import org.openmobileis.common.util.collection.Array;

public final class Account  implements Serializable{
  //add serial version to avoid serialization error if the version change
  // without class change. 
  static final long serialVersionUID = 5521257935120563452L;
  
  //define the category for all activity label in LabelManager
  public static final String ACCOUNT_ACTIVITY_LABEL_CATEGORY = "1";
  private String id;
  private String name, address, city;
  private String activity;
  private Array contactList;

  public Account(String id) {
    super();
    this.id = id;
    contactList = new Array();
  }
  
  public Array getAllContacts() {
    return contactList;
  }
  
  public void addContact(Contact contact)   {
    if (contact == null) return;
    for (int i=0; i<contactList.size(); i++)    {
      Contact c = (Contact)contactList.get(i);
      if (c.getId().equals(contact.getId()))  {
        contactList.replace(i, contact);
        return;
      }
    }    
    contactList.add(contact);
  }
  
  public Contact getContactById(String contactid){
    if (contactid == null) return null;
    Contact contact = null;    
    for (int i=0; i<contactList.size(); i++)    {
      Contact c = (Contact)contactList.get(i);
      if (c.getId().equals(contactid))  {
        contact = c;
      }
    }    
    return contact;
  }
  
  public void removeContact(String contactid)   {
    if (contactid == null) return;
    for (int i=0; i<contactList.size(); i++)    {
      Contact c = (Contact)contactList.get(i);
      if (c.getId().equals(contactid))  {
        contactList.remove(i);
      }
    }    
  }

  public String getActivity() {
    return activity;
  }

  public void setActivity(String activity) {
    this.activity = activity;
  }

  public String getAddress() {
    return address;
  }

  public void setAddress(String address) {
    this.address = address;
  }

  public String getCity() {
    return city;
  }

  public void setCity(String city) {
    this.city = city;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public String getId() {
    return id;
  }
    
  public boolean equals(Object obj)   {
      if (this.id.equals(((Account)obj).id))    {
          return true;
      }
      return false;
  }
  
  public int hashCode()   {
      return this.id.hashCode();
  }

}
