/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.data;

import java.io.Serializable;

import org.openmobileis.common.util.collection.Array;

public final class Contact implements Serializable {
  //add serial version to avoid serialization error if the version change
  // without class change. 
  static final long serialVersionUID = 5521257935120563452L;
 
  //define the category for all function label in LabelManager
  public static final String CONTACT_FUNCTION_LABEL_CATEGORY = "2";
  private String id;
  private String firstname, lastname;
  private int function;
  private Array reportList;

  public Contact(String id) {
    super();
    this.id = id;
    reportList = new Array();
 }
  
  public Array getAllReports() {
    return reportList;
  }
  
  public void addReport(Report report)   {
    if (report == null) return;
    for (int i=0; i<reportList.size(); i++)    {
      Report r = (Report)reportList.get(i);
      if (r.getId().equals(report.getId()))  {
        reportList.replace(i, report);
        return;
      }
    }    
    reportList.add(report);
  }
  
  public void removeReport(String reportid)   {
    if (reportid == null) return;
    for (int i=0; i<reportList.size(); i++)    {
      Report r = (Report)reportList.get(i);
      if (r.getId().equals(reportid))  {
        reportList.remove(i);
      }
    }    
  }
  
  public Report getReportById(String reportid){
    if (reportid == null) return null;
    Report report = null;    
    for (int i=0; i<reportList.size(); i++)    {
      Report r = (Report)reportList.get(i);
      if (r.getId().equals(reportid))  {
        report = r;
      }
    }    
    return report;
  }

  public String getFirstname() {
    return firstname;
  }

  public void setFirstname(String firstname) {
    this.firstname = firstname;
  }

  public int getFunction() {
    return function;
  }

  public void setFunction(int function) {
    this.function = function;
  }

  public String getLastname() {
    return lastname;
  }

  public void setLastname(String lastname) {
    this.lastname = lastname;
  }

  public String getId() {
    return id;
  }
  
  public boolean equals(Object obj)   {
      if (this.id.equals(((Contact)obj).id))    {
          return true;
      }
      return false;
  }
  
  public int hashCode()   {
      return this.id.hashCode();
  }

}
