/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.data.jdbc;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.util.Calendar;
import java.util.Date;

import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.database.AbstractQueryManager;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget;
import org.openmobileis.examples.mycrm.data.Account;
import org.openmobileis.examples.mycrm.data.Contact;
import org.openmobileis.examples.mycrm.data.Report;
import org.openmobileis.synchro.algo.replication.AlwaysUpdateServerSynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.utils.DefaultSynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.utils.SynchroAtomicObjectManager;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.client.db.DBImportFileCoder;
import org.openmobileis.synchro.security.auth.Credential;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class MyCrmAccountSynchroTarget extends AbstractQueryManager implements FODBSyncTarget {

  private static String queryCreateAccountTable = "CREATE TABLE ACCOUNT (" + " ID varchar(30) NOT NULL," + " NAME varchar(50) NOT NULL,"
      + " ADDRESS varchar(100) NOT NULL," + " CITY varchar(50) NOT NULL," + " ACTIVITY varchar(5) NOT NULL," + " CONSTRAINT idaccount PRIMARY KEY (ID)" + ")";

  private static String queryCreateContactTable = "CREATE TABLE CONTACT (" + " ID varchar(30) NOT NULL," + " IDCLIENT varchar(30) NOT NULL,"
      + " FIRSTNAME varchar(50) NOT NULL," + " LASTNAME varchar(100) NOT NULL," + " FUNCTION int ," + " CONSTRAINT idcontact PRIMARY KEY (ID)" + ")";

  private static String queryCreateLeadsTable = "CREATE TABLE LEADS (" + " ID varchar(30) NOT NULL," + " IDCLIENT varchar(30) NOT NULL," + " STATE int ,"
      + " AMOUNT int ," + " DATE bigint ," + " DESCRIPTION varchar(150) NOT NULL," + " CONSTRAINT idleads PRIMARY KEY (ID)" + ")";

  private static String queryCreateReportTable = "CREATE TABLE REPORT ("
    + " ID varchar(30) NOT NULL,"
    + " IDCONTACT varchar(30) NOT NULL,"
    + " ACTION int ,"
    + " DATE bigint ,"
    + " DESCRIPTION varchar(150) NOT NULL,"
    + " TERMINALUSERID varchar(5) NOT NULL,"
    + " CONSTRAINT idreport PRIMARY KEY (ID)"
    + ")";

  private static String queryInsertAccount = "INSERT INTO ACCOUNT (ID,NAME,ADDRESS,CITY,ACTIVITY )" + " VALUES ('%0%', '%1%', '%2%', '%3%', %4%)";

 // private static String queryUpdateAccount = "UPDATE ACCOUNT SET NAME='%0%', ADDRESS='%1%', CITY ='%2%' , ACTIVITY =%3% " + " WHERE ID='%4%'";

  private static String queryDeleteAccount = "DELETE FROM ACCOUNT where ID='%0%'";

  private static String queryGetAccount = "SELECT ID,NAME,ADDRESS,CITY,ACTIVITY from ACCOUNT where ID='%0%'";

  private static String queryGetAllAccountId = "SELECT ID from ACCOUNT";

  private static String queryInsertContact = "INSERT INTO CONTACT (ID,IDCLIENT,FIRSTNAME,LASTNAME,FUNCTION )" + " VALUES ('%0%', '%1%', '%2%', '%3%', %4%)";

 // private static String queryUpdateContact = "UPDATE CONTACT SET IDCLIENT='%0%', FIRSTNAME='%1%', LASTNAME ='%2%' , FUNCTION =%3% " + " WHERE ID='%4%'";

  private static String queryDeleteContact = "DELETE FROM CONTACT where IDCLIENT='%0%'";

  private static String queryGetContact = "SELECT ID,IDCLIENT,FIRSTNAME,LASTNAME,FUNCTION from CONTACT where IDCLIENT='%0%'";

  private static String queryInsertLeads = "INSERT INTO LEADS (ID,IDCLIENT,STATE,AMOUNT,DATE,DESCRIPTION )" + " VALUES ('%0%', '%1%', '%2%', %3%, %4%, '%5%')";

 // private static String queryUpdateLeads = "UPDATE LEADS SET IDCLIENT=%0%, STATE=%1%, AMOUNT =%2%, DATE =%3%, DESCRIPTION='%4%' " + " WHERE ID='%5%'";

 // private static String queryDeleteLeads = "DELETE FROM LEADS where ID='%0%'";

//  private static String queryGetLeads = "SELECT ID,IDCLIENT,STATE,AMOUNT,DATE,DESCRIPTION from LEADS where ID='%0%'";

  private static String queryInsertReport = "INSERT INTO REPORT (ID,IDCONTACT,ACTION,DATE,DESCRIPTION,TERMINALUSERID )" + " VALUES ('%0%', '%1%', %2%, %3%, '%4%', '%5%')";

 // private static String queryUpdateReport = "UPDATE REPORT SET IDCONTACT='%0%', ACTION=%1%, DATE =%2% , DESCRIPTION ='%3%' ,TERMINALUSERID='%4%'" + " WHERE ID='%5%'";

  private static String queryDeleteReport = "DELETE FROM REPORT where REPORT.IDCONTACT IN (SELECT CONTACT.ID FROM CONTACT WHERE CONTACT.IDCLIENT ='%0%')";

  private static String queryGetReport = "SELECT ID,IDCONTACT,ACTION,DATE,DESCRIPTION,TERMINALUSERID from REPORT where IDCONTACT='%0%'";

  /**
   * 
   */
  public MyCrmAccountSynchroTarget() throws ServiceException {
    super();
    try {
      this.createSyncTable();

    } catch (Throwable ex) {
      throw new ServiceException(ex);
    }
  }
  
  /**
   * Synchro meta data added to the synchro on the terminal side are given be the 
   * synchro process.
   * This method is call before all other synchro method.
   * @param metadata
   * @throws OpenMSPException
   */
  public void setSendSynchroMetaData(Object metadata)throws OpenMSPException    {
    
  }

  /*
   * check if the sync table exists. IF it does not exist, create it
   */
  private void createSyncTable() throws DatabaseException, java.sql.SQLException, SynchroException {
    try {
      Connection con = this.getDbManager().getConnection();
      DatabaseMetaData dbmd = con.getMetaData();
      String[] tableType = { "TABLE" };
      ResultSet result = dbmd.getTables(null, null, "ACCOUNT", tableType);
      if (!result.next()) {
        this.executeUpdate(MyCrmAccountSynchroTarget.queryCreateAccountTable, null);
        this.executeUpdate(MyCrmAccountSynchroTarget.queryCreateContactTable, null);
        this.executeUpdate(MyCrmAccountSynchroTarget.queryCreateLeadsTable, null);
        this.executeUpdate(MyCrmAccountSynchroTarget.queryCreateReportTable, null);
        //create default account
        String[] parameters = { "1111", "Green fields LTD", "123 yellow grass", "London", "0" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertAccount, parameters);
        SynchroAtomicObjectManager.getManager().updateAtomicObject(new DefaultSynchroAtomicObject("1111", ""), this.getCollectionName(), null);
        parameters = new String[] { "2222", "Blue Red", "23 Action Lake", "London", "1" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertAccount, parameters);
        SynchroAtomicObjectManager.getManager().updateAtomicObject(new DefaultSynchroAtomicObject("2222", ""), this.getCollectionName(), null);
        parameters = new String[] { "3333", "ADGR", "34 Open Garden", "London", "1" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertAccount, parameters);
        SynchroAtomicObjectManager.getManager().updateAtomicObject(new DefaultSynchroAtomicObject("3333", ""), this.getCollectionName(), null);
        parameters = new String[] { "4444", "EFFECT VIEW", "78 Red House", "London", "2" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertAccount, parameters);
        SynchroAtomicObjectManager.getManager().updateAtomicObject(new DefaultSynchroAtomicObject("4444", ""), this.getCollectionName(), null);

        //create default contact ID,IDCLIENT,FIRSTNAME,LASTNAME,FUNCTION
        parameters = new String[] { "1111", "1111", "Phil", "Gartner", "1" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertContact, parameters);
        parameters = new String[] { "2222", "1111", "Mac", "Urland", "2" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertContact, parameters);
        parameters = new String[] { "3333", "2222", "Tom", "Petite", "3" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertContact, parameters);

        //create default Leads ID,IDCLIENT,STATE,AMOUNT,DATE,DESCRIPTION
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.MONTH, 1);
        long time = cal.getTimeInMillis();
        String date = Long.toString(time);
        parameters = new String[] { "1111", "1111", "0", "250000", date, "Boat construction" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertLeads, parameters);
        parameters = new String[] { "2222", "1111", "0", "50000", date, "Boat transport" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertLeads, parameters);
        parameters = new String[] { "3333", "1111", "0", "100000", date, "Main store construction" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertLeads, parameters);
        parameters = new String[] { "4444", "2222", "0", "10000", date, "Server architecture" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertLeads, parameters);
        parameters = new String[] { "5555", "3333", "0", "150000", date, "Architecture update" };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertLeads, parameters);
      }
    } finally {
      this.getDbManager().garbageOpenedConnection();
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionName()
   */
  public String getCollectionName() {
    return "account";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionObjectWithId(java.lang.String)
   */
  public Object getCollectionObjectWithId(String id) throws OpenMSPException {
    try {
      Account account = null;
      String[] param = { id };
      ResultSet result = this.executeQuery(MyCrmAccountSynchroTarget.queryGetAccount, param);
      if (result.next()) {
        //ID,NAME,ADDRESS,CITY,ACTIVITY 
        account = new Account(result.getString(1));
        account.setName(DBImportFileCoder.getCoder().removeAntiSlash(result.getString(2)));
        account.setAddress(DBImportFileCoder.getCoder().removeAntiSlash(result.getString(3)));
        account.setCity(DBImportFileCoder.getCoder().removeAntiSlash(result.getString(4)));
        account.setActivity(result.getString(5));
      } else {
        return null;
      }
      result = this.executeQuery(MyCrmAccountSynchroTarget.queryGetContact, param);
      while (result.next()) {
        //ID,IDCLIENT,FIRSTNAME,LASTNAME,FUNCTION
        Contact c = new Contact(result.getString(1));
        c.setFirstname(DBImportFileCoder.getCoder().removeAntiSlash(result.getString(3)));
        c.setLastname(DBImportFileCoder.getCoder().removeAntiSlash(result.getString(4)));
        c.setFunction(result.getInt(5));
        account.addContact(c);
      }
      Array contactList = account.getAllContacts();
      for (int i = 0; i < contactList.size(); i++) {
        Contact c = (Contact) contactList.get(i);
       param = new String[] { c.getId() };
       result = this.executeQuery(MyCrmAccountSynchroTarget.queryGetReport, param);
        while (result.next()) {
          //ID,IDCONTACT,ACTION,DATE,DESCRIPTION
          Report r = new Report(result.getString(1));
          r.setAction(result.getInt(3));
          r.setReportDate(new Date(result.getLong(4)));
          r.setDescription(DBImportFileCoder.getCoder().removeAntiSlash(result.getString(5)));
          r.setTerminalUserId(result.getString(6));
          c.addReport(r);
        }
      }
      return account;
    } catch (Throwable ex) {
      throw new OpenMSPException(ex);
    } finally {
      this.getDbManager().garbageOpenedConnection();
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#updateCollectionObject(java.lang.Object)
   */
  public void updateCollectionObject(Object obj) throws OpenMSPException {
    try {
      Account account = (Account) obj;
      String[] param = { account.getId() };
      this.getDbManager().getConnection().setAutoCommit(false);
      try {
        //first delete all
        this.executeUpdate(MyCrmAccountSynchroTarget.queryDeleteReport, param);
        this.executeUpdate(MyCrmAccountSynchroTarget.queryDeleteContact, param);
        this.executeUpdate(MyCrmAccountSynchroTarget.queryDeleteAccount, param);

        //create all
        //ID,NAME,ADDRESS,CITY,ACTIVITY
        param = new String[] { 
            account.getId()
            , DBImportFileCoder.getCoder().cleanAndFormatStringforDB(account.getName())
            , DBImportFileCoder.getCoder().cleanAndFormatStringforDB(account.getAddress())
            , DBImportFileCoder.getCoder().cleanAndFormatStringforDB(account.getCity())
            , account.getActivity() 
           };
        this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertAccount, param);
        Array contactList = account.getAllContacts();
        for (int i = 0; i < contactList.size(); i++) {
          Contact c = (Contact) contactList.get(i);
          Array reportList = c.getAllReports();
          for (int j = 0; j < reportList.size(); j++) {
            Report r = (Report) reportList.get(j);
            //ID,IDCONTACT,ACTION,DATE,DESCRIPTION
            param = new String[] { 
                r.getId()
                , c.getId()
                , Integer.toString(r.getAction())
                , Long.toString(r.getReportDate().getTime())
                , DBImportFileCoder.getCoder().cleanAndFormatStringforDB(r.getDescription())
                , r.getTerminalUserId()
             };
            this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertReport, param);
          }
          //ID,IDCLIENT,FIRSTNAME,LASTNAME,FUNCTION
          param = new String[] { 
              c.getId()
              , account.getId()
              , DBImportFileCoder.getCoder().cleanAndFormatStringforDB(c.getFirstname())
              , DBImportFileCoder.getCoder().cleanAndFormatStringforDB(c.getLastname())
              , Integer.toString(c.getFunction()) };
          this.executeUpdate(MyCrmAccountSynchroTarget.queryInsertContact, param);
        }
        
        //update modification notification in Atomic Object table
        SynchroAtomicObjectManager.getManager().updateAtomicObject(new DefaultSynchroAtomicObject(account.getId(), ""), this.getCollectionName(), null);
        
        this.getDbManager().getConnection().commit();
      } finally {
        this.getDbManager().getConnection().setAutoCommit(true);
      }
    } catch (Throwable ex) {
      throw new OpenMSPException(ex);
    } finally {
      this.getDbManager().garbageOpenedConnection();
    }

  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#deleteCollectionObject(java.lang.String)
   */
  public void deleteCollectionObject(String id) throws OpenMSPException {
    try {
      String[] param = { id };
      this.getDbManager().getConnection().setAutoCommit(false);
      try {
        //first delete all
        this.executeUpdate(MyCrmAccountSynchroTarget.queryDeleteReport, param);
        this.executeUpdate(MyCrmAccountSynchroTarget.queryDeleteContact, param);
        this.executeUpdate(MyCrmAccountSynchroTarget.queryDeleteAccount, param);

        //update modification notification in Atomic Object table
        DefaultSynchroAtomicObject ao = new DefaultSynchroAtomicObject(id, "");
        ao.setModificationType(SynchroAtomicObject.DELETE);
        SynchroAtomicObjectManager.getManager().updateAtomicObject(ao, this.getCollectionName(), null);

        this.getDbManager().getConnection().commit();
      } finally {
        this.getDbManager().getConnection().setAutoCommit(true);
      }
    } catch (Throwable ex) {
      throw new OpenMSPException(ex);
    } finally {
      this.getDbManager().garbageOpenedConnection();
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getConflicResolver()
   */
  public SynchroConflicResolver getConflicResolver() {
    return new AlwaysUpdateServerSynchroConflicResolver(); //PDA WIN always update server
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getAllCollectionObject()
   */
  public Array getAllCollectionObject() throws OpenMSPException {

    Array ret = new Array();
    try {
      Array list = new Array();
      ResultSet result = this.executeQuery(MyCrmAccountSynchroTarget.queryGetAllAccountId, null);
      while (result.next()) {
        list.add(result.getString(1));
      }

      for (int i = 0; i < list.size(); i++) {
        ret.add(this.getCollectionObjectWithId((String) list.get(i)));
      }

    } catch (Throwable ex) {
      throw new OpenMSPException(ex);
    } finally {
      this.getDbManager().garbageOpenedConnection();
    }
    return ret;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionObjectId(java.lang.Object)
   */
  public String getCollectionObjectId(Object obj) {
    return ((Account) obj).getId();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getAllModifiedAtomicObjectForServiceSince(java.lang.String, long)
   */
  public SynchroAtomicObject[] getAllModifiedAtomicObjectSince(long date) throws OpenMSPException {
    try {
      return SynchroAtomicObjectManager.getManager().getAllModifiedAtomicObjectForServiceSince(this.getCollectionName(), date, null);
    } catch (SynchroException ex)   {
      throw new OpenMSPException(ex);
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getUpdateMaxNbRow()
   */
  public int getUpdateMaxNbRow() {
    return 10; //start complete when there is 2 or more account modifiation
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#updateSynchroDB(org.openmobileis.database.fastobjectdb.FastObjectDB)
   */
  public void updateSynchroDB(FastObjectDB db) throws OpenMSPException {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#connect(org.openmobileis.synchro.security.auth.Credential)
   */
  public void connect(Credential cred) throws UserNotFoundException, ServiceException {
    // do nothing
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#disconnect()
   */
  public void disconnect() {
    // do nothing
  }

}
