/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2005 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.data.jdbc;

import java.sql.ResultSet;

import org.openmobileis.common.user.User;
import org.openmobileis.common.user.UserAlreadyExistException;
import org.openmobileis.common.user.UserManagerFactory;
import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class MyCrmJDBCUserFactory implements UserManagerFactory {
  private MyCrmUserManagerDBQuery userquery;
  /**
   * 
   */
  public MyCrmJDBCUserFactory() {
    super();
    userquery = new MyCrmUserManagerDBQuery();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.entity.user.EntityUserFactory#createUser(java.lang.String, java.lang.String, java.lang.String)
   */
  public void deleteUser(String userID) throws ServiceException, UserNotFoundException	{
    throw new ServiceException("not implemented");
  }

  /* (non-Javadoc)
   * @see org.openmobileis.entity.user.EntityUserFactory#createUser(java.lang.String, java.lang.String, java.lang.String)
   */
  public void storeUser(User user) throws ServiceException, UserAlreadyExistException	{
    throw new UserAlreadyExistException("not implemented");
  }

  /* (non-Javadoc)
   * @see org.openmobileis.entity.user.EntityUserFactory#getUser(java.lang.String)
   */
  public User getUser(String userID) throws ServiceException, UserNotFoundException	{
		User user= null;
    try	{
			String[] param = new String[]{userID};
			ResultSet result = userquery.getUser(param);
			if (result.next()) {
			  user = new User(result.getString(1), result.getString(2), result.getString(3));
              user.setProfil("default");
              user.setGroup("default");
			} else	{
			  throw new UserNotFoundException();
			}
			result.close();
		} catch (Exception ex)	{
			throw new ServiceException(ex);
		} finally {
          userquery.getDbManager().garbageOpenedConnection();
		}
		return user;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.entity.user.EntityUserFactory#authenticateUser(java.lang.String, java.lang.String)
   */
  public String authenticateUser(String login, String pass) throws ServiceException, UserNotFoundException	{
		try {
			String[] params = {login, pass};
			ResultSet result = userquery.getUserIDWithloginPass(params);
			if (result.next()) {
				String id =  result.getString(1);
				result.close();
				return id;
			}
		}  catch (Throwable ex) {
			LogManager.traceError(0, ex);
			throw new ServiceException(ex);
		} finally {
		  userquery.getDbManager().garbageOpenedConnection();
		}
        throw new UserNotFoundException("User not founr for login :"+login);
  }
}
