/**
 *  OpenMobileIS - an open Java Framework for mobile applications
 *
 *   Copyright (C) 2005  Ubikis.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */
package org.openmobileis.examples.mycrm.data.jdbc;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;




import org.openmobileis.common.util.database.AbstractQueryManager;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.examples.mycrm.data.Account;
import org.openmobileis.examples.mycrm.data.Contact;
import org.openmobileis.examples.mycrm.data.Leads;
import org.openmobileis.examples.mycrm.data.Report;
import org.openmobileis.modules.common.data.Label;
import org.openmobileis.modules.common.database.fodb.FODBLabel;





/**
 * @author Manuel Gomez
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class MyCrmLabelQuery extends AbstractQueryManager {


	protected static String queryInsertLabel = "INSERT INTO label (labelId, categoryId, label, langue)"
		+" VALUES ('%0%', '%1%', '%2%', '%3%')";
	
	protected static String queryGetALlIDsLabels = "SELECT labelId from label";
	
	protected static String queryGetLabel = "SELECT labelId, categoryId, label from label where labelId='%0%' AND categoryId='%1%'";
	protected static String queryGetLabelsForCategory = "SELECT labelId, categoryId, label from label where categoryId='%0%'";
	protected static String queryGetALlLabels = "SELECT labelId, categoryId, label from label";
	protected static String queryDeleteLabel = "DELETE FROM label where labelId='%0%'AND categoryId='%1%'";
  
  public MyCrmLabelQuery() {
    super();
    try{
      this.init();  
    }catch(Throwable ex){
      LogManager.traceError(0, ex);
    }
  }
  
  public void init() throws ServiceException {
    try{
      
      this.createSyncTable();

    }catch(Throwable ex){
      throw new ServiceException(ex);
    }
    
    
  }

	protected String getCreateTableQuery()	{
		return "CREATE TABLE label ("
		+"	labelId varchar(5) NOT NULL,"
		+"	categoryId varchar(5) NOT NULL,"
		+"	label varchar(250),"
		+"	langue varchar(5) NOT NULL, "
		+"	CONSTRAINT label_id PRIMARY KEY (labelId, categoryId, langue)"
		+");";
	}
  
  public Label convertLabelFromResult(ResultSet result) throws Exception {
		try	{		
			return new FODBLabel(result.getString(1), result.getString(2), result.getString(3));
		} catch (Throwable ex)	{
			throw new ServiceException(ex);
		}
  }
  
  public void createLabel(String[] parameters) throws DatabaseException {
  	this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
  }
  

  public ResultSet getAllLabelIds() throws DatabaseException{
		return this.executeQuery(MyCrmLabelQuery.queryGetALlIDsLabels, null);
  }
  
  

	/*
	* check if the sync table exists. IF it does not exist, create it
	*/
	private void createSyncTable() throws DatabaseException, java.sql.SQLException {
		try {
			Connection con = this.getDbManager().getConnection();
			DatabaseMetaData dbmd = con.getMetaData();
			String[] tableType = {"TABLE"};
			ResultSet result = dbmd.getTables (null,null,"LABEL", tableType);
			if (!result.next()) {
				this.executeUpdate(this.getCreateTableQuery(), null);
                  //account activity
                  String[] parameters = {"0",Account.ACCOUNT_ACTIVITY_LABEL_CATEGORY,"Industry","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
                  parameters = new String[]{"1",Account.ACCOUNT_ACTIVITY_LABEL_CATEGORY,"Service","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
                  parameters = new String[]{"2",Account.ACCOUNT_ACTIVITY_LABEL_CATEGORY,"Pharmaceutique","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
                  parameters = new String[]{"3",Account.ACCOUNT_ACTIVITY_LABEL_CATEGORY,"Gouvernment","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);

                  //contact function
                  parameters = new String[]{"0",Contact.CONTACT_FUNCTION_LABEL_CATEGORY,"CEO","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
                  parameters = new String[]{"1",Contact.CONTACT_FUNCTION_LABEL_CATEGORY,"Commercial","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
                  parameters = new String[]{"2",Contact.CONTACT_FUNCTION_LABEL_CATEGORY,"Buyer","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
                  parameters = new String[]{"3",Contact.CONTACT_FUNCTION_LABEL_CATEGORY,"Manager","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);

                  //Leads state
                  parameters = new String[]{"0",Leads.LEADS_STATE_LABEL_CATEGORY,"Open","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
                  parameters = new String[]{"1",Leads.LEADS_STATE_LABEL_CATEGORY,"Closed","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);

                  //Report action
                  parameters = new String[]{"0",Report.REPORT_ACTION_LABEL_CATEGORY,"Presentation","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
                  parameters = new String[]{"1",Report.REPORT_ACTION_LABEL_CATEGORY,"Demo","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
                  parameters = new String[]{"2",Report.REPORT_ACTION_LABEL_CATEGORY,"Congres","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
                  parameters = new String[]{"3",Report.REPORT_ACTION_LABEL_CATEGORY,"Phoning","EN"};
                  this.executeUpdate(MyCrmLabelQuery.queryInsertLabel, parameters);
			}
		} finally {
			this.getDbManager().garbageOpenedConnection();
		}
	}

  /* (non-Javadoc)
   * @see org.openmobileis.modules.crm.server.synchro.services.common.FODBLabelSynchroQuery#getLabel(java.lang.String[])
   */
  public ResultSet getLabel(String[] param) throws DatabaseException{
		return this.executeQuery(MyCrmLabelQuery.queryGetLabel, param);
  }

  /* (non-Javadoc)
   * @see org.openmobileis.modules.crm.server.synchro.services.common.FODBLabelSynchroQuery#getAllLabel()
   */
  public ResultSet getAllLabel() throws DatabaseException{
		return this.executeQuery(MyCrmLabelQuery.queryGetALlLabels, null);
  }

}
