/**
 *  OpenMobileIS - an open Java Framework for mobile applications
 *
 *   Copyright (C) 2005  Ubikis.
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */
package org.openmobileis.examples.mycrm.data.jdbc;


import java.sql.ResultSet;



import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget;

import org.openmobileis.modules.common.data.Label;
import org.openmobileis.modules.common.database.fodb.FODBLabel;
import org.openmobileis.synchro.algo.replication.AlwaysUpdateServerSynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.utils.DefaultSynchroAtomicObject;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.security.auth.Credential;



/**
 * @author Manuel Gomez
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class MyCrmLabelSynchroTarget implements FODBSyncTarget {
	
//	public static final String LABEL_ATOMIC_OBJECT_UID="%labeluid1%";

  private MyCrmLabelQuery query = null;
	
  public MyCrmLabelSynchroTarget() {
    super();
    query = new MyCrmLabelQuery();

  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionName()
   */
  public String getCollectionName() {
    return "label";
  }
  
  /**
   * Synchro meta data added to the synchro on the terminal side are given be the 
   * synchro process.
   * This method is call before all other synchro method.
   * @param metadata
   * @throws OpenMSPException
   */
  public void setSendSynchroMetaData(Object metadata)throws OpenMSPException    {
    
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionObjectWithId(java.lang.String)
   */
  public Object getCollectionObjectWithId(String id) throws OpenMSPException {
    
		Label label = null;
		try	{
			String[] param = FODBLabel.getLabelIdAndCatIdFromId(id);
			ResultSet result = query.getLabel(param);
			while (result.next())	{
				label = query.convertLabelFromResult(result);
			}
		} catch (Throwable ex)	{
			throw new OpenMSPException(ex);
		} finally {
			query.getDbManager().garbageOpenedConnection();
		}
		return label;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#updateCollectionObject(java.lang.Object)
   */
  public void updateCollectionObject(Object obj) throws OpenMSPException {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#deleteCollectionObject(java.lang.String)
   */
  public void deleteCollectionObject(String id) throws OpenMSPException {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getConflicResolver()
   */
  public SynchroConflicResolver getConflicResolver() {
    return new AlwaysUpdateServerSynchroConflicResolver();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getAllCollectionObject()
   */
  public Array getAllCollectionObject() throws OpenMSPException {
    
		Array list = new Array();
			try	{
				ResultSet result = query.getAllLabel();
				while (result.next())	{
					list.add(query.convertLabelFromResult(result));
				}
			} catch (Throwable ex)	{
				throw new OpenMSPException(ex);
			} finally {
				query.getDbManager().garbageOpenedConnection();
			}
		return list;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionObjectId(java.lang.Object)
   */
  public String getCollectionObjectId(Object obj) {
    return ((FODBLabel)obj).getLabelCategoryId();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getAllModifiedAtomicObjectForServiceSince(java.lang.String, long)
   */
  public SynchroAtomicObject[] getAllModifiedAtomicObjectSince(long date) throws OpenMSPException {
    if (date == 0)  { //synchronize only during complete synchro
      SynchroAtomicObject[] sync = new  SynchroAtomicObject[1];
      sync[0] = new DefaultSynchroAtomicObject("", "");
      return sync;
    }
    return new SynchroAtomicObject[0];
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getUpdateMaxNbRow()
   */
  public int getUpdateMaxNbRow() {
    return 0;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#updateSynchroDB(org.openmobileis.database.fastobjectdb.FastObjectDB)
   */
  public void updateSynchroDB(FastObjectDB db) throws OpenMSPException {
  }
  
  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#connect(org.openmobileis.synchro.security.auth.Credential)
   */
  public void connect(Credential cred) throws UserNotFoundException, ServiceException {
    // do nothing
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#disconnect()
   */
  public void disconnect() {
    // do nothing
  }
}
