/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.data.jdbc;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.util.Calendar;
import java.util.Date;

import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.database.AbstractQueryManager;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget;
import org.openmobileis.examples.mycrm.data.Leads;
import org.openmobileis.synchro.algo.replication.AlwaysUpdateServerSynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.utils.DefaultSynchroAtomicObject;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.security.auth.Credential;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class MyCrmLeadsSynchroTarget extends AbstractQueryManager implements FODBSyncTarget {

  private static String queryCreateLeadsTable = "CREATE TABLE LEADS (" + " ID varchar(30) NOT NULL," + " IDCLIENT varchar(30) NOT NULL," + " STATE int ,"
      + " AMOUNT int ," + " DATE long ," + " DESCRIPTION varchar(150) NOT NULL," + " CONSTRAINT dataid PRIMARY KEY (ID)" + ")";

  private static String queryInsertLeads = "INSERT INTO LEADS (ID,IDCLIENT,STATE,AMOUNT,DATE,DESCRIPTION )" + " VALUES ('%0%', '%1%', '%2%', %3%, %4%, '%5%')";

//  private static String queryUpdateLeads = "UPDATE LEADS SET IDCLIENT=%0%, STATE=%1%, AMOUNT =%2%, DATE =%3%, DESCRIPTION='%4%' " + " WHERE ID='%5%'";

  private static String queryDeleteLeads = "DELETE FROM LEADS where ID='%0%'";

  private static String queryGetLeads = "SELECT ID,IDCLIENT,STATE,AMOUNT,DATE,DESCRIPTION from LEADS where ID='%0%'";
  private static String queryGetAllLeads = "SELECT ID from LEADS where STATE=0";

  /**
   * 
   */
  public MyCrmLeadsSynchroTarget() throws ServiceException {
    super();
    try {
      this.createSyncTable();

    } catch (Throwable ex) {
      throw new ServiceException(ex);
    }
  }
  
  /**
   * Synchro meta data added to the synchro on the terminal side are given be the 
   * synchro process.
   * This method is call before all other synchro method.
   * @param metadata
   * @throws OpenMSPException
   */
  public void setSendSynchroMetaData(Object metadata)throws OpenMSPException    {
    
  }

  /*
   * check if the sync table exists. IF it does not exist, create it
   */
  private void createSyncTable() throws DatabaseException, java.sql.SQLException {
    try {
      Connection con = this.getDbManager().getConnection();
      DatabaseMetaData dbmd = con.getMetaData();
      String[] tableType = { "TABLE" };
      ResultSet result = dbmd.getTables(null, null, "ACCOUNT", tableType);
      if (!result.next()) {
        this.executeUpdate(MyCrmLeadsSynchroTarget.queryCreateLeadsTable, null);

        //create default Leads ID,IDCLIENT,STATE,AMOUNT,DATE,DESCRIPTION
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.MONTH, 1);
        long time = cal.getTimeInMillis();
        String date = Long.toString(time);
        String[] parameters = new String[] { "1111", "1111", "0", "250000", date, "Boat construction" };
        this.executeUpdate(MyCrmLeadsSynchroTarget.queryInsertLeads, parameters);
        parameters = new String[] { "2222", "1111", "0", "50000", date, "Boat transport" };
        this.executeUpdate(MyCrmLeadsSynchroTarget.queryInsertLeads, parameters);
        parameters = new String[] { "3333", "1111", "0", "100000", date, "Main store construction" };
        this.executeUpdate(MyCrmLeadsSynchroTarget.queryInsertLeads, parameters);
        parameters = new String[] { "4444", "2222", "0", "10000", date, "Server architecture" };
        this.executeUpdate(MyCrmLeadsSynchroTarget.queryInsertLeads, parameters);
        parameters = new String[] { "5555", "3333", "0", "150000", date, "Architecture update" };
        this.executeUpdate(MyCrmLeadsSynchroTarget.queryInsertLeads, parameters);
      }
    } finally {
      this.getDbManager().garbageOpenedConnection();
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionName()
   */
  public String getCollectionName() {
    return "leads";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionObjectWithId(java.lang.String)
   */
  public Object getCollectionObjectWithId(String id) throws OpenMSPException {
    try {
      Leads lead = null;
      String[] param = { id };
      ResultSet result = this.executeQuery(MyCrmLeadsSynchroTarget.queryGetLeads, param);
      if (result.next()) {
        //ID,IDCLIENT,STATE,AMOUNT,DATE,DESCRIPTION 
        lead = new Leads(result.getString(1));
        lead.setIdaccount(result.getString(2));
        lead.setState(result.getInt(3));
        lead.setAmount(result.getInt(4));
        lead.setCloseDate(new Date(result.getLong(5)));
        lead.setDescription(result.getString(6));
       }
      return lead;
    } catch (Throwable ex) {
      throw new OpenMSPException(ex);
    } finally {
      this.getDbManager().garbageOpenedConnection();
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#updateCollectionObject(java.lang.Object)
   */
  public void updateCollectionObject(Object obj) throws OpenMSPException {
    try {
      Leads lead = (Leads) obj;
      String[] param = { lead.getId() };
      this.getDbManager().getConnection().setAutoCommit(false);
      try {
        //first delete all
        this.executeUpdate(MyCrmLeadsSynchroTarget.queryDeleteLeads, param);
 
        //create all
        //ID,IDCLIENT,STATE,AMOUNT,DATE,DESCRIPTION
        param = new String[] { lead.getId(), lead.getIdaccount(), Integer.toString(lead.getState()), Long.toString(lead.getCloseDate().getTime()), lead.getDescription() };
        this.executeUpdate(MyCrmLeadsSynchroTarget.queryInsertLeads, param);
        this.getDbManager().getConnection().commit();
      } finally {
        this.getDbManager().getConnection().setAutoCommit(true);
      }
    } catch (Throwable ex) {
      throw new OpenMSPException(ex);
    } finally {
      this.getDbManager().garbageOpenedConnection();
    }

  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#deleteCollectionObject(java.lang.String)
   */
  public void deleteCollectionObject(String id) throws OpenMSPException {
    try {
      String[] param = { id };
      this.getDbManager().getConnection().setAutoCommit(false);
      try {
        this.executeUpdate(MyCrmLeadsSynchroTarget.queryDeleteLeads, param);

        this.getDbManager().getConnection().commit();
      } finally {
        this.getDbManager().getConnection().setAutoCommit(true);
      }
    } catch (Throwable ex) {
      throw new OpenMSPException(ex);
    } finally {
      this.getDbManager().garbageOpenedConnection();
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getConflicResolver()
   */
  public SynchroConflicResolver getConflicResolver() {
    return new AlwaysUpdateServerSynchroConflicResolver(); //PDA WIN always update server
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getAllCollectionObject()
   */
  public Array getAllCollectionObject() throws OpenMSPException {

    Array ret = new Array();
    try {
      Array list = new Array();
      ResultSet result = this.executeQuery(MyCrmLeadsSynchroTarget.queryGetAllLeads, null);
      while (result.next()) {
        list.add(result.getString(1));
      }

      for (int i = 0; i < list.size(); i++) {
        ret.add(this.getCollectionObjectWithId((String) list.get(i)));
      }

    } catch (Throwable ex) {
      throw new OpenMSPException(ex);
    } finally {
      this.getDbManager().garbageOpenedConnection();
    }
    return ret;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionObjectId(java.lang.Object)
   */
  public String getCollectionObjectId(Object obj) {
    return ((Leads) obj).getId();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getAllModifiedAtomicObjectForServiceSince(java.lang.String, long)
   */
  public SynchroAtomicObject[] getAllModifiedAtomicObjectSince(long date) throws OpenMSPException {
    if (date == 0) { //synchronize only during complete synchro
      SynchroAtomicObject[] sync = new SynchroAtomicObject[1];
      sync[0] = new DefaultSynchroAtomicObject("", "");
      return sync;
    }
    return new SynchroAtomicObject[0];
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getUpdateMaxNbRow()
   */
  public int getUpdateMaxNbRow() {
    return 0;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#updateSynchroDB(org.openmobileis.database.fastobjectdb.FastObjectDB)
   */
  public void updateSynchroDB(FastObjectDB db) throws OpenMSPException {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#connect(org.openmobileis.synchro.security.auth.Credential)
   */
  public void connect(Credential cred) throws UserNotFoundException, ServiceException {
    // do nothing
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#disconnect()
   */
  public void disconnect() {
    // do nothing
  }

}
