package org.openmobileis.examples.mycrm.data.jdbc;

import org.openmobileis.common.util.database.AbstractQueryManager;
import org.openmobileis.common.util.database.ManagerDB;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.modules.common.data.DefaultTerminalUser;
import org.openmobileis.modules.common.data.TerminalUser;
import org.openmobileis.modules.common.database.jdbc.TerminalUserJDBCQuery;
import org.openmobileis.synchro.openmsp.client.db.DBImportFileCoder;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;

/**
 * Title:        Sanofi Project
 * Description: manage server side general TerminalUser synchrnisation  
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class MyCrmTerminalUserlQuery extends AbstractQueryManager implements TerminalUserJDBCQuery {

    private static  String getUserWithId = "SELECT code_rep, nom, prenom, email, type, actif, usePDA, langue FROM user WHERE code_rep='%0%'";
    private static  String getAllUsers = "SELECT code_rep, nom, prenom, email, type, actif, usePDA, langue FROM user";
	protected static String queryInsertUser = "INSERT INTO user (code_rep, nom, prenom, email, type, actif, usePDA, langue)"
		+" VALUES ('%0%', '%1%', '%2%', '%3%', '%4%', %5%, %6%, '%7%')";


	protected static String queryUpdateUser = "UPDATE user " 
		+"SET nom='%0%', prenom='%1%', email='%2%', type='%3%', actif=%4%, usePDA=%5%, langue='%6%'"
		+" WHERE code_rep='%7%'";

	protected static String queryUpdateLoginPass = "UPDATE user " 
		+"SET login='%0%',	password='%1%'"
		+" WHERE code_rep='%2%'";

	protected static String queryDeleteUser = "DELETE FROM user where code_rep='%0%'";

		
	private static String queryCreateTableUser  =
			"CREATE TABLE user ("
			+"	code_rep varchar(5) NOT NULL,"
			+"	nom varchar(30) NOT NULL,"
			+"	prenom varchar(30),"
			+"	email varchar(60),"
			+"	login varchar(15),"
			+"	password varchar(50),"
			+"	type varchar(5), "
			+"	actif bit, "
			+"	usePDA bit, "
			+"	langue varchar(2), "
			+"	CONSTRAINT rep_id PRIMARY KEY (code_rep)"
			+");"
			+"CREATE INDEX user_login ON user (login);";
 	
	public MyCrmTerminalUserlQuery()	{
		super();
        try {
          this.createSyncTable();
        } catch (Throwable ex)  {
          LogManager.traceError(0, ex);
        }
	}

	/*
	* check if the sync table exists. IF it does not exist, create it
	*/
	private void createSyncTable() throws DatabaseException, java.sql.SQLException {
		try {
			Connection con = this.getDbManager().getConnection();
			DatabaseMetaData dbmd = con.getMetaData();
			String[] tableType = {"TABLE"};
			ResultSet result = dbmd.getTables (null,null,"USER", tableType);
			if (!result.next()) {
				this.executeUpdate(queryCreateTableUser, null);
                String[] parameters = { "1111", "English", "testen", "emailen", "0", "1", "1", "EN" };
                this.executeUpdate(queryInsertUser, parameters);
                parameters = new String[]{"en", "en", "1111" };
                this.executeUpdate(queryUpdateLoginPass, parameters);
                parameters = new String[]{ "2222", "Franais", "testfr", "emailfr", "0", "1", "1", "FR" };
                this.executeUpdate(queryInsertUser, parameters);
                parameters = new String[]{"fr", "fr", "2222" };
                this.executeUpdate(queryUpdateLoginPass, parameters);
			}
		} finally {
			this.getDbManager().garbageOpenedConnection();
		}
	}
    
    public ResultSet getTerminalUser (String[] param) throws DatabaseException {
        return this.executeQuery(MyCrmTerminalUserlQuery.getUserWithId, param);
    }
    
    public ResultSet getAllTerminalUsers () throws DatabaseException {
        return this.executeQuery(MyCrmTerminalUserlQuery.getAllUsers, null);
    }

	/**
	 * @see org.openmobileis.server.synchro.database.queries.AbstractQueryManager#create(String[])
	 */
	public void insertTerminalUser(String[] parameters) throws DatabaseException {
		this.executeUpdate(MyCrmTerminalUserlQuery.queryInsertUser, parameters);
	}

	/**
	 * @see org.openmobileis.server.synchro.database.queries.AbstractQueryManager#delete(String[])
	 */
	public void deleteTerminalUser(String[] parameters) throws DatabaseException {
		this.executeUpdate(MyCrmTerminalUserlQuery.queryDeleteUser, parameters);
	}

	/**
	 * @see org.openmobileis.server.synchro.database.queries.AbstractQueryManager#update(String[])
	 */
	public void updateTerminalUser(String[] parameters) throws DatabaseException {
		this.executeUpdate(MyCrmTerminalUserlQuery.queryUpdateUser, parameters);
	}
	public void updateLoginPass(String[] parameters) throws DatabaseException {
		this.executeUpdate(MyCrmTerminalUserlQuery.queryUpdateLoginPass, parameters);
	}

	public ManagerDB getDbManager()	{
		return super.getDbManager();
	}

	
    public TerminalUser convertResultSetToTerminalUser (ResultSet result) throws DatabaseException  {
		try	{		
			//code_rep, nom, prenom, email, type, actif, usePDA
			DefaultTerminalUser rep =  new DefaultTerminalUser(result.getString(1));
			rep.setName(result.getString(2));
			rep.setProfil(result.getString(5));
			rep.setActif(DBImportFileCoder.getCoder().convertDBDataToBoolean(result.getByte(6)));
			rep.setLangue(result.getString(8));
			return rep;
		} catch (Throwable ex)	{
			throw new DatabaseException();
		}
	}
    public String[] getInsertParamFromTerminalUser(TerminalUser rep)    {return null;}
    public String[] getUpdateParamFromTerminalUser(TerminalUser rep)    {return null;}


}
