/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.server;

import java.io.File;
import java.util.Properties;

import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.user.profile.ProfileDataManager;
import org.openmobileis.common.user.profile.impl.XmlFileProfilDataFactory;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.database.JdbcPoolManagerDB;
import org.openmobileis.common.util.database.ManagerDB;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.FileLogManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBSynchroManager;
import org.openmobileis.examples.mycrm.data.jdbc.MyCrmAccountSynchroTarget;
import org.openmobileis.examples.mycrm.data.jdbc.MyCrmJDBCUserFactory;
import org.openmobileis.examples.mycrm.data.jdbc.MyCrmLabelSynchroTarget;
import org.openmobileis.examples.mycrm.data.jdbc.MyCrmLeadsSynchroTarget;
import org.openmobileis.examples.mycrm.data.jdbc.MyCrmTerminalUserlQuery;
import org.openmobileis.modules.common.database.jdbc.JDBCTerminalUserFODBSyncTarget;
import org.openmobileis.services.servlet.OpenMISInit;
import org.openmobileis.synchro.algo.replication.utils.DefaultSynchroAtomicObjectDelegate;
import org.openmobileis.synchro.algo.replication.utils.SynchroAtomicObjectManager;
import org.openmobileis.synchro.algo.replication.utils.impl.HSQLAtomicObjectQueryManager;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberManager;
import org.openmobileis.synchro.algo.syncnumber.impl.DefaultSyncNumberManagerDelegate;
import org.openmobileis.synchro.algo.syncnumber.impl.HSQLSyncNumberQueryManager;
import org.openmobileis.synchro.openmsp.server.synctarget.SynchroTargerManager;
import org.openmobileis.synchro.openmsp.server.synctarget.proxy.GenericProxySynchroTarget;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class MyCrmServerOpenMISInit implements OpenMISInit {

  /**
   * 
   */
  public MyCrmServerOpenMISInit() {
    super();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.servlet.OpenMISInit#preLoadingInit()
   */
  public void preLoadingInit() throws ServiceException {
    
    //init database.
    try {
      Properties  props = new Properties();
      props.put("org.openmobileis.common.log.file", System.getProperty("user.dir")+"/WEB-INF/log/serverlog.txt");
      File file = new File(System.getProperty("user.dir")+"/WEB-INF/log");
      if (file.exists())   FileLogManager.registerLogManager(props); //log to a file
      else LogManager.registerLogManager(null); //log to a console
     
      //init synchro database access.
      //use OpenMis db facilities. Any other JDBC DB API can be use.
      //load db properties
      PropertiesManager.getManager().addProperty("server.database.dbDriver", "org.hsqldb.jdbcDriver");
      PropertiesManager.getManager().addProperty("server.database.dbServer","jdbc:hsqldb:file:"+System.getProperty("user.dir")+"/hsql/basedb");
      PropertiesManager.getManager().addProperty("server.database.dbLogin","sa");
      PropertiesManager.getManager().addProperty("server.database.dbPassword","");
      PropertiesManager.getManager().addProperty("server.database.pool.monitorInterval", "300");
      PropertiesManager.getManager().addProperty("server.database.pool.maxConns", "10");
      PropertiesManager.getManager().addProperty("server.database.pool.idleTimeout", "300");
      PropertiesManager.getManager().addProperty("server.database.pool.checkoutTimeout", "120");
      PropertiesManager.getManager().addProperty("server.database.pool.maxCheckout", "3000");      
      JdbcPoolManagerDB dbmanager = new JdbcPoolManagerDB(); //init pool with loaded properties
      ManagerDB.registerManager(dbmanager);
      
      PropertiesManager.getManager().addProperty("org.openmobileis.common.user.profil.xmlprofilfile.path",System.getProperty("user.dir") + "/WEB-INF/conf/defaultprofils.xml");
      // init user manager with the default impl with one user. Login:test Pwd:test
      //to authenticate to another source, use your UserManagerFactory
      UserManager.getManager().registerUserManagerFactoryForGroup("default", new MyCrmJDBCUserFactory());
      //register the sync nyumber manager. Sync number identify a db modification in the time.
      //every modification on db data is tracked with a sync number.
      SyncNumberManager.getManager().registerDelegate(new DefaultSyncNumberManagerDelegate(new HSQLSyncNumberQueryManager()));
      
      // register the Synchro Atomic object manager.
      //manage data modification on the db server side.
      //It is use to get all modification since last synchro
      //It's use is not mandatotry. We provide it to facilitate the server db  modification management
      // But you can use any other db modification tracking process.
      // Use inside the FODBSynchroTarget to implements the getAllModifiedAtomicObjectSince method.
      SynchroAtomicObjectManager.getManager().registerDelegateManager(
            new DefaultSynchroAtomicObjectDelegate(
              new HSQLAtomicObjectQueryManager()  // default implementation for HSQL
            )
          );
      
      // init profil management with default profil. All synchronized collection must be in the profil.
      //edit the defaultprofils.xml file to add your collection.
      ProfileDataManager.getManager().registerProfilDataFactoryForGroup("default", new XmlFileProfilDataFactory());
      
      //init TerminalUser synchro
      FODBSynchroManager.getManager().registerCollection(new JDBCTerminalUserFODBSyncTarget(new MyCrmTerminalUserlQuery()));
      // register terminal synchro target for all terminal synchronized collection
      FODBSynchroManager.getManager().registerCollection(new MyCrmLabelSynchroTarget());
      FODBSynchroManager.getManager().registerCollection(new MyCrmAccountSynchroTarget());
      FODBSynchroManager.getManager().registerCollection(new MyCrmLeadsSynchroTarget());
      
      // to be added to work with a servlet server.
      SynchroTargerManager.getManager().registerProxyTargetForTerminal(new GenericProxySynchroTarget(), null);      
    } catch (Throwable ex)  {
      throw new ServiceException(ex);
    }
    
    
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.servlet.OpenMISInit#postLoadingInit()
   */
  public void postLoadingInit() throws ServiceException {
  }

}
