/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.terminal.services;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.examples.mycrm.data.Account;
import org.openmobileis.examples.mycrm.data.Contact;
import org.openmobileis.examples.mycrm.data.Leads;
import org.openmobileis.examples.mycrm.data.fodb.AccountFactory;
import org.openmobileis.examples.mycrm.data.fodb.LeadsFactory;
import org.openmobileis.modules.common.data.LabelManager;
import org.openmobileis.modules.common.history.FODBHistoryManager;
import org.openmobileis.modules.common.history.HistoryManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;
import org.openmobileis.services.servlet.IndexServlet;
import org.openmobileis.services.servlet.IndexServletContentListener;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title: OpenMobileIS project source <BR>
 * Description:
 * 
 * @author Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public final class DisplayAccountService extends TemplateService  implements NavigationBarService, IndexServletContentListener {
  private HistoryManager histomanager;

  /**
   * 
   */
  public DisplayAccountService() {
    super();
    histomanager = new FODBHistoryManager("account", 3);
    IndexServlet.getIndexServlet().addContentListener(this);
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.openmobileis.services.TemplateService#runTemplate(javax.servlet.http.HttpServletRequest,
   *      javax.servlet.http.HttpServletResponse,
   *      freemarker.template.TemplateModelRoot)
   */
  public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) throws ServletException, IOException {
    String accountid = req.getParameter("accountid");
    if (accountid != null) {
      try   {
        histomanager.addUID(accountid);
        Account account = AccountFactory.getManager().getAccount(accountid);
        templateData.put("name", new SimpleScalar(account.getName()));
        templateData.put("address", new SimpleScalar(account.getAddress()));
        templateData.put("city", new SimpleScalar(account.getCity()));
        templateData.put("activity", new SimpleScalar(LabelManager.getManager().getLabelWithIds(account.getActivity(), Account.ACCOUNT_ACTIVITY_LABEL_CATEGORY).getLabel()));     
        
        Array contactlist = account.getAllContacts();      
        // construct the SimpleList to fill HTML array (see the template).
        SimpleList showList = new SimpleList();
        SimpleHash dataStruct;
        for (int i = 0; i < contactlist.size(); i++) {
          dataStruct = new SimpleHash();
          Contact contact = (Contact)contactlist.get(i);
          dataStruct.put("id", contact.getId());
          dataStruct.put("firstname",  contact.getFirstname());
          dataStruct.put("lastname",  contact.getLastname());
          showList.add(dataStruct);
        }
        templateData.put("contactlist", showList);

        Array leadslist = LeadsFactory.getManager().getLeadsForAccount(accountid);      
        // construct the SimpleList to fill HTML array (see the template).
        java.text.SimpleDateFormat formater = new java.text.SimpleDateFormat("MM/dd/yyyy", java.util.Locale.FRANCE);
        showList = new SimpleList();
        for (int i = 0; i < leadslist.size(); i++) {
          dataStruct = new SimpleHash();
          Leads lead = (Leads)leadslist.get(i);
          dataStruct.put("leadid", lead.getId());
          dataStruct.put("amount",  Integer.toString(lead.getAmount()));
          dataStruct.put("date",  formater.format(lead.getCloseDate()).toString());
          showList.add(dataStruct);
        }
        templateData.put("leadslist", showList);
        
        templateData.put("accountid", new SimpleScalar(accountid));
      } catch (DatabaseException ex) {
        throw new ServletException(ex);
      }
    }
    return "crm/displayaccount.htm";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.servlet.IndexServletContentListener#addContent(freemarker.template.TemplateModelRoot)
   */
  public void addContent(TemplateModelRoot modelRoot) throws ServiceException {
    try {
    Array allids = histomanager.getAll();
    SimpleList showList = new SimpleList();
    SimpleHash dataStruct;
    for (int i = 0; i < allids.size(); i++) {
      dataStruct = new SimpleHash();
      String accountid = (String)allids.get(i);
      Account account = AccountFactory.getManager().getAccount(accountid);
      if (account != null)  {
        dataStruct.put("id", account.getId());
        dataStruct.put("name",  account.getName());
        dataStruct.put("city",  account.getCity());
        showList.add(dataStruct);
      } else  {
        histomanager.removeUID(accountid);
      }
    }
    modelRoot.put("accountList", showList);
    } catch (Throwable ex)  {
      throw new ServiceException(ex);
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.openmobileis.services.Service#getServiceUri()
   */
  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI() + "/crm/displayaccount";
  }

  /* 
   * Navigation bar methods
   */

  public String getNavigationBarLabel(HttpServletRequest req) {
    String paramId = req.getParameter("accountid");
    if (paramId != null) {
      try   {
        Account account = AccountFactory.getManager().getAccount(paramId);
        return account.getName();
      } catch (Throwable ex)    {
        LogManager.traceError(0, ex);
      }
    }
    return "Account";
  }

  public boolean displayFormExitMessage() {
    return false;
  }

  public boolean displayRecursive() {
    return false;
  }

}
