/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.terminal.services;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.examples.mycrm.data.Account;
import org.openmobileis.examples.mycrm.data.Contact;
import org.openmobileis.examples.mycrm.data.Report;
import org.openmobileis.examples.mycrm.data.fodb.AccountFactory;
import org.openmobileis.modules.common.data.LabelManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title: OpenMobileIS project source <BR>
 * Description:
 * 
 * @author Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public final class DisplayContactService extends TemplateService  implements NavigationBarService {

  /**
   * 
   */
  public DisplayContactService() {
    super();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.openmobileis.services.TemplateService#runTemplate(javax.servlet.http.HttpServletRequest,
   *      javax.servlet.http.HttpServletResponse,
   *      freemarker.template.TemplateModelRoot)
   */
  public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) throws ServletException, IOException {
    String accountid = req.getParameter("accountid");
    String contactid = req.getParameter("contactid");
    if (accountid != null && contactid != null) {
      try   {
        Account account = AccountFactory.getManager().getAccount(accountid);
        Contact contact = account.getContactById(contactid);
        templateData.put("name", new SimpleScalar(account.getName()));
        templateData.put("firstname", new SimpleScalar(contact.getFirstname()));
        templateData.put("lastname", new SimpleScalar(contact.getLastname()));
        templateData.put("function", new SimpleScalar(LabelManager.getManager().getLabelWithIds(Integer.toString(contact.getFunction()), Contact.CONTACT_FUNCTION_LABEL_CATEGORY).getLabel()));
       
        
        Array reportslist = contact.getAllReports();
        
        // construct the SimpleList to fill HTML array (see the template).
        SimpleList showList = new SimpleList();
        SimpleHash dataStruct;
        java.text.SimpleDateFormat formater = new java.text.SimpleDateFormat("EEEEE dd MMMMM yyyy", java.util.Locale.FRANCE);
        for (int i = 0; i < reportslist.size(); i++) {
          dataStruct = new SimpleHash();
          Report report = (Report)reportslist.get(i);
          dataStruct.put("id", report.getId());
          dataStruct.put("action",  LabelManager.getManager().getLabelWithIds(Integer.toString(report.getAction()), Report.REPORT_ACTION_LABEL_CATEGORY).getLabel());
          dataStruct.put("date", new SimpleScalar(formater.format(report.getReportDate()).toString()));         
          showList.add(dataStruct);
        }
        templateData.put("reportlist", showList);
        templateData.put("accountid", new SimpleScalar(accountid));
        templateData.put("contactid", new SimpleScalar(contactid));
      } catch (DatabaseException ex) {
        throw new ServletException(ex);
      }
    }
    return "crm/displaycontact.htm";
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.openmobileis.services.Service#getServiceUri()
   */
  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI() + "/crm/displaycontact";
  }

  /* 
   * Navigation bar methods
   */

  public String getNavigationBarLabel(HttpServletRequest req) {
    String accountid = req.getParameter("accountid");
    String contactid = req.getParameter("contactid");
    if (accountid != null && contactid != null) {
      try   {
        Account account = AccountFactory.getManager().getAccount(accountid);
        Contact contact = account.getContactById(contactid);
        return contact.getLastname();
      } catch (Throwable ex)    {
        LogManager.traceError(0, ex);
      }
    }
    return "Contact";
  }

  public boolean displayFormExitMessage() {
    return false;
  }

  public boolean displayRecursive() {
    return false;
  }

}
