/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.terminal.services;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.examples.mycrm.data.Account;
import org.openmobileis.examples.mycrm.data.Contact;
import org.openmobileis.examples.mycrm.data.Leads;
import org.openmobileis.examples.mycrm.data.Report;
import org.openmobileis.examples.mycrm.data.fodb.AccountFactory;
import org.openmobileis.examples.mycrm.data.fodb.LeadsFactory;
import org.openmobileis.modules.common.data.LabelManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title: OpenMobileIS project source <BR>
 * Description:
 * 
 * @author Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public final class DisplayLeadsService extends TemplateService  implements NavigationBarService {

  /**
   * 
   */
  public DisplayLeadsService() {
    super();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.openmobileis.services.TemplateService#runTemplate(javax.servlet.http.HttpServletRequest,
   *      javax.servlet.http.HttpServletResponse,
   *      freemarker.template.TemplateModelRoot)
   */
  public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) throws ServletException, IOException {
    String leadid = req.getParameter("leadid");
    if (leadid != null) {
      try   {
        Leads lead = LeadsFactory.getManager().getLeads(leadid);
        if (lead != null)   {
          Account account = AccountFactory.getManager().getAccount(lead.getIdaccount());
          templateData.put("accountid", new SimpleScalar(account.getId()));
          templateData.put("accountname", new SimpleScalar(account.getName()));
          templateData.put("description", new SimpleScalar(lead.getDescription()));
          templateData.put("amount", new SimpleScalar(Integer.toString(lead.getAmount())));
          templateData.put("state", new SimpleScalar(LabelManager.getManager().getLabelWithIds(Integer.toString(lead.getState()), Leads.LEADS_STATE_LABEL_CATEGORY).getLabel()));
          java.text.SimpleDateFormat formater = new java.text.SimpleDateFormat("EEEEE dd MMMMM yyyy", java.util.Locale.FRANCE);
          templateData.put("date", new SimpleScalar(formater.format(lead.getCloseDate()).toString()));
        }
       
        templateData.put("leadid", new SimpleScalar(leadid));
      } catch (DatabaseException ex) {
        throw new ServletException(ex);
      }
    }
    return "crm/displaylead.htm";
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.openmobileis.services.Service#getServiceUri()
   */
  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI() + "/crm/displaylead";
  }

  /* 
   * Navigation bar methods
   */

  public String getNavigationBarLabel(HttpServletRequest req) {
    return IntlResourceManager.getManager().getLocalizedProperty("mycrm.displayleadservicenavbar", "Lead"); 
  }

  public boolean displayFormExitMessage() {
    return false;
  }

  public boolean displayRecursive() {
    return false;
  }

}
