/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.terminal.services;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.examples.mycrm.data.Account;
import org.openmobileis.examples.mycrm.data.Contact;
import org.openmobileis.examples.mycrm.data.Report;
import org.openmobileis.examples.mycrm.data.fodb.AccountFactory;
import org.openmobileis.modules.common.data.LabelManager;
import org.openmobileis.modules.common.data.TerminalUser;
import org.openmobileis.modules.common.data.TerminalUserManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;
import org.openmobileis.services.util.TemplateUtils;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title: OpenMobileIS project source <BR>
 * Description:
 * 
 * @author Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public final class DisplayReportService extends TemplateService  implements NavigationBarService {

  /**
   * 
   */
  public DisplayReportService() {
    super();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.openmobileis.services.TemplateService#runTemplate(javax.servlet.http.HttpServletRequest,
   *      javax.servlet.http.HttpServletResponse,
   *      freemarker.template.TemplateModelRoot)
   */
  public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) throws ServletException, IOException {
    String accountid = req.getParameter("accountid");
    String contactid = req.getParameter("contactid");
    String reportid = req.getParameter("reportid");
    if (accountid != null && contactid != null && reportid!=null) {
      try   {
        Account account = AccountFactory.getManager().getAccount(accountid);
        Contact contact = account.getContactById(contactid);
        
        templateData.put("accountid", new SimpleScalar(accountid));
        templateData.put("contactid", new SimpleScalar(contactid));
        templateData.put("reportid", new SimpleScalar(reportid));
        
        Report  report = contact.getReportById(reportid);
        if (report != null) {        
          templateData.put("description", new SimpleScalar(report.getDescription()));
          templateData.put("action", new SimpleScalar(LabelManager.getManager().getLabelWithIds(Integer.toString(report.getAction()), Report.REPORT_ACTION_LABEL_CATEGORY).getLabel()));
          java.text.SimpleDateFormat formater = new java.text.SimpleDateFormat("EEEEE dd MMMMM yyyy", java.util.Locale.FRANCE);
          templateData.put("date", new SimpleScalar(formater.format(report.getReportDate()).toString()));
          TerminalUser user = TerminalUserManager.getManager().getTerminalUser(report.getTerminalUserId());
          if (user != null) {
            templateData.put("user", new SimpleScalar(user.getName()));
          }
       }
        
      } catch (DatabaseException ex) {
        throw new ServletException(ex);
      }
    }
    return "crm/displayreport.htm";
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.openmobileis.services.Service#getServiceUri()
   */
  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI() + "/crm/displayreport";
  }

  /* 
   * Navigation bar methods
   */

  public String getNavigationBarLabel(HttpServletRequest req) {
    return IntlResourceManager.getManager().getLocalizedProperty("mycrm.displayreportservicenavbar", "Report");
  }

  public boolean displayFormExitMessage() {
    return false;
  }

  public boolean displayRecursive() {
    return false;
  }

}
