/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.terminal.services;

import javax.servlet.http.HttpServletRequest;

import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.UniqueIdGenerator;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.examples.mycrm.data.Account;
import org.openmobileis.examples.mycrm.data.Contact;
import org.openmobileis.examples.mycrm.data.fodb.AccountFactory;
import org.openmobileis.modules.common.data.LabelManager;
import org.openmobileis.services.SimpleEditService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;
import org.openmobileis.services.util.TemplateUtils;

import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: SimpleEditService provide a base algorythm to develop and edit service.
 * It manage the edited object in a session created using the name return by getSessionDatasName().
 * Implements the NavigationBarService interface to use the naviagation bar that provide simple way to naviagate
 * along services.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class EditContactService extends SimpleEditService  implements NavigationBarService {

  class ContactSessionData     {
    String accountid;
    Contact contact;
  }
  /**
   * 
   */
  public EditContactService() {
    super();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#fillTemplateWithSessionData(java.lang.Object, freemarker.template.TemplateModelRoot)
   */
  protected void fillTemplateWithSessionData(Object sessionDatas, TemplateModelRoot templateData) throws ServiceException {
    ContactSessionData data = (ContactSessionData) sessionDatas;
   
    templateData.put("accountid", new SimpleScalar(data.accountid));
    templateData.put("contactid", new SimpleScalar(data.contact.getId()));
    templateData.put("firstname", new SimpleScalar(data.contact.getFirstname()));
    templateData.put("lastname", new SimpleScalar(data.contact.getLastname()));
    
    //for action
    Array array = LabelManager.getManager().getLabelListForCategorie(Contact.CONTACT_FUNCTION_LABEL_CATEGORY);
    templateData.put("functions", TemplateUtils.fillFormPopUpWithLabel(array, Integer.toString(data.contact.getFunction())));
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#storeSessionObjectInDB(java.lang.Object, javax.servlet.http.HttpServletRequest, freemarker.template.TemplateModelRoot)
   */
  protected void storeSessionObjectInDB(Object sessionDatas, HttpServletRequest req, TemplateModelRoot templateData) throws ServiceException {
    ContactSessionData data = (ContactSessionData)sessionDatas;
    
    
    //validate the edited object has an id.
    if (data.contact.getLastname() == null || data.contact.getLastname().trim().length() == 0) {
      this.setInputRequestParameterError(req, "Error :A contact lastname must be entered.");
      return;
    } 
    
    try {
      Account account = AccountFactory.getManager().getAccount(data.accountid);
      account.addContact(data.contact);
      AccountFactory.getManager().storeAccount(account);
    } catch (DatabaseException e) {
      throw new ServiceException(e);
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#fillSessionObjectWithRequestData(java.lang.Object, javax.servlet.http.HttpServletRequest)
   */
  protected void fillSessionObjectWithRequestData(Object sessionDatas, HttpServletRequest req) throws ServiceException {
    Contact data = ((ContactSessionData)sessionDatas).contact;
    
    // the Id of the object is edited. Its better to generate an unique id if new ID is needed.
    // Avoid Db problem with same Id . See UniqueIdGenerator class to generate an Id
    String temp = (String)req.getParameter("firstname");
    if (temp != null) {
      data.setFirstname(temp);
    }
    temp = (String)req.getParameter("lastname");
    if (temp != null) {
      data.setLastname(temp);
    }
    
    temp = (String)req.getParameter("function");
    if (temp != null) {
      try   {
        data.setFunction(Integer.parseInt(temp));
      } catch (NumberFormatException ex)   {
        data.setFunction(0);
      }
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#getSessionDatasName()
   */
  protected String getSessionDatasName() {
    return "contactdata";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#getTemplateName()
   */
  protected String getTemplateName() {
    return "crm/editcontact.htm";
  }

  /**
   * call to create the session object.
   * Before been created while it's edited, an object is store in a session.
   * The session object can be a new one (create it) or an object store in the database (get it from the database). 
   */
  protected Object createSessionObject(HttpServletRequest req) throws ServiceException {
    // the Id of the object is edited. Its better to generate an unique id if new ID is needed.
    // Avoid Db problem with same Id . See UniqueIdGenerator class to generate an Id
    String accountid = req.getParameter("accountid");
    String contactid = req.getParameter("contactid");
    ContactSessionData data = new ContactSessionData();
    data.accountid = accountid;
    if (accountid != null && contactid != null) {
      try {
        Account account = AccountFactory.getManager().getAccount(accountid);
        data.contact = account.getContactById(contactid);
      } catch (DatabaseException ex)  {
        throw new ServiceException(ex);
      }
    }
    if (data.contact == null) {
      data.contact = new Contact(UniqueIdGenerator.getManager().getNewStringID());
    }
    return data;
  }

  /** 
   * return the URI of the service that display data after the data edition.
   * ServiceBaseURI is not mandatory
   */
  protected String getDisplayServiceURI() {
     return "/crm/displaycontact";
  }

  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI() + "/crm/editcontact";
  }

  /* 
   * Navigation bar methods
   */
  public String getNavigationBarLabel(HttpServletRequest req) {
    return IntlResourceManager.getManager().getLocalizedProperty("mycrm.editservicenavbar", "Edit");
  }

  public boolean displayFormExitMessage() {
    return true;
  }

  public boolean displayRecursive() {
    return false;
  }

}
