/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.terminal.services;

import java.util.Calendar;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;

import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.UniqueIdGenerator;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.examples.mycrm.data.Account;
import org.openmobileis.examples.mycrm.data.Contact;
import org.openmobileis.examples.mycrm.data.Report;
import org.openmobileis.examples.mycrm.data.fodb.AccountFactory;
import org.openmobileis.modules.common.data.LabelManager;
import org.openmobileis.services.SimpleEditService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;
import org.openmobileis.services.util.TemplateUtils;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: SimpleEditService provide a base algorythm to develop and edit service.
 * It manage the edited object in a session created using the name return by getSessionDatasName().
 * Implements the NavigationBarService interface to use the naviagation bar that provide simple way to naviagate
 * along services.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class EditReportService extends SimpleEditService  implements NavigationBarService {

  class ReportSessionData     {
    String accountid;
    String contactid;
    Report report;
  }
  /**
   * 
   */
  public EditReportService() {
    super();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#fillTemplateWithSessionData(java.lang.Object, freemarker.template.TemplateModelRoot)
   */
  protected void fillTemplateWithSessionData(Object sessionDatas, TemplateModelRoot templateData) throws ServiceException {
    ReportSessionData data = (ReportSessionData) sessionDatas;
   
    templateData.put("accountid", new SimpleScalar(data.accountid));
    templateData.put("contactid", new SimpleScalar(data.contactid));
    templateData.put("reportid", new SimpleScalar(data.report.getId()));
    
    templateData.put("description", new SimpleScalar(data.report.getDescription()));
    
    //for action
    Array array = LabelManager.getManager().getLabelListForCategorie(Report.REPORT_ACTION_LABEL_CATEGORY);
    templateData.put("actions", TemplateUtils.fillFormPopUpWithLabel(array, Integer.toString(data.report.getAction())));

    Calendar calendar = Calendar.getInstance();
    calendar.setTime(data.report.getReportDate());
    // Fill start date and end date
    SimpleHash startDate =
        TemplateUtils.getInputDate(
            calendar.get(java.util.Calendar.DAY_OF_MONTH),
            calendar.get(java.util.Calendar.MONTH),
            calendar.get(java.util.Calendar.YEAR),
            calendar.get(java.util.Calendar.HOUR_OF_DAY),
            0);
    templateData.put("startDate", startDate);
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#storeSessionObjectInDB(java.lang.Object, javax.servlet.http.HttpServletRequest, freemarker.template.TemplateModelRoot)
   */
  protected void storeSessionObjectInDB(Object sessionDatas, HttpServletRequest req, TemplateModelRoot templateData) throws ServiceException {
    ReportSessionData data = (ReportSessionData)sessionDatas;
    
    //validate the edited object has an id.
 /*   if (accountid == null) {
      this.setInputRequestParameterError(req, "Error :No account id. Restart edit process.");
      return;
    } */
    
    try {
      Account account = AccountFactory.getManager().getAccount(data.accountid);
      Contact contact = account.getContactById(data.contactid);
      SessionContext context = SessionContextManager.getManager().getSessionContext();
      data.report.setTerminalUserId(context.getUserId());
      contact.addReport(data.report);
      AccountFactory.getManager().storeAccount(account);
    } catch (DatabaseException e) {
      throw new ServiceException(e);
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#fillSessionObjectWithRequestData(java.lang.Object, javax.servlet.http.HttpServletRequest)
   */
  protected void fillSessionObjectWithRequestData(Object sessionDatas, HttpServletRequest req) throws ServiceException {
    Report data = ((ReportSessionData)sessionDatas).report;
      Calendar caldate = Calendar.getInstance();
      caldate.setTime(data.getReportDate());
      String strStartDay = req.getParameter("startDay");
        if (strStartDay != null && strStartDay.length() != 0) {
            try {
              caldate.set(Calendar.DAY_OF_MONTH, Integer.parseInt(strStartDay));
            } catch (NumberFormatException ex) {
              this.setInputRequestParameterError(req, "Error :Bad day.");
              return;
            }
        }
        
        String strStartMonth = req.getParameter("startMonth");
        if (strStartMonth != null && strStartMonth.length() != 0) {
            try {
              caldate.set(Calendar.MONTH, Integer.parseInt(strStartMonth) - 1);
            } catch (NumberFormatException ex) {
              this.setInputRequestParameterError(req, "Error :Bad Month.");
              return;
            }
        }
        
        String strStartYear = req.getParameter("startYear");
        if (strStartYear != null && strStartYear.length() != 0) {
            try {
                 caldate.set(Calendar.YEAR, Integer.parseInt(strStartYear));
            } catch (NumberFormatException ex) {
              this.setInputRequestParameterError(req, "Error :Bad Year.");
              return;
            }
        }
        
        data.setReportDate(caldate.getTime());

    // the Id of the object is edited. Its better to generate an unique id if new ID is needed.
    // Avoid Db problem with same Id . See UniqueIdGenerator class to generate an Id
    String temp = (String)req.getParameter("description");
    if (temp != null) {
      data.setDescription(temp);
    }
    temp = (String)req.getParameter("action");
    if (temp != null) {
      try   {
        data.setAction(Integer.parseInt(temp));
      } catch (NumberFormatException ex)   {
        data.setAction(0);
      }
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#getSessionDatasName()
   */
  protected String getSessionDatasName() {
    return "reportdata";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#getTemplateName()
   */
  protected String getTemplateName() {
    return "crm/editreport.htm";
  }

  /**
   * call to create the session object.
   * Before been created while it's edited, an object is store in a session.
   * The session object can be a new one (create it) or an object store in the database (get it from the database). 
   */
  protected Object createSessionObject(HttpServletRequest req) throws ServiceException {
    // the Id of the object is edited. Its better to generate an unique id if new ID is needed.
    // Avoid Db problem with same Id . See UniqueIdGenerator class to generate an Id
    String accountid = req.getParameter("accountid");
    String contactid = req.getParameter("contactid");
    String reportid = req.getParameter("reportid");
    ReportSessionData data = new ReportSessionData();
    data.accountid = accountid;
    data.contactid = contactid;
    if (accountid != null && contactid != null && reportid != null) {
      try {
        Account account = AccountFactory.getManager().getAccount(accountid);
        Contact contact  = account.getContactById(contactid);
        data.report = contact.getReportById(reportid);
      } catch (DatabaseException ex)  {
        throw new ServiceException(ex);
      }
    }
    if (data.report == null) {
      data.report = new Report(UniqueIdGenerator.getManager().getNewStringID());
    }
    return data;
  }

  /** 
   * return the URI of the service that display data after the data edition.
   * ServiceBaseURI is not mandatory
   */
  protected String getDisplayServiceURI() {
     return "/crm/displayreport";
  }

  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI() + "/crm/editreport";
  }

  /* 
   * Navigation bar methods
   */
  public String getNavigationBarLabel(HttpServletRequest req) {
    return IntlResourceManager.getManager().getLocalizedProperty("mycrm.editservicenavbar", "Edit");
  }

  public boolean displayFormExitMessage() {
    return true;
  }

  public boolean displayRecursive() {
    return false;
  }

}
