/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.terminal.services;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.examples.mycrm.data.Account;
import org.openmobileis.examples.mycrm.data.Leads;
import org.openmobileis.examples.mycrm.data.fodb.AccountFactory;
import org.openmobileis.examples.mycrm.data.fodb.LeadsFactory;
import org.openmobileis.services.servlet.IndexServletContentListener;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.TemplateModelRoot;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class LeadsIndexList implements IndexServletContentListener {

  /**
   * 
   */
  public LeadsIndexList() {
    super();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.servlet.IndexServletContentListener#addContent(freemarker.template.TemplateModelRoot)
   */
  public void addContent(TemplateModelRoot modelRoot) throws ServiceException {
    try   {
      
      Array leadsList = LeadsFactory.getManager().getAllLeads();
      
      // construct the SimpleList to fill HTML array (see the template).
      SimpleList showList = new SimpleList();
      SimpleHash dataStruct;
      java.text.SimpleDateFormat formater = new java.text.SimpleDateFormat("MM/dd/yyyy", java.util.Locale.FRANCE);
      for (int i = 0; i < leadsList.size(); i++) {
        dataStruct = new SimpleHash();
        Leads lead = (Leads)leadsList.get(i);
        dataStruct.put("id", lead.getId());
        Account account = AccountFactory.getManager().getAccount(lead.getIdaccount());
        dataStruct.put("accountid",  account.getId());
        dataStruct.put("accountname",  account.getName());
        dataStruct.put("amount",  Integer.toString(lead.getAmount()));
        dataStruct.put("date",  formater.format(lead.getCloseDate()).toString());
         showList.add(dataStruct);
      }
      modelRoot.put("leadsList", showList);
    } catch (DatabaseException ex) {
      throw new ServiceException(ex);
    }
  }

}
