/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.mycrm.terminal.services;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.examples.mycrm.data.Account;
import org.openmobileis.examples.mycrm.data.fodb.AccountFactory;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;
import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title: OpenMobileIS project source <BR>
 * Description:
 * 
 * @author Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public final class SearchAccountService extends TemplateService  implements NavigationBarService {

  /**
   * 
   */
  public SearchAccountService() {
    super();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.openmobileis.services.TemplateService#runTemplate(javax.servlet.http.HttpServletRequest,
   *      javax.servlet.http.HttpServletResponse,
   *      freemarker.template.TemplateModelRoot)
   */
  public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) throws ServletException, IOException {
    String name = req.getParameter("searchname");
    if (name != null)   {
      try {      
        
        Array accountlist = AccountFactory.getManager().getAccountByName(name);
        
        // construct the SimpleList to fill HTML array (see the template).
        SimpleList showList = new SimpleList();
        SimpleHash dataStruct;
        for (int i = 0; i < accountlist.size(); i++) {
          dataStruct = new SimpleHash();
          Account account = (Account)accountlist.get(i);
          dataStruct.put("id", account.getId());
          dataStruct.put("name",  account.getName());
          dataStruct.put("city",  account.getCity());
          showList.add(dataStruct);
        }
        templateData.put("accountlist", showList);
        templateData.put("searchname", new SimpleScalar(name));
        //return the template name and path from the base template directory
        // defined in the openmis.property file, property org.openmobileis.services.templatesDir
      } catch (DatabaseException ex) {
        throw new ServletException(ex);
      }
    }
    return "crm/searchaccount.htm";
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.openmobileis.services.Service#getServiceUri()
   */
  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI() + "/crm/searchaccount";
  }

  /* 
   * Navigation bar methods
   */

  public String getNavigationBarLabel(HttpServletRequest req) {
    return IntlResourceManager.getManager().getLocalizedProperty("mycrm.searchaccountservicenavbar", "Search Account");
  }

  public boolean displayFormExitMessage() {
    return false;
  }

  public boolean displayRecursive() {
    return false;
  }

}
