/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.server.database;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;

import org.openmobileis.common.util.database.AbstractQueryManager;
import org.openmobileis.common.util.exception.DatabaseException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class JDBCBaseDataFactory extends AbstractQueryManager {

  private static String queryUpdate  =
    "UPDATE BASEDATA SET DATA='%0%' " +
    "WHERE ID='%1%'";

   private static String queryCreate  =
     "INSERT INTO BASEDATA (ID , DATA) " +
      "VALUES ('%0%', '%1%')";

  // query pattern used to create syncNB table
  private static String queryCreateTable  =
    "CREATE TABLE BASEDATA ("
    +" ID varchar(30) NOT NULL,"
    +" DATA varchar(50),"
    +" CONSTRAINT dataid PRIMARY KEY (ID)"
    +")";

  private static String queryDelete =
    "DELETE FROM BASEDATA WHERE ID = '%0%'";


  /********  SELECT QUERIES ********************/

  private static String queryGetBaseData  =
    "SELECT ID, DATA  FROM BASEDATA WHERE ID = '%0%' ";

  private static String queryGetAllBaseData  =
    "SELECT ID, DATA  FROM BASEDATA ";

  /**
   * 
   */
  public JDBCBaseDataFactory() {
    super();
  }

  /*
  * check if the sync table exists. IF it does not exist, create it
  */
  public void createSyncTable() throws DatabaseException, java.sql.SQLException {
    try {
      Connection con = this.getDbManager().getConnection();
      DatabaseMetaData dbmd = con.getMetaData();
      String[] tableType = {"TABLE"};
      ResultSet result = dbmd.getTables (null,null,"BASEDATA", tableType);
      if (!result.next()) {
        this.executeUpdate(queryCreateTable, null);
      }
    } finally {
      this.getDbManager().garbageOpenedConnection();
    }
  }

  public void create (String parameters[]) throws DatabaseException {
    this.executeUpdate(queryCreate, parameters);
  }

 public void delete (String parameters[]) throws DatabaseException {
    this.executeUpdate(queryDelete, parameters);
 }

 public void update (String parameters[]) throws DatabaseException {
    this.executeUpdate(queryUpdate, parameters);
 }

 public ResultSet getBaseData (String parameters[]) throws DatabaseException {
    return (this.executeQuery(queryGetBaseData, parameters));
  }

 public ResultSet getAllBaseData () throws DatabaseException {
    return (this.executeQuery(queryGetAllBaseData, null));
  }

}
