/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.simpleappli.terminal.services;

import javax.servlet.http.HttpServletRequest;

import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.examples.simpleappli.data.BaseData;
import org.openmobileis.examples.simpleappli.data.FODB.FODBBaseDataFactory;
import org.openmobileis.services.SimpleEditService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: SimpleEditService provide a base algorythm to develop and edit service.
 * It manage the edited object in a session created using the name return by getSessionDatasName().
 * Implements the NavigationBarService interface to use the naviagation bar that provide simple way to naviagate
 * along services.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class EditBaseDataService extends SimpleEditService  implements NavigationBarService {

  /**
   * 
   */
  public EditBaseDataService() {
    super();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#fillTemplateWithSessionData(java.lang.Object, freemarker.template.TemplateModelRoot)
   */
  protected void fillTemplateWithSessionData(Object sessionDatas, TemplateModelRoot templateData) throws ServiceException {
    BaseData data = (BaseData) sessionDatas;
   
    templateData.put("id", new SimpleScalar(data.getId()));
    templateData.put("data",new SimpleScalar(data.getData()));
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#storeSessionObjectInDB(java.lang.Object, javax.servlet.http.HttpServletRequest, freemarker.template.TemplateModelRoot)
   */
  protected void storeSessionObjectInDB(Object sessionDatas, HttpServletRequest req, TemplateModelRoot templateData) throws ServiceException {
    BaseData data = (BaseData) sessionDatas;
    
    //validate the edited object has an id.
    if ((data.getId() == null) || (data.getId().trim().length() == 0)) {
      this.setInputRequestParameterError(req, "Error :Data Id must be defined.");
      return;
    }
    
    try {
      FODBBaseDataFactory.getManager().storeBaseData(data);
    } catch (DatabaseException e) {
      throw new ServiceException(e);
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#fillSessionObjectWithRequestData(java.lang.Object, javax.servlet.http.HttpServletRequest)
   */
  protected void fillSessionObjectWithRequestData(Object sessionDatas, HttpServletRequest req) throws ServiceException {
    BaseData data = (BaseData)sessionDatas;
    
    // the Id of the object is edited. Its better to generate an unique id if new ID is needed.
    // Avoid Db problem with same Id . See UniqueIdGenerator class to generate an Id
    String temp = (String)req.getParameter("id");
    if (temp != null) {
      data.setId(temp);
    }
    temp = (String)req.getParameter("data");
    if (temp != null) {
      data.setData(temp);
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#getSessionDatasName()
   */
  protected String getSessionDatasName() {
    return "basedata";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.SimpleEditService#getTemplateName()
   */
  protected String getTemplateName() {
    return "appli/editdata.htm";
  }

  /**
   * call to create the session object.
   * Before been created while it's edited, an object is store in a session.
   * The session object can be a new one (create it) or an object store in the database (get it from the database). 
   */
  protected Object createSessionObject(HttpServletRequest req) throws ServiceException {
    // the Id of the object is edited. Its better to generate an unique id if new ID is needed.
    // Avoid Db problem with same Id . See UniqueIdGenerator class to generate an Id
   String idData = (String)req.getParameter("id");
    BaseData data = null;
    if (idData !=null)  {
      try {
        data = FODBBaseDataFactory.getManager().getBaseData(idData);
      } catch (DatabaseException ex)  {
        throw new ServiceException(ex);
      }
    }
    if (data == null) {
      data = new BaseData();
    }
    return data;
  }

  /** 
   * return the URI of the service that display data after the data edition.
   * ServiceBaseURI is not mandatory
   */
  protected String getDisplayServiceURI() {
     return "/basedata/showall";
  }

  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI() + "/basedata/edit";
  }

  /* 
   * Navigation bar methods
   */
  public String getNavigationBarLabel(HttpServletRequest req) {
    return "Edit";
  }

  public boolean displayFormExitMessage() {
    return true;
  }

  public boolean displayRecursive() {
    return false;
  }

}
