/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.examples.simpleappli.terminal.services;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.examples.simpleappli.data.BaseData;
import org.openmobileis.examples.simpleappli.data.FODB.FODBBaseDataFactory;
import org.openmobileis.services.SimpleEditService;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarService;

import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class ShowDataService extends TemplateService implements NavigationBarService {

  /**
   * 
   */
  public ShowDataService() {
    super();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.TemplateService#runTemplate(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse, freemarker.template.TemplateModelRoot)
   */
  public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) throws ServletException, IOException {
    
    String id = req.getParameter("id");
    if(id != null)  {    
      try {
        BaseData data = FODBBaseDataFactory.getManager().getBaseData(id);
        if (data != null) {
          templateData.put("id", new SimpleScalar(data.getId()));
          templateData.put("data",new SimpleScalar(data.getData()));
          //return the template name and path from the base template directory
          // defined in the openmis.property file, property org.openmobileis.services.templatesDir
          return "appli/showdata.htm";
        }
     } catch (DatabaseException ex)  {
        throw new ServletException(ex);
      }
    }
    throw new ServletException("Internal error could not find the specified data.");
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.Service#getServiceUri()
   */
  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI() + "/basedata/show";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.navigation.NavigationBarService#getNavigationBarLabel(javax.servlet.http.HttpServletRequest)
   */
  public String getNavigationBarLabel(HttpServletRequest req) {
    // TODO Auto-generated method stub
    return "Show";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.navigation.NavigationBarService#displayFormExitMessage()
   */
  public boolean displayFormExitMessage() {
    return false;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.navigation.NavigationBarService#displayRecursive()
   */
  public boolean displayRecursive() {
    return false;
  }

}
