/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.modules.common.data;

import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;

/**
 * Title:        OpenMobileIS Project
 * Description:   
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class TerminalUserManager {
	
	private static TerminalUserManager manager;
	private TerminalUserFactory factory;
  
	
	/**
	 * Constructor for TerminalUserManager.
	 */
	private TerminalUserManager() {
		super();		
	}
	
	public void registerFactory(TerminalUserFactory factory)	{
        this.factory = factory; 
	}
   
	public void removeFactory()	{
      this.factory = null;
	}

	public static TerminalUserManager getManager() {
      if (manager == null)  {
        synchronized(TerminalUserManager.class) {
          if (manager == null)  {
            manager = new TerminalUserManager();
            ApplicationContextManager.getManager().addManager(manager);
          }
        }
      }
		return manager;
	}
	
	public TerminalUserFactory getFactory() {
		return this.factory;
	}
	
	public String[] getTerminalUserIdsList()	{
		return this.factory.getTerminalUserIdsList();
	}
  
  public TerminalUser getTerminalUser(String id)	{
    return this.factory.getTerminalUser(id);
  }
  
  public TerminalUser getInstallTerminalUser()	{
		return this.factory.getInstallTerminalUser();
	}
  
	public void setInstallTerminalUser(String id) throws ServiceException, DatabaseException {
		this.factory.setInstallTerminalUser(id);
	}
    
  public TerminalUser[] getAllTerminalUsers() throws DatabaseException	{
		return this.factory.getAllTerminalUsers();
  }
   
  /**
   * call by synchro server to create the TerminalUser DB
   */
  public void addTerminalUser(TerminalUser rep) throws ServiceException, DatabaseException {
		this.factory.addTerminalUser(rep);
  }
  
  public void removeTerminalUser(String repId) throws ServiceException, DatabaseException {
  	this.factory.removeTerminalUser(repId);
  }
  
}
