/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  Creation P.Delrieu
 * 
 */
package org.openmobileis.modules.common.database.fodb;

import java.util.HashMap;

import org.odbms.Constraint;
import org.odbms.ObjectSet;
import org.odbms.Query;
import org.openmobileis.database.fastobjectdb.FODBIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.db.exception.FODBException;
import org.openmobileis.database.fastobjectdb.db.query.soda.FODBSodaObjectSet;
import org.openmobileis.common.util.exception.BadDataFormatException;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.modules.common.data.DefaultLabel;
import org.openmobileis.modules.common.data.Label;
import org.openmobileis.modules.common.data.LabelFactory;
import org.openmobileis.common.util.collection.Array;


/**
 * Title:        Sanofi Project
 * Description:  
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public abstract class FODBLabelFactory implements LabelFactory {
	private static final Label emptyLabel = new DefaultLabel ("", "", "");
	private HashMap categoriesMap;
	/**
	 * Constructor for FODBLabelFactory.
	 */
	public FODBLabelFactory() {
		super();
		try	{
			FastObjectDB db = FastObjectDBManager.getCurrentFODB();
			if (!db.isCollectionExist("label"))	{
				db.createCollection("label", this.getDataType());
				FODBStringIndexDescriptor IDDescriptor = new FODBStringIndexDescriptor("ID", FODBIndexDescriptor.UNIQUE, "getLabelCategoryId()", 12, this.getMaxLabelIdLength()+this.getMaxCategoryIdLength()+1);
				db.addIndex("label", IDDescriptor);
				FODBStringIndexDescriptor CategoryDescriptor = new FODBStringIndexDescriptor("cat", FODBIndexDescriptor.MULTIPLE, "getCategory()", 10, this.getMaxCategoryIdLength(), 10);
				db.addIndex("label", CategoryDescriptor);
				
				this.initDB(db);
			}
		} catch (Throwable ex)	{
			LogManager.traceError(0, "Label Factory FODB ERROR : Unknown error during creation");
			LogManager.traceError(0, ex);
		}
	}
	
	public Array getLabelListForCategorie(String categorie)	{
		try {
			Query q = FastObjectDBManager.getCurrentFODB().query();
			q.constrain(this.getDataType());
			Query subq = q.descend("getCategory()");
			Constraint c = subq.constrain(categorie).equal();
			ObjectSet set = q.execute(); 
			return ((FODBSodaObjectSet)set).getArrayFromSet();
		} catch (Exception e) {
			LogManager.traceError(LogServices.DATABASESERVICE, e);
		}
		return new Array();
	}
	
	public Label getLabelWithIds(String id, String category)	{
		try {
			Query q = FastObjectDBManager.getCurrentFODB().query();
			q.constrain(this.getDataType());
			Query subq = q.descend("getLabelCategoryId()");
			Constraint c = subq.constrain(FODBLabel.convertIdsToLabelObjectIds(id, category)).equal();
			ObjectSet set = q.execute(); 
			if (set.hasNext())	{
				return (Label)set.next();
			} else	{
				return emptyLabel;
			}
		} catch (Exception e) {
			LogManager.traceError(LogServices.DATABASESERVICE, e);
			return emptyLabel;
		} 
	} 
   
  /**
   * Called by synchro server to create the label DB
   */
  public void storeLabel(Label label) throws ServiceException, DatabaseException {
		try {
			Label dblabel = this.getLabelWithIds(label.getLabelId(), label.getCategory());
			FastObjectDB db = FastObjectDBManager.getCurrentFODB();
			if (dblabel == null)	{
				db.add("label", label);
			} else	{
				db.replace("label", label);
			}
		} catch (Throwable ex) {
			LogManager.traceError(0, ex);
		}
  }

	public void deleteLabel(String id, String category) throws ServiceException, DatabaseException {
		try {
			Label dblabel = this.getLabelWithIds(id, category);
			FastObjectDB db = FastObjectDBManager.getCurrentFODB();
			db.deleteWithId("label", FODBLabel.convertIdsToLabelObjectIds(id, category));
		} catch (Throwable ex) {
			LogManager.traceError(0, ex);
		}
	}
	
	public abstract String[][] getCategoriesList();
  
	/**
	 * return the max length of the label id.
	 */
	protected abstract int getMaxLabelIdLength();
  
	protected abstract int getMaxCategoryIdLength();
  
	protected abstract Class getDataType();
	
	protected abstract void initDB(FastObjectDB db) throws FODBException, BadDataFormatException;

}
