/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  Creation P.Delrieu
 * 
 */
package org.openmobileis.modules.common.database.jdbc;

import java.sql.ResultSet;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.modules.common.data.GlobalProperty;
import org.openmobileis.modules.common.data.GlobalPropertyFactory;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class JDBCGlobalPropertyFactory implements GlobalPropertyFactory {

 
	private static JDBCGlobalPropertyFactory factory = null;
	private JDBCGlobalPropertyQueryManager propsQuery = null;

 /**
   * 
   */
  public JDBCGlobalPropertyFactory() {
		super();
		propsQuery = new JDBCGlobalPropertyQueryManager();
		try	{
			propsQuery.createSyncTable();
		} catch (Throwable ex)	{
			LogManager.traceError(0, ex);
		}
  }

	public static JDBCGlobalPropertyFactory getManager() {
			if (factory == null)  {
			synchronized(JDBCGlobalPropertyFactory.class) {
				if (factory == null)  {
					factory = new JDBCGlobalPropertyFactory();
				}
			}
		}
		return factory;
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.modules.crm.data.common.GlobalPropertyFactory#getProperty(java.lang.String, java.lang.String)
	 */
	public GlobalProperty getProperty(String service, String key) throws  ServiceException {
		GlobalProperty prop = null;
		try	{
			String[] param = new String[]{service, key};
			ResultSet result = propsQuery.getGlobalProperty(param);
			if (result.next()) {
				prop =  propsQuery.convertResultSetToProperty(result);
			}
			result.close();
		} catch (Exception ex)	{
			throw new ServiceException(ex);
		} finally {
			propsQuery.getDbManager().garbageOpenedConnection();
		}
		return prop;
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.modules.crm.data.common.GlobalPropertyFactory#getProperty(java.lang.String, java.lang.String)
	 */
	public Array getPropertiesForService(String service) throws  ServiceException {
		Array ret = new Array();
		try	{
			String[] param = new String[]{service};
			ResultSet result = propsQuery.getPropertiesForService(param);
			while (result.next()) {
				ret.add(propsQuery.convertResultSetToProperty(result));
			}
			result.close();
		} catch (Exception ex)	{
			throw new ServiceException(ex);
		} finally {
			propsQuery.getDbManager().garbageOpenedConnection();
		}
		return ret;
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.modules.crm.data.common.GlobalPropertyFactory#getProperty(java.lang.String, java.lang.String)
	 */
	public Array getAllGlobalProperties() throws  ServiceException {
		Array ret = new Array();
		try	{
			ResultSet result = propsQuery.getAllGlobalProperties();
			while (result.next()) {
				ret.add(propsQuery.convertResultSetToProperty(result));
			}
			result.close();
		} catch (Exception ex)	{
			throw new ServiceException(ex);
		} finally {
			propsQuery.getDbManager().garbageOpenedConnection();
		}
		return ret;
	}

  /* (non-Javadoc)
   * @see org.openmobileis.modules.crm.data.common.GlobalPropertyFactory#storeProperty(org.openmobileis.modules.crm.data.common.GlobalProperty)
   */
  public void storeProperty(GlobalProperty property) throws DatabaseException  {
		try	{
			GlobalProperty dbprops = this.getProperty(property.getServiceProperty(), property.getPropertyKey());
			if (dbprops == null)	{
				propsQuery.create(propsQuery.getInsertParamFromProperty(property));
			} else	{
				propsQuery.update(propsQuery.getUpdateParamFromProperty(property));
			}			
		} catch (Throwable ex)	{
			throw new DatabaseException(ex);
		} finally {
			propsQuery.getDbManager().garbageOpenedConnection();
		}
  }

  /* (non-Javadoc)
   * @see org.openmobileis.modules.crm.data.common.GlobalPropertyFactory#deleteProperty(java.lang.String, java.lang.String)
   */
  public void deleteProperty(String service, String key) throws DatabaseException  {
		try	{
			String[] param = {service, key};
			propsQuery.delete(param);
		} catch (Throwable ex)	{
			throw new DatabaseException(ex);
		} finally {
			propsQuery.getDbManager().garbageOpenedConnection();
		}
  }

}
