/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  Creation P.Delrieu
 * 
 */
package org.openmobileis.modules.common.database.jdbc;

//import java.sql.ResultSet;
import java.sql.*;

import org.openmobileis.common.context.ApplicationContext;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.database.AbstractQueryManager;
import org.openmobileis.common.util.database.JdbcPoolManagerDB;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.modules.common.data.DefaultGlobalProperty;
import org.openmobileis.modules.common.data.GlobalProperty;

/**
 * @author Philippe Delrieu
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class JDBCGlobalPropertyQueryManager extends AbstractQueryManager	{
	
	protected static String queryGetGlobalProperty = "SELECT idservice, propkey, propvalue FROM oooglobalproperties where idservice='%0%' AND propkey='%1%'";
	protected static String queryGetPropertiesForService = "SELECT idservice, propkey, propvalue FROM oooglobalproperties where idservice='%0%'";
	protected static String queryGetAllGlobalProperties = "SELECT idservice, propkey, propvalue FROM oooglobalproperties";

	protected static String queryInsertGlobalProperty = "INSERT INTO oooglobalproperties (idservice, propkey, propvalue)"
		+" VALUES ('%0%', '%1%', '%2%')";

	//update from synoptic ONLT no fidelisation managed
	protected static String queryUpdateGlobalProperty = "UPDATE oooglobalproperties " 
		+"SET propvalue='%0%'"  
		+" WHERE idservice='%1%' AND propkey='%2%'";
	protected static String queryDeleteGlobalProperty = "DELETE FROM oooglobalproperties where  idservice='%0%' AND propkey='%1%'";	

	// query pattern used to create table
	private static String queryCreateTable  =
				"CREATE TABLE oooglobalproperties ("
				+"idservice varchar(25) NOT NULL,"
				+"propkey varchar(15) NOT NULL,"
				+"propvalue varchar(50) NOT NULL,"
				+"CONSTRAINT oooglobalpropertieskey PRIMARY KEY (idservice, propkey)"
				+");";

/**
   * 
   */
  public JDBCGlobalPropertyQueryManager() {
    super();
		ApplicationContext context = ApplicationContextManager.getManager().getApplicationContext();
		JdbcPoolManagerDB pool = (JdbcPoolManagerDB)context.getObject("server.querymanager.dbpool");
		this.registerManagerDB(pool);
  }

	/*
	* check if the sync table exists. IF it does not exist, create it
	*/
	public void createSyncTable() throws ServiceException, java.sql.SQLException, DatabaseException {
		try {
			Connection con = this.getDbManager().getConnection();
			DatabaseMetaData dbmd = con.getMetaData();
			String[] tableType = {"TABLE"};
			ResultSet result = dbmd.getTables (null,null,"oooglobalproperties", tableType);
			if (!result.next()) {
				this.executeUpdate(queryCreateTable, null);
			}
		} finally {
			this.getDbManager().garbageOpenedConnection();
		}
	}
	
	// insert a row
	public void create (String parameters[]) throws DatabaseException {
		this.executeUpdate(JDBCGlobalPropertyQueryManager.queryInsertGlobalProperty, parameters);
	}

	// delete a row
	public void delete (String parameters[]) throws DatabaseException  {
		this.executeUpdate(JDBCGlobalPropertyQueryManager.queryDeleteGlobalProperty, parameters);
	}

	
	// update a row
	public void update (String parameters[]) throws DatabaseException {
		this.executeUpdate(JDBCGlobalPropertyQueryManager.queryUpdateGlobalProperty, parameters);
	}
    
	// get all client in db
	public ResultSet getGlobalProperty (String[] param) throws DatabaseException {
		return this.executeQuery(JDBCGlobalPropertyQueryManager.queryGetGlobalProperty, param);
	}
    
	// get all client in db
	public ResultSet getPropertiesForService (String[] param) throws DatabaseException {
		return this.executeQuery(JDBCGlobalPropertyQueryManager.queryGetPropertiesForService, param);
	}
    
	// get all client in db
	public ResultSet getAllGlobalProperties () throws DatabaseException {
		return this.executeQuery(JDBCGlobalPropertyQueryManager.queryGetAllGlobalProperties, null);
	}

	public GlobalProperty convertResultSetToProperty (ResultSet result) throws DatabaseException	{
		try	{
			GlobalProperty property =  new DefaultGlobalProperty(result.getString(1), result.getString(2));
			//idservice, key, value
			property.setPropertyValue(result.getString(3));
			return property;
		} catch (Throwable ex)	{
			throw new DatabaseException(ex);
		}
	}
	public String[] getInsertParamFromProperty(GlobalProperty property)	{
		String[] param = {		  	
			property.getServiceProperty()
			, property.getPropertyKey()
			, property.getPropertyValue()
		};
		return param;
	}
	
	public String[] getUpdateParamFromProperty(GlobalProperty property)	{
		String[] param = {		  	
			property.getPropertyValue()
			, property.getServiceProperty()
			, property.getPropertyKey()
		};
		return param;
	}
}
