/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2005 Modification F.Salque
 * 
 */

package org.openmobileis.modules.common.database.jdbc;

import java.sql.ResultSet;

import org.openmobileis.modules.common.data.TerminalUser;
import org.openmobileis.synchro.algo.replication.AlwaysUpdateClientSynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.utils.DefaultSynchroAtomicObject;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.security.auth.Credential;
import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget;

/**
 * Title:        OpenMobileIS Project
 * Description:   
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class JDBCTerminalUserFODBSyncTarget implements FODBSyncTarget {

  protected TerminalUserJDBCQuery query;

  /**
   * Constructor for TerminalUserManager.
   */
  public JDBCTerminalUserFODBSyncTarget(TerminalUserJDBCQuery q) {
    super();
    this.query = q;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionName()
   */
  public String getCollectionName() {
    return "terminaluser";
  }
  
  /**
   * Synchro meta data added to the synchro on the terminal side are given be the 
   * synchro process.
   * This method is call before all other synchro method.
   * @param metadata
   * @throws OpenMSPException
   */
  public void setSendSynchroMetaData(Object metadata)throws OpenMSPException    {
    
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionObjectWithId(java.lang.String)
   */
  public Object getCollectionObjectWithId(String id) throws OpenMSPException {
    TerminalUser user = null;
    try {
      String[] param = new String[] { id };
      ResultSet result = query.getTerminalUser(param);
      if (result.next()) {
        user = query.convertResultSetToTerminalUser(result);
        SessionContext context = SessionContextManager.getManager().getSessionContext();
        String userId = context.getUserId();
        if (userId != null && userId.equals(user.getTerminalUserId())) {
          user.setInstallTerminalUser(true);
        }
      }
      result.close();
    } catch (Exception ex) {
      LogManager.traceError(0, ex);
    } finally {
      query.getDbManager().garbageOpenedConnection();
    }
    return user;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#updateCollectionObject(java.lang.Object)
   */
  public void updateCollectionObject(Object obj) throws OpenMSPException {
    TerminalUser user = (TerminalUser) obj;
    try {
      //test because of trigger PB could not delete that does not exist in db
      TerminalUser dbrep = (TerminalUser) this.getCollectionObjectWithId(user.getTerminalUserId());
      if (dbrep != null) {
        String[] param = query.getUpdateParamFromTerminalUser(user);
        query.updateTerminalUser(param);
      } else {
        String[] param = query.getInsertParamFromTerminalUser(user);
        query.insertTerminalUser(param);
      }
      this.notifyTerminalUserUpdate(user);
    } catch (Throwable ex) {
      throw new OpenMSPException(ex);
    } finally {
      query.getDbManager().garbageOpenedConnection();
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#deleteCollectionObject(java.lang.String)
   */
  public void deleteCollectionObject(String id) throws OpenMSPException {
    try {
      //test because of trigger PB could not delete that does not exist in db
      TerminalUser dbuser = (TerminalUser) this.getCollectionObjectWithId(id);
      if (dbuser != null) {
        //select tache if exist
        String[] param = { id };
        query.deleteTerminalUser(param);
        //update never delete because after each modification for complet synchro
        this.notifyTerminalUserdelete(id);
      }
    } catch (Throwable ex) {
      throw new OpenMSPException(ex);
    } finally {
      query.getDbManager().garbageOpenedConnection();
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getConflicResolver()
   */
  public SynchroConflicResolver getConflicResolver() {
    return new AlwaysUpdateClientSynchroConflicResolver(); //PDA WIN always update server
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getAllCollectionObject()
   */
  public Array getAllCollectionObject() throws OpenMSPException {
    Array idList = new Array();
    try {
      SessionContext context = SessionContextManager.getManager().getSessionContext();
      String userId = context.getUserId();
      ResultSet result = query.getAllTerminalUsers();
      while (result.next()) {
        TerminalUser user = query.convertResultSetToTerminalUser(result);
        if (userId != null && userId.equals(user.getTerminalUserId())) {
          user.setInstallTerminalUser(true);
        }
        idList.add(user);
      }
      result.close();
    } catch (Exception ex) {
      LogManager.traceError(0, ex);
    } finally {
      query.getDbManager().garbageOpenedConnection();
    }
    return idList;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getCollectionObjectId(java.lang.Object)
   */
  public String getCollectionObjectId(Object obj) {
    return ((TerminalUser) obj).getTerminalUserId();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getAllModifiedAtomicObjectForServiceSince(java.lang.String, long)
   */
  public SynchroAtomicObject[] getAllModifiedAtomicObjectSince(long date) throws OpenMSPException {
     //always do a complete synchro
      SynchroAtomicObject[] sync = new SynchroAtomicObject[1];
      sync[0] = new DefaultSynchroAtomicObject("", "");
      return sync;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#getUpdateMaxNbRow()
   */
  public int getUpdateMaxNbRow() {
    return 0; //start complete when there is 2 or more account modifiation
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncService#updateSynchroDB(org.openmobileis.database.fastobjectdb.FastObjectDB)
   */
  public void updateSynchroDB(FastObjectDB db) throws OpenMSPException {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#connect(org.openmobileis.synchro.security.auth.Credential)
   */
  public void connect(Credential cred) throws UserNotFoundException, ServiceException {
    // do nothing
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#disconnect()
   */
  public void disconnect() {
    // do nothing
  }

  public void notifyTerminalUserUpdate(TerminalUser rep) throws SynchroException {

  }

  public void notifyTerminalUserdelete(String repId) throws SynchroException {

  }

}
