/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  Creation P.Delrieu
 * 
 */

package org.openmobileis.modules.common.history;

import org.odbms.ObjectSet;
import org.odbms.Query;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.database.fastobjectdb.FODBCollectionDescriptor;
import org.openmobileis.database.fastobjectdb.FODBIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;




/**
 * Description : Create an FODB history manager. 
 * History is a fixed length stack. They are use to remember user action for example a
 * search result.
 * History is identified with a name and a size.
 * The name identify the history.
 * The maxsize : size of the history. Old entry are removed if size exceed maxside
 * 
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class FODBHistoryManager implements HistoryManager{

  private String histoName;
  private int sizeTab;
 
  
  public FODBHistoryManager (String name, int maxsize){
    super();
		histoName = name;
		sizeTab = maxsize;
		
    String dbPath = PropertiesManager.getManager().getProperty("fastobjectdb.database.path");
		if (dbPath == null)	{
			LogManager.traceError(0, "History Factory FODB ERROR : FODB path property is not set. Can't create db");
		}
		try	{
			FastObjectDB db = FastObjectDBManager.getCurrentFODB();
			if (!db.isCollectionExist("FWKHISTORY")) {
              FODBCollectionDescriptor coldesc = new FODBCollectionDescriptor("FWKHISTORY", HistoryData.class);
              coldesc.setSynchronize(false);
				db.createCollection(coldesc);
				FODBStringIndexDescriptor IDMDescriptor = new FODBStringIndexDescriptor("Service", FODBIndexDescriptor.UNIQUE, "getService()", 12, 20);
				db.addIndex("FWKHISTORY", IDMDescriptor);
			}
			
		}catch (Throwable ex){
		  
		  LogManager.traceError(0, "History Factory FODB ERROR : Unknown error during creation");
			LogManager.traceError(0, ex);
		}
		
  }

  
	private void setUIds(HistoryData hd) {
	 
	  try{
      Query q = FastObjectDBManager.getCurrentFODB().query();
      q.constrain(HistoryData.class);
      Query subq = q.descend("getService()");
      subq.constrain(histoName).equal();
      ObjectSet set  = q.execute();
	    
	    if(set.hasNext()){
	      FastObjectDBManager.getCurrentFODB().replace("FWKHISTORY",hd);
	    }
	    else{
	      
	      FastObjectDBManager.getCurrentFODB().add("FWKHISTORY",hd);
	    }
	    
	  }
	  catch(Throwable ex){
	  
	    LogManager.traceError(0, "History Factory FODB ERROR : Unknown error during creation");
			LogManager.traceError(0, ex);
	  }
	  
	}
  
	private HistoryData getUIds() {
	
	    try {
	      Query q = FastObjectDBManager.getCurrentFODB().query();
	      q.constrain(HistoryData.class);
	      Query subq = q.descend("getService()");
	      subq.constrain(histoName).equal();
	      ObjectSet set  = q.execute();
	      
	      if(!set.hasNext()){
	        HistoryData Hd = new HistoryData(this.histoName,this.sizeTab);
	        return Hd;
	      } else	{
	        
	        HistoryData Hd = (HistoryData)set.next();
		      return Hd;  
	      }    
	  
	    } 
	    catch (DatabaseException ex) {
	      LogManager.traceError(LogServices.WEBSERVICE, ex);
	      return null;
	    }
	    
	}

	public void addUID (String uid){
	
	  HistoryData hd = this.getUIds();
	  
	    hd.addId(uid);
	    setUIds(hd);
	}
	 
	
	public void removeUID (String uid){
	  
	}

	public Array getAll(){
	  
	  HistoryData Hd = this.getUIds();
	  String[] uidsTemps = Hd.getIds();	  
		Array results = new Array(5);
		for (int i=uidsTemps.length -1; i >= 0; i--) {
			if (uidsTemps[i] != null) {
				results.add(uidsTemps[i]);
			}
		}
		return results;
	}

	public String getHistoryName(){
	  
	  return this.histoName;
	}


}
