/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  Creation P.Delrieu
 * 
 */
package org.openmobileis.modules.common.history;

import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.util.collection.Array;


/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class SessionHistoryManager implements HistoryManager {
  
  private String histoName;
  /**
   * 
   */
  public SessionHistoryManager(String name) {
    super();
		histoName = name;
  }
	public void addUID (String uid) {
		String[] uids = this.getUidList();
		int pos = this.getPosition(uid, uids);
		if (pos != -1) { // does exists
			this.remove(pos, uids);
		}
	 // Add the new one at the end of the array
		this.addLast(uid, uids);
	 // save the list
//		this.saveProperties(uids);
	}
	
	private String[] getUidList()	{
		SessionContext context = SessionContextManager.getManager().getSessionContext();
		String[] uidList = (String[])context.getAttribute("%openmishisto%"+getHistoryName());
		if (uidList == null)	{
			uidList = new String[6];
			context.setAttribute("%openmishisto%"+getHistoryName(), uidList);
		}
		return uidList;
	}

	public void removeUID (String uid) {
		String[] uids = this.getUidList();
		int pos = this.getPosition(uid, uids);
		if (pos != -1) { // does not exists
				this.remove(pos, uids);
			}
	//	this.saveProperties(uids);
	}

	public Array getAll() {
		String[] uidsTemps = this.getUidList();
		Array results = new Array(5);
		for (int i=uidsTemps.length -1; i >= 0; i--) {
			if (uidsTemps[i] != null) {
				results.add(uidsTemps[i]);
			}
		}
		return results;
	}
	
	public String getHistoryName()	{
		return histoName;
	}

	// Private Methods
	 // return the position of the uid or -1 if it does not exist
	 private int getPosition(String id, String uids[]) {
		for (int i=0; i < uids.length; i++) {
		 if ((uids[i] != null) && (uids[i].equals(id))) {
			 return i;
		 }
		}
		return -1;
	 }

	 // add the element at the end, remove the first one and compact the array if the array is full
	 private void addLast(String uid, String uids[]) {
		 for (int i=0; i < uids.length; i++) {
			 if (uids[i] == null) {
				 uids[i] = uid; // not yet completed
				 return;
			 }
		 } // end for
		// otherwise, the array is full, remove the first one, compact and add the new one at the end
		 this.remove(0,uids);
		 uids[uids.length -1] = uid;
	 }

	 private void remove (int pos, String uids[]) {
		 for (int i=pos; i < uids.length-1; i++) {
			 uids[i] = uids[i+1];
		 }
			uids[uids.length-1] = null;
	 }

}
