/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  
 *  Modifications :
 *  2006 Creation P.Delrieu
 * 
 */

package org.openmobileis.modules.profiles.terminal;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.*;
import org.openmobileis.services.Service;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Class that load module rubric and manage the rubric life cycle.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public  abstract class RubricLoader {
 // private java.util.HashMap serviceMap;
  
  /** Creates a new instance of ModuleLoader */
  public RubricLoader() {
  //  serviceMap = new java.util.HashMap(10);
  }
  
  /**
   * call before the loading of the services
   * @param:
   * @return:
   **/
  public abstract void preLoadingInit() throws ServiceException;
  
  /**
   * call after the loading of the services
   * @param:
   * @return:
   **/
  public abstract void postLoadingInit() throws ServiceException;
  
  /**
   * load all service attached to this module.
   * @return and array of loaded service.
   **/
  public Array loadService() throws ServiceException{
    Array ret = new Array();
    try {
      String[] serviceList = this.getServiceClassList();
      if (serviceList != null)  {
        for (int i=0; i<serviceList.length;i++) {
          if (serviceList[i] != null) {
            Service  service = (Service) Class.forName(serviceList[i]).newInstance();
            ret.add(service);
          }
        }
      }
    } catch (Exception ex)  {
      LogManager.traceError(LogServices.WEBSERVICE, "Can't load module");
      throw new ServiceException (ex);
    }
    return ret;
  }
  
  /**
   * call to get all service classes to load.
   * @param:
   * @return:
   */
  public abstract String[] getServiceClassList() throws ServiceException;
  
}
