/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.services;

import java.io.IOException;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.common.ServiceManager;

import freemarker.template.TemplateModelRoot;

/**
 * Use for a MultiEdition
 * 
 * The MainSubEditService is the main Edit of the multi edition. 
 * The other edit service must be SubEditService
 *
 * The MainSubEditService implementation is the same that the implementation 
 * of the SubEditService or the SimpleEditService. It's transparency for the programmer.
 * 
 * author : Manuel Gomez
 */
public abstract class MainSubEditService extends SimpleEditService {
	
	public MainSubEditService()	{
		
	}

	public String runTemplate (HttpServletRequest req, HttpServletResponse res , TemplateModelRoot templateData) throws ServletException, IOException {
		try	{
			//Look after the session if it's not null
			
			if(req.getSession().getAttribute(this.getSessionDataTempName()) != null){
				Object sessionDatas = req.getSession().getAttribute(this.getSessionDataTempName());
				
				Map map2 = req.getParameterMap();
				map2.putAll((Map)sessionDatas);
				//remove the session
				req.getSession().removeAttribute(this.getSessionDataTempName());
			
			}else{
			
				String url = req.getParameter("redirectURL");
				if(url != null){
					//put the params request in the session and do redirect service
					HttpSession session = req.getSession(true);
					session.setAttribute(this.getSessionDataTempName(), req.getParameterMap());
					ServiceManager.getManager().redirectToServiceURI(url, req, res);
					return null;
				}
				

			}

			//run the SimpleEditService()
			return super.runTemplate(req,res,templateData);
			//return this.getTemplateName();
		} catch (Exception ex) {
			LogManager.traceError(0, ex);
			HttpSession session = req.getSession(true);
			session.removeAttribute(this.getSessionDatasName());
			session.removeAttribute(this.getErrorSessionAttributName());
			throw new ServletException(ex);
		}
	}
	
	protected boolean isStoreMode(HttpServletRequest req)	{
		
		String uri = req.getParameter("redirectURL");
		if (uri != null)	{
			EditService service = (EditService)ServiceManager.getManager().getServiceByURI(uri);
			
			if (service.hasSessionData(req))	{
				service.removeSessionData(req);
				return false;
			}
		}

		return true;
	}
	

	private String getSessionDataTempName(){
		return "Session.MultiEdit.temp";
	}
	
	protected abstract void fillTemplateWithSessionData(Object sessionDatas, TemplateModelRoot templateData) throws ServiceException;	
	protected abstract void storeSessionObjectInDB(Object sessionDatas, HttpServletRequest req, TemplateModelRoot templateData) throws ServiceException;	
	protected abstract void fillSessionObjectWithRequestData(Object sessionDatas, HttpServletRequest req) throws ServiceException;	
	protected abstract String getSessionDatasName();
	protected abstract String getTemplateName();
	protected abstract Object createSessionObject(HttpServletRequest req) throws ServiceException;
	protected abstract String getDisplayServiceURI();
	

}
