/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.services;

import org.openmobileis.services.cache.WebPageCacheManager;
import org.openmobileis.services.navigation.NavigationBarManager;
import org.openmobileis.services.navigation.NavigationBarService;
import org.openmobileis.services.security.SecurityManager;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  All services have to specialize this class and override run() method.
 * The name is set by the LocalServiceManager at launching (by reading serviceList.properties)<p>
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public abstract class Service {
	
// private String uri;
 private boolean isNavigationService = false;

 public Service() {
 	if (this instanceof NavigationBarService)	{
		isNavigationService = true;
 	}
 }

/* Call by Service manager to perform to internal process before run call
 * @param req the servlet request
 * @param req the servlet response
 *  @exception ServletException when an exception has occurred
 */
  public void runService(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException	{
	//	LogManager.traceNotice(0, "Service :Begin run");

  	// validate if its not a auth pass request
  	//if yes validate pass. Use by synchro to begin sync.
 		if (SecurityManager.getManager().isInitializedPass())	{
 			String type = req.getParameter("type");
 			if ((type !=null) && (type.equals("setPassword")))	{ 				
	 			String pwd = req.getParameter("pwd");
	 			if (SecurityManager.getManager().validateServicePass(pwd))	{
	 			    SecurityManager.getManager().setServiceCall();
	 			}
 			}
	  	//manage password authentification
//			if ((SecurityManager.getManager().getMaxTimeout()!=0) &&(pass != null))	{ //validate password timeout
			if ((SecurityManager.getManager().getMaxTimeout()!=0) && SecurityManager.getManager().isInitializedPass())	{ //validate password timeout
				if (SecurityManager.getManager().isServiceCallElapseTime())	{ // ask pass
				    res.sendRedirect(res.encodeRedirectURL("/showpassform"));
					return;
				}
			}
	  	SecurityManager.getManager().setServiceCall();
 		}
 		
 		//manage navigation bar.
 		if (isNavigationService)	{ 			
 			NavigationBarManager.getManager().setCurrentService((NavigationBarService)this, req);
 		}
 	
      	  
	  //manage cached data.
	  if (this.useWebCache())	{
  	  byte[] cachedResp = WebPageCacheManager.getManager().getCachedPage(req, this);
  	  if (cachedResp != null)	{
  	      res.getWriter().print(cachedResp);
  	  	//((WebServerConnection)res).setResponse(cachedResp);
  	  	return;
  	  }
  	}
      	   	
	//	LogManager.traceNotice(0, "Service :BEfore run service");
 		this.run(req, res);
	//	LogManager.traceNotice(0, "Service :AFTER run service");
  	
    //add to cache
    if (this.useWebCache())	{
        WebPageCacheManager.getManager().setCachedPage(req, res.getOutputStream().toString().getBytes(), this);
    	//WebPageCacheManager.getManager().setCachedPage(req, ((WebServerConnection)res).getResponseData(), this);
    }
  }

/* Process the service method to implement 
 * @param req the servlet request
 * @param req the servlet response
 *  @exception ServletException when an exception has occurred
 */
  public abstract void run( HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException;

  /*
  * * Return the service name
  */
 //  public abstract String getName();

  /**
   * to wen server page cache management extends this method and return true.
   * Register your service with the WebPageCacheManager.
   */
  public boolean useWebCache()	{
  	return false;
  }
	
	/**
	 * place the last service of the navigation bar at the specified service uri if found.
	 * if not needed return null.
	 * Place here has a helper for all NavigationBarService to avoid this method implementation if not needed. 
	 * @return
	 */
	public String getNavigationBarLastServiceURI()	{
		return null;
	}

	/**
	 * @return
	 */
	public boolean isNavigationService() {
		return isNavigationService;
	}
	
	public abstract  String getServiceUri();
	
	/*public void setServiceURI(String uri) {
		this.uri = uri;
	} */
	
	public boolean equals(Object obj)	{
		if( obj instanceof Service){
			Service in = (Service) obj;
			if (in.getServiceUri().equals(this.getServiceUri()))	{
				return true;
			}
		}
		return false;
	}
	
	public int hashCode()	{
		return this.getServiceUri().hashCode();
	}

}