/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.services.cache;

import java.util.Hashtable;

import org.openmobileis.services.Service;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Main manage to register a service that has its page cached.
 * The use of web page cache for service that use complet processis to generate an answer
 * with data that doen't change often. When a request arrive for this service, if it has already
 * been cached, the cached page is return with no service processing.
 * To register a service, you must use a WebPageCacheServiceStruct. A default implementation is provided.
 * This structure define cache properties and store cached page.
 * 
 * When registering a service register for a spcific event that will signal when the cache has to be cleared.
 * A service can register the same WebPageCacheServiceStruct for several event.
 * When an event is send, all cached page of all service registered for this event is validated with the WebPageCacheServiceStruct implementation.
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public interface WebPageCacheServiceStruct {
	public void addWebPage(Hashtable reqParam, byte[] cachedData);
	public byte[] getResponseForRequest(Hashtable req);
	public Service getService();
	public void notifyCacheEvent(WebPageCacheEvent event);
}
