/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.services.common;

import java.io.IOException;
import java.util.Hashtable;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.openmobileis.services.Service;
import org.openmobileis.services.navigation.NavigationBarManager;
import org.openmobileis.services.servlet.OpenmisHttpServletRequest;
import org.openmobileis.common.util.log.*;

/**
 * @author Philippe Delrieu
 *
 */
public class RedirectToFromCallingService extends Service {

	/**
	 * Constructor for RedirectToFromCallingService.
	 */
	public RedirectToFromCallingService() {
		super();
	}

	/**
	 * @see org.openmobileis.services.Service#run(HttpServletRequest, HttpServletResponse)
	 */
	public void run(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
		
	    OpenmisHttpServletRequest openmisReq = (OpenmisHttpServletRequest)req;
	    
		// begin search
		String to = req.getParameter("to");
		if (to != null)	{ // start redirect
			//remove to parameter
		    openmisReq.removeParameter("to");
			//res.getCallingServiceManager().pushCallingService(this.getServiceUri(), req);
		    HttpSession session = req.getSession();
			CallingServiceManager callServManager = CallingServiceManager.getManager();
			callServManager.pushCallingService(this.getServiceUri(), req);
			//update navigation bar
			String uri = (String)req.getParameter("from");
			if (uri != null)	{
				NavigationBarManager.getManager().updateRequestParametersForRedirectService(uri, req);
			}
		  res.sendRedirect(to);
		} else 	{ //manage redirect return
			String from = (String)req.getParameter("from");
			if (from == null)	{
			    HttpSession session = req.getSession();
				CallingServiceManager callServManager = CallingServiceManager.getManager();
				callServManager.peekToTheSpecifiedCallingService(this.getServiceUri());
				//res.getCallingServiceManager().peekToTheSpecifiedCallingService(this.getServiceUri());
				//Hashtable oldParameterList = res.getCallingServiceManager().getCallingServiceParameters();
				Hashtable oldParameterList = callServManager.getCallingServiceParameters();
				//res.getCallingServiceManager().popCallingService();
				callServManager.popCallingService();
				if (oldParameterList != null)	{
					//manage old request in new one
				    openmisReq.addParameters(oldParameterList);
					from = (String)req.getParameter("from");
				}
			}
			if (from!=null)	{
				res.sendRedirect(from);				
			} else {
	  		LogManager.traceError(LogServices.WEBSERVICE, "RedirectToFromCallingService can't find to service");
	  		ServletTools.sendErrorPage("Internal Error" , "No service specified to return", "/index", res);
			}	
		}
	}

  
  public String getName() {
    return "calendar.RedirectToFromCallingService";
  }
  
  /**
   * return the service URI
   */
  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI()+"/admin/redirectService";
  }

}
