/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */
package org.openmobileis.services.navigation;

import java.util.Enumeration;
import java.util.Hashtable;

import javax.servlet.http.HttpServletRequest;

import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.services.Service;

public class NavigationBarManager {

  private static NavigationBarManager manager;

  protected static NavigationBarDescription description;

  protected Array serviceList;

  protected int serviceCounter = 0;

  protected boolean displayFormExitMessage = false;

  /**
   *  
   */
  protected NavigationBarManager() {
    super();
    serviceList = new Array();
  }
  
  public static void registerNavigationBarManager(NavigationBarManager new_manager) {
    manager = new_manager;
  }

  public static NavigationBarManager getManager() {
    if (manager == null) {
      synchronized (NavigationBarManager.class) {
        if (manager == null) {
          manager = new NavigationBarManager();
          ApplicationContextManager.getManager().addManager(manager);
        }
      }
    }
    return manager;
  }

  public static void initializeNavigationBar(NavigationBarDescription desc) {
    description = desc;
  }

  public void setCurrentService(NavigationBarService service, HttpServletRequest req) {
/*    NavigationBarServiceData data = new NavigationBarServiceData(service, req.getPathTranslated(),
        (Hashtable) ((WebServerConnection) req).getParametersTable().clone());*/
/*    NavigationBarServiceData data = new NavigationBarServiceData(service, req.getRequestURI(),
            new Hashtable(req.getParameterMap()));*/
	NavigationBarServiceData data = new NavigationBarServiceData(service, ((Service)service).getServiceUri(),
	        new Hashtable(req.getParameterMap()));
    //validate that the service is not already registered.
    Object serv = null;

    if (!service.displayRecursive()) {
      for (int i = 0; i < serviceCounter; i++) {
        serv = serviceList.get(i);
        if (serv.equals(data)) {
          serviceCounter = i;
        }
      }
    }

    /*
     * If getNavigationBarLastServiceURI() is not null the fonction find
     * the equal between the uri done and the serviceList to place the 
     * specified service uri
     *      */

    NavigationBarServiceData serv2 = null;
    
    String servUri = service.getNavigationBarLastServiceURI(); 
    if (servUri != null) {
      if (servUri.equals(NavigationBarService.NAVBAR_ROOT)) {
      	serviceCounter = 0;
      } else {
		for (int i = 0; i < serviceCounter; i++) {
		  serv2 = (NavigationBarServiceData)serviceList.get(i);
		  String vartemp = serv2.getUri();
        
		  if(servUri.equals(vartemp)) {
			serviceCounter = i+1;
		  }
		}
      }
    }

    //*************************************************************//
    this.displayFormExitMessage = service.displayFormExitMessage();

    data.setNavigationBarLabel(service.getNavigationBarLabel(req));
    serviceList.add(serviceCounter++, data);
  }
	
	public void updateRequestParametersForRedirectService(String serviceUri, HttpServletRequest req)	{
		if (serviceList.size() == 0) return;
		for (int i=serviceList.size() -1; i>=0; i--)	{
			NavigationBarServiceData data = (NavigationBarServiceData)serviceList.get(i);
			if (data.getUri().equals(serviceUri))	{
				Hashtable oldParameterList = data.getParameters();
				//Hashtable newTable = ((WebServerConnection)req).getParametersTable();
				Hashtable newTable = new Hashtable(req.getParameterMap());
				Enumeration enumera = newTable.keys();
				while (enumera.hasMoreElements())	{
					Object key = enumera.nextElement();
					if ((!key.equals("to")) && (!key.equals("from")))	{  //remove specific redirect service parameters
						Object value = newTable.get(key);
						oldParameterList.put(key, value);  	
					}	
				}
				break;
			}
		}
	}

  public NavigationBarServiceData getCurrentServiceData() {
    if (serviceCounter == 0)
      return null; //no service
    NavigationBarServiceData data = (NavigationBarServiceData) serviceList.get(serviceCounter - 1);
    if (data != null) {
      return data;
    }
    return null;
  }

  public void popService() {
    serviceCounter--;
    serviceList.remove(serviceCounter);
  }

  public void popToService(int nb) {
    serviceCounter = nb + 1;
  }

  public void resetServiceTable() {
    if (serviceCounter == 0)
      return;
    serviceList.add(0, serviceList.get(serviceCounter - 1));
    serviceCounter = 1;
  }

  public String getNavigationBar() {
    if (description == null)
      return "";
    if (serviceCounter == 0)
      return "";
    StringBuffer str = new StringBuffer();
    NavigationBarServiceData data = null;
    for (int i = 0; i < serviceCounter - 1; i++) {
      data = (NavigationBarServiceData) serviceList.get(i);
      str.append(description.getPastLinkFontTag());
      if (this.displayFormExitMessage) {
        str.append("<a href=\"#\" onClick=\"displayNavFormMessageExit('/services/admin/managereturn?servnb=" + i + "')\">");
      } else {
        str.append("<a href=\"/services/admin/managereturn?servnb=" + i + "\">");
      }
      str.append(data.getNavigationBarLabel());
      str.append("</a>");
			str.append("</font>");
      str.append(description.getLinkSeparator());
    }
    str.append(description.getCurrentLinkFontTag());
    data = (NavigationBarServiceData) serviceList.get(serviceCounter - 1);
    str.append(data.getNavigationBarLabel());
		str.append("</font>");
    return str.toString();
  }

}