/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 * 
 */

package org.openmobileis.services.security;


import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class SecurityManager {
	private static SecurityManager manager;
 	private long passwordTime = 0;
 	private long maxTimeout = 0;
 	
 	private static SecurityStore store;
 	private static SecurityHash hash;
 	
	/**
	 * Constructor for SecurityManager.
	 */
 	
	protected SecurityManager() {
		super();
		if (SecurityManager.store == null) {
	        SecurityManager.registerSecurityStore(new DefaultSecurityStore());
		}
		if (SecurityManager.hash == null) {
	        SecurityManager.registerSecurityHash(new DefaultSecurityHash());
		}
 		String timeout = PropertiesManager.getManager().getProperty("webserver.securoty.password.timeout");
 		if (timeout != null)	{
 			maxTimeout = Integer.parseInt(timeout)*60*1000; 			
 		}
	}
	
	public static void registerSecurityStore(SecurityStore newStore) {
	      synchronized(SecurityManager.class) {
	          SecurityManager.store = newStore;
	      }
	}
	
	public static void registerSecurityHash(SecurityHash newHash) {
	      synchronized(SecurityManager.class) {
	          SecurityManager.hash = newHash;
	      }
	}
	
	public static SecurityManager getManager()	{
	    if (manager == null)  {
	        synchronized(SecurityManager.class) {
	          if (manager == null)  {
	            manager = new SecurityManager();
	            ApplicationContextManager.getManager().addManager(manager);
	          }
	        }
	      }
	    return manager;
	}
	
	public long getMaxTimeout()	{
		return maxTimeout;
	}
	
	public boolean isServiceCallElapseTime()	{
		long currentTime  = System.currentTimeMillis();
		long elapse = currentTime - passwordTime;
		if (elapse >maxTimeout)	{ 
			return true;
		}
		return false;
	}
	
	public void setServiceCall()	{
		passwordTime  = System.currentTimeMillis();
	}
	
	public boolean changeServicePass(String oldpassword, String newpassword) {
	    
	    // VALIDATE OLD PASSWOR
	    if (oldpassword == null || oldpassword.trim().length() == 0) {
	        if (this.isInitializedPass()) {
	            LogManager.traceWarning(LogServices.SECURITYSERVICE, "SecurityManager::changeServicePass : oldpassword is null while it's initialized... returning false");
	            return false;
	        }
	    } else {
	        if (!this.validateServicePass(oldpassword)) {
	            LogManager.traceWarning(LogServices.SECURITYSERVICE, "SecurityManager::changeServicePass : wrong old password... returning flase");
	            return false;
	        }
	    }
	    
	    // SAVE NEW PASSWORD
	    if (newpassword == null || newpassword.trim().length() == 0) {
	        LogManager.traceWarning(LogServices.SECURITYSERVICE, "SecurityManager::changeServicePass : newpassword is incorrect");
	        return false;
	    }
	    LogManager.traceWarning(LogServices.SECURITYSERVICE, "SecurityManager::changeServicePass : before setServicePass");
	    this.setServicePass(newpassword);
	    LogManager.traceWarning(LogServices.SECURITYSERVICE, "SecurityManager::changeServicePass : after setServicePass.. returning true");
	    return true;
	}
	
	private void setServicePass(String password)	{
	    if (password == null) {
	        return;
	    }
		byte[] hashedPass = SecurityManager.hash.hash(password.toCharArray());
		SecurityManager.store.store(new String(hashedPass));
	}
	
	public boolean validateServicePass(String password) {
	    if (password == null) {
	        return false;
	    }
	    byte[] hashedPass = SecurityManager.hash.hash(password.toCharArray());
	    byte[] hashedServicePass = SecurityManager.store.read();
	    if (hashedPass != null && hashedPass.equals(hashedServicePass)) {
	        return true;
	    }
	    return false;
	}
	
	public boolean isInitializedPass() {
	    byte[] pass = SecurityManager.store.read();
	    
	    if (pass == null || pass.length == 0) {
	        return false;
	    }
	    
	    return true;
	}
	
}
