/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.services.servlet;

import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.*;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.CallingServiceManager;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.navigation.NavigationBarManager;
import org.openmobileis.services.navigation.NavigationBarService;
import org.openmobileis.services.security.SecurityManager;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;
import org.openmobileis.synchro.openmsp.protocol.Status;

import HTTPClient.HTTPConnection;

import freemarker.template.*;
import java.io.*;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  show mobile portal. First page opened when the web server is started
 * use the property org.openmobileis.service.forcepass to define if password has to be asked.
 * The template name is openmischannel/index.htm"
 * URL is /services/index
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class IndexServlet extends TemplateService implements NavigationBarService {

  private static int uniqID = 0;
  protected static IndexServlet serverIndexServlet;
  private boolean forcePass = false;
  
  private Array contentListenerMap;
  
  public IndexServlet() {
    LogManager.traceWarning(0, "INIT INDEXSERVLET");
      contentListenerMap = new Array(5);
      serverIndexServlet = this;
      ApplicationContextManager.getManager().addManager(this);
    	String oslang = PropertiesManager.getManager().getProperty("org.openmobileis.service.forcepass");
    	if (oslang != null) forcePass = true;
    	else forcePass = false;
  }
  
  public static IndexServlet getIndexServlet()  {
    return serverIndexServlet;
  }
  
  public String getIndexServletTemplateName()   {
    return "openmischannel"+java.io.File.separator+"index.htm";
  }
  
  public synchronized void addContentListener(IndexServletContentListener listener) {
    contentListenerMap.add(listener);
  }

	public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot modelRoot)
		throws ServletException, IOException {
    try {  
	    // unregister pendding calling service
      CallingServiceManager calling = CallingServiceManager.getManager();
      calling.clearStack();
      
			NavigationBarManager.getManager().resetServiceTable();
	    //remove session attribut
	    SessionContext session = SessionContextManager.getManager().getSessionContext();
	    session.clearAttribute();
	    
	    
	    //force password fill if need
      if ((!SecurityManager.getManager().isInitializedPass())&&(forcePass))	{
        ServiceManager.getManager().redirectToServiceURI("/common/showparamform", req, res);
				return null;
      }
	    
	    
	    for (int i=0; i<contentListenerMap.size(); i++) {
	      IndexServletContentListener listener = (IndexServletContentListener)contentListenerMap.get(i);
	      listener.addContent(modelRoot);
	     
	    } 
	    // display date
	    java.text.SimpleDateFormat formatter
	      = new java.text.SimpleDateFormat ("dd/MM/yyyy", java.util.Locale.FRANCE);
	    java.util.Date newDate = new java.util.Date(org.openmobileis.embedded.util.SystemAPI.currentTimeInMillis());
	    modelRoot.put("date", new SimpleScalar(formatter.format(newDate)));
	    modelRoot.put("uniqID", new SimpleScalar(Integer.toString(uniqID++)));
	        
	        
		  // journal
		  if (OpenMSPSynchroManager.getManager().getGlobalSynchroStatut() == Status.STATUS_OK)
		    modelRoot.put("journalStatus", new SimpleScalar("journalOK"));
		  else
		  	modelRoot.put("journalStatus", new SimpleScalar("journalNOK"));
		} catch (Exception ex) {
		    LogManager.traceAlert(LogServices.WEBSERVICE, ex);
		    res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, "Erreur survenue lors de l'execution du service");
		}
		
	  return getIndexServletTemplateName();
  }

  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI()+"/index";
  }
  
	public String getNavigationBarLabel(HttpServletRequest req)	{
      String home = IntlResourceManager.getManager().getLocalizedProperty("org.openmobileis.services.IndexServlet.name", "Home");
	  return home;
	}
	
	public boolean displayFormExitMessage() {
		return false;
	}
	
	public boolean displayRecursive() {
		return false;
	}

}