/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.services.servlet;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.embedded.webserver.templates.TemplateManager;
import org.openmobileis.embedded.webserver.templates.TemplateNotFoundException;

import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * register this servlet with this URL /common
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class PasswordFormServlet extends HttpServlet {
  static final long serialVersionUID = 5521257935120563452L;

  public void service( HttpServletRequest req, HttpServletResponse res ) throws ServletException, java.io.IOException {
		String templateName="openmischannel/passwordform.htm";
    try {
			TemplateModelRoot modelRoot = TemplateManager.getManager().getTemplateModelRoot();
			//manage error
			String error = (String)req.getSession(true).getAttribute("security.pass.form");
			if (error !=null)	{
      	modelRoot.put("errorMessage", new SimpleScalar(error));
				req.getSession(true).removeAttribute("security.pass.form");
			}      
      
      TemplateManager.getManager().sendResponse(templateName, modelRoot, res);
    } catch (TemplateNotFoundException ex)  {
      try {
        IntlResourceManager resourceManager = IntlResourceManager.getManager();
        String title = resourceManager.getLocalizedProperty("TemplateService.ExecError");
        res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, title);
      } catch (Exception exi)  {
      }
      LogManager.traceAlert(LogServices.WEBSERVICE, "Error Template not found name :"+templateName);
      LogManager.traceAlert(LogServices.WEBSERVICE, ex);
    } catch (Exception ex) {
      try {
        IntlResourceManager resourceManager = IntlResourceManager.getManager();
        String title = resourceManager.getLocalizedProperty("TemplateService.ExecError");
        res.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, title);
      } catch (Exception exi)  {
      }
      LogManager.traceAlert(LogServices.WEBSERVICE, "Error during execution of Service show password form");
      LogManager.traceAlert(LogServices.WEBSERVICE, ex);
    }
  }

}
