/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.services.servlet;

import java.io.File;
import java.util.Properties;

import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.FileOpenCloseLogManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.synchro.client.SynchroFastObjectDBManager;
import org.openmobileis.services.servlet.OpenMISInit;

public class SimpleOpenMISInit implements OpenMISInit {

  public SimpleOpenMISInit() {
    super();
  }

  public void preLoadingInit() throws ServiceException {
    
    //init of log management. If the directory log exist, the logmanager log in the log/openmislog.txt file.
    // otherwise it log on the console
    Properties  props = new Properties();
    props.put("org.openmobileis.common.log.file", System.getProperty("user.dir")+"/WEB-INF/log/openmislog.txt");
    File file = new File(System.getProperty("user.dir")+"/WEB-INF/log");
    if (file.exists())   FileOpenCloseLogManager.registerLogManager(props); //log to a file
    else LogManager.registerLogManager(null); //log to a file
    
    //init db propeties.
    PropertiesManager.getManager().addProperty("fastobjectdb.database.path", System.getProperty("user.dir")+"/database");
    PropertiesManager.getManager().addProperty("fastobjectdb.database.name", "db");

    //init internationalization management.
    try	{
      String locale = IntlResourceManager.getManager().getSystemDefaultLocale();
      IntlResourceManager.getManager().addLocalizedProperties(System.getProperty("user.dir")+"/WEB-INF/conf/properties/systemmessages.properties", locale);
    } catch (Throwable ex)	{
      LogManager.traceError(0, ex);
    }

    //init synchro properties
    PropertiesManager.getManager().addProperty("org.openmobileis.synchro.client.openmsp.numsyncpropsfile",System.getProperty("user.dir") + "/WEB-INF/conf/numsync.txt");
    try {
      SynchroFastObjectDBManager.getCurrentFODB(); //init synchro db to be use as default db
    } catch (Throwable ex)  {
      throw new ServiceException(ex);
    }
    
 }

  public void postLoadingInit() throws ServiceException {
  }

}
