/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.services.util;

import java.util.Calendar;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.modules.common.data.Label;

import freemarker.template.SimpleHash;
import freemarker.template.SimpleList;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class TemplateUtils {



    /**
    * Use to fill form template with day, month, year, hour, minute combo box.
    * Return a simple hash containing 5 lists: dayList, monthList, yearList, hourList and minuteList set with the date giving by parameters.
    * Template code :
    *   <SELECT NAME="startDay">
        <list startDate.dayList as day>
          <OPTION VALUE="${day.value}" <if day.select>selected</if>>${day.value}</OPTION>
        </list>
    </SELECT>
    <SELECT NAME="startMonth">
        <list startDate.monthList as month>
          <OPTION VALUE="${month.value}" <if month.select>selected</if>>${month.value}</OPTION>
        </list>
    </SELECT>
    <SELECT NAME="startYear">
        <list startDate.yearList as year>
          <OPTION VALUE="${year.value}" <if year.select>selected</if>>${year.value}</OPTION>
        </list>
    </SELECT>

    */
    public static SimpleHash getInputDate(
        int selectedDay,
        int selectedMonth,
        int selectedYear,
        int selectedHours,
        int selectedMinutes
        ) {

        SimpleHash inputDate = new SimpleHash();
        int cpt;
        SimpleList list;
        SimpleHash element;
        // fill day
        list = new SimpleList();
        for (cpt = 1; cpt <= 31; cpt++) {
            element = new SimpleHash();
            element.put("select", selectedDay == cpt);
            element.put("value", format(cpt));
            list.add(element);
        }
        inputDate.put("dayList", list);
        // fill month
        list = new SimpleList();
        for (cpt = 1; cpt <= 12; cpt++) {
            element = new SimpleHash();
            element.put("select", selectedMonth + 1 == cpt);
            element.put("value", format(cpt));
            list.add(element);
        }
        inputDate.put("monthList", list);

        list = new SimpleList();
        Calendar cal = Calendar.getInstance();
        int currentYear = cal.get(Calendar.YEAR);
        for (cpt = currentYear - 1; cpt <= currentYear + 3; cpt++) {
            element = new SimpleHash();
            element.put("value", String.valueOf(cpt));
            element.put("select", selectedYear == cpt);
            list.add(element);
        }
        inputDate.put("yearList", list);
        // fill hours
        list = new SimpleList();
        for (cpt = 0; cpt <= 23; cpt++) {
            element = new SimpleHash();
            element.put("value", format(cpt));
            element.put("select", selectedHours == cpt);
            list.add(element);
        }
        inputDate.put("hourList", list);
        // fill minutes
        list = new SimpleList();
        for (cpt = 0; cpt <= 59; cpt += 5) {
            element = new SimpleHash();
            element.put("value", format(cpt));
            element.put("select", selectedMinutes == cpt);
            list.add(element);
        }
        inputDate.put("minutesList", list);

        // return the input date
        return inputDate;
    }
    
    public static SimpleHash getInputDuration(int selectedHours, int selectedMinutes, int maxHour){
        SimpleList list;
        SimpleHash element;
        SimpleHash inputDuration = new SimpleHash();
        int cpt;
      // fill hours
        list = new SimpleList();
        for (cpt = 0; cpt <= maxHour; cpt++) {
            element = new SimpleHash();
            element.put("value", format(cpt));
            element.put("select", selectedHours == cpt);
            list.add(element);
        }
        inputDuration.put("hourList", list);
        // fill minutes
        list = new SimpleList();
        for (cpt = 0; cpt <= 59; cpt += 5) {
            element = new SimpleHash();
            element.put("value", format(cpt));
            element.put("select", selectedMinutes == cpt);
            list.add(element);
        }
        inputDuration.put("minutesList", list);
        
        return inputDuration;
    }

    public static String format(int value) {
        if (value < 10) {
            return "0" + value;
        } else {
            return String.valueOf(value);
        }
    }
 
    
    /** 
     * Use to facilitate label fields edit. Returnt the template simplelist to use to fill your template.
     * Exemple of HTML template :
        <select name="type">
           <list typeVisit as fct>
               <option value="${fct.value}" <if fct.selected>selected</if>>${fct.name}</option>
           </list>
        </select>
     * generate the template list with fct.value and fct.selected.
     * If no selectedid is available  use null.
     * @param labelArray : list of label to display in the list.
     * @param selectedId : default label to show in the list.
     * @return
     */
    public static SimpleList fillFormPopUpWithLabel(Array labelArray, String selectedId)    {
        SimpleList list = new SimpleList();
        SimpleHash listElement;
        for (int i=0; i<labelArray.size(); i++) {
            Label lab = (Label) labelArray.get(i);
            listElement = new SimpleHash();
            listElement.put("name", lab.getLabel());
            if ((selectedId!=null)&&(selectedId.equals(lab.getLabelId())))  {
                listElement.put("selected", true);
            } else  {
                listElement.put("selected", false);
            }
            listElement.put("value", lab.getLabelId());
            list.add(listElement);      
        }
        return list;
    }

}
