/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.algo.replication;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Base interface to implements to handle synchro conflicts.
 * Interface implementation are use as a service, so instance member must be declared.
 * If data has to be share betwin different method call use the SessionContext. 
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public interface SynchroConflicResolver {

  public final static int RETURN_ERROR_CONFLIC = 0;
  public final static int UPDATE_SERVER = 1;
  public final static int UPDATE_PDA = 2;
  public final static int CREATE_NEW_OBJECT= 3;
  public final static int DO_NOTHING = 4;

  /**
   * resolve the synchro conflic when an add command is send by the pda when the same object is already in database.
   * @return the ReplicationSAOEvent that correspond to the action to be done.
   */
  public ReplicationSAOEvent resolveAddandPresent(SynchroAtomicObject clientao);

  /**
   * resolve the synchro conflic when a replace command is send by the pda when the same object has been modified in database.
   * @return the ReplicationSAOEvent that correspond to the action to be done.
   */
  public ReplicationSAOEvent resolveReplaceandModified(SynchroAtomicObject ao);

  /**
   * resolve the synchro conflic when a replace command is send by the pda when the same object has been deleted in database.
   * @return the ReplicationSAOEvent that correspond to the action to be done.
   */
  public ReplicationSAOEvent resolveReplaceandDeleted(SynchroAtomicObject ao);

  /**
   * resolve the synchro conflic when a delete command is send by the pda when the same object has been modified in database.
   * @return the ReplicationSAOEvent that correspond to the action to be done.
   */
  public ReplicationSAOEvent resolveDeleteandModified(SynchroAtomicObject ao);

}