/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  
 *  Modifications :
 *  2006 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.algo.replication.utils;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.SynchroException;
import java.sql.ResultSet;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.*;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.utils.impl.AtomicObjectQueryManager;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberManager;
import org.openmobileis.synchro.openmsp.client.db.DBImportFileCoder;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Manage Atomic Object data needed for synchronisation.
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class DefaultSynchroAtomicObjectDelegate implements SynchroAtomicObjectDelegate {
  protected AtomicObjectQueryManager queryManager;

	public DefaultSynchroAtomicObjectDelegate(AtomicObjectQueryManager query) {
		queryManager = query;
		try {
			queryManager.createTable();
		}  catch (Exception e) {
			LogManager.traceCritique(LogServices.SERVERSERVICE, new ServiceException("Error init Syncho Atomic Object Manager. Exit System",e));
		}
	}

  /**
   * return all atomic object uid modified since specified date for service
   */
  public SynchroAtomicObject[] getAllModifiedAtomicObjectForServiceSince(String serviceName, long date, String userLink) throws SynchroException{
    Array retList = new Array();
    try {
    	if (userLink==null)	userLink = "";
      String[] uids = {serviceName, Long.toString(date), userLink};
      ResultSet result = queryManager.getAllModifiedAtomicObjectForServiceSince(uids);
      SynchroAtomicObject syncObject = null;
      while (result.next()) {
        syncObject = new DefaultSynchroAtomicObject(DBImportFileCoder.getCoder().removeAntiSlash(result.getString(1)), null, 0, result.getShort(2), result.getLong(3));
        retList.add(syncObject);
      }
    }  catch (java.sql.SQLException ex) {
      throw new SynchroException(ex);
    } catch (DatabaseException ex) {
      throw new SynchroException(ex);
    } finally {
			queryManager.getDbManager().garbageOpenedConnection();
    }
    SynchroAtomicObject[] ret = new SynchroAtomicObject[retList.size()];
    retList.toArray(ret);
    return ret;
  }

  /**
   * return the AtomicObject with specified uid. return null if not found
   */
  public SynchroAtomicObject getAtomicObject(String uid, String serviceName) throws SynchroException{
    try {
      String[] params = {DBImportFileCoder.getCoder().cleanAndFormatStringforDB(uid), serviceName};
      ResultSet result = queryManager.getAtomicObject(params);
      if(result.next()) {
        return new DefaultSynchroAtomicObject(DBImportFileCoder.getCoder().removeAntiSlash(result.getString(1)), result.getString(2), result.getLong(3), result.getShort(4), result.getLong(5));
      }
      
    }  catch (java.sql.SQLException ex) {
      throw new SynchroException(ex);
    } catch (DatabaseException ex) {
      throw new SynchroException(ex);
    } finally {
			queryManager.getDbManager().garbageOpenedConnection();
    }
    return null;
  }

  public void updateAtomicObject(SynchroAtomicObject object, String serviceName, String userLink) throws SynchroException {
    try {
      if (object.getModificationType() == SynchroAtomicObject.ADD)  {
        object.setModificationType(SynchroAtomicObject.REPLACE);
      }
      long modifSyncNumbner = object.getModifSyncNumber();
      if (modifSyncNumbner == 0)  {
        //if not syncnumber is defined, for to the MAX sync number.
        //During the synchronisation, if not db modification is done the MAX sync number == the new synchro number.
        modifSyncNumbner = SyncNumberManager.getManager().getNextSynchroNumber().getSynchroNumber();
      }
	 if (userLink==null)	userLink = "";
      String[] data = {Long.toString(modifSyncNumbner), object.getCheckSum(), Short.toString(object.getModificationType()), userLink, DBImportFileCoder.getCoder().cleanAndFormatStringforDB(object.getUID()), serviceName};
      queryManager.updateAtomicObject(data);
    } catch (DatabaseException ex) {
      throw new SynchroException(ex);
    }
  }

  public void createAtomicObject(SynchroAtomicObject object, String serviceName, String userLink) throws SynchroException {
    try {
			if (userLink==null)	userLink = "";
      long createSyncNumbner = object.getModifSyncNumber();
      if (createSyncNumbner == 0)  {
        createSyncNumbner = SyncNumberManager.getManager().getNextSynchroNumber().getSynchroNumber();
      }
      String[] data = {DBImportFileCoder.getCoder().cleanAndFormatStringforDB(object.getUID()), serviceName, Long.toString(createSyncNumbner), object.getCheckSum(), Short.toString(SynchroAtomicObject.ADD), Long.toString(createSyncNumbner), userLink};
      queryManager.createAtomicObject(data);
    } catch (DatabaseException ex) {
      throw new SynchroException(ex);
    }
  }

  /**
   * delete all atomic object of the specified service that are older than the specified time
   * @param service id
   * @param deletedTime : time defining the date where DELETE Atomic Object Time Stamp that are before it are removed.
  */
  public void purgeAtomicObjcet(String serviceName, long deletedTime) throws SynchroException  {
    try {
      String[] data = {serviceName, Long.toString(deletedTime)};
      queryManager.purgeDeletedAtomicObject(data);
    } catch (DatabaseException ex) {
      throw new SynchroException(ex);
    }
  }

}