/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  
 *  Modifications :
 *  2006 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.algo.replication.utils;

import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Manage Atomic Object. Base manager to access all atomic Object.
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class SynchroAtomicObjectManager {
  private static SynchroAtomicObjectManager manager;
  private SynchroAtomicObjectDelegate delegate;

  private SynchroAtomicObjectManager() {
  }

  public static SynchroAtomicObjectManager getManager() {
     if (manager == null)  {
        synchronized(SynchroAtomicObjectManager.class) {
        if (manager == null)  {
          manager = new SynchroAtomicObjectManager();
        }
      }
    }
    return manager;
  }
  
  public SynchroAtomicObjectDelegate getDelegate() {
  	return this.delegate;
  }

  /**
   * delete all atomic object of the specified service that are older than the specified time
   * @param service id
   * @param deletedTime : time defining the date where DELETE Atomic Object Time Stamp that are before it are removed.
  */
  public void purgeAtomicObjcet(String serviceName, long deletedTime) throws SynchroException  {
    delegate.purgeAtomicObjcet(serviceName, deletedTime);
  }

  /**
   * return all atomic object uid modified since specified date for service
   * the user link is use to get AO associated to a user. NULL if none
   */
  public SynchroAtomicObject[] getAllModifiedAtomicObjectForServiceSince(String serviceName, long date, String userLink) throws SynchroException{
    return delegate.getAllModifiedAtomicObjectForServiceSince(serviceName, date, userLink);
  }

  /**
   * Update the modification information (sync number) if the specified SynchroAtomicObject.
   * 
   * The sync number update policy is defined by the SynchroAtomicObjectManager delegate.
   * the user link is use to get Atomic object associated to a user. NULL if none
   * It is use to get all mofication ofr a service and an user.
   */
  public synchronized void updateAtomicObject(SynchroAtomicObject object, String serviceName, String userLink) throws SynchroException  {
    Object obj  = delegate.getAtomicObject(object.getUID(), serviceName);
    if (obj != null)	{
    	delegate.updateAtomicObject(object, serviceName, userLink);
    } else	{
			delegate.createAtomicObject(object, serviceName, userLink);
    }
  }

  public void registerDelegateManager(SynchroAtomicObjectDelegate d) {
    delegate = d;
  }
  /**
   * return the AtomicObject with specified uid. return null if not found
   */
  public SynchroAtomicObject getAtomicObject(String uid, String serviceName) throws SynchroException{
    return delegate.getAtomicObject(uid, serviceName);
  }
}