/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  
 *  Modifications :
 *  2006 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.algo.replication.utils.impl;

import java.sql.*;

import org.openmobileis.common.util.database.AbstractQueryManager;
import org.openmobileis.common.util.exception.DatabaseException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Manage physical storae of AtomicObject synchro data (modification timestamp, ....).
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public abstract class AtomicObjectQueryManager extends AbstractQueryManager {

 /*
 * check if the sync table exists. IF it does not exist, create it
 */
 public abstract void createTable() throws DatabaseException, java.sql.SQLException;

  public abstract void createAtomicObject(String[] parameters) throws DatabaseException;

  public abstract void updateAtomicObject(String[] parameters) throws DatabaseException;

	public abstract void purgeDeletedAtomicObject(String[] parameters) throws DatabaseException;

  /* getAtomicObject() get the atomic object for specified UID
  * @param     : array containing atomic object uid
  * @return    ResultSet : rows containing UID, CHECKSUM, TIME_STAMP, MODIFICATION_TYPE, CREATION_DATE
  * @exception ServiceException
  */
  public abstract ResultSet getAtomicObject(String parameters[]) throws DatabaseException;

  /* getAtomicObjectsForService() return all atomic object uid modified since specified date for service
  * @param     : array containing service id and date
  * @return    ResultSet : rows containing UID
  * @exception ServiceException
  */
  public abstract ResultSet getAllModifiedAtomicObjectForServiceSince (String parameters[]) throws DatabaseException;
 
}
