/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  
 *  Modifications :
 *  2006 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.algo.replication.utils.impl;

import java.sql.*;

import org.openmobileis.common.util.exception.DatabaseException;


/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Manage physical storae of AtomicObject synchro data (modification timestamp, ....).
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class PostgreSQLAtomicObjectQueryManager extends AtomicObjectQueryManager {
  // query pattern used to insert a new UID
	private static String queryUpdate  =
		"UPDATE SYNC_UID SET TIME_STAMP=%0%, CHECKSUM='%1%', MODIFICATION_TYPE =%2% , USERLINK ='%3%' " +
		"WHERE UID='%4%' AND SERVICEID='%5%'";

	 private static String queryCreate  =
		 "INSERT INTO SYNC_UID (UID , SERVICEID, TIME_STAMP, CHECKSUM, MODIFICATION_TYPE, CREATION_DATE, USERLINK) " +
			"VALUES ('%0%', '%1%', %2%, '%3%', %4%, %5%, '%6%')";

  private static String createIndex =
    "CREATE INDEX serviceid ON SYNC_UID (SERVICEID)";

  private static String uidIndex =
    "CREATE INDEX uidid ON SYNC_UID (UID, SERVICEID)";

  private static String modifIndex =
    "CREATE INDEX modifid ON SYNC_UID (SERVICEID, TIME_STAMP,  MODIFICATION_TYPE)";

  // query pattern used to create syncNB table
  private static String queryCreateTable  =
    "CREATE TABLE SYNC_UID ("
    +"UID	VARCHAR(250) NOT NULL,"
    +"SERVICEID VARCHAR(50) NOT NULL,"
    +"TIME_STAMP BIGINT,"
    +"CHECKSUM	VARCHAR(25) NOT NULL,"
    +"MODIFICATION_TYPE	SMALLINT,"
    +"CREATION_DATE BIGINT,"
	  +"USERLINK	VARCHAR(25) NOT NULL,"
	  +" CONSTRAINT uid PRIMARY KEY (UID, SERVICEID)"
    +") ";

  // delete all atomic object that are deleted and with a TS inferior to the specified date.
  private static String purgeDetetedAtomicObject =
    "DELETE FROM SYNC_UID WHERE SERVICEID = '%0%' AND TIME_STAMP < %1% AND MODIFICATION_TYPE=3";


  /********  SELECT QUERIES ********************/

  // query pattern used to get all Atomic Object for a service id
  private static String queryGetAtomicObject  =
    "SELECT UID, CHECKSUM, TIME_STAMP, MODIFICATION_TYPE, CREATION_DATE FROM SYNC_UID WHERE UID='%0%' AND SERVICEID = '%1%'";

  // query pattern used to get all modified Atomic Object for a service id since date
	private static String queryGetAllModifiedAtomicObjectForServiceSince  =
		"SELECT UID, MODIFICATION_TYPE, CREATION_DATE FROM SYNC_UID WHERE SERVICEID = '%0%' AND TIME_STAMP > %1% AND USERLINK='%2%'";

  
   public void createAtomicObject(String[] parameters) throws DatabaseException {
    this.executeUpdate(queryCreate, parameters);
  }

 /*
 * check if the sync table exists. IF it does not exist, create it
 */
 public void createTable() throws DatabaseException, java.sql.SQLException {
     try  {
      Connection con = this.getDbManager().getConnection();
      DatabaseMetaData dbmd = con.getMetaData();
      String[] tableType = {"TABLE"};
      ResultSet result = dbmd.getTables (null,null,"sync_uid", tableType);
      if (!result.next()) {
        this.executeUpdate(queryCreateTable, null);
        this.executeUpdate(createIndex, null);
        this.executeUpdate(uidIndex, null);
        this.executeUpdate(modifIndex, null);
        //init default AO
        String[] data = {"0", "0", "0","0", "0", "0", ""};
        this.createAtomicObject(data);
      }
    } finally {
			this.getDbManager().garbageOpenedConnection();
    }
  }

 public void updateAtomicObject(String[] parameters) throws DatabaseException {
    this.executeUpdate(queryUpdate, parameters);
  }

	public void purgeDeletedAtomicObject(String[] parameters) throws DatabaseException {
		this.executeUpdate(purgeDetetedAtomicObject, parameters);
	}

  /* getAtomicObject() get the atomic object for specified UID
  * @param     : array containing atomic object uid
  * @return    ResultSet : rows containing UID, CHECKSUM, TIME_STAMP, MODIFICATION_TYPE, CREATION_DATE
  * @exception ServiceException
  */
  public ResultSet getAtomicObject(String parameters[]) throws DatabaseException {
    return (this.executeQuery(queryGetAtomicObject, parameters));
  }

  /* getAtomicObjectsForService() return all atomic object uid modified since specified date for service
  * @param     : array containing service id and date
  * @return    ResultSet : rows containing UID
  * @exception ServiceException
  */
  public ResultSet getAllModifiedAtomicObjectForServiceSince (String parameters[]) throws DatabaseException {
    return (this.executeQuery(queryGetAllModifiedAtomicObjectForServiceSince, parameters));
  }

}
