/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.algo.syncnumber;

import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.synchro.algo.syncnumber.impl.DefaultSyncNumberManagerDelegate;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class SyncNumberManager {
  private static SyncNumberManager manager;
  private SyncNumberManagerDelegate delegate;

  private SyncNumberManager() {
  }

  public static SyncNumberManager getManager() {
     if (manager == null)  {
        synchronized(SyncNumberManager.class) {
        if (manager == null)  {
          manager = new SyncNumberManager();
        }
      }
    }
    return manager;
  }
  
  public void registerDelegate(SyncNumberManagerDelegate delegate) {
  	this.delegate = delegate;
  	this.delegate.initDelegate();
  }
  
  public SyncNumberManagerDelegate getDelegate() {
  	if (this.delegate == null) {
  		this.delegate = new DefaultSyncNumberManagerDelegate();
  	}
  	return this.delegate;
  }

  public SynchroNumber getSynchroNumber(long ns) throws SyncNumberNotFoundException {
  	return this.getDelegate().getSynchroNumber(ns);
  }

  public SynchroNumber getNextSynchroNumber() throws DatabaseException {
    return this.getDelegate().getNextSynchroNumber();
  }

  /**
   * return current current synchro number. 
   * @return
   * @throws SyncNumberNotFoundException
   */
  public long getCurrentSynchroNumber()  throws DatabaseException {
    return this.getDelegate().getCurrentSynchroNumber();
  } 

  public void deleteSyncNumber(SynchroNumber ns) throws DatabaseException {
  	this.getDelegate().deleteSyncNumber(ns);
  }

  public void createSynchroNumber(long ns, long timestamp)  throws DatabaseException  {
  	this.getDelegate().createSynchroNumber(ns, timestamp);
  }
  
  /**
   * set the new timestamp for the specified syncnumber. If it does not exist it is created.
   */
  public void setTimeStampForSynchroNumber(long syncNumber)	 throws SyncNumberNotFoundException {
  	this.getDelegate().setTimeStampForSynchroNumber(syncNumber);
  }
  
  /**
   * return the last timestamp of the specified sync number. Throws SyncNumberNotFoundException if not found.
   */
  public long getTimeStampForSynchroNumber(long syncNumber)  throws SyncNumberNotFoundException {
  	return this.getDelegate().getTimeStampForSynchroNumber(syncNumber);
  }

}