/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.algo.syncnumber.impl;

import java.sql.*;

import org.openmobileis.common.util.database.AbstractQueryManager;
import org.openmobileis.common.util.database.JdbcPoolManagerDB;
import org.openmobileis.common.util.exception.DatabaseException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: This class manages the synchronization number
 * For each user, a number is associated with the synchronization
 * to manage the incremental sync. 
 * Data mapping is made for InstantDB database
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class MSSQLSyncNumberQueryManager extends AbstractQueryManager  implements SyncNumberQueryManager {

  // query pattern used to insert a new version number
  private static String queryUpdate  =
    "UPDATE SYNC_ID SET SYNCNUMBER=%0% , TIME_STAMP=%1% " +
    "WHERE SYNCNUMBER=%2%";

   private static String queryCreate  =
     "INSERT INTO SYNC_ID (SYNCNUMBER , TIME_STAMP) " +
      "VALUES (%0%, %1%)";

  // query pattern used to create syncNB table
  private static String queryCreateTable  =
		"CREATE TABLE SYNC_ID ("
		+" SYNCNUMBER BIGINT NOT NULL,"
		+" TIME_STAMP BIGINT NOT NULL,"
		+" CONSTRAINT syncnum PRIMARY KEY (SYNCNUMBER)"
		+")";

 /* private static String existsTable  =
   "select count(*) from SYSTEM_COLUMNS where TABLE_NAME like 'SYNC_ID'";*/

  // query pattern used to delete a user and its version number
  private static String queryDelete =
    "DELETE FROM SYNC_ID WHERE SYNCNUMBER = %0%";


  /********  SELECT QUERIES ********************/

  // query pattern used to get the current version number
  private static String queryGetTimeStamp  =
    "SELECT TIME_STAMP  FROM SYNC_ID WHERE SYNCNUMBER = %0% ";

  private static String queryGetMax  =
    "SELECT Max(SYNCNUMBER)  FROM SYNC_ID";
    
	public MSSQLSyncNumberQueryManager(){}
    
	public MSSQLSyncNumberQueryManager(JdbcPoolManagerDB pool){
		this.registerManagerDB(pool);
	}
  
 /**
  * create() insert a new version number for a user ID. Create the user Id if it
  * does not exist.
  * @param     Array containing a String parameters representing the variable
  * values giving the USERID, VERSION_NB
  * Refer to the database schema for more details
  * @return    None
  * @exception None
*/
  public void create (String parameters[]) throws DatabaseException {
    this.executeUpdate(queryCreate, parameters);
  }

  /*
  * check if the sync table exists. IF it does not exist, create it
  */
  public void createSyncTable() throws DatabaseException, java.sql.SQLException {
    try {
      Connection con = this.getDbManager().getConnection();
      DatabaseMetaData dbmd = con.getMetaData();
      String[] tableType = {"TABLE"};
      ResultSet result = dbmd.getTables (null,null,"sync_id", tableType);
      if (!result.next()) {
        this.executeUpdate(queryCreateTable, null);
        String[] data = {"0", "0"};
        this.create(data);
      }
    } finally {
			this.getDbManager().garbageOpenedConnection();
    }
  }
 /**
  * createTable() create SyncNB table
  * @return    None
  * @exception None
*/
 // public void createTable() throws ServiceException {
 //   this.executeUpdate(queryCreateTable, new String[0]);
 // }

/**
  * delete() delete a synchronisation from the USERID
  * @param     Array containing a String parameters : USERID
  * @return    None
  * @exception None
*/
 public void delete (String parameters[]) throws DatabaseException {
    this.executeUpdate(queryDelete, parameters);
 }

/**
  * update() update a synchronisation : Not used here (same as create)
  * @param     Array containing a String parameters
  * @return    None
  * @exception None
*/
 public void update (String parameters[]) throws DatabaseException {
    this.executeUpdate(queryUpdate, parameters);
 }


/**
  * getLastSyncNumberbysn() get the sync number with specified ns
  * @param     : array containing sync number
  * @return    ResultSet : A unique row containing the sunc num  (int)
  * @exception None
*/
 public ResultSet getTimeStampForsn (String parameters[]) throws DatabaseException {
    return (this.executeQuery(queryGetTimeStamp, parameters));
  }

  /**
    * getLastSyncNumber() get the last sync number
    * @return    ResultSet : A unique row containing the sunc num  (int)
    * @exception None
  */
   public ResultSet getLastSyncNumber () throws DatabaseException {
      return (this.executeQuery(queryGetMax, null));
  }

  /**
    * getLastSyncNumberbysn() get the sync number with specified ns
    * @param     : array containing sync number
    * @return    ResultSet : A unique row containing the sunc num  (int)
    * @exception None
  */
  public ResultSet getSyncNumberbysn (String parameters[]) throws DatabaseException  {
    return this.getTimeStampForsn(parameters);
  }

}


