/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.algo.syncnumber.impl;

import java.sql.ResultSet;
import org.openmobileis.common.util.log.*;
import org.openmobileis.common.util.database.ManagerDB;
import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberManagerDelegate;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberNotFoundException;
import org.openmobileis.synchro.algo.syncnumber.SynchroNumber;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class OracleSyncNumberManagerDelegate implements SyncNumberManagerDelegate {
  private long currentSyncNumber;
  protected static OracleSafeServerSyncNumberQueryManager syncQueryManager = null;

  public OracleSyncNumberManagerDelegate() {
  }
  
	public void initDelegate()	{
		syncQueryManager = new OracleSafeServerSyncNumberQueryManager();
		try {
			syncQueryManager.createSyncTable();
      
			// get last sync number
			ResultSet result = null;
			result = syncQueryManager.getLastSyncNumber();
			if (result.next())  {
				currentSyncNumber =  result.getLong(1);
			} else  {
				LogManager.traceAlert(LogServices.SERVERSERVICE," No current synchro number set to 0");
				currentSyncNumber = 0;
			}
		}  catch (java.sql.SQLException e) {
			LogManager.traceCritique(LogServices.SERVERSERVICE, new ServiceException("Error init Syncho Number manager. Exit System",e));
			System.exit(1);
		}  catch (DatabaseException e) {
			LogManager.traceCritique(LogServices.SERVERSERVICE, new ServiceException("Error init Syncho Number manager. Exit System",e));
			System.exit(1);
		}
	}

  
  public synchronized void setSyncNumberFactory(SyncNumberQueryManager fac)  {
    syncQueryManager = (OracleSafeServerSyncNumberQueryManager)fac;
  }

  public SynchroNumber getSynchroNumber(long ns) throws SyncNumberNotFoundException {
    String[] syncParams = {String.valueOf(ns)};
    try {
      ResultSet result = syncQueryManager.getSyncNumberbysn(syncParams);
      if (!result.next()) {
        throw new SyncNumberNotFoundException(LogPriorities.INFO);
      } else {
//        return new SynchroNumber(ns, result.getDate(1).getTime());
				return new SynchroNumber(ns, result.getTimestamp(1).getTime());
      }
    }  catch (java.sql.SQLException e) {
      LogManager.traceError(LogServices.SERVERSERVICE, new ServiceException("Error during getSynchroNumberBySN for synchro number :"+ns,e));
      throw new SyncNumberNotFoundException(LogPriorities.INFO);
    }  catch (DatabaseException e) {
      LogManager.traceError(LogServices.SERVERSERVICE, e);
      throw new SyncNumberNotFoundException(LogPriorities.INFO);
    } finally {
      ManagerDB.getManager().garbageOpenedConnection();;
    }
  }

  public long getCurrentSynchroNumber() throws DatabaseException {
    try {
      // update sync number
      ResultSet result = syncQueryManager.getLastSyncNumber();
      if (result.next())  {
       return result.getLong(1);
      }
    } catch (Throwable ex)  {
      throw new DatabaseException(ex);
    } finally {
      ManagerDB.getManager().garbageOpenedConnection();;
    }
    throw new DatabaseException("No sync number in db. Db init error");
  }

  public SynchroNumber getNextSynchroNumber() throws DatabaseException {
		long ns = this.incrementSynchoNumber();
		String[] syncParams = {String.valueOf(ns)};
    try {
	    // update sync number
	    syncQueryManager.create(syncParams);
	    ResultSet result = syncQueryManager.getSyncNumberbysn(syncParams);
			SynchroNumber syncNumber = null;
	    if (result.next()) {
//				syncNumber = new SynchroNumber(ns, result.getDate(1).getTime());
				syncNumber = new SynchroNumber(ns, result.getTimestamp(1).getTime());
	    }
	    return syncNumber;
		}  catch (java.sql.SQLException e) {
			LogManager.traceError(LogServices.SERVERSERVICE, new ServiceException("Error during getNextSynchroNumber for synchro number :"+ns,e));
			throw new DatabaseException(e);
    } catch (DatabaseException ex)  {
      throw new DatabaseException(ex);
    }
  }

  public void deleteSyncNumber(SynchroNumber ns) throws DatabaseException {
    try {
	    String[] syncParams = {String.valueOf(ns.getSynchroNumber())};
	    syncQueryManager.delete(syncParams);
    } catch (DatabaseException ex)  {
      LogManager.trace(ex);
    }
  }

  public void createSynchroNumber(long ns, long timestamp)  throws DatabaseException  {
    String[] syncParams = {Long.toString(ns)};
    syncQueryManager.create(syncParams);
  }
  
  /**
   * set the new timestamp for the specified syncnumber. If it does not exist it is created.
   */
  public void setTimeStampForSynchroNumber(long syncNumber)	throws SyncNumberNotFoundException {
  }
  
  /**
   * return the last timestamp of the specified sync number. Throws SyncNumberNotFoundException if not found.
   */
  public long getTimeStampForSynchroNumber(long syncNumber) throws SyncNumberNotFoundException {
    return this.getSynchroNumber(syncNumber).getTimeStamp();
  }

  private synchronized long incrementSynchoNumber() {
    return ++currentSyncNumber;
  }
}