/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.algo.syncnumber.impl;

import java.sql.ResultSet;

import org.openmobileis.common.util.exception.DatabaseException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  
 * Define the operation use to manage synchro Number in the database
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public interface SyncNumberQueryManager {
 /**
  * check if the sync table exists. IF it does not exist, create it
  */
  public void createSyncTable() throws DatabaseException, java.sql.SQLException;
 /**
  * create() insert a new version number for a user ID. Create the user Id if it
  * does not exist.
  * @param     Array containing a String parameters representing the variable
  * values giving the USERID, VERSION_NB
  * Refer to the database schema for more details
  * @return    None
  * @exception None
*/
  public void create (String parameters[]) throws DatabaseException;

 /**
  * createTable() create SyncNB table
  * @return    None
  * @exception None
*/
//  public void createTable() throws ServiceException;

/**
  * delete() delete a synchronisation from the USERID
  * @param     Array containing a String parameters : USERID
  * @return    None
  * @exception None
*/
 public void delete (String parameters[]) throws DatabaseException;

/**
  * update() update a synchronisation : Not used here (same as create)
  * @param     Array containing a String parameters
  * @return    None
  * @exception None
*/
 public void update (String parameters[]) throws DatabaseException;


/**
  * getLastSyncNumber() get the last sync number
  * @return    ResultSet : A unique row containing the sunc num  (int)
  * @exception None
*/
 public ResultSet getLastSyncNumber () throws DatabaseException;

/**
  * getLastSyncNumberbysn() get the sync number with specified ns
  * @param     : array containing sync number
  * @return    ResultSet : A unique row containing the sunc num  (int)
  * @exception None
*/
 public ResultSet getSyncNumberbysn (String parameters[]) throws DatabaseException;
}

