/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.synchro.client;

import java.util.Enumeration;
import java.util.Hashtable;

import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:Manage all registered SynchroManager.  Use to start and manage a synchro.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class SynchroManager {

  private static SynchroManager manager;
  private Hashtable procTable;

  /**
   * 
   */
  private SynchroManager() {
    super();
    procTable = new Hashtable(10);
  }

  public static SynchroManager getManager() {
    if (manager == null)  {
      synchronized (SynchroManager.class) {
        if (manager == null)  {
          manager = new SynchroManager();
          ApplicationContextManager.getManager().addManager(manager);
        }
      }
    }
    return manager;
  }
 
  public void registerSynchroProcessor(SynchroProcessor proc)  {
    procTable.put(proc.getProcessorName(), proc);
  }
  
  public SynchroProcessor getSynchroProcessor(String processorName)  {
    return (SynchroProcessor) procTable.get(processorName);
  }
  
  public void doSynchro(Credential cred, SynchroDescriptor descriptor)  throws SynchroException {
    //open ras
    descriptor.synchroConduit.openRAS();
    try {
      boolean error = false;
      Enumeration proclist = procTable.elements();
      while (proclist.hasMoreElements())  {
        try {
          SynchroProcessor proc = (SynchroProcessor) proclist.nextElement();
          proc.doSynchro(cred, descriptor);
        } catch (Throwable ex)  {
          LogManager.traceError(0, ex);
          error = true;
        }
      }
      if (error)  {
        throw new SynchroException("Error during SynchroManager doSynchro processing");
      }
    } finally {
      descriptor.synchroConduit.closeRAS();
    }
  }

}
