/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.journal;

import org.openmobileis.common.util.OpenMISSerializable;



/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  An entry for the journal. Each synchro listener result is show by the 
 * Journal Service.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 * @see DefaultLogRenderer, JournalService
 */
 public class JournalEntry implements OpenMISSerializable {
   static final long serialVersionUID = 5521257935120563452L;

  // contants for entryID :
    // general error on server for a sync PDA command(SERVER -> PDA)
    public static short ERROR_SYNC = 0;
    // error on PDA (add, replace or delete action)
    public static short ERROR_OP_MOBILE = 1;
    // map error
    public static short ERROR_MAP = 2;
    // general error for server sync command (PDA -> SERVER)
    public static short ERROR_SYNC_SERVER = 3;
    // error Add operation
    public static short ERROR_ADD = 4;
    // error Replace operation
    public static short ERROR_REPLACE = 5;
    // error Delete operation
    public static short ERROR_DELETE = 6;
    // error during sync command request on PDA side
    public static short ERROR_SYNC_MOBILE = 7;

    private String serviceName;
    private short entryID;
    private String entryData;
    private long date;
    private int status;

 public JournalEntry() { }

  public JournalEntry(String service, short id, String data, int status, long date) {
    serviceName = service;
    entryID = id;
    entryData = data;
    this.date = date;
    this.status = status;
  }

	public String getServiceName()  {
		return serviceName;
	}
  
  /**
   * TO avoid collection name and parameter conflict in fodbsearch change servicename that is indexed
   */
/*  public static String convertServiceNameToCollectionEntry (String name)	{
  	return "FWKJ"+name;
  }*/

  public void setDate(long date)  {
    this.date = date;
  }

  public void setServiceName(String service)  {
    serviceName = service;
  }

  public short getServiceEntryID()  {
    return entryID;
  }

  public void setServiceEntryID(short id)  {
    entryID = id;
  }

  public String getEntryData()  {
    return entryData;
  }

  public void setEntryData(String data)  {
    entryData = data;
  }

  public void setStatus (int status) {
    this.status = status;
  }

  public int getStatus() {
    return status;
  }

  public long getDate() {
    return date;
  }

  public String getFormatedDate() {
    java.text.SimpleDateFormat formater = new java.text.SimpleDateFormat("'le' dd/MM/yy '' HH'h'mm'.'", java.util.Locale.FRANCE);
    return formater.format(new java.util.Date(date));
  }

  public int hashCode() {
    return (serviceName+"%"+Short.toString(entryID)).hashCode();
  }

  public boolean equals(Object obj) {
    if (obj instanceof JournalEntry)  {
      return ((((JournalEntry)obj).getServiceName().equals(this.getServiceName()))
             && (((JournalEntry)obj).entryID == this.getServiceEntryID()));
    }
    return false;
  }
  
  public String getEntryID()	{
  	return this.serviceName+Short.toString(this.entryID);
  }

}