/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  Creation P.Delrieu
 * 
 */
package org.openmobileis.synchro.openmsp.client;

import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.Message;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.security.auth.Credential;

/**
 * @author phil
 *
 */
public abstract class EmptyOpenMSPSyncListener implements OpenMSPSyncListener {

	/**
	 * 
	 */
	public EmptyOpenMSPSyncListener() {
	}
 
	public void initListener() throws OpenMSPException	{
  }

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#endSync()
	 */
	public void endSync() throws OpenMSPException {
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#getSynchroStatus()
	 */
	public int getSynchroStatus() {
		return OpenMSPSyncListener.SYNCHRO_OK;
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#isSyncOK()
	 */
	public boolean isSyncOK() {
		return true;
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#notifySynchroFailure()
	 */
	public void notifySynchroFailure() {
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveMapCommand(org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
	 */
	public void receiveMapCommand(ContainerMessage mapContainer) throws OpenMSPException {
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveResultCommand(org.openmobileis.synchro.openmsp.protocol.ContainerMessage, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
	 */
	public void receiveResultCommand(ContainerMessage resultContainer, ContainerMessage initialCommand) throws OpenMSPException {
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveStatusCommand(org.openmobileis.synchro.openmsp.protocol.Status, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
	 */
	public void receiveStatusCommand(Status statusCommande, ContainerMessage initialCommand) throws OpenMSPException {
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveSyncCommand(org.openmobileis.synchro.openmsp.protocol.ContainerMessage, long)
	 */
	public void receiveSyncCommand(ContainerMessage syncContainer, long newSyncNumber) throws OpenMSPException {
	}

	/**
	 * Call when the server answer container contains a GET command.
	 * Extends this method to process GET command.
	 * @param ContainerMessage container : container containing the GET message
	 * @param long newSyncNumber : the synchro number use for this synchro for this listener message.
	* @param Credential cred : the credential is any added in the GET command.
	 */
	public void receiveGetCommand( Credential cred, ContainerMessage replaceContainer, long newSyncNumber)  throws OpenMSPException	{
		
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#sendData(org.openmobileis.synchro.openmsp.protocol.Message)
	 */
	public void sendData(Message message) throws OpenMSPException {
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#startSync(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.client.SynchroDescriptor)
	 */
	public void startSync(Credential cred, SynchroDescriptor synchrodescriptor) throws OpenMSPException {
	}

}
