/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.conduit;

import java.io.*;

import HTTPClient.*;

import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.synchro.openmsp.server.OpenMSPService;
import org.openmobileis.test.mock.embedded.MockHttpServletRequest;
import org.openmobileis.test.mock.embedded.MockHttpServletResponse;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Synchronisation conduit use for debug purpose. Only generate 
 * the synchronisation request in a file sendOpenML.xml in the synchro directory.
 * Use tu see if the send synchro message is correctly generated.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class LocalOpenMSPSynchroConduit implements SynchroConduit {

  private String userAgent;
  private String intallPath;
  
  public LocalOpenMSPSynchroConduit(String synchroDirectoryPath, String userAgent)  {
    this.userAgent = userAgent; // "OpenMSPPlug-WI-V1.0-WINCE[DR-FR-WINCE/04-02-00-00]"
    intallPath = synchroDirectoryPath;
  }

  public boolean isConnected() {
    return true;
  }

  public String sendRequest(ConduitParameter[]parameters, String openMSPMessage, String url) throws SynchroException, UserNotFoundException{
    intallPath = FileUtilities.convertFileNameToSystem(intallPath);
    try {
      MockHttpServletRequest req = new MockHttpServletRequest();
      MockHttpServletResponse res = new MockHttpServletResponse();
      
      req.addParameter("openMISData", openMSPMessage);
      req.addParameter("openMISGroup", parameters[0].value);
      req.headerNames.add("user-agent");
      req.headerValues.add(this.userAgent);
	 
      NVPair form_data[] = new NVPair[2];
      form_data[0] = new NVPair("openMISData", openMSPMessage);
      form_data[1] = new NVPair("openMISGroup", parameters[0].value);
      
      OpenMSPService syncservice = new OpenMSPService();
      syncservice.run(req, res);

      if (res.status >= 300) {
       if ((res.status == 403) || (res.status == 401)) {
          throw  new UserNotFoundException("Error during HTTP send request conduit. Server answer :"+res.status);
        } else  {
          throw  new SynchroException("Error during HTTP send request conduit. Server answer :"+res.status);
        }
      } else {
        byte[] data = res.outStream.toByteArray();
         ByteArrayInputStream inst  = new ByteArrayInputStream(data);
        try {
          org.openmobileis.common.util.file.ZipUtilities.unZipData(inst, intallPath);
        } finally {
          inst.close();
        }

        String openMLFileName = intallPath+File.separator+"cyberML.xml";
        StringBuffer buffer = new StringBuffer();
        java.io.FileInputStream in = new java.io.FileInputStream(openMLFileName);
        try {
          byte[] bytes = new byte[512];
            int len;
            while ( ( len = in.read( bytes ) ) != -1 ) {
                buffer.append(new String(bytes, 0, len));
            }
        } finally {
          in.close();
        }
        return buffer.toString();
      }
    } catch (UserNotFoundException ioe) {
      throw ioe;
    } catch (Throwable ioe) {
      throw  new SynchroException(ioe);
    }
  }

  /*
  * Open RAS connection
  * @param : isSSLRequired : boolean indicates wether or not is a SSL connection
  * @return: boolean (true : connection OK, false : connection failed
  */
  public void openRAS() throws SynchroException { //do nothing
   }

  /*
  * Close RAS connection
  * @return boolean (status OK, not OK)
  */
  public void closeRAS() throws SynchroException { // do nothing
   }

  public String getRASConnectionNames() {
    return "connection1;connection2";
  }

  public String getIntallPath() {
    return intallPath;
  }
}
