/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.core;

import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.log.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Base manager to get and save synchro number.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class NumSyncManagerDB {
  private static NumSyncManagerDB manager;
  private NumSyncQueryDB queryManager;

  private NumSyncManagerDB() {
    queryManager = new NumSyncQueryDB();
  }

  public static NumSyncManagerDB getManager() {
    if (manager == null)  {
      synchronized (NumSyncManagerDB.class) {
        if (manager == null)  {
          manager = new NumSyncManagerDB();
          ApplicationContextManager.getManager().addManager(manager);
        }
      }
    }
    return manager;
  }

  public void saveSyncNumberForService(long sn, String serviceName) {
    queryManager.saveSyncNumber(serviceName, String.valueOf(sn));
  }

  /**
    return the last sync number for the specified service.
    if no sn is found return 0.
  */
  public long getSyncNumberForService(String serviceName) {
    String nsString = queryManager.getServiceSN(serviceName);
    if (nsString != null) {
      try {
        return Long.parseLong(nsString);
      } catch (NumberFormatException e) {
        LogManager.traceError(LogServices.WEBSERVICE, "Incorrect ns : " + nsString);
        return 0;
      }
    } else {
      return 0;
    }
  }

  public void deleteSyncNumberForService(String serviceName) {
    queryManager.delete(serviceName);
  }
  
  public void initAllSyncNumber()	{
  	queryManager.initAllSyncNumber();
  }
}