/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.db;

import org.openmobileis.common.util.log.*;
import org.openmobileis.common.util.exception.SynchroException;
//import org.openmobileis.embedded.syncho.db.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Default implementation of the listener use to decode synchro ROw from the synchronisation,process
 * If you change the synchronisation process the the way dtabase update rows are encoded you have to modify this listener. 
 * from the synchro process. Abstrct helper methods are provided to get the database.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class ImportDecodedRowListener implements DecodedRowListener {

  private ImportQueryManager importQM;

  public ImportDecodedRowListener(ImportQueryManager qm) {
    importQM = qm;
  }

  public void notifyNewRowDecoded(char action, String[] rowData) {
    try {
      //make query
      if (action == DBImportFileCoder.ADDAction)  {
        importQM.executeCreateQuery(rowData);
      } else if (action == DBImportFileCoder.REPLACEAction)  {
        importQM.executeUpdateQuery(rowData);
      } else if (action == DBImportFileCoder.DELETEAction)  {
        // get PDA uid for delete. TBD
        importQM.executeDeleteQuery(rowData);
      }
    } catch (Exception ex)  {
      LogManager.trace(new SynchroException("ImportDecodedRowListener Error in notifyNewRowDecoded", ex) );
    }

  }

  public void notifyError(int ligne, char errorCharacter) {
    LogManager.traceError(LogServices.SYNCHROSERVICE, "Error during decoding import Data file at line"+ligne+" for char '"+errorCharacter+"'" );
  }
}