/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.db;

import org.openmobileis.common.util.exception.DatabaseException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Base interface that all query manager implements.
 * A query manager is use to interface a database with a synchronisation process
 * Server update are send to the mobile in a row format. Rows are decoded with a DecodedROwListener
 * and database are updated with a ImportQueryManager. A default implemantation is provided
 * to interfce with the base synchronisation process.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 * @see DefaultDBImportQueryManager
 */
public interface ImportQueryManager {

  /**
   * execute the creation query
   */
  public void executeCreateQuery(String[] data) throws DatabaseException;
  /**
   *  execute the update query
   */
  public void executeUpdateQuery(String[] data) throws DatabaseException;

  /**
   *  execute the delete query
   */
  public void executeDeleteQuery(String[] data) throws DatabaseException;

  /**
   *  import all data in the specified file
   */
  public void importDataFile(String fileName);

}