/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.db;

import org.openmobileis.common.util.exception.DatabaseException;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.common.util.log.LogManager;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Base implemtnation of ImportQueryManager use to import object to the database.
 * All snchronisation rows are serialized object.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public abstract class ObjectDBImportQueryManager implements ImportQueryManager {

  /** Creates new ObjectDBImportQueryManager */
  public ObjectDBImportQueryManager() {
  }

	public void importDataFile(String fileName)  {
		ImportDecodedRowListener listener = new ImportDecodedRowListener(this);
		String goodFileName = FileUtilities.convertFileNameToSystem(fileName);
		try {
			this.beginImport();
			DBImportFileCoder.getCoder().decodeFileData(goodFileName, listener);
		} catch (Throwable ex)  {
			LogManager.trace(new SynchroException("DefaultDBImportQueryManager importDataFile Error ", ex));
		} finally {
			try {
				this.endImport();
			} catch (Throwable ex)  {
				LogManager.trace(new SynchroException("DefaultDBImportQueryManager importDataFile Error ", ex));
			}
		}
	}


	protected String[] cleanData(String[] data) {

		for (int i=0; i<data.length; i++) {
			data[i] =  DBImportFileCoder.getCoder().removeAntiSlash(data[i]);
		}
		return data;
	}

   
  /**
   * execute the creation query
   */
  public void executeCreateQuery(String[] data) throws DatabaseException  {
    try {
      Object obj = DBImportFileCoder.getCoder().unserializeDBObject(data);
      this.addObject(data[0], obj);
    } catch (Throwable ex)  {
      throw new DatabaseException(ex);
    }
  }

  /**
   *  execute the update query
   */
  public void executeUpdateQuery(String[] data) throws DatabaseException  {
    try {
      Object obj = DBImportFileCoder.getCoder().unserializeDBObject(data);
      this.replaceObject(data[0], obj);
    } catch (Throwable ex)  {
      throw new DatabaseException(ex);
    }
  }

  /**
   *  execute the delete query
   */
  public void executeDeleteQuery(String[] data) throws DatabaseException  {
    this.deleteObject(data[0]);
  }

  public abstract void addObject(String uid, Object object) throws DatabaseException;
    
  public abstract void replaceObject(String uid, Object object) throws DatabaseException;

  public abstract void deleteObject(String uid) throws DatabaseException;

	protected abstract void beginImport() throws DatabaseException;
	protected abstract void endImport() throws DatabaseException;

}
