/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.client.db;

import org.openmobileis.common.util.log.*;
import org.openmobileis.common.util.collection.Array;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  DB Action manager that allow to get or store in a persistent way 
 * embedded database action.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class SyncActionDBManager{

  private static SyncActionDBManager manager;
  private static SyncActionDBQueryManager actiondbQueryManager;

  public static SyncActionDBManager getManager () {
    if (manager == null)  {
      synchronized (SyncActionDBManager.class) {
        if (manager == null)  {
          manager = new SyncActionDBManager();
        }
      }
    }
    return manager;
  }

  private SyncActionDBManager(){

    try {
      actiondbQueryManager.initDB();
    } catch (Exception e) {
      LogManager.traceError(LogServices.WEBSERVICE, e);
    }
  }
  
  public static void registerSyncActionDBQueryManager(SyncActionDBQueryManager actionManager)	{
		actiondbQueryManager = actionManager;
  }
  
  public ActionDB[] getActionsForService(String serviceName) {
    try {
      Array result = actiondbQueryManager.getActionsForService(serviceName);
      ActionDB[] actionRes = new ActionDB[result.size()];
      result.toArray(actionRes);
      return actionRes;
    } catch (Exception e) {
      LogManager.traceError(LogServices.WEBSERVICE, e);
    }
    return new ActionDB[0];
  }


  public ActionDB getActionDB(String serviceName, String uid) {
    try {
      return actiondbQueryManager.getActionDB(serviceName, uid);
    } catch (Exception e) {
      LogManager.traceError(LogServices.WEBSERVICE, e);
    }
    return null;
  }
  
  public void addActionDB(ActionDB action)	{
    try { 
      actiondbQueryManager.create(action);
    } catch (Exception e) {
        LogManager.traceError(LogServices.WEBSERVICE, e);
    }
  }


  /*
  * insert or update an add actionDB.
  */
  public void setAddActionDB (String serviceName, String uid) {

    try { 
      ActionDB action = new ActionDB(serviceName, uid, ActionDB.ADD_ACTION);
      actiondbQueryManager.create(action);
   } catch (Exception e) {
        LogManager.traceError(LogServices.WEBSERVICE, e);
    }
  }

  /*
  * insert or update an update actionDB.
  */
  public void setUpdateActionDB (String serviceName, String uid) {

    try { 
      ActionDB action = new ActionDB(serviceName, uid, ActionDB.UPDATE_ACTION);
      actiondbQueryManager.create(action);
    } catch (Exception e) {
        LogManager.traceError(LogServices.WEBSERVICE, e);
    }
  }

  /*
  * insert or update an delete actionDB.
  */
  public void setDeleteActionDB (String serviceName, String uid) {
    try { 
      ActionDB action = new ActionDB(serviceName, uid, ActionDB.DELETE_ACTION);
      actiondbQueryManager.create(action);
    } catch (Exception e) {
        LogManager.traceError(LogServices.WEBSERVICE, e);
    }
  }

  /*
  * delete the action associated with the service and uid.
  */
  public void deleteActionDB (String serviceName, String uid) {

    try { 
      actiondbQueryManager.delete(serviceName, uid);
    } catch (Exception e) {
        LogManager.traceError(LogServices.WEBSERVICE, e);
    }
  }

  /*
  * delete the action associated with the service and uid.
  */
  public void deleteAllActionDBForService (String serviceName) {

    try { 
      actiondbQueryManager.deleteAllActionForService(serviceName);
    } catch (Exception e) {
        LogManager.traceError(LogServices.WEBSERVICE, e);
    }
  }



}