/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.synchro.openmsp.client.services;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.context.Plateform;
import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.embedded.util.SystemAPI;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.common.ServletTools;
import org.openmobileis.services.navigation.NavigationBarService;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.client.SynchroManager;
import org.openmobileis.synchro.journal.JournalManager;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;
import org.openmobileis.synchro.openmsp.client.conduit.HttpOpenMSPSynchroConduit;
import org.openmobileis.synchro.security.auth.Credential;

import freemarker.template.SimpleScalar;
import freemarker.template.TemplateModelRoot;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class DirectSynchroService extends TemplateService implements NavigationBarService{
  private static final String templatePath="openmischannel"+java.io.File.separator+"synchroresult.htm";
  private boolean synchroPending = false;
  private SynchroDescriptor descriptor;

  class SynchroThread implements Runnable {
    String login, password;
    SynchroThread(String login, String password) {
      this.login = login;
      this.password = password;
    }

    public void run() {
      try {       
        this.doSynchro(login, password);
      } catch (Throwable e) {
        LogManager.traceError(LogServices.WEBSERVICE, "Error in synchro thread: "+e.getMessage());
        LogManager.traceError(0, e);
      } finally	{
        synchronized(this) {synchroPending = false;} //end before start IE
        Plateform plateform = ApplicationContextManager.getManager().getApplicationContext().getPlateform();
        if (plateform.getOS().equals(Plateform.POCKETPCOS)) {
  	      if (PropertiesManager.getManager().getProperty("org.openmobileis.embedded.execprog.browser")!=null)	{
  	        String indexurl = PropertiesManager.getManager().getProperty("org.openmobileis.embedded.execprog.browser.args");
  	        if(indexurl != null)	{
  	          int index = indexurl.indexOf("//");
  	          String crackurl = indexurl.substring(index+2, indexurl.length());
  	          int index2 = crackurl.indexOf('/');
  	          if (index2 == -1)	{
  	            index2 =crackurl.length()-1;
  	          }
  	          indexurl = indexurl.substring(0, index+2+index2);
   	        } else	{
  	          indexurl = "http://127.0.0.1:9090";
  	        }
  	        //miss base url not have to crack it.
  	        SystemAPI.execProgram(PropertiesManager.getManager().getProperty("org.openmobileis.embedded.execprog.browser"), indexurl+"/services/synchro/direct/"+Long.toString(System.currentTimeMillis())+"?showres=true");
  	      }
        }
      }
      
    }
    
    private void doSynchro(String login, String password) throws SynchroException {
      Credential cred = new Credential(login, password);   
      SynchroManager.getManager().doSynchro(cred, descriptor);
    }
  }
  /**
   * 
   */
  public DirectSynchroService() {
    super();
    try {
      String synchroInstallPath = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.installpath");
      if (synchroInstallPath == null)	{
	      String propFilePath = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.properties.file.path");
	      PropertiesManager.getManager().addPropertiesFileFromFilePath(propFilePath);
	      synchroInstallPath = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.installpath");
      }
      
      descriptor = new SynchroDescriptor();
      descriptor.addProperty("OpenMSPsynchrotype", "DR");
      String useragent = OpenMSPSynchroManager.getManager().getSynchroUserAgent(descriptor);
      descriptor.setSynchroConduit(new HttpOpenMSPSynchroConduit(synchroInstallPath ,useragent));
      
      String group = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.group");
      descriptor.setSynchroGroup(group);
      String url = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.targeturl");
      descriptor.setServerURL(url);
    } catch (Throwable ex)  {
      LogManager.traceError(0, "Error during init of DirectSynchroService. No direct synchro available");
      LogManager.traceError(0, ex);      
    }
    
  }

  /* (non-Javadoc)
   * @see org.openmobileis.services.common.TemplateService#runTemplate(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse, freemarker.template.TemplateModelRoot)
   */
  public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) throws ServletException, IOException {
    String login = req.getParameter("login");
    String pwd = req.getParameter("pwd");
    String showres = req.getParameter("showres");
    if (showres != null)  {
      LogManager.traceDebug(0, "DirectSyncService showres.");
      return this.showResultMessage(req, res, templateData);
    }
   // manage auhentication always ask user password
    if ((login == null) || (pwd == null)) {
     //return password error authentication required
      return null;
    }
    if (!synchroPending) {// should be synchronized but to synchro at the same time never arrive
      synchronized(this) {synchroPending = true;}
      // start synchro thread
      SynchroThread sync = new SynchroThread(login, pwd);
      Thread thread = new Thread(sync);
      Plateform plateform = ApplicationContextManager.getManager().getApplicationContext().getPlateform();
      // show wait page on POCKETPC if browser property is set.
      if ((plateform.getOS().equals(Plateform.POCKETPCOS)) && (PropertiesManager.getManager().getProperty("org.openmobileis.embedded.execprog.browser")!=null)) {
        thread.start();  
        return this.showWaitMessage(res, templateData);
     } else  {
         thread.run();     
        return this.showResultMessage(req, res, templateData);
     }                       
    } else  {
      return this.showWaitMessage(res, templateData);
    }
    
  }
  
  public String showWaitMessage(HttpServletResponse res, TemplateModelRoot templateData) {
    IntlResourceManager resourceManager = IntlResourceManager.getManager();
    String title = resourceManager.getLocalizedProperty("org.openmobileis.synchro.DirectSynchroService.SyncStarted");
    if (title == null) title = "Synchronizationn started.";
    String message = resourceManager.getLocalizedProperty("org.openmobileis.synchro.DirectSynchroService.Wait");
    if (message == null) message = "Please wait until the end.";
    ServletTools.sendConfirmationPage(title, message, "/index", res);
    return null;
  }
  
  public String showResultMessage(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData)  throws ServletException, IOException {
    int status = OpenMSPSynchroManager.getManager().getGlobalSynchroStatut();
    String syncMessage = JournalManager.getManager().getErrorMessageForSynchroStatus(status);
    templateData.put("syncstatus", new SimpleScalar(syncMessage));
    SynchroJournalDisplayService journalservice = new SynchroJournalDisplayService();
    journalservice.runTemplate(req, res, templateData);
    return DirectSynchroService.templatePath;
  }

  
  public String getNavigationBarLabel(HttpServletRequest req) {
    return "Synchronisation";
  }
  
  public boolean displayFormExitMessage() {
    return false;
  }
  
  public boolean displayRecursive() {
    return false;
  }

  public String getServiceUri() {
    return ServiceManager.getManager().getServiceBaseURI()+"/synchro/direct";
  }

}
