/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

/**
 * Title:        OpenMobileIS project source <BR>
* Description:  Superclass for OpenMSP commands :
 *  Get, Sync, Add, Delete, Replace, Map, Sequence, Result, Status
 *  An abstract command has a type (See Element for the command list) and a command id.
 *  See Message and MessageFactory to have example for building commands
 *  Direct subclass : RequestCommand, ResponseCommand, Sequence
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 * @see: Message
 * @see: MessageFactory
 * @see: RequestCommand
 * @see: ResponseCommand
 * @see: Sequence
 */

public abstract class AbstractCommand  implements Element {
    /**
     * Set version of the serialized data to make it compatible with
     * new class versions.
    */
  final static long serialVersionUID = 1L;

  // command type (the value is taken among the command types)
  protected int commandType;
  // command identifier
  protected int cmdId = 0;

  public  AbstractCommand (int commandType) {
    this.commandType = commandType;
  }

  public int getElementType() {
    return commandType;
  }

  public int getCmdId() {
    return cmdId;
    }

   public boolean hasCommmandId() {
    return true;
  }

 public String toString() {
  return "cmdId : " + String.valueOf(cmdId) + " type : " + commandType;
 }

 //************ implements Element inferface **********

   public boolean hasCommandId() {
    return true;
   }

   public void setCmdId (int cmdId) {
    this.cmdId = cmdId;
   }

   public void setElementType(int elementType) {
    this.commandType = elementType;
   }

   public void setCredentialData (String data) {
   // do nothing by default
   }

   public void setCredentialMeta (String meta){
   // do nothing by default
   }

   public void setSource(String source){
   // do nothing by default
   }

   public void setSourceSessionID(long sessionID){
   // do nothing by default
   }

   public void setTarget (String target){
   // do nothing by default
   }

   public void setMetaInformation (String meta){
   // do nothing by default
   }

   public void setCmdRef (int cmdRef){
   }

   public void setSourceRef(String source){
   // do nothing by default
   }

   public void setTargetRef(String target){
   // do nothing by default
   }

   public void setData (String data){
   // do nothing by default
   }


  public void writeBeginTag(StringBuffer buffer) {
     MessageFactory.writeBeginTagForElement(buffer, commandType);
  }

 public void writeContent(StringBuffer buffer) {
     if (cmdId != 0) {
      MessageFactory.writeElement(buffer, MessageFactory.TAG_CMDID, String.valueOf(cmdId));
      buffer.append("\n");
     }
 }

 public void writeEndTag(StringBuffer buffer) {
     MessageFactory.writeEndTagForElement(buffer, commandType);
     buffer.append("\n");
 }

}