/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.synchro.openmsp.OpenMSPException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Container of OpenMSP messages
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class ContainerMessage  implements java.io.Serializable {
  /**
   * Set version of the serialized data to make it compatible with
   * new class versions.
  */
  final static long serialVersionUID = 1L;


  // array of ContainerMessage
  protected Array children;

  protected int cursor = -1;
  protected Element content;

  protected ContainerMessage() {
    children = new Array(5);
  }

  public ContainerMessage(Element element) {
    content = element;
    children = new Array(5);
  }

  public void add (Element element)  throws OpenMSPException {
    children.add(new AtomicMessage(element));
  }

  public void add (ContainerMessage message)  throws OpenMSPException {
    children.add(message);
  }

  public Array getAllMessages() {
    return children;
  }

  /*
  * Get the next element pointed by the cursor
  * @return: CmlContent if it exists or null if is the end of the children list
  */
  public ContainerMessage nextMessage() {
    cursor++;
    if (!(cursor < children.size()))
      return null;
    else
      return (ContainerMessage)children.get(cursor);
  }

  /*
  * Return true if the cursor is not at the end of the children list
  * @return : boolean
  */
  public boolean hasMoreMessage() {
    return cursor + 1 < children.size();
  }

  public void resetCursor() {
    cursor = -1;
    for (int i=0; i<children.size(); i++)   {
      Object obj = children.get(i);
      if (obj != null && obj instanceof ContainerMessage)  {
        ContainerMessage cont = (ContainerMessage)obj;
        cont.resetCursor();
      }
    }
  }

  public int getMessagesSize() {
    return children.size();
  }

  public Element getElement() {
    return content;
  }
  protected void encode (StringBuffer buffer) {
    content.writeBeginTag(buffer);
    buffer.append("\n");
    content.writeContent(buffer);
    for (int i=0; i < children.size(); i++) {
      ((ContainerMessage)children.get(i)).encode(buffer);
    }
    content.writeEndTag(buffer);
  }

}