/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  OpenMSP header
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public class Header implements Element {
  /**
   * Set version of the serialized data to make it compatible with
   * new class versions.
  */
  final static long serialVersionUID = 1L;


  private static String version = "1.0";
  protected String target;
  protected String source;
  protected long sessionID;
  // not used in the current version
  protected int msgID = 1;
  // credential : String[0] : Meta information and String[1] : data authentification
  protected String[] credential;

  protected Header() {}

  public Header(long sessionID, String source, String target) {
    this.target = target;
    this.sessionID = sessionID;
    this.source = source;
  }

  public Header( long sessionID, String source, String target, String credentialMeta, String credentialData) {
      this(sessionID, source, target);
      credential = new String[2];
      credential[0] = credentialMeta;
      credential[1] = credentialData;
  }

  public boolean hasCommmandId() {
    return false;
  }

  public int getElementType() {
    return Element.HEADER;
  }

  public int getCmdId() {
    return 0;
  }


  public int getMsgId() {
    return msgID;
    }

  public long getSessionID() {
    return sessionID;
    }

  public String getSource() {
    return source;
    }

  public String getTarget() {
    return target;
    }

  public void setCredential (String meta, String data) {
    credential = new String[2];
    credential[0] = meta;
    credential[1] = data;
  }

  public String[] getCredential() {
    return credential;
   }

  public String getVersion() {
    return version;
    }

  public String toString() {
    return " header : sessionID " + sessionID + " source : " + source + " target : " + target;
  }

  //************ implements Element inferface **********

   public void setCmdId (int cmdId) {}

   public void setElementType(int elementType) {}

   public void setCredentialData (String data) {
     if (credential == null)
        credential = new String[2];
    credential[1] = data;
   }

   public void setCredentialMeta (String meta){
      if (credential == null)
          credential = new String[2];
      credential[0] = meta;
   }

   public void setSource(String source){
      this.source = source;
   }

   public void setTarget (String target){
      this.target = target;
   }

   public void setMetaInformation (String meta){}

   public void setCmdRef (int cmdRef){}

   public void setSourceRef(String source){}

   public void setSourceSessionID(long sessionid){
   // do nothing by default
   }

   public void setTargetRef(String target){}

   public void setData (String data){ }

  public void setSessionID(long ns) {
    sessionID = ns;
  }

  public void writeBeginTag(StringBuffer buffer) {
    MessageFactory.writeBeginTag(buffer, MessageFactory.TAG_HEADER);
    buffer.append("\n");
  }

   public void writeEndTag(StringBuffer buffer) {
     MessageFactory.writeEndTag(buffer, MessageFactory.TAG_HEADER);
     buffer.append("\n");
  }

 public void writeContent(StringBuffer buffer) {
    MessageFactory.writeElement(buffer, MessageFactory.TAG_VERSION, version);
    buffer.append("\n");
    MessageFactory.writeElement(buffer, MessageFactory.TAG_SESSION, String.valueOf(sessionID));
    buffer.append("\n");
    MessageFactory.writeElement(buffer, MessageFactory.TAG_MSGID, String.valueOf(msgID));
    buffer.append("\n");
    MessageFactory.writeSource(buffer, source, -1);
    buffer.append("\n");
    MessageFactory.writeTarget(buffer, target);
    buffer.append("\n");
    if (credential != null) {
        MessageFactory.writeCredential(buffer, credential[0], credential[1]);
        buffer.append("\n");
    }
 }

}