/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Item used for Request commands and Result command which have not to manage data. (For instance Map command)
 * If an item contains data, use DataItem
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 * @See: DataItem
 */

public class Item implements Element {
  static final long serialVersionUID = 5521257935120563452L;

  protected String target;
  protected String source;
  protected int itemType;
  protected long sessionID = -1;


  public Item(int itemType, String source, String target) {
    this.target = target;
    this.source = source;
    this.itemType = itemType;
  }


  protected Item (int itemType) {
     this.itemType = itemType;
  }

 public int getCmdId() {
    return 0;
  }

 public boolean hasCommmandId() {
    return false;
  }

  public String getSource() {
    return source;
    }

  /**
   * return the optional sessionID of source element. If not present return -1.
   *
   */
  public long getSourceSessionID()  {
    return sessionID;
  }

  public String getTarget() {
    return target;
    }

  public int getElementType() {
    return itemType;
  }

 public String toString() {
  return "type : " + itemType + " source : " + source + " target : " + target;
 }

//** CmlElements interface **//

 public void setTarget (String target) {
    this.target = target;
    }

  public void setSource (String source) {
    this.source = source;
    }

 public void setSourceSessionID(long sessionid) {
  this.sessionID = sessionid;
 }

   public void setElementType(int elementType) {
    this.itemType = elementType;
    }

   public void setCmdId (int cmdId) {}
   public void setCredentialData (String data) {}
   public void setCredentialMeta (String meta){}
   public void setMetaInformation (String meta){}
   public void setCmdRef (int cmdRef){}
   public void setSourceRef(String source){}
   public void setTargetRef(String target){}
   public void setData (String data){ }


  public void writeBeginTag(StringBuffer buffer) {
   MessageFactory.writeBeginTagForElement(buffer, itemType);
   buffer.append("\n");
  }


 public void writeContent(StringBuffer buffer) {
     if (source != null) {
        MessageFactory.writeSource(buffer, source, sessionID);
        buffer.append("\n");
     }
     if (target != null) {
        MessageFactory.writeTarget(buffer, target);
        buffer.append("\n");
     }
  }

 public void writeEndTag(StringBuffer buffer) {
     MessageFactory.writeEndTagForElement(buffer, itemType);
     buffer.append("\n");
 }

}