/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

import org.openmobileis.synchro.openmsp.OpenMSPException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Factory in charge of providing Message.
 * Either by creating new message : MessageFactory.getFactory().createMessage(aHeader)
 * or from XML String MessageFactory.getFactory().createMessage(aXMLString)
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public class MessageFactory extends Handler {


  public static String TAG_ROOT = "OpenMSP";
  public static String TAG_HEADER = "SyncHdr";
  public static String TAG_VERSION = "Version";
  public static String TAG_SESSION = "SessionID";
  public static String TAG_MSGID = "MsgID";
  public static String TAG_TARGET = "Target";
  public static String TAG_SOURCE = "Source";
  public static String TAG_URI = "LocURI";
  public static String TAG_BODY = "SyncBody";
  public static String TAG_SYNC = "Sync";
  public static String TAG_CMDID = "CmdID";
  public static String TAG_CRED = "Cred";
  public static String TAG_META = "Meta";
  public static String TAG_DATA = "Data";
  public static String TAG_ADD = "Add";
  public static String TAG_REPLACE = "Replace";
  public static String TAG_DELETE = "Delete";
  public static String TAG_MAP = "Map";
  public static String TAG_RESULT = "Result";
  public static String TAG_ITEM = "Item";
  public static String TAG_GET = "Get";
  public static String TAG_CMD_REF = "CmdRef";
  public static String TAG_SOURCE_REF = "SourceRef";
  public static String TAG_TARGET_REF = "TargetRef";
  public static String TAG_SEQUENCE = "Sequence";
  public static String TAG_STATUS = "Status";
  public static String TAG_MAP_ITEM = "MapItem";
  public static String TAG_FINAL = "Final";

  // link between elementType and Tags
  protected static String[] tags =
    {TAG_HEADER, TAG_SYNC, TAG_GET, TAG_ADD, TAG_REPLACE, TAG_DELETE, TAG_MAP, TAG_RESULT, TAG_STATUS, TAG_ITEM, TAG_MAP_ITEM, TAG_SEQUENCE, TAG_FINAL};

  private static MessageFactory factory;


  public static MessageFactory getFactory() {
     if (factory == null)  {
        synchronized(org.openmobileis.synchro.openmsp.protocol.MessageFactory.class) {
        if (factory == null)  {
          factory = new MessageFactory();
        }
      }
    }
    return factory;
  }


  public Message getMessage (String openML) throws OpenMSPException {
     DefaultHandler handler = new DefaultHandler();
     Parser parser = new Parser(handler);
     parser.parse(openML);
     return handler.getMessage();
 }

  public Message createMessage (Header header, boolean isFinal) {
    return new Message(header, isFinal);
  }

  public Message createMessage (Header header) {
    return new Message(header, true);
  }

 /** Static methods for writing XML tags **/
  protected static void writeElement (StringBuffer buffer, String tag, String value) {
    writeBeginTag(buffer, tag);
    buffer.append(value);
    writeEndTag(buffer, tag);
  }

  protected static void writeBeginTagForElement (StringBuffer buffer, int elementType) {
    writeBeginTag (buffer, tags[elementType]);
  }

  protected static void writeEndTagForElement (StringBuffer buffer, int elementType) {
    writeEndTag (buffer, tags[elementType]);
  }


  protected static void writeBeginTag (StringBuffer buffer, String tag) {
    buffer.append("<");
    buffer.append(tag);
    buffer.append(">");
    }

  protected static void writeSingleTag (StringBuffer buffer, String tag) {
    buffer.append("<");
    buffer.append(tag);
    buffer.append("/>");
  }

  protected static void writeEndTag (StringBuffer buffer, String tag) {
    buffer.append("</");
    buffer.append(tag);
    buffer.append(">");
    }

  protected static void writeCredential(StringBuffer buffer, String meta, String data) {
      writeBeginTag(buffer, MessageFactory.TAG_CRED);
      if (meta != null)
          writeElement(buffer, MessageFactory.TAG_META, meta);
      writeElement(buffer, MessageFactory.TAG_DATA, data);
      writeEndTag(buffer, MessageFactory.TAG_CRED);
    }


  protected static void writeTarget(StringBuffer buffer, String target) {
      writeBeginTag(buffer, MessageFactory.TAG_TARGET);
      writeElement(buffer, MessageFactory.TAG_URI, target);
      writeEndTag(buffer, MessageFactory.TAG_TARGET);
    }

  protected static void writeSource(StringBuffer buffer, String source, long sessionID) {
      writeBeginTag(buffer, MessageFactory.TAG_SOURCE);
      writeElement(buffer, MessageFactory.TAG_URI, source);
      if (sessionID != -1)  {
        writeElement(buffer, MessageFactory.TAG_SESSION, Long.toString(sessionID));
      }
      writeEndTag(buffer, MessageFactory.TAG_SOURCE);
    }
}