/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:   Request command class manages the following commands : Sync, Map
 * In fact, it manages all request commands which not use source and target attributes
 * Request commands manages optional meta information and a credential
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 * @See: Command
 */

public class RequestCommand extends AbstractCommand  implements Element {
  /**
   * Set version of the serialized data to make it compatible with
   * new class versions.
  */
  final static long serialVersionUID = 1L;


  // credential : String[0] : Meta information and String[1] : data authentification
  protected String[] credential;
  // meta information (blob)
  protected String metaInfo;

  public  RequestCommand (int commandType) {
    super(commandType);
  }

  public void setCredential (String meta, String data) {
    credential = new String[2];
    credential[0] = meta;
    credential[1] = data;
  }

  public void setCredentialMeta (String meta) {
    if (credential == null)
      credential = new String[2];
    credential[0] = meta;
  }

   public void setCredentialData (String data) {
    if (credential == null)
      credential = new String[2];
    credential[1] = data;
  }

  public String[] getCrendential() {
    return credential;
   }

  public String getMetaInformation() {
    return metaInfo;
    }

  public void setMetaInformation (String meta) {
    this.metaInfo = meta;
    }

 public String toString() {
  String temp =  super.toString() + " meta : " + metaInfo;
  if (credential != null)
    temp = temp + " cred : " + credential[0] + " " + credential[1];
  return temp;
 }


  /*** Implements Element interface **/
 public void writeContent(StringBuffer buffer) {
   super.writeContent(buffer);
    if (credential != null) {
      MessageFactory.writeCredential(buffer, credential[0], credential[1]);
      buffer.append("\n");
    }
     if (metaInfo != null) {
      MessageFactory.writeElement(buffer, MessageFactory.TAG_META, metaInfo);
      buffer.append("\n");
     }
 }


 }