/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Response command class manages the following commands : Result and Status
 * A response command have an attribute CmdRef (command reference) which refer to the request command identifier.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 * @See: Result
 * @See: Status
 */

public abstract class ResponseCommand extends AbstractCommand implements Element {
  /**
   * Set version of the serialized data to make it compatible with
   * new class versions.
  */
  final static long serialVersionUID = 1L;


  protected int cmdRef;

  protected ResponseCommand (int commandType) {
    super(commandType);
  }

  public ResponseCommand(int commandType, int cmdRef) {
    super(commandType);
    this.cmdRef = cmdRef;
  }

  public int getCmdRef() {
    return cmdRef;
    }

  public String toString() {
    return super.toString() + " cmdRef : " + cmdRef;
 }

   /*** Implements Element interface **/

 public void setCmdRef (int cmdRef){
    this.cmdRef = cmdRef;
   }

 public void writeContent(StringBuffer buffer) {
      super.writeContent(buffer);
      MessageFactory.writeElement(buffer,MessageFactory.TAG_CMD_REF , String.valueOf(cmdRef));
      buffer.append("\n");
 }


}