/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Set a status for request command
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public class Status extends ResponseCommand implements Element {
  /**
   * Set version of the serialized data to make it compatible with
   * new class versions.
  */
  final static long serialVersionUID = 1L;

  public final static int STATUS_SYNC_IN_PROGRESS=101;
  public final static int STATUS_OK = 200;
  public final static int STATUS_NO_CONTENT = 204;
  public final static int STATUS_PARTIAL_CONTENT = 206;
  public final static int STATUS_WRONG_FORMAT = 400;
  public final static int STATUS_UNAUTHORIZED = 401;
  public final static int STATUS_NOT_FOUND = 404;
  public final static int STATUS_AUTHENTIFICATION_REQUIRED = 407;
  public final static int STATUS_TOO_LARGE = 413;
  public final static int STATUS_UNSUPPORTED_MEDIA = 415;
  public final static int STATUS_EXIST_TARGET = 418;
  public final static int STATUS_DEVICE_FULL = 420;
  public final static int STATUS_FAILED = 500;
  public final static int STATUS_COMMAND_NOT_IMPLEMENTED = 501;
  public final static int STATUS_SYNCHRO_CONFLIC = 1001;


  protected int status;

  protected Status () {
     super(Element.STATUS);
  }

  public Status(int cmdRef, int status) {
    super(AbstractCommand.STATUS, cmdRef);
    this.status = status;
  }

  public int getStatus() {
    return status;
  }

 public String toString() {
  return super.toString() + "status : " + status ;
 }

   /*** Implements Element interface **/
 public boolean isAtomic() {
    return true;
   }

 public void setData (String data) {
  if ((data != null) && (data.length() != 0))
    this.status = Integer.parseInt(data);
 }

 public void writeContent(StringBuffer buffer) {
      super.writeContent(buffer);
      MessageFactory.writeElement(buffer, MessageFactory.TAG_DATA, String.valueOf(status));
      buffer.append("\n");
  }

}