/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.protocol.test;

import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: An example to show how to use OpenMSP protocol API.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public class Example extends Handler {

  String tabs = "";

  public Example() {
  }

  /* build the following sample message :
          <OpenMSP>
        <SyncHdr>
        <Version>1.0</Version>
        <SessionID>200</SessionID>
        <MsgID>1</MsgID>
        <Source><LocURI>header source</LocURI></Source>
        <Target><LocURI>target header</LocURI></Target>
        <Cred><Meta>cred meta</Meta><Data>cred data</Data></Cred>
        </SyncHdr>
        <SyncBody>
        <Sync>
        <CmdID>1</CmdID>
        <Source><LocURI>sync source</LocURI></Source>
        <Target><LocURI>sync target</LocURI></Target>
        <Add>
        <CmdID>2</CmdID>
        <Cred><Meta>credential add meta</Meta><Data>credential add data</Data></Cred>
        <Meta>add meta information</Meta>
        <Item>
        <Source><LocURI>source item 2</LocURI></Source>
        <Target><LocURI>target item 2</LocURI></Target>
        <Meta>meta item 2</Meta>
        <Data><![CDATA[<xml><truc>
        <titi>TOTO</titi>
        </truc></xml>]]></Data>
        </Item>
        <Item>
        <Source><LocURI>source item 1</LocURI></Source>
        <Target><LocURI>target item 1</LocURI></Target>
        <Meta>meta item 1</Meta>
        <Data>data item 1</Data>
        </Item>
        </Add>
        </Sync>
        </Final></SyncBody></OpenMSP>
        */
  public static Message buildMessage() throws OpenMSPException {
    Message message = MessageFactory.getFactory().createMessage(new Header(200, "header source", "target header", "cred meta", "cred data"));
    // create a sync command
    ContainerMessage syncMessage = new ContainerMessage(new Command(Element.SYNC, "sync source", "sync target"));
    // create add command for sync command
    ContainerMessage addMessage = new ContainerMessage(new RequestCommand(Element.ADD));
    ((RequestCommand)addMessage.getElement()).setCredential("credential add meta", "credential add data");
    ((RequestCommand)addMessage.getElement()).setMetaInformation("add meta information");
    // add two data items for add sub command
    addMessage.add(new DataItem(Element.ITEM,"meta item 2", "<xml><truc>\n<titi>TOTO</titi>\n</truc></xml>", "source item 2", "target item 2"));
    addMessage.add(new DataItem(Element.ITEM,"meta item 1", "data item 1", "source item 1", "target item 1"));
    syncMessage.add(addMessage);
    message.add(syncMessage);
    return message;
   }

  /*
  * Second Message
  *<OpenMSP>
          <SyncHdr>
          <Version>1.0</Version>
          <SessionID>200</SessionID>
          <MsgID>1</MsgID>
          <Source><LocURI>header source</LocURI></Source>
          <Target><LocURI>target header</LocURI></Target>
          <Cred><Meta>cred meta</Meta><Data>cred data</Data></Cred>
          </SyncHdr>
          <SyncBody>
          <Sequence>
          <CmdID>5</CmdID>
          <Map>
          <CmdID>4</CmdID>
          <Source><LocURI>sync1 source</LocURI></Source>
          <Target><LocURI>sync1 target</LocURI></Target>
          <MapItem>
          <Source><LocURI>source item 1</LocURI></Source>
          <Target><LocURI>target item 1</LocURI></Target>
          </MapItem>
          <MapItem>
          <Source><LocURI>source item 2</LocURI></Source>
          <Target><LocURI>target item 2</LocURI></Target>
          </MapItem>
          </Map>
          <Sync>
          <CmdID>1</CmdID>
          <Source><LocURI>sync source</LocURI></Source>
          <Target><LocURI>sync target</LocURI></Target>
          <Add>
          <CmdID>2</CmdID>
          <Cred><Meta>credential add meta</Meta><Data>credential add data</Data></Cred>
          <Meta>add meta information</Meta>
          <Item>
          <Source><LocURI>source item 1</LocURI></Source>
          <Target><LocURI>target item 1</LocURI></Target>
          <Meta>meta item 1</Meta>
          <Data>data item 1</Data>
          </Item>
          <Item>
          <Source><LocURI>source item 2</LocURI></Source>
          <Target><LocURI>target item 2</LocURI></Target>
          </Item>
          </Add>
          <Delete>
          <CmdID>3</CmdID>
          <Item>
          <Meta>meta item 1 for delete</Meta>
          <Data>data item 1  for delete</Data>
          </Item>
          </Delete>
          </Sync>
          </Sequence>
          <Get>
          <CmdID>6</CmdID>
          <Item>
          <Source><LocURI>source item 1</LocURI></Source>
          <Target><LocURI>target item 1</LocURI></Target>
          <Meta>meta item 1</Meta>
          <Data>data item 1</Data>
          </Item>
          <Item>
          <Source><LocURI>source item 1</LocURI></Source>
          <Target><LocURI>target item 1</LocURI></Target>
          <Meta>meta item 1</Meta>
          <Data>data item 1</Data>
          </Item>
          </Get>
          <Result>
          <CmdID>7</CmdID>
          <CmdRef>300</CmdRef>
          <SourceRef>sourc Ref 0</SourceRef>
          <TargetRef>target ref 0</TargetRef>
          <Item>
          <Source><LocURI>source item 1</LocURI></Source>
          <Target><LocURI>target item 1</LocURI></Target>
          <Meta>meta item 1</Meta>
          <Data>data item 1</Data>
          </Item>
          <Item>
          <Source><LocURI>source item 2</LocURI></Source>
          <Target><LocURI>target item 2</LocURI></Target>
          <Meta>meta item 2</Meta>
          <Data><![CDATA[<xml><truc>TOTO</truc></xml>]]></Data>
          </Item>
          </Result>
          <Result>
          <CmdID>8</CmdID>
          <CmdRef>301</CmdRef>
          <SourceRef>sourc Ref 1</SourceRef>
          <TargetRef>target ref 1</TargetRef>
          <Item>
          <Source><LocURI>source item 1</LocURI></Source>
          <Target><LocURI>target item 1</LocURI></Target>
          <Meta>meta item 1</Meta>
          <Data>data item 1</Data>
          </Item>
          <Item>
          <Source><LocURI>source item 2</LocURI></Source>
          <Target><LocURI>target item 2</LocURI></Target>
          <Meta>meta item 2</Meta>
          <Data><![CDATA[<xml><truc>TOTO</truc></xml>]]></Data>
          </Item>
          </Result>
          <Result>
          <CmdID>9</CmdID>
          <CmdRef>302</CmdRef>
          <SourceRef>sourc Ref 2</SourceRef>
          <TargetRef>target ref 2</TargetRef>
          <Item>
          <Source><LocURI>source item 1</LocURI></Source>
          <Target><LocURI>target item 1</LocURI></Target>
          <Meta>meta item 1</Meta>
          <Data>data item 1</Data>
          </Item>
          <Item>
          <Source><LocURI>source item 2</LocURI></Source>
          <Target><LocURI>target item 2</LocURI></Target>
          <Meta>meta item 2</Meta>
          <Data><![CDATA[<xml><truc>TOTO</truc></xml>]]></Data>
          </Item>
          </Result>
          <Status><CmdID>10</CmdID>
          <CmdRef>4000</CmdRef>
          <Data>200</Data>
          </Status>
          </SyncBody></OpenMSP>
  */
  public static Message buildCompleteMessage() throws OpenMSPException {
    Message message = MessageFactory.getFactory().createMessage(new Header(200, "header source", "target header", "cred meta", "cred data"), false);
    //// create a sync command
    ContainerMessage syncMessage = new ContainerMessage(new Command(Element.SYNC, "sync source", "sync target"));
    // create add command for sync command
    ContainerMessage addMessage = new ContainerMessage(new RequestCommand(Element.ADD));
    ((RequestCommand)addMessage.getElement()).setCredential("credential add meta", "credential add data");
    ((RequestCommand)addMessage.getElement()).setMetaInformation("add meta information");
    // add two data items for add sub command
    addMessage.add(new DataItem(Element.ITEM,"meta item 1", "data item 1", "source item 1", "target item 1"));
    addMessage.add(new Item(Element.ITEM, "source item 2", "target item 2"));
    syncMessage.add(addMessage);
    // create delete command
     ContainerMessage deleteMessage = new ContainerMessage(new RequestCommand(Element.DELETE));
    // add item
    deleteMessage.add(new DataItem(Element.ITEM,"meta item 1 for delete", "data item 1  for delete", null, null));
    syncMessage.add(deleteMessage);
    //// create map command
    ContainerMessage mapMessage = new ContainerMessage(new Command(Element.MAP, "sync1 source", "sync1 target"));
    // add two data items for add sub command
    mapMessage.add(new Item(Element.MAP_ITEM, "source item 1", "target item 1"));
    mapMessage.add(new Item(Element.MAP_ITEM, "source item 2", "target item 2"));
    // create sequence to process map command before sync  command
    ContainerMessage sequence = new ContainerMessage(new Sequence());
    sequence.add(mapMessage);
    sequence.add(syncMessage);
    message.add(sequence);
    /// add get message
    ContainerMessage getMessage = new ContainerMessage(new Command(Element.GET, "get1 source", "get1 target"));
    getMessage.add(new DataItem(Element.ITEM,"meta item 1", "data item 1", "source item 1", "target item 1"));
    getMessage.add(new DataItem(Element.ITEM,"meta item 1", "data item 1", "source item 1", "target item 1"));
    message.add(getMessage);
    /// add 3 results
    for (int i = 0; i < 3; i++) {
      ContainerMessage resultMessage = new ContainerMessage(new Result(300 + i, "sourc Ref " + i, "target ref " + i));
      resultMessage.add(new DataItem(Element.ITEM,"meta item 1", "data item 1", "source item 1", "target item 1"));
      resultMessage.add(new DataItem(Element.ITEM,"meta item 2", "<xml><truc>TOTO</truc></xml>", "source item 2", "target item 2"));
      message.add(resultMessage);
    }
    // add a status
    message.add(new Status(4000, Status.STATUS_OK));
    return message;
  }

  private static void printMessage (String tabs, ContainerMessage message) {
     System.out.println(tabs + message.getElement());
     message.resetCursor();
     while (message.hasMoreMessage()) {
      ContainerMessage child = message.nextMessage();
      printMessage (tabs + "\t", child);
     }
  }

  public static void main(String[] args) {
    // create a sample openMSP message
    try {

     Message message = buildMessage();
     String cyberML = message.encode();
     System.out.println("************ Sample message *********");
     System.out.println(cyberML);
     Message newMessage = MessageFactory.getFactory().getMessage(message.encode());
     //System.out.println("************ Sample message after encoding *********");
     System.out.println(newMessage.encode());
     // go through the structure
   printMessage ("", newMessage.getRoot());

   Message message1 = buildCompleteMessage();
   System.out.println(message1.encode());
   // go through the structure
   printMessage ("", message1.getRoot());
   newMessage = MessageFactory.getFactory().getMessage(message1.encode());
   newMessage.add(new Status(3, Status.STATUS_FAILED));
   printMessage("", newMessage.getRoot());
   System.out.println(newMessage.encode());
   System.out.println("Is empty : (should not be...) " + newMessage.isEmpty());
   System.out.println("Is empty : (should be) " + MessageFactory.getFactory().createMessage(new Header(1,"source","target")).isEmpty());
    } catch (OpenMSPException e) {
      e.printStackTrace();
      }
  }

}