/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.server;

import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Properties;

import org.openmobileis.common.util.log.FileLogTracer;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Tracer to log user synchro.
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class OpenMSPServiceTracer extends FileLogTracer {
  private java.text.SimpleDateFormat formatter;

  public OpenMSPServiceTracer() {
    super();
    formatter  = new java.text.SimpleDateFormat ("[dd/MMM/yyyy:HH:mm:ss]", java.util.Locale.FRANCE);
  }

	/**
	 * Two property are defined :
	 *  LOGFILE : file where log are written.
	 *  LOGDIRARCHIVE : directory where old log are archived.
	 */

	public void setLogTracerProperties(Properties props)  {
		try {
			logFile = props.getProperty("LOGFILE");
			if (logFile == null)  {
				System.out.println("LOGFILE property is not defined for OpenMSPServiceTracer");
				System.out.println("LOG to console");
				outStream = System.out;
				return;
			}

		} catch (Exception e) {
			System.out.println("Error in FileTraceManager Init" +e);
		}
	}
  /**
   * Trace the message to the stream defined by the specified priority and service.
   */
  public void trace(int service, int priority, String message)  {
		try {
			fileStream = new FileOutputStream(logFile, true);
			try	{
				outStream = new PrintStream(fileStream, true);
		    outStream.print(formatter.format(new java.util.Date())
		      +"-"+message
		      +"\n");
		     outStream.flush();
			} catch (Throwable ex) {
				System.out.println("Error in OpenMSPServiceTracer trace" +ex);
			} finally	{
				fileStream.close();
			}
		} catch (Throwable ex) {
			System.out.println("Error in OpenMSPServiceTracer trace" +ex);
		}
  }
}
