/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.server.synctarget.proxy;

import org.openmobileis.common.user.profile.Profile;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.*;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.AbstractCommand;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.Message;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener;
import org.openmobileis.synchro.openmsp.server.synctarget.SyncTargetAnswer;
import org.openmobileis.synchro.openmsp.server.synctarget.SynchroTargerManager;
import org.openmobileis.synchro.openmsp.server.util.FileSystem;
import org.openmobileis.synchro.security.auth.Credential;
import org.openmobileis.common.context.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * 
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class GenericProxySynchroTarget implements ProxySyncroTarget {
  
  public GenericProxySynchroTarget() {
  }
  
  
  public void executeOpenMSPCommande(Credential cred, String listenerName, ContainerMessage container, Message answer, UserTerminal terminal, String version, Profile profil, FileSystem fileSystem) throws OpenMSPException  {
    // validate profil access
    if (!this.validateProfilAccess(listenerName, profil))  {
      Status status = new  Status (((AbstractCommand)(container.getElement())).getCmdId(), Status.STATUS_UNAUTHORIZED) ;
      answer.add(status);
      LogManager.traceInfo(0, "Error rubric not found in profil validation for synchro listener :"+listenerName);
      return;
    }
    OpenMSPSynchroTargetListener listener = SynchroTargerManager.getManager().getSynchroTargetListener(listenerName, version, terminal); //no version
    if (listener != null) {
      SyncTargetAnswer listenerAnswer = listener.processCommand(cred, container);   
      
      // process send files if any
      if (listenerAnswer.answerfiles != null)  {
          int size = listenerAnswer.answerfiles.length;
          for (int i=0; i<size; i++)  {
            try {
              fileSystem.addFile(listenerAnswer.answerfiles[i]);
            } catch (Exception ex) {
              LogManager.trace(new SynchroException("file not synchronized continue", ex));
              Status status = new  Status (((AbstractCommand)(container.getElement())).getCmdId(), Status.STATUS_FAILED) ;
              answer.add(status);
              return;
            }
          }
        }
        if (listenerAnswer.containerMessage != null)  {
          int size = listenerAnswer.containerMessage.length;
          for (int i=0; i<size; i++)  {
            answer.add(listenerAnswer.containerMessage[i]);
          }
       }
    } else {
      throw new OpenMSPException("No synchro target defined for OpenMSP synchro target :"+listenerName);
    }
  }

  protected boolean validateProfilAccess(String listenerName, Profile profil) {
    int size = profil.profilRubric.length;
    for (int i=0; i<size; i++)  {
      if (listenerName.equals(profil.profilRubric[i].rubricName)) {
        return true;
      }
    }
    return false;
  }
}