/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.server.util;

import java.util.zip.*;
import java.io.*;
import java.util.*;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: virtual file system in memory.
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class MemoryFileSystem implements FileSystem {
  static final long serialVersionUID = 5521257935120563452L;

  private HashMap fileMap = new HashMap(20);

  public MemoryFileSystem() {
  }


  // Manage zip files
  public MemoryFileSystem (ZipInputStream zipFile) throws IOException {
    ZipEntry entry;
    while ( (entry = zipFile.getNextEntry()) != null) {
      String completeName = entry.getName();
      OpenMISFile file = createCyberFile(completeName, zipFile);
      this.addFile(file);
      zipFile.closeEntry();
    }
    zipFile.close();
  }

  public MemoryFileSystem (OpenMISFile filelist[])  {
  }

  /**
   * extends this methods to create the rigth type of OpenMISFile with the file system is created using a ZIP Stream
   */
  protected OpenMISFile createCyberFile(String completeName, InputStream input) throws IOException  {
    return new MemoryFile(completeName, input);
  }

  public void addAll(FileSystem fileSystem) throws IOException {
    OpenMISFile[] fileList = fileSystem.getFileList();
    for (int i= 0; i<fileList.length; i++)  {
      this.addFile(fileList[i]);
    }
  }


 /**
  * return null if not found. Specify the the completeFileName
  */
  public OpenMISFile getFile(String completefileName) {
    return  (OpenMISFile)fileMap.get(completefileName);
  }

  /**
   * Extends this method to add your own file management before adding a file.
   * call super.addFile to add the file to the existing file memory management.
   */
  public void addFile(OpenMISFile file) throws IOException  {
    String completeName = file.getFileCompleteName();
    fileMap.put(completeName,file);
  }

  public OpenMISFile[] getFileList() {
    Object[] array = fileMap.values().toArray();
    OpenMISFile[] retArray = new OpenMISFile[array.length];
    for (int i=0; i<array.length; i++)  {
      retArray[i] = (OpenMISFile)array[i];
    }
    return retArray;
  }

  /**
   save all manager file to disk using beginpath for base file repository
   @param String beginPath : base path of file. Must be terminated by the the file separator
  */
  public void saveToDisk(String beginPath) throws IOException {

    // validate beginPath format
    if (!beginPath.endsWith(File.separator))  {
      beginPath = beginPath+File.separator;
    }
    // create base dirctory
    File beginRep = new File(beginPath);
    if (!beginRep.exists())  {
      beginRep.mkdirs();
    }
    Iterator iter = fileMap.values().iterator();
    while (iter.hasNext())  {
      OpenMISFile file = (OpenMISFile) iter.next();
      File rep = new File(beginPath+file.getFilePath());
      rep.mkdirs();// create directory if not exist

      // save file
      FileOutputStream stream = new FileOutputStream(beginPath+file.getFileCompleteName());
      try {
        stream.write(file.getFileData());
      } finally {
        stream.close();
      }
    }
  }

	public int getFileCount()	{
		return fileMap.size();
	}

  /**
   * clear all file in the file system.
   */
  public void clear() {
    fileMap.clear();
  }

  public void removeFile(String completefileName) {
    fileMap.remove(completefileName);
  }


}
