/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  2004 Creation P.Delrieu
 * 
 */

package org.openmobileis.synchro.openmsp.server.util;

import org.openmobileis.common.context.UserTerminal;
import org.openmobileis.common.user.profile.Profile;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.common.util.exception.SynchroException;

import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.zip.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class WinCESynchroFileGenerator implements SynchroFileGenerator {
  class ZipArrayEntryFile {
    ByteArrayOutputStream array;

    ZipOutputStream zip;

    String fileName;

    public int hashCode() {
      return fileName.hashCode();
    }

    public boolean equals(Object obj) {
      if (obj instanceof ZipArrayEntryFile) {
        return ((ZipArrayEntryFile) obj).fileName.equals(this.fileName);
      }
      return false;
    }

  }

  public WinCESynchroFileGenerator() {
  }

  public void generateSynchroFile(UserTerminal terminal, Profile profil, HttpServletRequest request, HttpServletResponse res, FileSystem fileSystem,
      String newSessionId) throws SynchroException {
    ByteArrayOutputStream array = new ByteArrayOutputStream(100000);
    try {
      ZipOutputStream zip = new ZipOutputStream(array);
      try {
        //manage added zipEntryMemoryFiles
        HashMap zipFileMap = new HashMap(10);
        OpenMISFile[] syncfileList = fileSystem.getFileList();
        for (int i = 0; i < syncfileList.length; i++) {
          if (syncfileList[i] instanceof ZipEntryMemoryFile) {
            String zipFileName = ((ZipEntryMemoryFile) syncfileList[i]).getZipFileName();
            ZipArrayEntryFile zipEntry = (ZipArrayEntryFile) zipFileMap.get(zipFileName);
            if (zipEntry == null) {
              zipEntry = new ZipArrayEntryFile();
              zipEntry.array = new ByteArrayOutputStream(100000);
              zipEntry.zip = new ZipOutputStream(zipEntry.array);
              zipEntry.fileName = zipFileName;
              zipFileMap.put(zipFileName, zipEntry);
            }
            ZipEntry entry = new ZipEntry(FileUtilities.convertToUnixFomat(syncfileList[i].getFileName()));
            entry.setTime(syncfileList[i].getFileDate());
            entry.setMethod(ZipEntry.DEFLATED);
            zipEntry.zip.putNextEntry(entry);
            zipEntry.zip.write(syncfileList[i].getFileData());
            zipEntry.zip.closeEntry();
          } else {
            ZipEntry entry = new ZipEntry(FileUtilities.convertToUnixFomat(syncfileList[i].getFileCompleteName()));
            entry.setTime(syncfileList[i].getFileDate());
            entry.setMethod(ZipEntry.DEFLATED);
            zip.putNextEntry(entry);
            zip.write(syncfileList[i].getFileData());
            zip.closeEntry();
          }
        }
        
        //add zip entry files to synchro file.
        Iterator iter = zipFileMap.keySet().iterator();
        while (iter.hasNext())  {
          String filename = (String) iter.next();
          ZipArrayEntryFile zipEntry = (ZipArrayEntryFile) zipFileMap.get(filename);
          zipEntry.zip.flush();
          zipEntry.zip.close();
          ZipEntry entry = new ZipEntry(FileUtilities.convertToUnixFomat(filename));
         // entry.setTime(syncfileList[i].getFileDate());
          entry.setMethod(ZipEntry.DEFLATED);
          zip.putNextEntry(entry);
          zip.write(zipEntry.array.toByteArray());
          zip.closeEntry();          
        }
      } finally {
        zip.flush();
        zip.close();
      }
      res.addHeader("SyncNum", newSessionId);
      res.setContentType("application/zip");
      res.getOutputStream().write(array.toByteArray());
    } catch (IOException ex) {
      throw new SynchroException("Error during synchro file genration ", ex);
    }
  }

}