/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.synchro.security.auth;

import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.*;

import org.openmobileis.common.util.codec.GeneralCoder;

/**
* @title:        OpenMobileIS project source
* Description:   Manage credential convertion between string format and Credential class
* Implement the OpenMSP and BasicAuth format. 
* Description:
* @author	Philippe Delrieu
* @since	JDK 1.1
* @version 1.0.
*/

public class CredentialCodec {
  private static final int BASIC_AUTH = 1;
  private static final int TEXT_FORMAT = 1;
  private static final int B64_FORMAT = 2;

  private static Credential getCredentialFromBasicAuth(String data)  {
    Credential cred = null;
    int index = data.indexOf(":");
    if (index != -1)  {
      try {
        cred = new Credential(data.substring(0, index), data.substring(index+1, data.length()));
      } catch (ArrayIndexOutOfBoundsException ex) {
      }
    }
    return cred;
  }

  public static Credential getOpenMSPCredential(String[] credential) throws OpenMSPException {
    if (credential == null)  return null;
    try {
      int authType = 0;
      int format = 0;
      ElementData[] credMeta = SimpleOpenMLParser.parseXML(credential[0]);
      for (int i=0; i<credMeta.length; i++)  {
        if (credMeta[i].getElement().equals("Type")) {
          if (credMeta[i].getData().equals("OpenMSP:auth-basic"))  {
            authType = CredentialCodec.BASIC_AUTH;
          }
        } else if (credMeta[i].getElement().equals("Format")) {
          //manage format
          if (credMeta[i].getData().equals("plain/text"))  {
            format = CredentialCodec.TEXT_FORMAT;
          } else if (credMeta[i].getData().equals("b64"))  {
            format = CredentialCodec.B64_FORMAT;
          }
        }
      }

      // get credential data ;
      if (( authType ==  CredentialCodec.BASIC_AUTH) && ( format== CredentialCodec.TEXT_FORMAT)) {
        return CredentialCodec.getCredentialFromBasicAuth(credential[1]);
      } else if (( authType ==  CredentialCodec.BASIC_AUTH) && ( format== CredentialCodec.B64_FORMAT)) {
        return CredentialCodec.getCredentialFromBasicAuth(new String(GeneralCoder.decodeBase64(credential[1].getBytes())));
      }
    } catch (java.io.IOException ex)  {
      throw new OpenMSPException(ex);
    }
    return null;
  }
}