/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.synchro.sync4j.server;

import java.security.Principal;
import java.sql.Timestamp;
import java.util.ArrayList;

import org.openmobileis.common.util.log.LogManager;

import sync4j.framework.engine.SyncItem;
import sync4j.framework.engine.SyncItemKey;
import sync4j.framework.engine.SyncItemState;
import sync4j.framework.engine.source.AbstractSyncSource;
import sync4j.framework.engine.source.SyncSource;
import sync4j.framework.engine.source.SyncSourceException;

/**
 * @author Administrateur
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public abstract class  FODBSync4JSource extends AbstractSyncSource implements SyncSource {
  protected static final String MIMETYPE    = "text/plain";

  /**
   * 
   */
  public FODBSync4JSource() {
    super();
    setType(MIMETYPE);
  }

  /**
   * @param arg0
   * @param arg1
   * @param arg2
   */
  public FODBSync4JSource(String arg0, String arg1, String arg2) {
    super(arg0, arg1, arg2);
    setType(MIMETYPE);
  }

  /**
   * @param arg0
   */
  public FODBSync4JSource(String arg0) {
    super(arg0);
    setType(MIMETYPE);
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#getAllSyncItems(java.security.Principal)
   */
  public SyncItem[] getAllSyncItems(Principal principal) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling getAllSyncItems [" + sourceURI + "]");
    return getSyncItems(principal, null, null);
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#getDeletedSyncItemKeys(java.security.Principal, java.sql.Timestamp)
   */
  public SyncItemKey[] getDeletedSyncItemKeys(Principal arg0, Timestamp arg1) throws SyncSourceException {
   LogManager.traceDebug(0, "Calling getDeletedSyncItemKeys [" + sourceURI + "]");
   return new SyncItemKey[0];
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#getDeletedSyncItems(java.security.Principal, java.sql.Timestamp)
   */
  public SyncItem[] getDeletedSyncItems(Principal principal, Timestamp since) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling getDeletedSyncItems [" + sourceURI + "]");

    return getSyncItems(principal, String.valueOf(SyncItemState.DELETED), since);
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#getNewSyncItemKeys(java.security.Principal, java.sql.Timestamp)
   */
  public SyncItemKey[] getNewSyncItemKeys(Principal arg0, Timestamp arg1) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling getNewSyncItemKeys [" + sourceURI + "]");
    return new SyncItemKey[0];
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#getNewSyncItems(java.security.Principal, java.sql.Timestamp)
   */
  public SyncItem[] getNewSyncItems(Principal principal, Timestamp since) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling getNewSyncItemKeys [" + sourceURI + "]");
    return getSyncItems(principal, String.valueOf(SyncItemState.NEW), since);
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#getSyncItemFromId(java.security.Principal, sync4j.framework.engine.SyncItemKey)
   */
  public SyncItem getSyncItemFromId(Principal principal, SyncItemKey syncItemKey) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling getSyncItemFromId [" + sourceURI + "]");
    return getSyncItem(principal, syncItemKey.getKeyAsString());
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#getSyncItemsFromIds(java.security.Principal, sync4j.framework.engine.SyncItemKey[])
   */
  public SyncItem[] getSyncItemsFromIds(Principal principal, SyncItemKey[] syncItemsKeys) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling getSyncItemsFromIds [" + sourceURI + "]");
    ArrayList listItem = new ArrayList();

    int num = syncItemsKeys.length;
    for (int i = 0; i < num; i++) {
        listItem.add(getSyncItemFromId(principal, syncItemsKeys[i]));
    }

    SyncItem[] toReturn = (SyncItem[]) (listItem.toArray(new SyncItem[
        0]));

    return toReturn;
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#getUpdatedSyncItems(java.security.Principal, java.sql.Timestamp)
   */
  public SyncItem[] getUpdatedSyncItems(Principal principal, Timestamp since) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling getUpdatedSyncItems [" + sourceURI + "]");
    return getSyncItems(principal, String.valueOf(SyncItemState.UPDATED), since);
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#removeSyncItem(java.security.Principal, sync4j.framework.engine.SyncItem)
   */
  public void removeSyncItem(Principal arg0, SyncItem arg1) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling removeSyncItem [" + sourceURI + "]");
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#removeSyncItems(java.security.Principal, sync4j.framework.engine.SyncItem[])
   */
  public void removeSyncItems(Principal principal, SyncItem[] syncItems) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling removeSyncItem [" + sourceURI + "]");
    for (int i = 0; (syncItems != null) && (i < syncItems.length); i++) {
      removeSyncItem(principal, syncItems[i]);
    }
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#setSyncItem(java.security.Principal, sync4j.framework.engine.SyncItem)
   */
  public SyncItem setSyncItem(Principal arg0, SyncItem arg1) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling setSyncItem [" + sourceURI + "]");
    return null;
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#setSyncItems(java.security.Principal, sync4j.framework.engine.SyncItem[])
   */
  public SyncItem[] setSyncItems(Principal principal, SyncItem[] syncItems) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling setSyncItems [" + sourceURI + "]");
    for (int i = 0; (syncItems != null) && (i < syncItems.length); i++) {
      setSyncItem(principal, syncItems[i]);
  }

  return syncItems;
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#getSyncItemFromTwin(java.security.Principal, sync4j.framework.engine.SyncItem)
   */
  public SyncItem getSyncItemFromTwin(Principal arg0, SyncItem arg1) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling getSyncItemFromTwin [" + sourceURI + "]");
    return null;
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#getSyncItemsFromTwins(java.security.Principal, sync4j.framework.engine.SyncItem[])
   */
  public SyncItem[] getSyncItemsFromTwins(Principal arg0, SyncItem[] arg1) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling getSyncItemsFromTwins [" + sourceURI + "]");
    return new SyncItem[0];
  }

  /* (non-Javadoc)
   * @see sync4j.framework.engine.source.SyncSource#getUpdatedSyncItemKeys(java.security.Principal, java.sql.Timestamp)
   */
  public SyncItemKey[] getUpdatedSyncItemKeys(Principal arg0, Timestamp arg1) throws SyncSourceException {
    LogManager.traceDebug(0, "Calling getUpdatedSyncItemKeys [" + sourceURI + "]");
    return new SyncItemKey[0];
  }
  
  /**
   * Get the SyncItem[] in accord with the given parameters
   * @param principal Principal
   * @param type String
   * @param since Timestamp
   * @return SyncItem[]
   * @throws SyncSourceException
   */
  protected abstract SyncItem[] getSyncItems(Principal principal,
                                          String type,
                                          Timestamp since) throws SyncSourceException;

  /**
   * Get the SyncItem with the given itemId for the given principal
   * @param principal Principal
   * @param itemId String
   * @return SyncItem
   * @throws SyncSourceException
   */
  protected abstract SyncItem getSyncItem(Principal principal, String itemId)
          throws SyncSourceException;

}
