/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.test.bundle.profil;

import java.io.File;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.Reader;

import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.user.profile.Profile;
import org.openmobileis.common.user.profile.ProfileDataManager;
import org.openmobileis.common.user.profile.ProfileProperty;
import org.openmobileis.common.user.profile.ProfileRubric;
import org.openmobileis.common.user.profile.impl.TerminalXmlFileProfilDataFactory;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.module.core.DependProfileModule;
import org.openmobileis.module.core.ProfileModule;
import org.openmobileis.module.core.ProfileModuleRubric;
import org.openmobileis.module.core.XmlModuleParser;
import org.openmobileis.module.profiles.server.ProfilManagerSyncTarget;
import org.openmobileis.module.profiles.terminal.OpenMSPProfileSyncListener;
import org.openmobileis.module.profiles.terminal.ProfileRubricSynchroListener;
import org.openmobileis.module.server.ModuleJarSynchroTarget;
import org.openmobileis.module.server.ModuleServerManager;
import org.openmobileis.module.terminal.ModuleJarSyncListener;
import org.openmobileis.module.terminal.ModuleManager;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.client.SynchroManager;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;
import org.openmobileis.synchro.openmsp.client.conduit.HttpOpenMSPSynchroConduit;
import org.openmobileis.synchro.openmsp.client.conduit.LocalOpenMSPSynchroConduit;
import org.openmobileis.synchro.openmsp.server.synctarget.SynchroTargerManager;
import org.openmobileis.synchro.openmsp.server.synctarget.proxy.GenericProxySynchroTarget;
import org.openmobileis.synchro.security.auth.Credential;
import org.openmobileis.test.mock.common.MockUserManagerFactory;
import org.openmobileis.test.mock.synchro.SimpleMockSynchroManager;

import junit.framework.Assert;
import junit.framework.TestCase;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public class TestBaseProfil extends TestCase {

  /**
   * @param name
   */
  public TestBaseProfil(String name) {
    super(name);
  }

  /* (non-Javadoc)
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp() throws Exception {
    super.setUp();
    LogManager.registerLogManager(null);
  }

  /* (non-Javadoc)
   * @see junit.framework.TestCase#tearDown()
   */
  protected void tearDown() throws Exception {
    super.tearDown();
  }

  /**
   * Test method for {@link org.openmobileis.module.core.XmlModuleParser#XmlModuleParser(java.io.Reader)}.
   */
  public void testXmlModuleParser() {
    try {
      Reader reader = new InputStreamReader(PropertiesManager.getManager().getRessourceAsStream("/org/openmobileis/test/bundle/profil/testmodule.xml"));
      XmlModuleParser parser = new XmlModuleParser(reader);
      parser.parse();
      //validate parsing
      ProfileModule module = parser.getModuletoload();
      Assert.assertEquals("TEST", module.getName());
      Assert.assertEquals("1.0.0", module.getVersion());
      Assert.assertEquals("testsnchro.jar", module.getServerjar());
      Assert.assertEquals("testterminal.jar", module.getTerminaljar());
      Array deplist = module.getDependModuleList();
      Assert.assertTrue(deplist.contains(new DependProfileModule("openMobileIS", "1.0.0")));

      Array rubriclist = module.getModuleRubrics();

      //testrub1
      ProfileModuleRubric rub1 = (ProfileModuleRubric) rubriclist.get(0);
      Assert.assertEquals("testrub1", rub1.getName());
      Assert.assertEquals("testloader1", rub1.getLoaderClass());
      Assert.assertTrue(rub1.getDependsList().length == 0);
      Array synclist = rub1.getSynchroListenerList();
      ProfileRubricSynchroListener sync = (ProfileRubricSynchroListener) synclist.get(0);
      Assert.assertEquals("listerner1", sync.name);
      Assert.assertNull(sync.dependsList);
      sync = (ProfileRubricSynchroListener) synclist.get(1);
      Assert.assertEquals("listerner3", sync.name);
      Assert.assertNull(sync.dependsList);
      sync = (ProfileRubricSynchroListener) synclist.get(2);
      Assert.assertEquals("listerner2", sync.name);
      Assert.assertNotNull(sync.dependsList);
      Assert.assertEquals("listerner1", sync.dependsList[0]);
      Assert.assertEquals("listerner3", sync.dependsList[1]);
      Assert.assertNotNull(rub1.getProperty("description"));
      Assert.assertEquals("Test rubric loader", rub1.getProperty("description"));

      //testrub2
      rub1 = (ProfileModuleRubric) rubriclist.get(1);
      Assert.assertEquals("testrub2", rub1.getName());
      String[] deprublist = rub1.getDependsList();
      Assert.assertEquals("testrub1", deprublist[0]);
      Assert.assertEquals("testloader21", rub1.getLoaderClass());
      synclist = rub1.getSynchroListenerList();
      sync = (ProfileRubricSynchroListener) synclist.get(0);
      Assert.assertEquals("listerner1", sync.name);
      Assert.assertNull(sync.dependsList);

      //testrub3
      rub1 = (ProfileModuleRubric) rubriclist.get(2);
      Assert.assertEquals("testrub3", rub1.getName());
      deprublist = rub1.getDependsList();
      Assert.assertEquals("testrub1", deprublist[0]);
      Assert.assertEquals("testrub2", deprublist[1]);
      Assert.assertEquals("testloader3", rub1.getLoaderClass());
      synclist = rub1.getSynchroListenerList();
      sync = (ProfileRubricSynchroListener) synclist.get(0);
      Assert.assertEquals("listerner31", sync.name);
      Assert.assertNull(sync.dependsList);

    } catch (Exception ex) {
      Assert.fail("Exception occurs during testXmlModuleParser :" + ex.getMessage());
    }
  }

  /**
   * Test method for {@link org.openmobileis.common.user.profile.impl.TerminalXmlFileProfilDataFactory#TerminalXmlFileProfilDataFactory(java.io.Reader)}.
   */
  public void testTerminalProfilParser() {
    try {
      Reader reader = new InputStreamReader(PropertiesManager.getManager().getRessourceAsStream("/org/openmobileis/test/bundle/profil/testprofils.xml"));
      TerminalXmlFileProfilDataFactory parser = new TerminalXmlFileProfilDataFactory(reader);
      Profile profil = parser.getParsedProfil();
      Assert.assertEquals("defname", profil.profilName);
      //validate ribric
      Assert.assertEquals(profil.profilRubric.length, 4);

      Assert.assertEquals("openMobileIS", profil.profilRubric[0].moduleName);
      Assert.assertEquals("Core OpenMobileIS", profil.profilRubric[0].rubricName);
      Assert.assertEquals("1.0.0", profil.profilRubric[0].version);
      Assert.assertNull(profil.profilRubric[0].description);

      Assert.assertEquals("TEST", profil.profilRubric[1].moduleName);
      Assert.assertEquals("testrub1", profil.profilRubric[1].rubricName);
      Assert.assertEquals("1.0.0", profil.profilRubric[1].version);
      Assert.assertEquals("test desc", profil.profilRubric[1].description);

      Assert.assertEquals("TEST", profil.profilRubric[2].moduleName);
      Assert.assertEquals("testrub2", profil.profilRubric[2].rubricName);
      Assert.assertEquals("test desc2", profil.profilRubric[2].description);
      Assert.assertNull(profil.profilRubric[2].version);

      Assert.assertEquals("TEST", profil.profilRubric[3].moduleName);
      Assert.assertEquals("testrub3", profil.profilRubric[3].rubricName);
      Assert.assertNull(profil.profilRubric[3].version);
      Assert.assertNull(profil.profilRubric[3].description);

      //validate property
      Assert.assertEquals(profil.profilProperties.length, 2);
      Assert.assertEquals("testkey1", profil.profilProperties[0].key);
      Assert.assertEquals("value1", profil.profilProperties[0].value);
      Assert.assertNull(profil.profilProperties[0].moduleName);

      Assert.assertEquals("testkey2", profil.profilProperties[1].key);
      Assert.assertEquals("value2", profil.profilProperties[1].value);
      Assert.assertEquals("TEST", profil.profilProperties[1].moduleName);

    } catch (Exception ex) {
      Assert.fail("Exception occurs during testXmlModuleParser :" + ex.getMessage());
    }

  }

  /**
   * Test method for {@link org.openmobileis.common.user.profile.Profile#equals(java.lang.Object)}.
   */
  public void testProfilEquals() {
    try {
      Reader reader = new InputStreamReader(PropertiesManager.getManager().getRessourceAsStream("/org/openmobileis/test/bundle/profil/testprofils.xml"));
      TerminalXmlFileProfilDataFactory parser = new TerminalXmlFileProfilDataFactory(reader);
      Profile profil1 = parser.getParsedProfil();

      reader = new InputStreamReader(PropertiesManager.getManager().getRessourceAsStream("/org/openmobileis/test/bundle/profil/testprofils.xml"));
      parser = new TerminalXmlFileProfilDataFactory(reader);
      Profile profil2 = parser.getParsedProfil();

      Assert.assertEquals(profil1, profil2);

      //test modif name
      profil2.profilName = "toto";
      Assert.assertNotSame(profil1, profil2);
      profil2.profilName = profil1.profilName;
      Assert.assertEquals(profil1, profil2);

      profil2.profilRubric = new ProfileRubric[] { profil1.profilRubric[0], profil1.profilRubric[1], profil1.profilRubric[2] };
      Assert.assertNotSame(profil1, profil2);
      profil2.profilRubric = profil1.profilRubric;
      Assert.assertEquals(profil1, profil2);
      profil2.profilRubric[0].moduleName = "toto";
      Assert.assertNotSame(profil1, profil2);
      profil2.profilRubric = profil1.profilRubric;
      Assert.assertEquals(profil1, profil2);
      profil2.profilRubric[0].rubricName = "toto";
      Assert.assertNotSame(profil1, profil2);
      profil2.profilRubric = profil1.profilRubric;
      Assert.assertEquals(profil1, profil2);
      profil2.profilRubric[0].version = "toto";
      Assert.assertNotSame(profil1, profil2);
      profil2.profilRubric = profil1.profilRubric;
      Assert.assertEquals(profil1, profil2);
      profil2.profilRubric[0].description = "toto";
      Assert.assertNotSame(profil1, profil2);
      profil2.profilRubric = profil1.profilRubric;
      Assert.assertEquals(profil1, profil2);
      profil2.profilRubric[1].description = "toto";
      Assert.assertNotSame(profil1, profil2);
      profil2.profilRubric = profil1.profilRubric;
      Assert.assertEquals(profil1, profil2);
      profil2.profilRubric[3].version = "toto";
      Assert.assertNotSame(profil1, profil2);
      profil2.profilRubric = profil1.profilRubric;
      Assert.assertEquals(profil1, profil2);
      profil2.profilRubric[1].version = null;
      Assert.assertNotSame(profil1, profil2);
      profil2.profilRubric = profil1.profilRubric;
      Assert.assertEquals(profil1, profil2);
      profil2.profilRubric[1].description = null;
      Assert.assertNotSame(profil1, profil2);
      profil2.profilRubric = profil1.profilRubric;
      Assert.assertEquals(profil1, profil2);

      profil2.profilProperties = new ProfileProperty[] { profil1.profilProperties[0] };
      Assert.assertNotSame(profil1, profil2);
      profil2.profilProperties = profil1.profilProperties;
      Assert.assertEquals(profil1, profil2);
      profil2.profilProperties[0].key = "toto";
      Assert.assertNotSame(profil1, profil2);
      profil2.profilProperties = profil1.profilProperties;
      Assert.assertEquals(profil1, profil2);
      profil2.profilProperties[0].value = "toto";
      Assert.assertNotSame(profil1, profil2);
      profil2.profilProperties = profil1.profilProperties;
      Assert.assertEquals(profil1, profil2);
      profil2.profilProperties[0].moduleName = "toto";
      Assert.assertNotSame(profil1, profil2);
      profil2.profilProperties = profil1.profilProperties;
      Assert.assertEquals(profil1, profil2);
      profil2.profilProperties[1].moduleName = "toto";
      Assert.assertNotSame(profil1, profil2);
      profil2.profilProperties = profil1.profilProperties;
      Assert.assertEquals(profil1, profil2);
      profil2.profilProperties[1].moduleName = null;
      Assert.assertNotSame(profil1, profil2);
      profil2.profilProperties = profil1.profilProperties;
      Assert.assertEquals(profil1, profil2);

    } catch (Exception ex) {
      Assert.fail("Exception occurs during testXmlModuleParser :" + ex.getMessage());
    }

  }

  /**
   * Test method for test profil synchronization}.
   */
  public void testProfilSynchro() {
    try {

      String synchronizedProfilFile = System.getProperty("user.dir") + "/WEB-INF/conf/userprofil.xml";
      PropertiesManager.getManager().addProperty("org.openmobileis.profil.file", synchronizedProfilFile);

      //remove terminal files
      File file = new File(synchronizedProfilFile);
      if (file.exists())
        file.delete();

      //init db properties.
      PropertiesManager.getManager().addProperty("fastobjectdb.database.path", System.getProperty("user.dir") + "/database");
      PropertiesManager.getManager().addProperty("fastobjectdb.database.name", "testdb");

      Credential cred = new Credential("test", "test");
      UserManager.getManager().registerUserManagerFactoryForGroup("defgroup", new MockUserManagerFactory("defgroup", "defname", cred));

      Reader reader = new InputStreamReader(PropertiesManager.getManager().getRessourceAsStream("/org/openmobileis/test/bundle/profil/testprofils.xml"));
      TerminalXmlFileProfilDataFactory serverprofilmanager = new TerminalXmlFileProfilDataFactory(reader);
      ProfileDataManager.getManager().registerProfilDataFactoryForGroup("defgroup", serverprofilmanager);

      //manage session
      SessionContext context = SessionContextManager.getManager().createSessionContext("testsession");
      SessionContextManager.getManager().joinSessionContext("testsession");
      context.setUserGroup("defgroup");
      context.setUserId("defname");

      String synchroInstallPath = System.getProperty("user.dir") + "/synchro";
      SimpleMockSynchroManager synchromock = new SimpleMockSynchroManager(synchroInstallPath);
      OpenMSPProfileSyncListener listener = new OpenMSPProfileSyncListener();
      ProfilManagerSyncTarget target = new ProfilManagerSyncTarget();
      synchromock.registerTerminalSynchroListener(listener);
      synchromock.registerServerSynchroTarget(target);
      synchromock.synchronize("defgroup", cred, new SynchroDescriptor());

      //validate the created profil is the same as the server one.
      Profile serverprofil = serverprofilmanager.getParsedProfil();
      Reader termreader = new FileReader(synchronizedProfilFile);
      TerminalXmlFileProfilDataFactory parser = new TerminalXmlFileProfilDataFactory(termreader);
      Profile tremprofil = parser.getParsedProfil();
      Assert.assertEquals(serverprofil, tremprofil);

    } catch (Exception ex) {
      LogManager.traceError(0, ex);
      Assert.fail("Exception occurs during testXmlModuleParser :" + ex.getMessage());
    }

  }

  public void testJarSynchro() {
    try {
      //copy files needed for the test
      String basepath = System.getProperty("user.dir") + "/testfiles/testprofile/testjarsynchro/";
      String destPath = System.getProperty("user.dir") + "/repository/";
      FileUtilities.copyFile(basepath+"openmobileis.xml", destPath+"openmobileis.xml");
      FileUtilities.copyFile(basepath+"openmisclient.jar", destPath+"openmisclient.jar");
      FileUtilities.copyFile(basepath+"openmobileis.xml", System.getProperty("user.dir")+"/WEB-INF/conf/xml/openMobileIS.xml");
      
      //init server parameters.
      String xmlservermodulefile = System.getProperty("user.dir") + "/repository/openmobileis.xml";
      PropertiesManager.getManager().addProperty("org.openmobileis.bundle.module.repositorypath", System.getProperty("user.dir") + "/repository");

      //init terminal parameter
      PropertiesManager.getManager().addProperty("org.openmobileis.module.repository.jar", System.getProperty("user.dir") + "/WEB-INF/lib");
      PropertiesManager.getManager().addProperty("org.openmobileis.module.repository.xml", System.getProperty("user.dir") + "/WEB-INF/conf/xml");
      
      //test if the terminal jar and xml file exist. If yes delete them.
      File jarfile = new File(PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.jar") + "/openmisclient.jar");
      if (jarfile.exists())
        jarfile.delete();
      File xmlffile = new File(PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.xml") + "/openMobileIS.xml");
 //     if (xmlffile.exists())
 //       xmlffile.delete();

      //init context data
      Credential cred = new Credential("test", "test");
      UserManager.getManager().registerUserManagerFactoryForGroup("defgroup", new MockUserManagerFactory("defgroup", "defname", cred));

      Reader reader = new InputStreamReader(PropertiesManager.getManager().getRessourceAsStream("/org/openmobileis/test/bundle/profil/testprofils.xml"));
      TerminalXmlFileProfilDataFactory serverprofilmanager = new TerminalXmlFileProfilDataFactory(reader);
      ProfileDataManager.getManager().registerProfilDataFactoryForGroup("defgroup", serverprofilmanager);

      //manage session
      SessionContext context = SessionContextManager.getManager().createSessionContext("testsession");
      SessionContextManager.getManager().joinSessionContext("testsession");
      context.setUserGroup("defgroup");
      context.setUserId("defname");
      
      ProfileModule module = ModuleManager.getManager().readModuleXmlFiles("openMobileIS");
      ModuleJarSyncListener listener = new ModuleJarSyncListener();
      listener.setModule(module);
      ModuleJarSynchroTarget target = new ModuleJarSynchroTarget(new File(xmlservermodulefile));

      //init synchro path.
      String synchroInstallPath = System.getProperty("user.dir") + "/synchro";
      PropertiesManager.getManager().addProperty("org.openmobileis.synchro.direct.installpath", synchroInstallPath);
      SynchroDescriptor descriptor = new SynchroDescriptor();
      descriptor.setSynchroConduit(new HttpOpenMSPSynchroConduit(synchroInstallPath, ""));
      SimpleMockSynchroManager synchromock = new SimpleMockSynchroManager(synchroInstallPath);

      synchromock.registerTerminalSynchroListener(listener);
      synchromock.registerServerSynchroTarget(target);
      synchromock.synchronize("defgroup", cred, descriptor);

      //validate synchronisation first synchro
      Assert.assertTrue(jarfile.exists());
      Assert.assertTrue(xmlffile.exists());

      //delete terminal xml and jar. do synchro  Test nothing synchronized.
      jarfile.delete();
      xmlffile.delete();
      synchromock.synchronize("defgroup", cred, descriptor);
      Assert.assertTrue(!jarfile.exists());
      Assert.assertTrue(!xmlffile.exists());

      //modif server xml date test file synchronized
      jarfile.delete();
      xmlffile.delete();

      File sxmlfile = new File(xmlservermodulefile);
      sxmlfile.setLastModified(new java.util.Date().getTime());

      //way 1 second
      Thread.currentThread().sleep(1000);

      synchromock.synchronize("defgroup", cred, descriptor);
      Assert.assertTrue(jarfile.exists());
      Assert.assertTrue(xmlffile.exists());

      //delete terminal xml and jar. Modif server jar . test file synchronized.
      jarfile.delete();
      xmlffile.delete();
      File sjarfile = new File(PropertiesManager.getManager().getProperty("org.openmobileis.bundle.module.repositorypath") + "/"
          + target.getProfilModule().getTerminaljar());
      sjarfile.setLastModified(new java.util.Date().getTime());

      //way 1 second
      Thread.currentThread().sleep(1000);
      synchromock.synchronize("defgroup", cred, descriptor);
      Assert.assertTrue(jarfile.exists());
      Assert.assertTrue(xmlffile.exists());

      // delete terminal jar Test noting synchronized.
      jarfile.delete();
      xmlffile.delete();
      synchromock.synchronize("defgroup", cred, descriptor);
      Assert.assertTrue(!jarfile.exists());
      Assert.assertTrue(!xmlffile.exists());

    } catch (Exception ex) {
      LogManager.traceError(0, ex);
      Assert.fail("Exception occurs during testXmlModuleParser :" + ex.getMessage());
    }

  }

  public void testCompleteProfilJarSynchro() {
    try {
      //copy files needed for the test
      String basepath = System.getProperty("user.dir") + "/testfiles/testprofile/testCompleteProfilJarSynchro/";
      String destPath = System.getProperty("user.dir");
      FileUtilities.copyFile(basepath+"openmobileis.xml", destPath+"/repository/openmobileis.xml");
      FileUtilities.copyFile(basepath+"openmisclient.jar", destPath+"/repository/openmisclient.jar");
      FileUtilities.copyFile(basepath+"mycrm.xml", destPath+"/repository/mycrm.xml");
      FileUtilities.copyFile(basepath+"mycrm.jar", destPath+"/repository/mycrm.jar");
      FileUtilities.copyFile(basepath+"initprofil.xml", destPath+"/WEB-INF/conf/userprofil.xml");
      FileUtilities.copyFile(basepath+"baseprofil.xml", destPath+"/server_conf/baseprofil.xml");
      //put base file to load openmis module.
      FileUtilities.copyFile(basepath+"openmobileis.xml", destPath+"/WEB-INF/module/xml/openMobileIS.xml");

      
      //DO ALL INIT
      //init profil path. Done because modified by other test, otherwise done by module manager
      String synchronizedProfilFile = System.getProperty("user.dir") + "/WEB-INF/conf/userprofil.xml";
      PropertiesManager.getManager().addProperty("org.openmobileis.profil.file", synchronizedProfilFile);
      //init db properties.
      PropertiesManager.getManager().addProperty("fastobjectdb.database.path", System.getProperty("user.dir") + "/database");
      PropertiesManager.getManager().addProperty("fastobjectdb.database.name", "testdb");
      //init server parameters.
 //     String xmlservermodulefile = System.getProperty("user.dir") + "/repository/openmobileis.xml";
      PropertiesManager.getManager().addProperty("org.openmobileis.bundle.module.repositorypath", System.getProperty("user.dir") + "/repository");

      //init terminal parameter
      //done by module manager
  //    PropertiesManager.getManager().addProperty("org.openmobileis.module.repository.jar", System.getProperty("user.dir") + "/WEB-INF/lib");
  //    PropertiesManager.getManager().addProperty("org.openmobileis.module.repository.xml", System.getProperty("user.dir") + "/WEB-INF/conf/xml");

      //test if the terminal jar and xml file exist. If yes delete them.

      //init context data
      Credential cred = new Credential("test", "test");
      UserManager.getManager().registerUserManagerFactoryForGroup("defgroup", new MockUserManagerFactory("defgroup", "defname", cred));

      Reader reader = new FileReader(System.getProperty("user.dir") + "/server_conf/baseprofil.xml");
      TerminalXmlFileProfilDataFactory serverprofilmanager = new TerminalXmlFileProfilDataFactory(reader);
      ProfileDataManager.getManager().registerProfilDataFactoryForGroup("defgroup", serverprofilmanager);

      //manage session
      SessionContext context = SessionContextManager.getManager().createSessionContext("testsession");
      SessionContextManager.getManager().joinSessionContext("testsession");
      context.setUserGroup("defgroup");
      context.setUserId("defname");
      
      //init module management
      TestModuleManagerListener moduleManagerListenernew  = new TestModuleManagerListener();
      //delete synchro files
      PropertiesManager.getManager().addProperty("org.openmobileis.module.repository.jar", moduleManagerListenernew.getJarInstallPath());
      PropertiesManager.getManager().addProperty("org.openmobileis.module.repository.xml", moduleManagerListenernew.getXMLInstallPath());
      File todeletefile = new File(PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.jar")+"/openmisclient.jar");
      todeletefile.delete();
      todeletefile = new File(PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.xml")+"/openMobileIS.xml");
      todeletefile.delete();
      todeletefile = new File(PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.jar")+"/mycrm.jar");
      todeletefile.delete();
      todeletefile = new File(PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.xml")+"/MyCRM.xml");
      todeletefile.delete();
      ModuleManager.getManager().registerModuleManagerListener(moduleManagerListenernew);
      ModuleManager.getManager().initModule();
      Assert.assertNotNull(moduleManagerListenernew.loadedModuleNames[0]);
      Assert.assertEquals("openMobileIS", moduleManagerListenernew.loadedModuleNames[0].getName());
      
      //init synchro
      String synchroInstallPath = System.getProperty("user.dir") + "/synchro";
      PropertiesManager.getManager().addProperty("org.openmobileis.synchro.direct.installpath", synchroInstallPath);
      SynchroDescriptor descriptor = new SynchroDescriptor();
      descriptor.setSynchroGroup("defgroup");
      String useragent = OpenMSPSynchroManager.getManager().getSynchroUserAgent(descriptor);
      descriptor.setSynchroConduit(new LocalOpenMSPSynchroConduit(synchroInstallPath, useragent));
      
      //server configuration. Add profil target and module target
      PropertiesManager.getManager().addProperty("org.openmobileis.bundle.module.repositorypath", System.getProperty("user.dir") + "/repository");
      ModuleServerManager.getManager();
      ProfilManagerSyncTarget target = new ProfilManagerSyncTarget();
      SynchroTargerManager.getManager().addSynchroTargetListener(target, null, null);
      SynchroTargerManager.getManager().registerProxyTargetForTerminal(new GenericProxySynchroTarget(), null);      
      //start globale synchro
      SynchroManager.getManager().registerSynchroProcessor(OpenMSPSynchroManager.getManager());
      SynchroManager.getManager().doSynchro(cred, descriptor);
      
  //    validate XML and JAR files has been synchronized
      File jarfile = new File(PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.jar")+"/openmisclient.jar");
      Assert.assertTrue(jarfile.exists());
      jarfile = new File(PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.jar")+"/mycrm.jar");
      Assert.assertTrue(jarfile.exists());
      
      File xmlfile = new File(PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.xml")+"/openMobileIS.xml");
      Assert.assertTrue(xmlfile.exists());
      xmlfile = new File(PropertiesManager.getManager().getProperty("org.openmobileis.module.repository.xml")+"/MyCRM.xml");
      Assert.assertTrue(xmlfile.exists());
      
      //mise en place de la partie static synchro. Module initiale. dmarrage et rechargement.
     
    } catch (Exception ex) {
      LogManager.traceError(0, ex);
      Assert.fail("Exception occurs during testXmlModuleParser :" + ex.getMessage());
    }

  }

}
