/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.test.database;

import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget;
import org.openmobileis.synchro.algo.replication.AlwaysUpdateServerSynchroConflicResolver;
import org.openmobileis.synchro.algo.replication.SynchroAtomicObject;
import org.openmobileis.synchro.algo.replication.SynchroConflicResolver;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.security.auth.Credential;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class TestDummyFODBSyncTarget implements FODBSyncTarget {
  public boolean connectCall = false;
  public String deleteCollectionObjectId = null;
  public boolean disconnectCall = false;
  public SynchroAtomicObject currentAO;
  public int updateMaxNbRow = 2;
  public Object synchroMetaData;
  public Object updateCollectionObject;
  public boolean updateSynchroDBCall = false;
  /**
   * 
   */
  public TestDummyFODBSyncTarget() {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#connect(org.openmobileis.synchro.security.auth.Credential)
   */
  public void connect(Credential cred) throws UserNotFoundException, ServiceException {
    connectCall = true;
    deleteCollectionObjectId = null;
    disconnectCall = false;
    updateCollectionObject = null;
    synchroMetaData = null;
    updateSynchroDBCall = false;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#deleteCollectionObject(java.lang.String)
   */
  public void deleteCollectionObject(String id) throws OpenMSPException {
    deleteCollectionObjectId = id;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#disconnect()
   */
  public void disconnect() {
    disconnectCall = true;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#getAllCollectionObject()
   */
  public Array getAllCollectionObject() throws OpenMSPException {
    Array ret = new Array();
    DummyObject dum = new DummyObject("COMPLETE");
    dum.value = "complete synchro";
    ret.add(dum);
    return ret;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#getAllModifiedAtomicObjectSince(long)
   */
  public SynchroAtomicObject[] getAllModifiedAtomicObjectSince(long syncnumber) throws OpenMSPException {
    if (currentAO != null)  {
      SynchroAtomicObject[] ret = new SynchroAtomicObject[1];
      ret[0] = currentAO;
      return ret;
    } else  {
      return new SynchroAtomicObject[0];
    }
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#getCollectionName()
   */
  public String getCollectionName() {
    return "dummy";
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#getCollectionObjectWithId(java.lang.String)
   */
  public Object getCollectionObjectWithId(String id) throws OpenMSPException {
    DummyObject ret = new DummyObject(id);
    ret.value = "getCollectionObjectWithId";
    return ret;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#getConflicResolver()
   */
  public SynchroConflicResolver getConflicResolver() {
    return new AlwaysUpdateServerSynchroConflicResolver();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#getUpdateMaxNbRow()
   */
  public int getUpdateMaxNbRow() {
    return updateMaxNbRow;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#setSendSynchroMetaData(java.lang.Object)
   */
  public void setSendSynchroMetaData(Object metadata) throws OpenMSPException {
    synchroMetaData = metadata;

  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#updateCollectionObject(java.lang.Object)
   */
  public void updateCollectionObject(Object obj) throws OpenMSPException {
    updateCollectionObject = obj;

  }

  /* (non-Javadoc)
   * @see org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget#updateSynchroDB(org.openmobileis.database.fastobjectdb.FastObjectDB)
   */
  public void updateSynchroDB(FastObjectDB db) throws OpenMSPException {
    updateSynchroDBCall = true;
  }

}
