/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.test.mock.common;

import org.openmobileis.common.user.User;
import org.openmobileis.common.user.UserAlreadyExistException;
import org.openmobileis.common.user.UserManagerFactory;
import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.synchro.security.auth.Credential;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class MockUserManagerFactory implements UserManagerFactory {
  private Credential cred;
  private String group;
  private String profilid;
  /**
   * 
   */
  public MockUserManagerFactory(String group, String profilid, Credential cred) {
    this.cred = cred;
    this.group = group;
    this.profilid = profilid;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.common.user.UserManagerFactory#authenticateUser(java.lang.String, java.lang.String)
   */
  public String authenticateUser(String login, String pass) throws ServiceException, UserNotFoundException {
    if (login.equals(cred.getPrincipal()) && pass.equals(cred.getPassword()))   {
      return "1";
    }
    throw new UserNotFoundException();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.common.user.UserManagerFactory#deleteUser(java.lang.String)
   */
  public void deleteUser(String userID) throws ServiceException, UserNotFoundException {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.common.user.UserManagerFactory#getUser(java.lang.String)
   */
  public User getUser(String userID) throws ServiceException, UserNotFoundException {
    User user = new User("1", cred.getPrincipal(), cred.getPassword());
    user.setGroup(this.group);
    user.setProfil(this.profilid);
    return user;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.common.user.UserManagerFactory#storeUser(org.openmobileis.common.user.User)
   */
  public void storeUser(User user) throws ServiceException, UserAlreadyExistException {
  }

}
