/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.test.mock.embedded;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.security.Principal;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Map;
import java.util.Vector;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletInputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public class MockHttpServletRequest implements HttpServletRequest {
  public String authType;
  public  String contextPath;
  public Cookie[] cookies;
  public long dateHeader = 0;
  public String method;
  public String pathinfo;
  public String query;
  public String requestURI;
  public String servletPath;
  public String characterEncoding;
  public String contentType;
  public ServletInputStream inputStream;
  public Locale locale;
  public String protocol;
  public String realPath;
  
  private Hashtable attributes = new Hashtable();
  private Hashtable parameters = new Hashtable();;

  public Vector headerNames = new Vector();
  public Vector headerValues = new Vector();
  
  
  /**
   * 
   */
  public MockHttpServletRequest() {
    authType =  HttpServletRequest.BASIC_AUTH;
    contextPath = "";
    cookies = new Cookie[0];
    locale = Locale.US;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getAuthType()
   */
  public String getAuthType() {
    return authType;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getContextPath()
   */
  public String getContextPath() {
    return contextPath;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getCookies()
   */
  public Cookie[] getCookies() {
    return cookies;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getDateHeader(java.lang.String)
   */
  public long getDateHeader(String arg0) {
    return dateHeader;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getHeader(java.lang.String)
   */
  public String getHeader(String name) {
    int i = headerNames.indexOf(name.toLowerCase());
    if (i == -1)
        return null;
    return (String) headerValues.elementAt(i);
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getHeaderNames()
   */
  public Enumeration getHeaderNames() {
    return headerNames.elements();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getHeaders(java.lang.String)
   */
  public Enumeration getHeaders(String arg0) {
    return headerValues.elements();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getIntHeader(java.lang.String)
   */
  public int getIntHeader(String name) {
    String val = getHeader(name);
    if (val == null)
        return -1;
    
    // We don't use any try catch, as this method must return
    // a NumberFormatException in case of problem
    // (According to the Servlet 2.4 specifications)
    return Integer.parseInt(val);
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getMethod()
   */
  public String getMethod() {
    return method;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getPathInfo()
   */
  public String getPathInfo() {
    return pathinfo;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getPathTranslated()
   */
  public String getPathTranslated() {
    return getRealPath(getPathInfo());
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getQueryString()
   */
  public String getQueryString() {
    return query;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getRemoteUser()
   */
  public String getRemoteUser() {
    return null;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getRequestURI()
   */
  public String getRequestURI() {
    return requestURI;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getRequestURL()
   */
  public StringBuffer getRequestURL() {
    return new StringBuffer("http://localhost:8080").append(getRequestURI());
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getRequestedSessionId()
   */
  public String getRequestedSessionId() {
    return this.getSession().getId();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getServletPath()
   */
  public String getServletPath() {
    return servletPath;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getSession()
   */
  public HttpSession getSession() {
    return this.getSession(true);
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getSession(boolean)
   */
  public HttpSession getSession(boolean arg0) {
    // Only one seesion context
    SessionContext session = SessionContextManager.getManager().getSessionContext("OPENMOBILEISSESSION");
    if (session == null)  {
      synchronized (SessionContextManager.getManager()) {
        session = SessionContextManager.getManager().getSessionContext("OPENMOBILEISSESSION");
        if (session == null)  {
          session = SessionContextManager.getManager().createSessionContext("OPENMOBILEISSESSION");
        }
      }
    }
    SessionContextManager.getManager().joinSessionContext(session.getId());
    return SessionContextManager.getManager().getSessionContext();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#getUserPrincipal()
   */
  public Principal getUserPrincipal() {
    return null;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#isRequestedSessionIdFromCookie()
   */
  public boolean isRequestedSessionIdFromCookie() {
    return false;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#isRequestedSessionIdFromURL()
   */
  public boolean isRequestedSessionIdFromURL() {  
    return false;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#isRequestedSessionIdFromUrl()
   */
  public boolean isRequestedSessionIdFromUrl() {
    return false;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#isRequestedSessionIdValid()
   */
  public boolean isRequestedSessionIdValid() {
    return false;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletRequest#isUserInRole(java.lang.String)
   */
  public boolean isUserInRole(String arg0) {
    return false;
  }
 
  public Object getAttribute(String name) {
    return attributes.get(name);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getAttribute(java.lang.String)
   */
  public void setAttribute(String name, Object o) {
    attributes.put(name, o);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getAttributeNames()
   */
  public Enumeration getAttributeNames() {
    return attributes.keys();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getCharacterEncoding()
   */
  public String getCharacterEncoding() {
    return characterEncoding;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getContentLength()
   */
  public int getContentLength() {
    return 0;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getContentType()
   */
  public String getContentType() {
    return contentType;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getInputStream()
   */
  public ServletInputStream getInputStream() throws IOException {
    return inputStream;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getLocalAddr()
   */
  public String getLocalAddr() {
     return null;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getLocalName()
   */
  public String getLocalName() {
    return null;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getLocalPort()
   */
  public int getLocalPort() {
    return 8080;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getLocale()
   */
  public Locale getLocale() {
    return locale;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getLocales()
   */
  public Enumeration getLocales() {
    return null;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getParameter(java.lang.String)
   */
  public String getParameter(String name) {
    String[] params = getParameterValues(name);
    if (params == null || params.length == 0)
        return null;

    return params[0];
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getParameterMap()
   */
  public Map getParameterMap() {
    return (java.util.Map) parameters;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getParameterNames()
   */
  public Enumeration getParameterNames() {
    return parameters.keys();
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getParameterValues(java.lang.String)
   */
  public String[] getParameterValues(String name) {
    if (parameters == null)
      getParameterNames();

    return (String[]) parameters.get(name);
  }
  
  public void addParameter(String name, String value)    {
    String[] params = this.getParameterValues(name);
    if (params == null) {
      params = new String[]{value};
      parameters.put(name, params);
    } else  {
      String[] newparam = new String[params.length+1];
      System.arraycopy(params, 0, newparam, 0, params.length);
      newparam[params.length] = value;
      parameters.put(name, newparam);
    }
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getProtocol()
   */
  public String getProtocol() {
    return protocol;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getReader()
   */
  public BufferedReader getReader() throws IOException {
    if (characterEncoding != null)
    try {
      return new BufferedReader(new InputStreamReader(inputStream, characterEncoding));
    } catch (UnsupportedEncodingException uee) {
    }
    return new BufferedReader(new InputStreamReader(inputStream));
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getRealPath(java.lang.String)
   */
  public String getRealPath(String arg0) {
    return realPath;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getRemoteAddr()
   */
  public String getRemoteAddr() {
    return null;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getRemoteHost()
   */
  public String getRemoteHost() {
    return "localhost";
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getRemotePort()
   */
  public int getRemotePort() {
    return 0;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getRequestDispatcher(java.lang.String)
   */
  public RequestDispatcher getRequestDispatcher(String arg0) {
    return null;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getScheme()
   */
  public String getScheme() {
    return "http";
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getServerName()
   */
  public String getServerName() {
    return "localhost";
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#getServerPort()
   */
  public int getServerPort() {
    return 8080;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#isSecure()
   */
  public boolean isSecure() {
    return false;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#removeAttribute(java.lang.String)
   */
  public void removeAttribute(String arg0) {
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletRequest#setCharacterEncoding(java.lang.String)
   */
  public void setCharacterEncoding(String arg0) throws UnsupportedEncodingException {
    characterEncoding = arg0;
  }

}
