/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.test.mock.embedded;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;
import java.util.Locale;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.util.log.LogManager;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public class MockHttpServletResponse implements HttpServletResponse {
  public final static String CONTENTLENGTH = "Content-Length";
  public final static String CONTENTTYPE = "Content-Type";

  public Hashtable headers = new Hashtable();
  protected static final SimpleDateFormat datefmt = new SimpleDateFormat(
      "EEE, dd-MMM-yyyy HH:mm:ss 'GMT'");
  
  public int status;
  public String statusMessage;
  public Locale locale;
  public ByteArrayOutputStream outStream;
  public String characterEncoding;


  /**
   * 
   */
  public MockHttpServletResponse() {
    locale = Locale.US;
    outStream = new ByteArrayOutputStream();
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#addCookie(javax.servlet.http.Cookie)
   */
  public void addCookie(Cookie arg0) {
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#addDateHeader(java.lang.String, long)
   */
  public void addDateHeader(String header, long date) {
    addHeader(header, datefmt.format(new Date(date)));
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#addHeader(java.lang.String, java.lang.String)
   */
  public void addHeader(String header, String value) {
    Object o = headers.get(header);
    if (o == null)
        setHeader(header, value);
    else {
        if (o instanceof String[]) {
            String[] oldVal = (String[]) o;
            String[] newVal = new String[oldVal.length + 1];
            System.arraycopy(oldVal, 0, newVal, 0, oldVal.length);
            newVal[oldVal.length] = value;
            headers.put(header, newVal);
        } else if (o instanceof String) {
            String[] newVal = new String[2];
            newVal[0] = (String) o;
            newVal[1] = value;
            headers.put(header, newVal);
        } else
            throw new RuntimeException("Invalid content of header hash - "
                    + o.getClass().getName());
    }
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#addIntHeader(java.lang.String, int)
   */
  public void addIntHeader(String header, int value) {
    addHeader(header, Integer.toString(value));
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#containsHeader(java.lang.String)
   */
  public boolean containsHeader(String name) {
    return headers.contains(name);
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#encodeRedirectURL(java.lang.String)
   */
  public String encodeRedirectURL(String url) {
    return url;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#encodeRedirectUrl(java.lang.String)
   */
  public String encodeRedirectUrl(String url) {
    return url;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#encodeURL(java.lang.String)
   */
  public String encodeURL(String url) {
    return url;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#encodeUrl(java.lang.String)
   */
  public String encodeUrl(String url) {
    return url;
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#sendError(int)
   */
  public void sendError(int resCode) throws IOException {
    setStatus(resCode);
    LogManager.traceError(0, "MockHttpServletResponse Error code:"+resCode);
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#sendError(int, java.lang.String)
   */
  public void sendError(int resCode, String resMessage) throws IOException {
    setStatus(resCode, resMessage);
    LogManager.traceError(0, "MockHttpServletResponse Error code:"+resCode+" message :"+resMessage);
  }
  
  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#sendRedirect(java.lang.String)
   */
  public void sendRedirect(String url) throws IOException {
    LogManager.traceWarning(0, "MockHttpServletResponse send redirect call for url:"+url);
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#setDateHeader(java.lang.String, long)
   */
  public void setDateHeader(String name, long value) {
    setHeader(name, datefmt.format(new Date(value)));
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#setHeader(java.lang.String, java.lang.String)
   */
  public void setHeader(String name, String value) {
    headers.put(name, value);
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#setIntHeader(java.lang.String, int)
   */
  public void setIntHeader(String name, int value) {
    setHeader(name, Integer.toString(value));
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#setStatus(int)
   */
  public void setStatus(int resCode) {
    setStatus(resCode, "");
  }

  /* (non-Javadoc)
   * @see javax.servlet.http.HttpServletResponse#setStatus(int, java.lang.String)
   */
  public void setStatus(int resCode, String resMessage) {
    this.status = resCode;
    this.statusMessage = resMessage;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#flushBuffer()
   */
  public void flushBuffer() throws IOException {
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#getBufferSize()
   */
  public int getBufferSize() {
    return 0;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#getCharacterEncoding()
   */
  public String getCharacterEncoding() {
    String ct = (String) headers.get(CONTENTTYPE);
    if (ct != null) {
        int scp = ct.indexOf(';');
        if (scp > 0) {
            scp = ct.toLowerCase().indexOf("charset=", scp);
            if (scp >= 0) {
                ct = ct.substring(scp + 8);
                scp = ct.indexOf(' ');
                if (scp > 0)
                    ct = ct.substring(0, scp);
                scp = ct.indexOf(';');
                if (scp > 0)
                    ct = ct.substring(0, scp);
                return ct;
            }
        }
    }
    return null;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#getContentType()
   */
  public String getContentType() {
    return getHeader(CONTENTTYPE);
  }
  public String getHeader(String name) {
    
    return (String) headers.get(name);
}

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#getLocale()
   */
  public Locale getLocale() {
    return locale;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#getOutputStream()
   */
  public ServletOutputStream getOutputStream() throws IOException {
    return new MockServletOutputStream(outStream);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#getWriter()
   */
  public PrintWriter getWriter() throws IOException {
    PrintWriter printWriter = null; 
    String encoding = getCharacterEncoding();
    if (encoding != null) {
      //return new PrintWriter(new OutputStreamWriter(out, encoding));
        printWriter = new PrintWriter(new OutputStreamWriter(outStream, encoding));
    } else {
      //return new PrintWriter(out);
        printWriter = new PrintWriter(outStream);
    }
    return printWriter;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#isCommitted()
   */
  public boolean isCommitted() {
    return false;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#reset()
   */
  public void reset() {
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#resetBuffer()
   */
  public void resetBuffer() {
 }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#setBufferSize(int)
   */
  public void setBufferSize(int arg0) {
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#setCharacterEncoding(java.lang.String)
   */
  public void setCharacterEncoding(String enc) {
    characterEncoding = enc;
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#setContentLength(int)
   */
  public void setContentLength(int length) {
    setIntHeader(CONTENTLENGTH, length);
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#setContentType(java.lang.String)
   */
  public void setContentType(String type) {
    setHeader(CONTENTTYPE, type != null ? type : "Unknown");
  }

  /* (non-Javadoc)
   * @see javax.servlet.ServletResponse#setLocale(java.util.Locale)
   */
  public void setLocale(Locale arg0) {
    this.locale = locale;
  }

}
