/**
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.test.mock.synchro;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener;
import org.openmobileis.synchro.openmsp.client.core.OpenMSPSyncMessageFactory;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.Header;
import org.openmobileis.synchro.openmsp.protocol.Message;
import org.openmobileis.synchro.openmsp.protocol.MessageFactory;
import org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener;
import org.openmobileis.synchro.openmsp.server.synctarget.SyncTargetAnswer;
import org.openmobileis.synchro.openmsp.server.util.ZipEntryMemoryFile;
import org.openmobileis.synchro.security.auth.Credential;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class SimpleMockSynchroManager {
  class ZipArrayEntryFile {
    ByteArrayOutputStream array;

    ZipOutputStream zip;

    String fileName;

    public int hashCode() {
      return fileName.hashCode();
    }

    public boolean equals(Object obj) {
      if (obj instanceof ZipArrayEntryFile) {
        return ((ZipArrayEntryFile) obj).fileName.equals(this.fileName);
      }
      return false;
    }

  }

  private OpenMSPSyncListener listener;

  private OpenMSPSynchroTargetListener target;
  private String terminalSynchroDirectory;

  private long sessionNumber = 0;

  public SimpleMockSynchroManager(String terminalSynchroDirectory) {
    this.terminalSynchroDirectory = terminalSynchroDirectory;
    File newDir = new File(this.terminalSynchroDirectory+"/dbfiles");
    if (!newDir.exists()) newDir.mkdirs();
    File file = new File (terminalSynchroDirectory+"/filedir");
    if (!file.exists()) file.mkdirs();
  }

  public void registerTerminalSynchroListener(OpenMSPSyncListener listener) {
    this.listener = listener;
  }

  public OpenMSPSyncListener getTerminalSynchroListener() {
    return listener;
  }

  public void registerServerSynchroTarget(OpenMSPSynchroTargetListener target) {
    this.target = target;
  }

  public OpenMSPSynchroTargetListener getServerSynchroTarget() {
    return target;
  }

  public void synchronize(String group, Credential cred, SynchroDescriptor synchrodescriptor) throws OpenMSPException {
    PropertiesManager.getManager().addProperty("org.openmobileis.synchro.client.openmsp.numsyncpropsfile", System.getProperty("user.dir") + File.separator +"numsync.txt");
    OpenMSPSyncMessageFactory messageFactory = new OpenMSPSyncMessageFactory();
    Array listenerList = new Array();
    listenerList.add(this.listener);
    try {
      messageFactory.beginSynchro(cred, synchrodescriptor);
      listener.startSync(cred, synchrodescriptor);
      
      while (true) {
        Message message = messageFactory.getOpenMSPMessage(listenerList);
        if (message.isEmpty()) {
          //get next listener phase.
          break;
        }
        Message CMLAnswer = MessageFactory.getFactory().createMessage(new Header(this.getNextSessionNumber(), target.getTargetName(), listener.getSyncName(), null, null));
        message.resetCursor();
        while (message.hasMoreMessage()) {
          ContainerMessage CMLRequest = message.nextMessage();
          SyncTargetAnswer listenerAnswer = target.processCommand(cred, CMLRequest);
          //manage synchronized files.
          HashMap zipFileMap = new HashMap(10);
          if (listenerAnswer.answerfiles != null)   {
            for (int i=0; i<listenerAnswer.answerfiles.length; i++) { 
               if (listenerAnswer.answerfiles[i] instanceof ZipEntryMemoryFile)    {
                String zipFileName = ((ZipEntryMemoryFile) listenerAnswer.answerfiles[i]).getZipFileName();
                ZipArrayEntryFile zipEntry = (ZipArrayEntryFile) zipFileMap.get(zipFileName);
                if (zipEntry == null) {
                  zipEntry = new ZipArrayEntryFile();
                  zipEntry.array = new ByteArrayOutputStream(100000);
                  zipEntry.zip = new ZipOutputStream(zipEntry.array);
                  zipEntry.fileName = zipFileName;
                  zipFileMap.put(zipFileName, zipEntry);
                }
                ZipEntry entry = new ZipEntry(FileUtilities.convertToUnixFomat(listenerAnswer.answerfiles[i].getFileName()));
                entry.setTime(listenerAnswer.answerfiles[i].getFileDate());
                entry.setMethod(ZipEntry.DEFLATED);
                zipEntry.zip.putNextEntry(entry);
                zipEntry.zip.write(listenerAnswer.answerfiles[i].getFileData());
                zipEntry.zip.closeEntry();
              } else    {
                String path = this.terminalSynchroDirectory+listenerAnswer.answerfiles[i].getFilePath(); //
                File newDir = new File(path);
                if (!newDir.exists()) newDir.mkdirs();
                FileOutputStream out = new FileOutputStream(path+listenerAnswer.answerfiles[i].getFileName());
                try   {
                  out.write(listenerAnswer.answerfiles[i].getFileData());
                } finally {
                  out.flush();
                  out.close();
                } //end try / finally
              } //end if else
            } //end for
            //unzip zipped collection files.
            Iterator iter = zipFileMap.keySet().iterator();
            while (iter.hasNext())  {
              String filename = (String) iter.next();
              ZipArrayEntryFile zipEntry = (ZipArrayEntryFile) zipFileMap.get(filename);
              zipEntry.zip.flush();
              zipEntry.zip.close();
              FileOutputStream out = new FileOutputStream(this.terminalSynchroDirectory+filename);
              try   {
                out.write(zipEntry.array.toByteArray());
              } finally {
                out.flush();
                out.close();
              } //end try / finally
            }
          }
          
          if (listenerAnswer.containerMessage != null) {
            int size = listenerAnswer.containerMessage.length;
            for (int i = 0; i < size; i++) {
              CMLAnswer.add(listenerAnswer.containerMessage[i]);
            }
          }
        }

        // third step : Process responses
        messageFactory.receiveOpenMSPMessage(listenerList, CMLAnswer);
      }
      listener.endSync();
    } catch (Throwable ex) {
      listener.notifySynchroFailure();
      throw new OpenMSPException(ex);
    } finally {
      messageFactory.endSynchro();
      //remove created files
      File file = new File(System.getProperty("user.dir") + File.separator +"numsync.txt");
      if (file.exists()) file.delete();

    }

  }

  private long getNextSessionNumber() {
    return sessionNumber + 1;
  }

}
